//
//  BreakBlocksViewController.m
//  BreakBlocks
//
//  Created by Dayle Flowers on 1/6/12.
//  Copyright (c) 2012 Greater Good Games. All rights reserved.
//

#import <QuartzCore/CADisplayLink.h>
#import "xViewController.h"
#import "xView.h"
#import "iOSMain.h"
#import "Input.h"
#import "Texture2D.h"
#import "CToObjectiveC.h"
#import "Paper.h"

BOOL loadShader(NSString* VertexShaderName, NSString* FragShaderName, uint* ProgramIndex, int AttribCount, char** AttribNames, int** AttribLocations, int UniformCount, char** UniformNames, int** UniformLocations);
BOOL compileShader(GLuint* shader, GLenum type, NSString* file);
BOOL linkProgram(GLuint prog);
BOOL validateProgram(GLuint prog);

@interface xViewController ()
{
	double previousTime;
}

@property (strong, nonatomic) EAGLContext *context;
@property (nonatomic, assign) CADisplayLink *displayLink;
@property (strong, nonatomic) Texture2D *stamp;
@property (strong, nonatomic) Texture2D *colorbar;
@property (strong, nonatomic) Texture2D *bricks;

- (void)resetFramebuffer;
@end

@implementation xViewController

@synthesize animating;
@synthesize context = _context;
@synthesize displayLink;
@synthesize stamp = _stamp;
@synthesize colorbar = _colorbar;
@synthesize bricks = _bricks;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];

    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)dealloc
{
    [_context release];
    [_bricks release];
    [super dealloc];
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];
	
	previousTime = CACurrentMediaTime();

    self.context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
	
    if (!self.context)
        NSLog(@"Failed to create ES context");
	
    xView *view = (xView *)self.view;
    [view setContext:self.context];
    [view setFramebuffer];
	
	char* DDSAttribNames[] = {"position", "inputTextureCoordinate"};
	int* DDSAttribLocations[] = {&g_xGraphics.m_xGrayScalePixel.m_iPosition, &g_xGraphics.m_xGrayScalePixel.m_iTextureCoord};
	char* DDSUniformNames[] = {"modelViewProjectionMatrix", "textureIndex", "blendColor"};
	int* DDSUniformLocations[] = {&g_xGraphics.m_xGrayScalePixel.m_iMVP, &g_xGraphics.m_xGrayScalePixel.m_iTexture, &g_xGraphics.m_xGrayScalePixel.m_iColor};
	loadShader(@"DirectDisplayShader", @"DirectDisplayShader", &g_xGraphics.m_uiGrayScalePixelProgramIndex, 2, DDSAttribNames, DDSAttribLocations, 3, DDSUniformNames, DDSUniformLocations);
	
	int* DCSAttribLocations[] = {&g_xGraphics.m_xDualColorPixel.m_iPosition, &g_xGraphics.m_xDualColorPixel.m_iTextureCoord};
	char* DCSUniformNames[] = {"modelViewProjectionMatrix", "textureIndex", "secondColor", "colorDifference"};
	int* DCSUniformLocations[] = {&g_xGraphics.m_xDualColorPixel.m_iMVP, &g_xGraphics.m_xDualColorPixel.m_iTexture, &g_xGraphics.m_xDualColorPixel.m_iSecondColor, &g_xGraphics.m_xDualColorPixel.m_iColorDifference};
	loadShader(@"DirectDisplayShader", @"DualColorShader", &g_xGraphics.m_uiDualColorPixelProgramIndex, 2, DDSAttribNames, DCSAttribLocations, 4, DCSUniformNames, DCSUniformLocations);
	
	PaperEngineMain(self);
	
	UISwipeGestureRecognizer *swipeLeft = [UISwipeGestureRecognizer alloc];
	[swipeLeft initWithTarget:self action:@selector(didSwipeLeft:)];
	[swipeLeft setDirection:UISwipeGestureRecognizerDirectionLeft];
	swipeLeft.delegate = self;
	
	UISwipeGestureRecognizer *swipeRight = [UISwipeGestureRecognizer alloc];
	[swipeRight initWithTarget:self action:@selector(didSwipeRight:)];
	[swipeRight setDirection:UISwipeGestureRecognizerDirectionRight];
	swipeRight.delegate = self;
	
	UISwipeGestureRecognizer *swipeUp = [UISwipeGestureRecognizer alloc];
	[swipeUp initWithTarget:self action:@selector(didSwipeUp:)];
	[swipeUp setDirection:UISwipeGestureRecognizerDirectionUp];
	swipeUp.delegate = self;
	
	UISwipeGestureRecognizer *swipeDown = [UISwipeGestureRecognizer alloc];
	[swipeDown initWithTarget:self action:@selector(didSwipeDown:)];
	[swipeDown setDirection:UISwipeGestureRecognizerDirectionDown];
	swipeDown.delegate = self;
	
	UITapGestureRecognizer *doubleTap = [UITapGestureRecognizer alloc];
	[doubleTap initWithTarget:self action:@selector(didDoubleTap:)];
	[doubleTap setNumberOfTapsRequired:2];
	doubleTap.delegate = self;
	
	[view setGestureRecognizers:[NSArray arrayWithObjects:swipeLeft, swipeRight, swipeUp, swipeDown, doubleTap, nil]];
	
	[swipeLeft release];
	[swipeRight release];
	[swipeUp release];
	[swipeDown release];
	[doubleTap release];

	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : nil;
	strcpy(g_strDocumentDir, [basePath UTF8String]);
}

- (void)viewDidUnload
{
    [super viewDidUnload];

    [EAGLContext setCurrentContext:self.context];
	
	MainDestroy();

    if ([EAGLContext currentContext] == self.context)
        [EAGLContext setCurrentContext:nil];

    self.context = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationLandscapeRight);
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
	[super didRotateFromInterfaceOrientation:fromInterfaceOrientation];
	
	g_xPaperAPI.m_xPlatform.m_bIsPhone = [[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone;
	
	MainViewChanged(self.view.bounds.size.width, self.view.bounds.size.height);
}

- (void)startAnimation
{
    if (!animating)
	{
        CADisplayLink *aDisplayLink = [[UIScreen mainScreen] displayLinkWithTarget:self selector:@selector(drawFrame)];
        [aDisplayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSDefaultRunLoopMode];
        self.displayLink = aDisplayLink;
        
        animating = TRUE;
    }
}

- (void)stopAnimation
{
    if (animating)
	{
        [self.displayLink invalidate];
        self.displayLink = nil;
        animating = FALSE;
    }
}

- (void)drawFrame
{
	g_xPaperAPI.m_dwCurrentTime = CACurrentMediaTime() * 1000;
	
	g_dwCurrentTime = g_xPaperAPI.m_dwCurrentTime;

    [(xView*)self.view setFramebuffer];

	MainUpdate();
	//MainDraw();
	
	[(xView*)self.view presentFramebuffer];
	
	g_xPaperAPI.m_dwLastTime = g_xPaperAPI.m_dwCurrentTime;
}

- (void)resetFramebuffer
{
	[(xView*)self.view setFramebuffer];
}

void ResetFramebuffer(void *pViewController)
{
	[(xViewController*)pViewController resetFramebuffer];
}

BOOL loadShader(NSString* VertexShaderName, NSString* FragShaderName, uint* ProgramIndex, int AttribCount, char** AttribNames, int** AttribLocations, int UniformCount, char** UniformNames, int** UniformLocations)
{
    GLuint vertShader, fragShader;
    NSString *vertShaderPathname, *fragShaderPathname;

	*ProgramIndex = glCreateProgram();
    
    // Create and compile vertex shader.
    vertShaderPathname = [[NSBundle mainBundle] pathForResource:VertexShaderName ofType:@"vsh"];
    if (!compileShader(&vertShader, GL_VERTEX_SHADER, vertShaderPathname)) {
        NSLog(@"Failed to compile vertex shader");
        return NO;
    }
    
    // Create and compile fragment shader.
    fragShaderPathname = [[NSBundle mainBundle] pathForResource:FragShaderName ofType:@"fsh"];
    if (!compileShader(&fragShader, GL_FRAGMENT_SHADER, fragShaderPathname)) {
        NSLog(@"Failed to compile fragment shader");
        return NO;
    }
    
    // Attach vertex shader to program.
    glAttachShader(*ProgramIndex, vertShader);
    
    // Attach fragment shader to program.
    glAttachShader(*ProgramIndex, fragShader);

    // Link program.
    if (!linkProgram(*ProgramIndex)) {
        NSLog(@"Failed to link program: %d", *ProgramIndex);
        
        if (vertShader) {
            glDeleteShader(vertShader);
            vertShader = 0;
        }
		
        if (fragShader) {
            glDeleteShader(fragShader);
            fragShader = 0;
        }
		
        if (*ProgramIndex) {
            glDeleteProgram(*ProgramIndex);
            *ProgramIndex = 0;
        }
        
        return NO;
    }
    
	// Get attribute locations.
	for (int i = 0; i < AttribCount; i++) {
		*(AttribLocations[i]) = glGetAttribLocation(*ProgramIndex, AttribNames[i]);
	}
    
    // Get uniform locations.
	for (int i = 0; i < UniformCount; i++) {
		*(UniformLocations[i]) = glGetUniformLocation(*ProgramIndex, UniformNames[i]);
	}
    
    // Release vertex and fragment shaders.
    if (vertShader) {
        glDetachShader(*ProgramIndex, vertShader);
        glDeleteShader(vertShader);
    }

    if (fragShader) {
        glDetachShader(*ProgramIndex, fragShader);
        glDeleteShader(fragShader);
    }
    
    return YES;
}

BOOL compileShader(GLuint* shader, GLenum type, NSString* file)
{
    GLint status;
    const GLchar *source;
    
    source = (GLchar *)[[NSString stringWithContentsOfFile:file encoding:NSUTF8StringEncoding error:nil] UTF8String];
    if (!source) {
        NSLog(@"Failed to load vertex shader");
        return NO;
    }
    
    *shader = glCreateShader(type);
    glShaderSource(*shader, 1, &source, NULL);
    glCompileShader(*shader);
    
#if defined(DEBUG)
    GLint logLength;
    glGetShaderiv(*shader, GL_INFO_LOG_LENGTH, &logLength);
    if (logLength > 0) {
        GLchar *log = (GLchar *)malloc(logLength);
        glGetShaderInfoLog(*shader, logLength, &logLength, log);
        NSLog(@"Shader compile log:\n%s", log);
        free(log);
    }
#endif
    
    glGetShaderiv(*shader, GL_COMPILE_STATUS, &status);
    if (status == 0) {
        glDeleteShader(*shader);
        return NO;
    }
    
    return YES;
}

BOOL linkProgram(GLuint prog)
{
    GLint status;
    glLinkProgram(prog);
    
#if defined(DEBUG)
    GLint logLength;
    glGetProgramiv(prog, GL_INFO_LOG_LENGTH, &logLength);
    if (logLength > 0) {
        GLchar *log = (GLchar *)malloc(logLength);
        glGetProgramInfoLog(prog, logLength, &logLength, log);
        NSLog(@"Program link log:\n%s", log);
        free(log);
    }
#endif
    
    glGetProgramiv(prog, GL_LINK_STATUS, &status);
    if (status == 0) {
        return NO;
    }
    
    return YES;
}

BOOL validateProgram(GLuint prog)
{
    GLint logLength, status;
    
    glValidateProgram(prog);
    glGetProgramiv(prog, GL_INFO_LOG_LENGTH, &logLength);
    if (logLength > 0) {
        GLchar *log = (GLchar *)malloc(logLength);
        glGetProgramInfoLog(prog, logLength, &logLength, log);
        NSLog(@"Program validate log:\n%s", log);
        free(log);
    }
    
    glGetProgramiv(prog, GL_VALIDATE_STATUS, &status);
    if (status == 0) {
        return NO;
    }
    
    return YES;
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	for(UITouch *touch in touches)
	{
		CGPoint loc = [touch locationInView:self.view];
		
		MouseDown((int)touch, loc.x, loc.y);
	}
}

-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	for(UITouch *touch in touches)
	{
		CGPoint loc = [touch locationInView:self.view];
		
		MouseMove((int)touch, loc.x, loc.y);
	}
}

-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	for(UITouch *touch in touches)
	{
		CGPoint loc = [touch locationInView:self.view];
		
		MouseUp((int)touch, loc.x, loc.y);
	}
}

-(void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
	for(UITouch *touch in touches)
	{
		CGPoint loc = [touch locationInView:self.view];
		
		MouseUp((int)touch, loc.x, loc.y);
	}
}

- (BOOL) gestureRecognizerShouldBegin:(UIGestureRecognizer *)gestureRecognizer
{
    return YES;
}

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
{
	CGPoint loc = [touch locationInView:self.view];
	
	return !GameWantsTouch(loc.x, loc.y);
}

- (void) didSwipeLeft:(UIGestureRecognizer*)recognizer
{
	MainSwipe(0);
}

- (void) didSwipeRight:(UIGestureRecognizer*)recognizer
{
	MainSwipe(1);
}

- (void) didSwipeUp:(UIGestureRecognizer*)recognizer
{
	MainSwipe(2);
}

- (void) didSwipeDown:(UIGestureRecognizer*)recognizer
{
	MainSwipe(3);
}

- (void) didDoubleTap:(UIGestureRecognizer*)recognizer
{
	MainSwipe(4);
}

void LoadAppStore()
{
	[[UIApplication sharedApplication] openURL: [NSURL URLWithString:@"http://itunes.apple.com/WebObjects/MZStore.woa/wa/viewSoftware?id=545044287&mt=8&uo=6"]];
}

@end
