#include <unistd.h>
#include <stdio.h>
#include <math.h>
#include "Paper.h"
#include "iOSMain.h"
#include "Main.h"
#include "Input.h"
#include "CToObjectiveC.h"
#include "peGame.h"

typedef struct
{
    float m_f3Position[3];
    float m_f2TexCoords0[2];
} Vertex;

typedef struct
{
	float m_f4Color[4];
} Color;

typedef struct
{
	float m_fX;
	float m_fY;
	float m_fZ;
	float m_fWidth;
	float m_fHeight;
	
	GLuint m_uiVertexBuffer;
	GLuint m_uiTexture;
	Color m_colorBlend;
	
	int m_bEnableBlending;
	GLenum m_eSourceBlendMode;
	GLenum m_eDestBlendMode;
} TexturedQuad;

enum MySkyTouchOrigin
{
	MSTO_ColorBar,
	MSTO_DrawingArea,
};

const Vertex Vertices[] = {
    {{ 100, -100, 0}, {1, 0}},
    {{ 100,  100, 0}, {1, 1}},
    {{-100,  100, 0}, {0, 1}},
    {{-100, -100, 0}, {0, 0}}
};

const Vertex StampVertices[] = {
    {{ 20, -20, 0}, {1, 0}},
    {{ 20,  20, 0}, {1, 1}},
    {{-20,  20, 0}, {0, 1}},
    {{-20, -20, 0}, {0, 0}}
};

Vertex TexturedQuadVertices[] = {
    {{10,  0, 0}, {1, 0}},
    {{10, 10, 0}, {1, 1}},
    {{ 0, 10, 0}, {0, 1}},
    {{ 0,  0, 0}, {0, 0}}
};

const GLushort Indices[] = {
    0, 1, 2,
    2, 3, 0
};

void *g_pViewController;

Matrix4 g_matProjection;
Matrix4 g_matModelView;
Matrix4 g_matDrawingProjection;

GLuint g_uiVertexBuffer;
GLuint g_uiTexture;
Color g_colorNormalColor = {1.0, 1.0, 1.0, 1.0};

TexturedQuad g_quadStamp = {0, 0, -0.3, 40, 40, 0, 0, {0.0, 1.0, 1.0, 1.0}, 1, GL_ONE, GL_ONE_MINUS_SRC_ALPHA};

TexturedQuad g_quadQuads[1] = 
{
	{0, 0, -0.3, 512, 512, 0, 0, {1.0, 1.0, 1.0, 1.0}, 1, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA},
	//{0.00, 0.82, -0.3, 1.0, 0.18, 0, 0, {1.0, 1.0, 1.0, 1.0}, 0, 0, 0},
	//{0.01, 0.84, -0.3, 0.98, 0.14, 0, 0, {1.0, 1.0, 1.0, 1.0}, 0, 0, 0},
};

enum MySkyTouchOrigin g_eTouchOrigin[MAX_MICE];

float g_fRotation;

GLuint g_uiDrawingFrameBuffer = 0;

xPaper	g_xPaperAPI;
//WindowPtr			gGameWindow			= NULL;
//EventLoopTimerRef	gMainLoopTimer		= NULL;
//EventHandlerUPP		gEventHandlerUPP	= NULL;
//EventHandlerRef		gEventHandlerRef	= 0;

extern void GameInit();
extern void GamePreUpdate(DWORD dwTime);
extern void GameUpdate(double dTime);
extern void GameShutdown();

int PaperEngineMain(void *pViewController)
{
	g_pViewController = pViewController;

#ifdef __APPLE__
	{
		char path[255];
		CFURLRef url = CFBundleCopyBundleURL(CFBundleGetMainBundle());
		CFStringRef str = CFURLCopyFileSystemPath(url,kCFURLPOSIXPathStyle);
		CFStringGetCString(str,path,255,CFStringGetSystemEncoding());
		CFRelease(str);
		CFRelease(url);
		sprintf(path, "%s/", path);
		chdir(path);
	}
#endif

	xFileIO fileio;

#ifdef _DEBUG
	char temp[256] = "FreePlay", output[256];
	uint32_t encrypted[100] = {0};
	int i;

	//xEncryptString(temp, encrypted, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3);
	//xDecryptString(encrypted, 4, temp, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3);

/*	strcpy(temp, "BreakBlocks.lic");
	memset(output, 0, 255);
	memset(encrypted, 0, 400);
	xEncryptString(temp, encrypted, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3);
	for(i = 0; i < 4; i++)
		sprintf(output, "%s, %u", output, encrypted[i]);
*/
#endif

	g_xPaperAPI.Init();

	if(g_xPaperAPI.m_bRunning)
	{
		//Create Game Objects
		//Load Maps/Menus/etc.
		GameInit();

		g_xPaperAPI.m_xInput.m_lWindowOffsetX = g_xPaperAPI.m_xPlatform.GetScreenRect().left;
		g_xPaperAPI.m_xInput.m_lWindowOffsetY = g_xPaperAPI.m_xPlatform.GetScreenRect().top;

		g_xPaperAPI.m_dwLastTime = g_xPaperAPI.m_xPlatform.GetTime();

		//SetMainLoopEventTimer(GameLoop);
	
		//RunApplicationEventLoop();
	}
	
	return 0;
}

void MainViewChanged(float fWidth, float fHeight)
{
	g_xOptions.m_iScreenWidth = fWidth;
	g_xOptions.m_iScreenHeight = fHeight;
	
	g_xPaperAPI.m_xEngineSettings.m_iScreenWidth = fWidth;
	g_xPaperAPI.m_xEngineSettings.m_iScreenHeight = fHeight;
	
	g_xGraphics.m_iScreenWidth = fWidth;
	g_xGraphics.m_iScreenHeight = fHeight;
	
	g_xPaperAPI.UpdateScreenOffsetsAndRatios();

	Matrix4GenOrtho(&g_xGraphics.m_matProjection, 0, fWidth, 0, fHeight, 0.0, 10.0);
	Matrix4GenPerspective(&g_xGraphics.m_matPerspective, 50.0f, fWidth / fHeight, 0.5f, 300.0f);
}

bool MainGameWantsTouch(float fX, float fY)
{
	float x0, y0, x1, y1, x2, y2, width, height;
	
	if(!g_xGame.m_xPlayer.m_pColorButton[0] || !g_xGame.m_xPlayer.m_pColorButton[0]->m_xRenderable2D.m_bVisible)
		return 0;

	fX = (fX - g_xPaperAPI.m_iScreenOffsetX) / g_xPaperAPI.m_fScreenRatioX;
	fY = (fY - g_xPaperAPI.m_iScreenOffsetY) / g_xPaperAPI.m_fScreenRatioY;
	
	x0 = g_xGame.m_xPlayer.m_pColorButton[0]->m_xRenderable2D.m_fX;
	y0 = g_xGame.m_xPlayer.m_pColorButton[0]->m_xRenderable2D.m_fY;
	x1 = g_xGame.m_xPlayer.m_pColorButton[1]->m_xRenderable2D.m_fX;
	y1 = g_xGame.m_xPlayer.m_pColorButton[1]->m_xRenderable2D.m_fY;
	x2 = g_xGame.m_xPlayer.m_pColorButton[2]->m_xRenderable2D.m_fX;
	y2 = g_xGame.m_xPlayer.m_pColorButton[2]->m_xRenderable2D.m_fY;
	width = g_xGame.m_xPlayer.m_pColorButton[0]->m_xRenderable2D.m_fWidth;
	height = g_xGame.m_xPlayer.m_pColorButton[0]->m_xRenderable2D.m_fHeight;
	
	return (fX > x0 && fX < x0 + width && fY > y0 && fY < y0 + height) ||
		   (fX > x1 && fX < x1 + width && fY > y1 && fY < y1 + height) ||
		   (fX > x2 && fX < x2 + width && fY > y2 && fY < y2 + height);
}

void MainTouchOrDrag(int bNewTouch,  MouseStatus *pMouseStatus, int iIndex, float fX, float fY)
{
	g_xPaperAPI.m_xInput.HandleMouseButtonPressed(0, false);
	g_xPaperAPI.m_xInput.HandleMouseButtonPressed(0, pMouseStatus->m_bInUse);
	
	if(iIndex == 0)
	{
		g_xPaperAPI.m_xInput.m_lMouseX = (fX - g_xPaperAPI.m_iScreenOffsetX) / g_xPaperAPI.m_fScreenRatioX;
		g_xPaperAPI.m_xInput.m_lMouseY = (fY - g_xPaperAPI.m_iScreenOffsetY) / g_xPaperAPI.m_fScreenRatioY;
		g_xPaperAPI.m_xInput.m_lMouse2X = 999999;
		g_xPaperAPI.m_xInput.m_lMouse2Y = 999999;
	}
	else if(iIndex == 1)
	{
		g_xPaperAPI.m_xInput.m_lMouse2X = (fX - g_xPaperAPI.m_iScreenOffsetX) / g_xPaperAPI.m_fScreenRatioX;
		g_xPaperAPI.m_xInput.m_lMouse2Y = (fY - g_xPaperAPI.m_iScreenOffsetY) / g_xPaperAPI.m_fScreenRatioY;
	}
	
	//printf("MainTouchOrDrag %i %i %i %f,%f %i,%i\n", pMouseStatus->m_bInUse, iIndex, g_xPaperAPI.m_xInput.m_buttonMouseLeft.JustPressed(), fX, fY, g_xPaperAPI.m_xInput.m_lMouseX, g_xPaperAPI.m_xInput.m_lMouseY);
}

// 0 = Left, 1 = Right, 2 = Up, 3 = Down
void MainSwipe(int Direction)
{
	switch(Direction)
	{
		case 0:
			g_xPaperAPI.m_xInput.m_buttonMouseExtra1.UpdateState(g_xPaperAPI.m_bHasFocus, g_xPaperAPI.m_dwCurrentTime);
			//DEBUGLOG("SwipedLeft %i\n", g_xPaperAPI.m_xInput.m_buttonMouseExtra1.JustPressed());
			break;
			
		case 1:
			g_xPaperAPI.m_xInput.m_buttonMouseExtra2.UpdateState(g_xPaperAPI.m_bHasFocus, g_xPaperAPI.m_dwCurrentTime);
			//DEBUGLOG("SwipedRight %i\n", g_xPaperAPI.m_xInput.m_buttonMouseExtra2.JustPressed());
			break;
			
		case 2:
			g_xPaperAPI.m_xInput.m_buttonMouseExtra3.UpdateState(g_xPaperAPI.m_bHasFocus, g_xPaperAPI.m_dwCurrentTime);
			//DEBUGLOG("SwipedUp %i\n", g_xPaperAPI.m_xInput.m_buttonMouseExtra3.JustPressed());
			break;
			
		case 3:
			g_xPaperAPI.m_xInput.m_buttonMouseExtra4.UpdateState(g_xPaperAPI.m_bHasFocus, g_xPaperAPI.m_dwCurrentTime);
			//DEBUGLOG("SwipedDown %i\n", g_xPaperAPI.m_xInput.m_buttonMouseExtra4.JustPressed());
			break;
			
		case 4:
			g_xPaperAPI.m_xInput.m_buttonMouseExtra5.UpdateState(g_xPaperAPI.m_bHasFocus, g_xPaperAPI.m_dwCurrentTime);
			//DEBUGLOG("DoubleTap %i\n", g_xPaperAPI.m_xInput.m_buttonMouseExtra5.JustPressed());

			xTexture* current = g_xGraphics.m_xTexturePool.m_pNext;
			while(current)
			{
				DEBUGLOG("Textures: %s - %i\n", current->m_strObjectPath, current->m_iReferenceCount)
				current = current->m_pNext;
			}
			
			DEBUGLOG("\n\n\n");
			break;
	}
}

void MainUpdate()
{
	if(g_xPaperAPI.m_bHasFocus || g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
	{
		g_xPaperAPI.PreUpdate();
		GamePreUpdate(g_xPaperAPI.m_dwCurrentTime);
	}
	
	GameUpdate((double)(g_xPaperAPI.m_dwCurrentTime - g_xPaperAPI.m_dwLastTime) * 0.001);
	
	g_xPaperAPI.Update(); //Updates GameTime and Renders/Flips Screen
	
	g_xPaperAPI.m_xInput.PostUpdate();
	
	if(!g_xPaperAPI.m_bHasFocus)
		sleep(100);	
}

void MainDraw()
{	
	Matrix4 modelViewMatrix, modelViewProjectionMatrix;
	
	glUseProgram(g_xGraphics.m_uiGrayScalePixelProgramIndex);
	
	Matrix4GenOrtho(&g_matProjection, 0, g_xGraphics.m_iScreenWidth, 0, g_xGraphics.m_iScreenHeight, 0.0, 1.0);
	Matrix4MakeTranslation(&modelViewMatrix, 125.0f, 150.0f, -0.3f);
	Matrix4Rotate(&modelViewMatrix, g_fRotation * M_PI / 180.0f, 0, 0, 1);
	Matrix4Multiply(&modelViewProjectionMatrix, &g_matProjection, &modelViewMatrix);
	
	glClearColor(1.0, 0.0, 0.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	//glActiveTexture(GL_TEXTURE0);
	//glBindTexture(GL_TEXTURE_2D, g_uiTexture);
	
	glUniformMatrix4fv(g_xGraphics.m_xGrayScalePixel.m_iMVP, 1, 0, modelViewProjectionMatrix.m);
	glUniform1i(g_xGraphics.m_xGrayScalePixel.m_iTexture, 0);
	glUniform4fv(g_xGraphics.m_xGrayScalePixel.m_iColor, 1, g_colorNormalColor.m_f4Color);
	
    glBindBuffer(GL_ARRAY_BUFFER, g_uiVertexBuffer);
	
	glVertexAttribPointer(g_xGraphics.m_xGrayScalePixel.m_iPosition, 3, GL_FLOAT, GL_FALSE, sizeof(Vertex), (const GLvoid *)0);
	glVertexAttribPointer(g_xGraphics.m_xGrayScalePixel.m_iTextureCoord, 2, GL_FLOAT, GL_FALSE, sizeof(Vertex), (const GLvoid *)12);
	
	glDrawElements(GL_TRIANGLES, sizeof(Indices)/sizeof(Indices[0]), GL_UNSIGNED_BYTE, 0);
}

void MainDestroy()
{
	glDeleteFramebuffers(1, &g_uiDrawingFrameBuffer);

	GameShutdown();

	g_xPaperAPI.Shutdown();

	if(g_xPaperAPI.m_strExitCommand[0] != '\0')
		system(g_xPaperAPI.m_strExitCommand);
}

void HandleAsyncNetworkMessage(xNetworkClient* pNetwork, DWORD lparam)
{
/*	switch(WSAGETSELECTEVENT(lparam))
	{
		case FD_CONNECT:
		{
			pNetwork->SetConnected(true);
			break;
		}

		case FD_READ:
		{
			if(int retvalu = WSAGETSELECTERROR(lparam))
			{
				if(retvalu != 0 && retvalu != 10035)
				{
					return;
				}
			}

			pNetwork->Read();
			break;
		}

		case FD_WRITE:
		{
			if(int retvalu = WSAGETSELECTERROR(lparam))
			{
				if(retvalu != 0 && retvalu != 10035)
				{
					return;
				}
			}

			pNetwork->SetWritable(true);
			break;
		}

		case FD_ACCEPT:
		{
			if(int retvalu = WSAGETSELECTERROR(lparam))
			{
				if(retvalu != 0 && retvalu != 10035)
				{
					return;
				}
			}

			if(!pNetwork->Accept())
				pNetwork->SetConnected(true);

			break;
		}

		case FD_CLOSE:
		{
			pNetwork->Close();
			break;
		}
	}
*/}
