#include "Paper.h"

int xAudioStream::Open(const char *strPathName, bool bLooping, bool bAnalyzing)
{
	OSStatus err = noErr;
	AudioStreamBasicDescription fileformat, outputformat;
	UInt32 propertysize;
	SInt64 fileframes;
	
	m_bLooping = bLooping;
	m_bAnalyzing = bAnalyzing;

	CFStringRef cfstring = CFStringCreateWithCStringNoCopy(0, strPathName, kCFStringEncodingASCII, kCFAllocatorNull);
	CFURLRef url = CFURLCreateWithFileSystemPath(0, cfstring, kCFURLPOSIXPathStyle, 0);
	err = ExtAudioFileOpenURL(url, &m_refExtAudioFile);
	
	if(err)
		return PE_RES_OPENALMUSICLOADFILEERROR;
	
	propertysize = sizeof(fileformat);
	err = ExtAudioFileGetProperty(m_refExtAudioFile, kExtAudioFileProperty_FileDataFormat, &propertysize, &fileformat);

	if(err)
		return PE_RES_OPENALMUSICLOADFILEERROR;
	
	// Set read format
	outputformat.mChannelsPerFrame	= fileformat.mChannelsPerFrame;
	outputformat.mSampleRate		= fileformat.mSampleRate;
	outputformat.mFormatID			= kAudioFormatLinearPCM;
	outputformat.mBytesPerPacket	= 2 * outputformat.mChannelsPerFrame;
	outputformat.mFramesPerPacket	= 1;
	outputformat.mBytesPerFrame		= outputformat.mBytesPerPacket * outputformat.mFramesPerPacket;
	outputformat.mBitsPerChannel	= 16;
	outputformat.mFormatFlags		= kAudioFormatFlagsNativeEndian | kAudioFormatFlagIsPacked | kAudioFormatFlagIsSignedInteger;
	err = ExtAudioFileSetProperty(m_refExtAudioFile, kExtAudioFileProperty_ClientDataFormat, sizeof(outputformat), &outputformat);

	if(err)
		return PE_RES_OPENALMUSICLOADFILEERROR;
	
	m_ulChannels	= fileformat.mChannelsPerFrame;
	m_ulFrequency	= fileformat.mSampleRate;
	m_ulFormat		= 0;
	m_ulBufferSize	= 0;

	switch(m_ulChannels)
	{
		case 1:
			m_ulFormat		 = AL_FORMAT_MONO16;
			m_ulBufferSize	 = (m_ulFrequency >> 1) * (m_bAnalyzing ? 10 : 1);
			m_ulBufferSize	-= m_ulBufferSize % 2;
			break;
		case 2:
			m_ulFormat		 = AL_FORMAT_STEREO16;
			m_ulBufferSize	 = m_ulFrequency * (m_bAnalyzing ? 10 : 1);
			m_ulBufferSize	-= m_ulBufferSize % 4;
			break;
		case 4:
			m_ulFormat		 = alGetEnumValue("AL_FORMAT_QUAD16");
			m_ulBufferSize	 = (m_ulFrequency << 1) * (m_bAnalyzing ? 10 : 1);
			m_ulBufferSize	-= m_ulBufferSize % 8;
			break;
		case 6:
			m_ulFormat		 = alGetEnumValue("AL_FORMAT_51CHN16");
			m_ulBufferSize	 = (m_ulFrequency * 3) * (m_bAnalyzing ? 10 : 1);
			m_ulBufferSize	-= m_ulBufferSize % 12;
			break;
		default:
			return PE_RES_OPENALMUSICLOADFILEERROR;
	}

	if(m_pDecodeBuffer)
		delete [] m_pDecodeBuffer;

	// Allocate a buffer to be used to store decoded data for all Buffers
	m_pDecodeBuffer = new char[m_ulBufferSize];
	m_listAudioBuffer.mNumberBuffers = 1;
	m_listAudioBuffer.mBuffers[0].mDataByteSize = m_ulBufferSize;
	m_listAudioBuffer.mBuffers[0].mNumberChannels = outputformat.mChannelsPerFrame;
	m_listAudioBuffer.mBuffers[0].mData = m_pDecodeBuffer;

	if(!m_bAnalyzing)
	{
		// Generate some AL Buffers for streaming
		alGenBuffers(PE_OPENAL_OGG_NUMBUFFERS, m_uiBuffers);

		// Get a Source to playback the Buffers
		m_uiSource = g_xPaperAPI.m_xAudio.GetSource(true);

		// Fill all the Buffers with decoded audio data from the OggVorbis file
		for(int i = 0; i < PE_OPENAL_OGG_NUMBUFFERS - 1; i++)
		{
			unsigned long bytesloaded = DecodeStream();
			if(bytesloaded)
			{
				alBufferData(m_uiBuffers[i], m_ulFormat, m_pDecodeBuffer, bytesloaded, m_ulFrequency);
				//DEBUGLOG("AudioBuffer[%i] %u: size:%u\n", i, m_uiBuffers[i], bytesloaded);
				
				while(alGetError())
				{
					alGenBuffers(1, &m_uiBuffers[i]);
					alBufferData(m_uiBuffers[i], m_ulFormat, m_pDecodeBuffer, bytesloaded, m_ulFrequency);
				}
			}
		}
	}

	propertysize = sizeof(fileframes);
	err = ExtAudioFileGetProperty(m_refExtAudioFile, kExtAudioFileProperty_FileLengthFrames, &propertysize, &fileframes);
	m_dwLength = (fileframes / (outputformat.mSampleRate / 1000.0));

	return PE_RES_NOERROR;
}

int xAudioStream::Play()
{
	//DEBUGLOG("%i AudioStream.Play - OldError=%i WasPlaying=%i WasDone=%i\n", (int)this, alGetError(), m_bPlaying, m_bDone);

	alSourceQueueBuffers(m_uiSource, PE_OPENAL_OGG_NUMBUFFERS - 1, m_uiBuffers);
	
	//int error = alGetError();
	//if(error)
	//	DEBUGLOG("Play - Queue Buffers Error: %s\n", alGetString(error));

	alSourcePlay(m_uiSource);

	m_bPlaying = true;
	m_bDone = false;

	return PE_RES_NOERROR;
}

void xAudioStream::ContinuePlaying()
{
	unsigned long bytesloaded;
	ALuint buffer;
	int buffersprocessed, i;
	
	//DEBUGLOG("%i AudioStream.ContinuePlaying - IsPlaying=%i IsDone=%i\n", (int)this, m_bPlaying, m_bDone);

	if(m_bPlaying)
	{
		// Request the number of OpenAL Buffers have been processed (played) on the Source
		buffersprocessed = 0;
		alGetSourcei(m_uiSource, AL_BUFFERS_PROCESSED, &buffersprocessed);

		// For each processed buffer, remove it from the Source Queue, read next chunk of audio
		// data from disk, fill buffer with new data, and add it to the Source Queue
		while(buffersprocessed)
		{
			// Remove the Buffer from the Queue.  (uiBuffer contains the Buffer ID for the unqueued Buffer)
			buffer = 0;
			alSourceUnqueueBuffers(m_uiSource, 1, &buffer);
			//DEBUGLOG("Unqueue Buffer %u/%i %i\n", buffer, buffersprocessed, alGetError());

			if(buffer == m_uiBuffers[0])
			{
				i = PE_OPENAL_OGG_NUMBUFFERS;
				buffer = m_uiBuffers[PE_OPENAL_OGG_NUMBUFFERS - 1];
			}
			else
			{
				for(i = 1; i < PE_OPENAL_OGG_NUMBUFFERS; i++)
				{
					if(buffer == m_uiBuffers[i])
					{
						buffer = m_uiBuffers[i - 1];
						break;
					}
				}
				
				if(i == PE_OPENAL_OGG_NUMBUFFERS)
					return;
			}
			
			// Read more audio data (if there is any)
			bytesloaded = DecodeStream();
			
			if(!bytesloaded && m_bLooping)
			{
				ExtAudioFileSeek(m_refExtAudioFile, 0);
				// For god knows what reason, when we reach the end of the file, this gets set to 0 so no more data would ever be read if we dont set this again
				m_listAudioBuffer.mBuffers[0].mDataByteSize = m_ulBufferSize; 
				bytesloaded = DecodeStream();
				//DEBUGLOG("!bytesloaded\n");
			}

			if(bytesloaded)
			{				
				alBufferData(buffer, m_ulFormat, m_pDecodeBuffer, bytesloaded, m_ulFrequency);
				
				int error = alGetError();
				while(error)
				{
					//DEBUGLOG("Buffer Data Error: %u %s\n", buffer, alGetString(error));
					
					alGenBuffers(1, &m_uiBuffers[i - 1]);
					buffer = m_uiBuffers[i - 1];
					alBufferData(buffer, m_ulFormat, m_pDecodeBuffer, bytesloaded, m_ulFrequency);

					//DEBUGLOG("Buffer Data Retry %u\n", buffer);

					error = alGetError();
				}

				alSourceQueueBuffers(m_uiSource, 1, &buffer);				
				//DEBUGLOG("Queue Buffer %u\n", buffer);
			}

			buffersprocessed--;
		}

		// Check the status of the Source.  If it is not playing, then playback was completed,
		// or the Source was starved of audio data, and needs to be restarted.
		int state = -1;
		alGetSourcei(m_uiSource, AL_SOURCE_STATE, &state);
		if(state != AL_PLAYING)
		{
			//DEBUGLOG("Source not playing!\n");
			// If there are Buffers in the Source Queue then the Source was starved of audio
			// data, so needs to be restarted (because there is more audio data to play)
			int queuedbuffers = 0;
			alGetSourcei(m_uiSource, AL_BUFFERS_QUEUED, &queuedbuffers);
			if(queuedbuffers)
				alSourcePlay(m_uiSource);
			else
				m_bDone = true;
		}
	}
}

int xAudioStream::Stop()
{
	//DEBUGLOG("%i AudioStream.Stop\n", (int)this);

	alSourceStop(m_uiSource);
	m_bPlaying = false;

	ALuint tempbuffers[10];
	alSourceUnqueueBuffers(m_uiSource, PE_OPENAL_OGG_NUMBUFFERS, tempbuffers);

	return PE_RES_NOERROR;
}

int xAudioStream::Release()
{
	if(m_pDecodeBuffer)
	{
		delete [] m_pDecodeBuffer;
		m_pDecodeBuffer = 0;
	}

	alDeleteBuffers(PE_OPENAL_OGG_NUMBUFFERS, m_uiBuffers);

	if(m_refExtAudioFile)
		ExtAudioFileDispose(m_refExtAudioFile);
	
	return PE_RES_NOERROR;
}

void xAudioStream::SetVolume(float fVolume)
{
	alSourcef(m_uiSource, AL_GAIN, fVolume);
}

DWORD xAudioStream::GetTime()
{
	SInt64 position;
	float decodetime;
	int bufferoffset;
	
	ExtAudioFileTell(m_refExtAudioFile, &position);
	decodetime = position / (m_ulFrequency / 1000.0);

	alGetSourcei(m_uiSource, AL_SAMPLE_OFFSET,  &bufferoffset);

	DWORD ret = (DWORD)(decodetime + ((float)bufferoffset / (float)m_ulFrequency) * 1000.0f) - 1000;

	//DEBUGLOG("AudioStream.GetTime: %f %i %f %f %lu %lu\n", decodetime, bufferoffset, (float)bufferoffset / (float)m_ulFrequency, (decodetime + (float)bufferoffset / (float)m_ulFrequency) * 1000.0, (unsigned long)((decodetime - (0.25 + (0.25 - (float)bufferoffset / (float)m_ulFrequency))) * 1000.0), ret);

	return ret;
}

int xAudioStream::SetTime(DWORD dwMilliseconds)
{
	if(GetTime() != dwMilliseconds)
	{
		//DEBUGLOG("AudioStream.SetTime: %f\n", (double)dwMilliseconds / 1000.0);
		ExtAudioFileSeek(m_refExtAudioFile, (dwMilliseconds * m_ulFrequency) / 1000);

		if(!m_bAnalyzing)
		{
			alSourcei(m_uiSource, AL_BUFFER, 0);

			// Fill all the Buffers with decoded audio data from the OggVorbis file
			for(int i = 0; i < PE_OPENAL_OGG_NUMBUFFERS; i++)
			{
				unsigned long bytesloaded = DecodeStream();
				if(bytesloaded)
				{
					alBufferData(m_uiBuffers[i], m_ulFormat, m_pDecodeBuffer, bytesloaded, m_ulFrequency);

					//DEBUGLOG("AudioStream.SetTime - AudioBuffer[%i] %u: %ul %s\n", i, m_uiBuffers[i], bytesloaded, m_pDecodeBuffer);
				}
			}
		}
	}

	return PE_RES_NOERROR;
}

unsigned long xAudioStream::DecodeStream()
{
	UInt32 readsize = m_ulBufferSize / (m_ulChannels * 2);
	
	OSStatus err = ExtAudioFileRead(m_refExtAudioFile, &readsize, &m_listAudioBuffer);
	if(err == noErr)
		return readsize * m_ulChannels * 2;

	return 0;
}
