#include "peGame.h"

#if WITH_BALANCESTATS

void xBalanceStats::Reset()
{
	m_xBalanceStats.m_u16BlocksCleared		= 0;
	m_xBalanceStats.m_u32NumBlocks			= 0;
	m_xBalanceStats.m_u16NumPieces			= 0;
	m_xBalanceStats.m_u16WrongColors		= 0;
	m_xBalanceStats.m_u16MissedBeats		= 0;
	m_xBalanceStats.m_u16MaxComboCount		= 0;

	m_xBalanceStats.m_byteVersion = BB_BALANCESTATSVERSION;
}

void xBalanceStats::Commit()
{
	// Start connecting to the FTP server
	Connect();

	// Reset Everything for the Next Session
	m_bCommitting	= true;
	m_iCommitStep	= 0;
	m_bStatsSent	= false;
}

void xBalanceStats::Update()
{
	DWORD time;
	char receive[MAX_NETWORK_BUFFERSIZE + 1] = {0};

	if(!m_bCommitting || !g_xPaperAPI.m_xFTPNetwork.IsConnected())
		return;

	if(m_iCommitStep == 0)
	{
		g_xPaperAPI.m_xFTPNetwork.Receive(receive, time);

		if(receive[0] != '\0')
		{
			sprintf(receive, "USER %s\r\n", BB_BALANCESTATSFTPUSER);
			g_xPaperAPI.m_xFTPNetwork.Send(receive, strlen(receive));
			m_iCommitStep++;
		}
	}
	else if(m_iCommitStep == 1)
	{
		g_xPaperAPI.m_xFTPNetwork.Receive(receive, time);

		if(receive[0] != '\0')
		{
			sprintf(receive, "PASS %s\r\n", BB_BALANCESTATSFTPPASS);
			g_xPaperAPI.m_xFTPNetwork.Send(receive, strlen(receive));
			m_iCommitStep++;
		}
	}
	else if(m_iCommitStep == 2)
	{
		g_xPaperAPI.m_xFTPNetwork.Receive(receive, time);

		if(receive[0] != '\0')
		{
			sprintf(receive, "CWD /stats.greatergoodgames.org\r\n");
			g_xPaperAPI.m_xFTPNetwork.Send(receive, strlen(receive));
			m_iCommitStep++;
		}
	}
	else if(m_iCommitStep == 3)
	{
		g_xPaperAPI.m_xFTPNetwork.Receive(receive, time);

		if(receive[0] != '\0')
		{
			sprintf(receive, "TYPE I\r\n");
			g_xPaperAPI.m_xFTPNetwork.Send(receive, strlen(receive));
			m_iCommitStep++;
		}
	}
	else if(m_iCommitStep == 4)
	{
		g_xPaperAPI.m_xFTPNetwork.Receive(receive, time);

		if(receive[0] != '\0')
		{
			sprintf(receive, "PASV\r\n");
			g_xPaperAPI.m_xFTPNetwork.Send(receive, strlen(receive));
			m_iCommitStep++;
		}
	}
	else if(m_iCommitStep == 5)
	{
		g_xPaperAPI.m_xFTPNetwork.Receive(receive, time);

		if(receive[0] != '\0')
		{
			char filename[BB_MAXPROFILENAMELENGTH + 16];
			int dateyear, datemonth, dateday, timehours24, timeminutes, timeseconds;
			g_xPaperAPI.m_xPlatform.GetDateAndTime(dateyear, datemonth, dateday, timehours24, timeminutes, timeseconds);
			sprintf(filename, "%i-%s-%i%02i%02i-%02i%02i%02i", BB_BALANCESTATSVERSION, m_strPlayerName, 1900 + dateyear, datemonth + 1, dateday, timehours24, timeminutes, timeseconds);

			char temp[100];
			sprintf(temp, "STOR %s\r\n", filename);
			g_xPaperAPI.m_xFTPNetwork.Send(temp, strlen(temp));
			g_xPaperAPI.m_xFTPNetwork.Update();

			ParsePassiveResponse(receive);

			g_xPaperAPI.m_xFTPFileTransfer.Connect(m_strFileTransferIP, m_iFileTransferPort);

			m_iCommitStep++;
		}
	}
	else if(m_iCommitStep == 6 || m_iCommitStep == 7)
	{
		g_xPaperAPI.m_xFTPNetwork.Receive(receive, time);

		if(receive[0] != '\0')
			m_iCommitStep++;

		if(!m_bStatsSent && g_xPaperAPI.m_xFTPFileTransfer.IsConnected())
		{
			m_bStatsSent = true;
			g_xPaperAPI.m_xFTPFileTransfer.Send((char*)&m_xBalanceStats, sizeof(xBalanceStatStruct));
			m_iCommitStep++;
		}
	}
	else if(g_xPaperAPI.m_xFTPFileTransfer.m_iWriteBufferOffset <= 0)
	{
		// We're done Committing
		g_xPaperAPI.m_xFTPNetwork.Close();
		g_xPaperAPI.m_xFTPFileTransfer.Close();

		m_bCommitting = false;

		Reset();
	}

	g_xPaperAPI.m_xFTPNetwork.Update();
	g_xPaperAPI.m_xFTPFileTransfer.Update();
}

void xBalanceStats::Connect()
{
	char strIP[20] = {0};

	g_xPaperAPI.m_xFTPNetwork.Close();

	addrinfo hints, *result = NULL;
	memset(&hints, 0, sizeof(hints));
	hints.ai_flags = AI_PASSIVE;
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = IPPROTO_TCP;

	DWORD retval;
	if((retval = getaddrinfo(BB_BALANCESTATSFTP, NULL, &hints, &result)) != 0)
		return;

	if(getnameinfo(result->ai_addr, result->ai_addrlen, strIP, 256, NULL, 0, NI_NUMERICHOST) != 0)
		return;

	freeaddrinfo(result);

	g_xPaperAPI.m_xFTPNetwork.Connect(strIP, BB_BALANCESTATSFTPPORT);
}

void xBalanceStats::ParsePassiveResponse(char *strResponse)
{
	int responselength = strlen(strResponse);

	bool foundstart = false;
	int section = 0;
	int index = 0;
	int sectionstart;

	for(int i = 0; i < responselength; i++)
	{
		if(!foundstart)
		{
			if(strResponse[i] == '(')
				foundstart = true;
		}
		else if(strResponse[i] == ',' || strResponse[i] == ')')
		{
			if(section < 3)
				m_strFileTransferIP[index++] = '.';
			else if(section == 4)
			{
				strResponse[i] = '\0';
				m_iFileTransferPort = atoi(strResponse + sectionstart) * 256;
			}
			else if(section == 5)
			{
				strResponse[i] = '\0';
				m_iFileTransferPort += atoi(strResponse + sectionstart);
			}

			section++;
			sectionstart = i + 1;
		}
		else if(section < 4)
			m_strFileTransferIP[index++] = strResponse[i];
	}
}

#endif