#ifndef XEMITTER_H_
#define XEMITTER_H_

#include "xList.h"
#include "xVector.h"
#include "xColor.h"

class xParticle
{
public:
	xParticle()
	{
		m_pRenderable	= 0;
		m_bDead			= 1;
		m_dCurLife		= 0.0;
		m_fX			= 0.0;
		m_fY			= 0.0;
		m_fWidth		= -1.0;
		m_fHeight		= -1.0;
		m_fScale		= 1.0;
		m_iCurFrame		= 0;
		m_fFullAlpha	= 1.0;
	};

	void Update(double dTime);

	xRenderable2DQuad	*m_pRenderable;

	bool	m_bDead;
	double	m_dCurLife;
	float	m_fX;
	float	m_fY;
	float	m_fWidth;
	float	m_fHeight;
	xColor	m_xColor;
	float	m_fScale;
	int		m_iCurFrame;
	double	m_dFrameTime;

	double	m_dLifespan;
	float	m_fFadeInTime;
	float	m_fFadeOutTime;
	xVector m_xVelocity;
	float	m_fGravity;
	float	m_fStartingScale;
	float	m_fFullScale;
	float	m_fScaleInTime;
	float	m_fScaleOutTime;
	float	m_fRotation;
	float	m_fRotationSpeed;
	float	m_fFullAlpha;
	xColor	m_xStartColor;
	xColor	m_xEndColor;
};

class xParticleSystem
{
public:
	xParticleSystem()
	{
		m_bEnabled					= 1;
		m_bIsDone					= 0;
		m_bAnimated					= 0;
		m_dCurTime					= 0.0;
		m_fX						= 0.0;
		m_fY						= 0.0;
		m_fWidth					= 0.0;
		m_fHeight					= 0.0;
		m_fDirection1				= 1.0;
		m_fDirection2				= 1.0;
		m_fLastDirection			= 1.0;
		m_bMovingFromDirection1to2	= 1;

		m_dMinSpawnTime			= 0.2;
		m_dMaxSpawnTime			= 0.6;
		m_iMinParticlesPerSecond= 200;
		m_iMaxParticlesPerSecond= 240;
		m_fMinStartX			= 0.5;
		m_fMaxStartX			= 0.5;
		m_fMinStartY			= 0.5;
		m_fMaxStartY			= 0.5;
		m_dMinLifespan			= 4.0;
		m_dMaxLifespan			= 8.0;
		m_fMinDirection			= -1.25;
		m_fMaxDirection			= -1.75;
		m_fMinDirection2		= -6.3;
		m_fMaxDirection2		= -6.3;
		m_fMinDirectionSpeed	= 0.0;
		m_fMaxDirectionSpeed	= 0.0;
		m_fMinSpeed				= 11.0;
		m_fMaxSpeed				= 15.0;
		m_fMinFadeInTime		= 0.2;
		m_fMaxFadeInTime		= 0.5;
		m_fMinFadeOutTime		= 0.3;
		m_fMaxFadeOutTime		= 0.9;
		m_fMinGravity			= 0.3;
		m_fMaxGravity			= 0.6;
		m_fMinRotation			= 0.0;
		m_fMaxRotation			= -6.28319;
		m_fMinRotationSpeed		= 0.0;
		m_fMaxRotationSpeed		= 0.2;
		m_fMinStartingScale		= 0.1;
		m_fMaxStartingScale		= 0.3;
		m_fMinScale				= 0.75;
		m_fMaxScale				= 1.25;
		m_fMinScaleInTime		= 0.3;
		m_fMaxScaleInTime		= 0.5;
		m_fMinScaleOutTime		= 0.6;
		m_fMaxScaleOutTime		= 0.8;
		m_fMinFullAlpha			= 0.6;
		m_fMaxFullAlpha			= 1.0;
		m_bColorRanges			= 0;
		m_bColorTransitions		= 0;
		m_iColorArrayLength		= 0;
		m_pColorArray			= 0;
		m_iColorIndex			= -1;
		m_iMaxParticles			= 0;
		m_pParticles			= 0;
		m_pSceneGraph			= 0;
	}

	void	Init(xTexture *pTexture, float fDepth);
	void	InitAnimated(xTexture **pTextures, float fDepth, int iNumTextures, int iFPS);
	void	CopyParticleSettings(xParticleSystem *pCopy);
	void	Update(double dTime);
	void	Delete();
	void	CreateNextParticle(float fX = -99999.0, float fY = -99999.0);
	void	UpdatedSpawnTimes();
	xVector CreateVelocityVector();
	xColor	GenerateColorInRange(xColor &xColor1, xColor &xColor2);

	bool	m_bEnabled;
	bool	m_bIsDone;
	double	m_dCurTime;
	double	m_dNextSpawnTime;
	double	m_dMinSpawnTime;
	double	m_dMaxSpawnTime;
	float	m_fX;
	float	m_fY;
	float	m_fWidth;
	float	m_fHeight;
	float	m_fDirection1;
	float	m_fDirection2;
	float	m_fLastDirection;
	bool	m_bMovingFromDirection1to2;

	int		 m_iMinParticlesPerSecond;
	int		 m_iMaxParticlesPerSecond;
	double	 m_dMinLifespan;
	double	 m_dMaxLifespan;
	float	 m_fMinStartX;
	float	 m_fMaxStartX;
	float	 m_fMinStartY;
	float	 m_fMaxStartY;
	float	 m_fMinDirection;
	float	 m_fMaxDirection;
	float	 m_fMinDirection2;
	float	 m_fMaxDirection2;
	float	 m_fMinDirectionSpeed;
	float	 m_fMaxDirectionSpeed;
	float	 m_fMinSpeed;
	float	 m_fMaxSpeed;
	float	 m_fMinFadeInTime;
	float	 m_fMaxFadeInTime;
	float	 m_fMinFadeOutTime;
	float	 m_fMaxFadeOutTime;
	float	 m_fMinGravity;
	float	 m_fMaxGravity;
	float	 m_fMinRotation;
	float	 m_fMaxRotation;
	float	 m_fMinRotationSpeed;
	float	 m_fMaxRotationSpeed;
	float	 m_fMinStartingScale;
	float	 m_fMaxStartingScale;
	float	 m_fMinScale;
	float	 m_fMaxScale;
	float	 m_fMinScaleInTime;
	float	 m_fMaxScaleInTime;
	float	 m_fMinScaleOutTime;
	float	 m_fMaxScaleOutTime;
	float	 m_fMinFullAlpha;
	float	 m_fMaxFullAlpha;
	bool	 m_bColorRanges;
	bool	 m_bColorTransitions;
	xColor	*m_pColorArray;
	int		 m_iColorArrayLength;
	int		 m_iColorIndex;

	xSceneGraph	*m_pSceneGraph;
	xParticle	*m_pParticles;
	int			 m_iMaxParticles;
	int			 m_iCurParticle;

	// Animated Particle Helpers
	bool		  m_bAnimated;
	xTexture	**m_pTextures;
	int			  m_iFPS;
	int			  m_iNumFrames;
};
typedef xList<xParticleSystem> xParticleSystemList;

class xEmitter
{
public:
	xEmitter()
	{
		m_fX				= 0.0;
		m_fY				= 0.0;
		m_fWidth			= 0.0;
		m_fHeight			= 0.0;
		m_iReferenceCount	= 0;
		m_bIsLoaded			= 0;
		m_dLifeTime			= 0.0;
		m_pPoolNext			= 0;
		m_pPoolPrev			= 0;
	}

	void Init(const char *strFilePath, float fDepth, bool bAnimated, xSceneGraph *pSceneGraph);
	void Update(double dTime);
	void RefreshRenderables(const char *strFilePath);
	void Reset();
	void Kill();
	void Delete();

	void LoadParticleParameters(xParticleSystem *pParticleSystem, xFileIO *pFile);

	void	SetPosition(float fX, float fY);
	float	GetWidth() { return m_fWidth; }
	float	GetHeight() { return m_fHeight; }
	void	SetSize(float fWidth, float fHeight);
	void	SetColorIndex(int iIndex);

	bool IsLoaded() { return m_bIsLoaded; }
	bool IsDone() { return m_bIsDone; }

	xParticleSystemList	  m_xParticleList;
	xTexture			**m_pTextures;

	double	m_dLifeTime;

	bool	m_bIsDone;
	float	m_fX;
	float	m_fY;
	float	m_fWidth;
	float	m_fHeight;

	// Emitter Pool Helpers
	char		 m_strObjectPath[256];
	int			 m_iReferenceCount;
	bool		 m_bIsLoaded;
	xEmitter	*m_pPoolNext;
	xEmitter	*m_pPoolPrev;
};
typedef xList<xEmitter> xEmitterList;

#endif