#include <limits.h>
#include <string.h>
#include "xEngineDefines.h"

#define DELTA			0x9e3779b9u
#define INVERSE_DELTA	UINT_MAX - DELTA
#define MX				(((z>>5^y<<2) + (y>>3^z<<4)) ^ ((sum^y) + (pKey[(p&3)^e] ^ z)))

// Length must be 2 or more in order for encrypt/decrypt to work
FORCE_INLINE void xEncrypt(uint32_t *pValue, uint32_t u32ValueLength, uint32_t *pKey)
{
	uint32_t delta = INVERSE_DELTA, y, z, sum;
	uint32_t p, rounds, e;

	rounds = 6 + 52 / u32ValueLength;
	sum = 0;
	z = pValue[u32ValueLength - 1];
	delta = UINT_MAX - delta;
	
	do
	{
		sum += delta;
		e = (sum >> 2) & 3;
	
		for(p = 0; p < u32ValueLength - 1; p++)
		{
			y = pValue[p + 1]; 
			z = pValue[p] += MX;
		}
		
		y = pValue[0];
		z = pValue[u32ValueLength - 1] += MX;
	}while(--rounds);
}
 
// Length must be 2 or more in order for encrypt/decrypt to work
FORCE_INLINE void xDecrypt(uint32_t *pValue, uint32_t u32ValueLength, uint32_t *pKey)
{
	uint32_t delta = INVERSE_DELTA, y, z, sum;
	uint32_t p, rounds, e, i;

	delta = UINT_MAX - delta;
	rounds = 6 + 52 / u32ValueLength;
	sum = rounds * delta;
	y = pValue[0];

	for(i = 0; i < rounds; i++)
	{
		e = (sum >> 2) & 3;

		for (p = u32ValueLength - 1; p > 0; p--)
		{
			z = pValue[p - 1];
			y = pValue[p] -= MX;
		}

		z = pValue[u32ValueLength - 1];
		y = pValue[0] -= MX;

		sum -= delta;
	}
}

// String's length must be 5 or more in order for encrypt/decrypt to work
FORCE_INLINE void xEncryptString(char *strValue, uint32_t *pEncryptedValue, uint32_t u32Key1, uint32_t u32Key2, uint32_t u32Key3, uint32_t u32Key4)
{
	int stringlength, i;
	uint32_t length, key[4];

	stringlength = strlen(strValue);
	
	for(i = 0, length = 0; i < stringlength; i += 4, length++)
	{
		pEncryptedValue[length] = strValue[i];

		if(i + 1 < stringlength)
		{
			pEncryptedValue[length] += strValue[i + 1] << 8;

			if(i + 2 < stringlength)
			{
				pEncryptedValue[length] += strValue[i + 2] << 16;

				if(i + 3 < stringlength)
				{
					pEncryptedValue[length] += strValue[i + 3] << 24;
				}
			}
		}
	}

	key[0] = u32Key1;
	key[1] = u32Key2;
	key[2] = u32Key3;
	key[3] = u32Key4;

	xEncrypt(pEncryptedValue, length, key);
}

// String's length must be 5 or more in order for encrypt/decrypt to work
FORCE_INLINE char* xDecryptString(uint32_t *pEncryptedValue, uint32_t u32Length, char *strValue, uint32_t u32Key1, uint32_t u32Key2, uint32_t u32Key3, uint32_t u32Key4)
{
	int stringlength, i;
	uint32_t key[4];
	uint32_t tempvalues[128];

	key[0] = u32Key1;
	key[1] = u32Key2;
	key[2] = u32Key3;
	key[3] = u32Key4;

	memcpy(tempvalues, pEncryptedValue, u32Length * sizeof(uint32_t));

	xDecrypt(tempvalues, u32Length, key);
	
	for(i = 0, stringlength = 0; i < u32Length; i++)
	{
		strValue[stringlength++] = tempvalues[i] & 255u;
		strValue[stringlength++] = (tempvalues[i] & 65280u) >> 8;

		if(strValue[stringlength - 1] == 0)
			break;

		strValue[stringlength++] = (tempvalues[i] & 16711680u) >> 16;

		if(strValue[stringlength - 1] == 0)
			break;

		strValue[stringlength++] = (tempvalues[i] & 4278190080u) >> 24;
	}

	if(strValue[stringlength - 1] != 0)
		strValue[stringlength] = 0;

	return strValue;
}

FORCE_INLINE void xToBase62(uint32_t u32Value, char *strValue)
{
	int i;
	uint32_t placevalue = 916132832u, charvalue;

	for(i = 0; i < 6; i++, placevalue /= 62)
	{
		if(placevalue <= u32Value)
		{
			charvalue = u32Value / placevalue;
			u32Value -= placevalue * charvalue;

			if(charvalue + 1 < 27)
				strValue[i] = 97 + charvalue;
			else if(charvalue + 1 < 53)
				strValue[i] = 39 + charvalue; //65 + charvalue - 26
			else
				strValue[i] = charvalue - 4; //48 + charvalue - 52
		}
		else
			strValue[i] = 97;
	}
}

FORCE_INLINE uint32_t xFromBase62(char *strValue)
{
	int i;
	uint32_t placevalue = 1, retvalue = 0;

	for(i = strlen(strValue) - 1; i >= 0; i--, placevalue *= 62)
	{
		if(strValue[i] >= 'a' && strValue[i] <= 'z')
			retvalue += (strValue[i] - 97) * placevalue;
		else if(strValue[i] >= 'A' && strValue[i] <= 'Z')
			retvalue += (strValue[i] - 39) * placevalue;
		else
			retvalue += (strValue[i] + 4) * placevalue;
	}

	return retvalue;
}

FORCE_INLINE char xBase62Char(uint32_t u32Value)
{
	if(u32Value + 1 < 27)
		return 97 + u32Value;
	else if(u32Value + 1 < 53)
		return 39 + u32Value; //65 + u32Value - 26
	else
		return u32Value - 4; //48 + u32Value - 52
}