#include "Paper.h"

char g_strDocumentDir[256];

int xFileIO::StartFindFiles(const char* strFilter, bool bDirectoriesOnly)
{
	if(m_globFindFilesStruct.gl_pathv)
		globfree(&m_globFindFilesStruct);
	
	m_bFindDirectoriesOnly = bDirectoriesOnly;
	m_iFindFilesIndex = 0;

	glob(strFilter, 0, nil, &m_globFindFilesStruct);

	while(bDirectoriesOnly && m_iFindFilesIndex < m_globFindFilesStruct.gl_pathc && !DirectoryExists(m_globFindFilesStruct.gl_pathv[m_iFindFilesIndex]))
		m_iFindFilesIndex++;

	m_bFindFilesComplete = m_iFindFilesIndex >= m_globFindFilesStruct.gl_pathc;

	return PE_RES_NOERROR;
}

bool xFileIO::GetNextFile(char* strFileOut)
{
	const char* curpath;
	int fullpathlength, i, j;
	
	if(m_bFindFilesComplete)
		return false;
	
	curpath = m_globFindFilesStruct.gl_pathv[m_iFindFilesIndex];
	
	fullpathlength = strlen(curpath);
	
	for(i = fullpathlength - 1; i >= 0 ; i--)
		if(curpath[i] == '\\' || curpath[i] == '/')
			break;
	
	i++;
	
	for(j = 0; i < fullpathlength; i++, j++)
		strFileOut[j] = curpath[i];

	m_iFindFilesIndex++;

	while(m_bFindDirectoriesOnly && m_iFindFilesIndex < m_globFindFilesStruct.gl_pathc && !DirectoryExists(m_globFindFilesStruct.gl_pathv[m_iFindFilesIndex]))
		m_iFindFilesIndex++;

	m_bFindFilesComplete = m_iFindFilesIndex >= m_globFindFilesStruct.gl_pathc;
	
	return true;
}

int xFileIO::EndFindFiles()
{
//	m_hFindHandle = NULL;

	return PE_RES_NOERROR;
}

int xFileIO::OpenFile(const char* strFileName, bool bReadOnly, bool bBinary)
{
	m_strPathName = strFileName;

	if(bReadOnly == true)
	{
		if(bBinary == true)
		{
			m_pFile = fopen(strFileName, "rb");
			if(m_pFile == NULL)
				return PE_RES_FILEDOESNOTEXIST;
		}
		else
		{
			m_pFile = fopen(strFileName, "r");
			if(m_pFile == NULL)
				return PE_RES_FILEDOESNOTEXIST;
		}
	}
	else
	{
		if(bBinary == true)
		{
			m_pFile = fopen(strFileName, "wb");
			if(m_pFile == NULL)
				return PE_RES_FILEOPENERROR;
		}
		else
		{
			m_pFile = fopen(strFileName, "w+");
			if(m_pFile == NULL)
				return PE_RES_FILEOPENERROR;
		}
	}

	return PE_RES_NOERROR;
}

int xFileIO::ReadData(void* pOutputData, long lSize, int *iReadSize)
{
	int readsize;

	if(iReadSize == NULL)
		iReadSize = &readsize;

	*iReadSize = 0;

	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	*iReadSize = fread(pOutputData, 1, lSize, m_pFile);

	if(*iReadSize < lSize)
		return PE_RES_FILEREADERROR;

	return PE_RES_NOERROR;
}

int xFileIO::ReadLine(char* pOutputData, int iMaxSize)
{
	int length;
    
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	if(fgets(pOutputData, iMaxSize, m_pFile) == NULL)
		return PE_RES_FILEREADERROR;
    
	length = strlen(pOutputData);

	//Remove line feed and new line characters from end of line
	if(pOutputData[length - 2] == '\r')
		pOutputData[length - 2] = '\0';
	else
		pOutputData[length - 1] = '\0';

	return PE_RES_NOERROR;
}

int xFileIO::ReadToEnd(void* pOutputData)
{
	long lSize = 0;

	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fseek (m_pFile, 0, SEEK_END);
	lSize = ftell(m_pFile);
	rewind(m_pFile);

	fread(pOutputData, 1, lSize, m_pFile);

	if(pOutputData == NULL)
		return PE_RES_FILEREADERROR;

	return PE_RES_NOERROR;
}

int xFileIO::SkipAhead(long lBytesToSkip)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fseek(m_pFile, lBytesToSkip, SEEK_CUR);

	return PE_RES_NOERROR;
}

int xFileIO::SetPosition(long lBytesOffset)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fseek(m_pFile, lBytesOffset, SEEK_SET);

	return PE_RES_NOERROR;
}

int xFileIO::WriteData(void* pInputData, long lSize)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fwrite(pInputData, 1, lSize, m_pFile);
	fflush(m_pFile);

	return PE_RES_NOERROR;
}

int xFileIO::WriteString(const char* strOutputText)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fputs(strOutputText, m_pFile);
	fflush(m_pFile);

	return PE_RES_NOERROR;
}

long xFileIO::GetCurrentOffset()
{
	return ftell(m_pFile);
}

int xFileIO::CloseFile()
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fclose(m_pFile);

	return PE_RES_NOERROR;
}

bool xFileIO::FileExists(const char *strPath)
{
	struct stat filedata;
	
	if(stat(strPath, &filedata))
		return false;
	
	return !(filedata.st_mode & S_IFDIR);
}

bool xFileIO::DirectoryExists(const char *strPath)
{
	struct stat filedata;
	
	if(stat(strPath, &filedata))
		return false;
	
	return (filedata.st_mode & S_IFDIR);
}
