/////////////////////////////////////////////////////////////////
// Name: xFont.h
/////////////////////////////////////////////////////////////////
// Description:
// - Holds font.
/////////////////////////////////////////////////////////////////
// - 26.10.2004, Update, Arto Ruotsalainen (Arex)
//   Fixed a very bad memory leak from Update function. 
//
// - 8.8.2004, Update, Arto Ruotsalainen (Arex)
//   Added Update function. Can be used for non-static
//   text stuff like counters, etc.
//   Handles string cases:
//	 * Same
//   * Bigger
//   * Smaller
//
// - 6.8.2004, Update, Arto Ruotsalainen(Arex)
//   File created,
//
// Note!
// *.p2d characters must be in ASCII order, like the first
// is space (' '), second is '!', and so on...
/////////////////////////////////////////////////////////////////
#ifndef XFONT_H_
#define XFONT_H_

#include <string.h>
#include "xEngineDefines.h"
#include "xColor.h"
#include "xPoolItem.h"

struct xFont
{
	xRECTf		*m_rectCharRects;
	int			*m_iCharWidths;
	int			*m_iCharHeights;
	uint32_t	 m_iNumCharacters;
	int			 m_iSize;
	int			 m_iResult;

	xTexture	*m_pTexture;
	xColor		 m_clrColor1;
	xColor		 m_clrColor2;

	// Font Pool Helpers
	char	 m_strObjectPath[256];
	int		 m_iReferenceCount;
	bool	 m_bLoaded;
	xFont	*m_pNext;
	xFont	*m_pPrev;
};

void xFontLoad(xFont *pThis, const char *strFontName);
void xFontRefreshRenderables(xFont *pThis);

FORCE_INLINE int xFontGetSize(xFont *pThis) { return pThis->m_iSize; }
FORCE_INLINE int xFontGetWidth(xFont *pThis, const char cCharacter) { return pThis->m_iCharWidths[(int)cCharacter - 32]; }
FORCE_INLINE int xFontGetHeight(xFont *pThis, const char cCharacter) { return pThis->m_iCharHeights[(int)cCharacter - 32]; }
FORCE_INLINE float xFontGetCharacterLeft(xFont *pThis, const char cCharacter) { return pThis->m_rectCharRects[(int)cCharacter - 32].left; }
FORCE_INLINE float xFontGetCharacterRight(xFont *pThis, const char cCharacter) { return pThis->m_rectCharRects[(int)cCharacter - 32].right; }
FORCE_INLINE float xFontGetCharacterTop(xFont *pThis, const char cCharacter) { return pThis->m_rectCharRects[(int)cCharacter - 32].top; }
FORCE_INLINE float xFontGetCharacterBottom(xFont *pThis, const char cCharacter) { return pThis->m_rectCharRects[(int)cCharacter - 32].bottom; }

FORCE_INLINE void xFontInit(xFont *pThis)
{
	pThis->m_iCharWidths		= 0;
	pThis->m_iCharHeights		= 0;
	pThis->m_pTexture			= 0;
	pThis->m_iReferenceCount	= 0;
	pThis->m_bLoaded			= 0;
	pThis->m_pNext				= 0;
	pThis->m_pPrev				= 0;
}

FORCE_INLINE void xFontDelete(xFont *pThis)
{
	if(pThis->m_pTexture)
		xGraphicsDeleteTexture(pThis->m_pTexture);

	if(pThis->m_iCharWidths)
		dlfree(pThis->m_iCharWidths);

	if(pThis->m_iCharHeights)
		dlfree(pThis->m_iCharHeights);
}

FORCE_INLINE int xFontGetWidth(xFont *pThis, const char *strString)
{
	int iwidth = 0;

	for(int i = 0; i < strlen(strString); i++)
		iwidth += xFontGetWidth(pThis, strString[i]);

	return iwidth;
}

FORCE_INLINE void xFontUpdateNumbericString(int iValue, char* strOut)
{
	itoa(iValue, strOut, 10);

	for(int i = 0; i < strlen(strOut); i++)
		strOut[i] -= '0' - 32;
}

#endif