#include "Paper.h"

xGraphics g_xGraphics;

const GLushort g_u8RectIndices[] = {
     0,  1,  2,     2,  3,  0,
     4,  5,  6,     6,  7,  4,
     8,  9, 10,    10, 11,  8,
    12, 13, 14,    14, 15, 12,
    16, 17, 18,    18, 19, 16,
    20, 21, 22,    22, 23, 20,
    24, 25, 26,    26, 27, 24,
    28, 29, 30,    30, 31, 28,
    32, 33, 34,    34, 35, 32
};

FORCE_INLINE void SkipExtension(uint8_t u8ExtensionType, xFileIO *pFileIO);

int xGraphicsInit(void* pWindow, int iWindowWidth, int iWindowHeight, bool bFullscreenApp, int iBPP)
{
    glGenBuffers(1, &g_xGraphics.m_uiRectIndexBuffer);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_xGraphics.m_uiRectIndexBuffer);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(g_u8RectIndices) / 9, g_u8RectIndices, GL_STATIC_DRAW);
	
    glGenBuffers(1, &g_xGraphics.m_ui3RectIndexBuffer);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_xGraphics.m_ui3RectIndexBuffer);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, (sizeof(g_u8RectIndices) / 9) * 3, g_u8RectIndices, GL_STATIC_DRAW);
	
    glGenBuffers(1, &g_xGraphics.m_ui4RectIndexBuffer);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_xGraphics.m_ui4RectIndexBuffer);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, (sizeof(g_u8RectIndices) / 9) * 4, g_u8RectIndices, GL_STATIC_DRAW);
	
    glGenBuffers(1, &g_xGraphics.m_ui9RectIndexBuffer);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, g_xGraphics.m_ui9RectIndexBuffer);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(g_u8RectIndices), g_u8RectIndices, GL_STATIC_DRAW);
	
	glDisable(GL_CULL_FACE);

	return PE_RES_NOERROR;
}

void xGraphicsUpdate(double dTime)
{
	xRenderDataShader *shaderdata;
	xRenderDataTexture *texturedata;
	xRenderDataVerts *vertdata;
	Matrix4 modelViewMatrix, modelViewProjectionMatrix;
	int indicelength;
		
	shaderdata = g_xGraphics.m_pSceneGraph->m_xRenderData.m_pNext;

	if(g_xGraphics.m_iFrameSkip <= 0)
	{
		Matrix4MakeTranslation(&modelViewMatrix, 0.0, 0.0, 0.0);
		
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glEnable(GL_BLEND);

		while(shaderdata)
		{
			if(shaderdata->m_bRender)
			{
				switch(shaderdata->m_iBlendMode)
				{
					case PE_BLEND_MODE_NONE:
					case PE_BLEND_MODE_NONE_OUTLINED:
						glEnable(GL_DEPTH_TEST);
						glEnableVertexAttribArray(g_xGraphics.m_xGrayScalePixel.m_iPosition);
						glEnableVertexAttribArray(g_xGraphics.m_xGrayScalePixel.m_iTextureCoord);
						glUseProgram(g_xGraphics.m_uiGrayScalePixelProgramIndex);
						glDisable(GL_BLEND);
						break;

					case PE_BLEND_MODE_ALPHA:
					case PE_BLEND_MODE_ALPHA_NOSHADER:
						glEnableVertexAttribArray(g_xGraphics.m_xGrayScalePixel.m_iPosition);
						glEnableVertexAttribArray(g_xGraphics.m_xGrayScalePixel.m_iTextureCoord);
						glUseProgram(g_xGraphics.m_uiGrayScalePixelProgramIndex);
						glEnable(GL_BLEND);
						glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
						break;

					// NOT USED in Break Blocks
					case PE_BLEND_MODE_INVERTED_COLOR:
						glUseProgram(g_xGraphics.m_uiAlphaKillPixelProgramIndex);
						glEnable(GL_BLEND);
						glBlendFunc(GL_ONE_MINUS_DST_COLOR, GL_ZERO);
						break;
			
					// NOT USED in Break Blocks
					case PE_BLEND_MODE_MASKED:
						glUseProgram(g_xGraphics.m_uiAlphaMaskPixelProgramIndex );
						glEnable(GL_BLEND);
						glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
						break;

					case PE_BLEND_MODE_2COLOR:
						glEnableVertexAttribArray(g_xGraphics.m_xDualColorPixel.m_iPosition);
						glEnableVertexAttribArray(g_xGraphics.m_xDualColorPixel.m_iTextureCoord);
						glUseProgram(g_xGraphics.m_uiDualColorPixelProgramIndex);
						glEnable(GL_BLEND);
						glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
						break;
				}
				
				texturedata = shaderdata->m_xTextureDataList.m_pNext;

				while(texturedata)
				{
					if(texturedata->m_bRender)
					{
						switch(shaderdata->m_iBlendMode)
						{
							case PE_BLEND_MODE_MASKED:
								// set the grayScaleFactor value
								glUniform4f(g_xGraphics.m_xAlphaMaskPixel.m_iTextureOffset,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].R,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].G,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].B,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].A);

								glUniform4f(g_xGraphics.m_xAlphaMaskPixel.m_iTextureRatio,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].R,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].G,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].B,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].A);

								glActiveTexture(GL_TEXTURE0);
								glBindTexture(GL_TEXTURE_2D, texturedata->m_uiTextureIndexArray[1]);
								glUniform1i(g_xGraphics.m_xAlphaMaskPixel.m_iMask, 0);

								glActiveTexture(GL_TEXTURE1);
								break;

							case PE_BLEND_MODE_2COLOR:
								glUniform4f(g_xGraphics.m_xDualColorPixel.m_iSecondColor,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].R,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].G,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].B,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].A);
								glUniform4f(g_xGraphics.m_xDualColorPixel.m_iColorDifference,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].R,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].G,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].B,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].A);
								break;
						}

						if(texturedata->m_iTextureIndexArrayLength > 0)
						{

							glBindTexture(GL_TEXTURE_2D, texturedata->m_uiTextureIndexArray[0]);

							if(shaderdata->m_iBlendMode == PE_BLEND_MODE_MASKED)
								glUniform1i(g_xGraphics.m_xAlphaMaskPixel.m_iTexture, 1);
							else if(shaderdata->m_iBlendMode == PE_BLEND_MODE_INVERTED_COLOR)
								glUniform1i(g_xGraphics.m_xAlphaKillPixel.m_iTexture, 0);
							else if(shaderdata->m_iBlendMode == PE_BLEND_MODE_2COLOR)
								glUniform1i(g_xGraphics.m_xDualColorPixel.m_iTexture, 0);
							else if(shaderdata->m_iBlendMode == PE_BLEND_MODE_NONE_OUTLINED || shaderdata->m_iBlendMode == PE_BLEND_MODE_ALPHA)
								glUniform1i(g_xGraphics.m_xGrayScalePixel.m_iTexture, 0);
						}
						else
						{
							if(texturedata->m_uiRenderType == GL_LINES || texturedata->m_uiRenderType == GL_LINE_LOOP || texturedata->m_uiRenderType == GL_LINE_STRIP)
								glLineWidth(texturedata->m_fLineWidth);
						}

						vertdata = texturedata->m_xVertDataList.m_pNext;

						while(vertdata && vertdata->m_uiVertexBuffer)
						{
							if(vertdata->m_bRender)
							{
								if(shaderdata->m_bIs3D)
								{
									Matrix4Rotate(&modelViewMatrix, vertdata->m_xRotation.x, 1.0, 0.0, 0.0);
									Matrix4Rotate(&modelViewMatrix, vertdata->m_xRotation.y, 0.0, 1.0, 0.0);
									Matrix4Rotate(&modelViewMatrix, vertdata->m_xRotation.z, 0.0, 0.0, 1.0);

									Matrix4Multiply(&modelViewProjectionMatrix, &g_xGraphics.m_matPerspective, &modelViewMatrix);
									
									indicelength = vertdata->m_iVectorArrayLength;
								}
								else
								{
									if(vertdata->m_xRotation.x)
										Matrix4Rotate(&modelViewMatrix, vertdata->m_xRotation.x, 1.0, 0.0, 0.0);

									Matrix4Multiply(&modelViewProjectionMatrix, &g_xGraphics.m_matProjection, &modelViewMatrix);
									
									indicelength = 6 * vertdata->m_iVectorArrayLength / 4;
								}
								
								glBindBuffer(GL_ARRAY_BUFFER, vertdata->m_uiVertexBuffer);
								
								if(shaderdata->m_iBlendMode == PE_BLEND_MODE_2COLOR)
								{
									glUniformMatrix4fv(g_xGraphics.m_xDualColorPixel.m_iMVP, 1, 0, modelViewProjectionMatrix.m);
									glUniform1i(g_xGraphics.m_xDualColorPixel.m_iTexture, 0);
									
									glVertexAttribPointer(g_xGraphics.m_xDualColorPixel.m_iPosition, 3, GL_FLOAT, GL_FALSE, sizeof(xTexturedVert), (const GLvoid *)0);
									glVertexAttribPointer(g_xGraphics.m_xDualColorPixel.m_iTextureCoord, 2, GL_FLOAT, GL_FALSE, sizeof(xTexturedVert), (const GLvoid *)12);
								}
								else
								{
									glUniformMatrix4fv(g_xGraphics.m_xGrayScalePixel.m_iMVP, 1, 0, modelViewProjectionMatrix.m);
									glUniform1i(g_xGraphics.m_xGrayScalePixel.m_iTexture, 0);
									glUniform4fv(g_xGraphics.m_xGrayScalePixel.m_iColor, 1, (float*)&vertdata->m_xColor);

									glVertexAttribPointer(g_xGraphics.m_xGrayScalePixel.m_iPosition, 3, GL_FLOAT, GL_FALSE, sizeof(xTexturedVert), (const GLvoid *)0);
									glVertexAttribPointer(g_xGraphics.m_xGrayScalePixel.m_iTextureCoord, 2, GL_FLOAT, GL_FALSE, sizeof(xTexturedVert), (const GLvoid *)12);
								}

								glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, vertdata->m_uiIndexBuffer);
								glDrawElements(GL_TRIANGLES, indicelength, GL_UNSIGNED_SHORT, 0);
								
								if(shaderdata->m_bIs3D || vertdata->m_xRotation.x)
									Matrix4MakeTranslation(&modelViewMatrix, 0.0, 0.0, 0.0);
							}

							vertdata = vertdata->m_pNext;
						}
					}

					texturedata = texturedata->m_pNext;
				}

				switch(shaderdata->m_iBlendMode)
				{
					case PE_BLEND_MODE_NONE:
					case PE_BLEND_MODE_NONE_OUTLINED:
						glDisable(GL_DEPTH_TEST);
						glEnable(GL_BLEND);
						break;

					case PE_BLEND_MODE_MASKED:
						glActiveTexture(GL_TEXTURE1);
						glDisable(GL_TEXTURE_2D);
						glActiveTexture(GL_TEXTURE0);
						break;
				}
			}

			shaderdata = shaderdata->m_pNext;
		}

		glDisable(GL_BLEND);
	}
	else
		g_xGraphics.m_iFrameSkip--;
}

void xGraphicsShutdown()
{
	//wglMakeCurrent(NULL, NULL);
	//wglDeleteContext(g_xGraphics.m_hRC);
	//ReleaseDC(g_xGraphics.m_hWnd, g_xGraphics.m_hDC);
}

void xGraphicsGetDesktopResolution(int *pScreenWidth, int *pScreenHeight)
{
	*pScreenWidth = g_xGraphics.m_iScreenWidth;
	*pScreenHeight = g_xGraphics.m_iScreenHeight;
}

void xGraphicsStartGettingAllDesktopResolutions()
{
	g_xGraphics.m_iAllScreenResolutionsIndex = 0;
	g_xGraphics.m_dwDisplayFrequency = 0;
}

bool GetDictionaryBoolean(CFDictionaryRef theDict, const void* key) 
{
	Boolean value = false;
	CFBooleanRef boolRef;
	boolRef = (CFBooleanRef)CFDictionaryGetValue(theDict, key);

	if (boolRef != NULL)
		value = CFBooleanGetValue(boolRef); 	
	
	return value;
}

long GetDictionaryLong(CFDictionaryRef theDict, const void* key) 
{
	long value = 0;
	CFNumberRef numRef;
	numRef = (CFNumberRef)CFDictionaryGetValue(theDict, key);
	
	if (numRef != NULL)
		CFNumberGetValue(numRef, kCFNumberLongType, &value);
	
	return value;
}

bool xGraphicsGetNextDesktopResolution(int *pScreenWidth, int *pScreenHeight)
{
	return 0;
}

bool xGraphicsIsResolutionAvailable(int iScreenWidth, int iScreenHeight, bool bFullscreen)
{
	return 1;
}

xTexture* xGraphicsGetTexture(const char* strFilePath, int iFrame)
{
	int i;
	char strFile[256];
	xTexture *temptexture, *ret;
	xFileIO objectfile;
	xP2DFileHeader fileheader;
	xP2DSpriteHeader spriteheader;
	RECT temprect;

	sprintf(strFile, "%s%i", strFilePath, iFrame);

	ret = xGraphicsTexturePoolNew(strFile);

	if(!ret->m_bLoaded)
	{
		if(objectfile.OpenFile(strFilePath, true, true))
			assert(0);

		if(objectfile.ReadData(&fileheader, sizeof(xP2DFileHeader)))
			assert(0);

		if(objectfile.ReadData(&spriteheader, sizeof(xP2DSpriteHeader)))
			assert(0);

		for(i = 0; i < spriteheader.m_iNumFrames; i++)
		{
			if(i == iFrame)
			{
				xTextureCreateTexture(ret, &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
				strcpy(ret->m_strObjectPath, strFile);
				ret->m_bLoaded = true;
			}
			else
			{
				sprintf(strFile, "%s%i", strFilePath, i);
				temptexture = xGraphicsTexturePoolNew(strFile);
				xTextureCreateTexture(temptexture, &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
				strcpy(temptexture->m_strObjectPath, strFile);
				temptexture->m_bLoaded = true;
			}

			objectfile.ReadData(&temprect, sizeof(RECT));
		}

		objectfile.CloseFile();
	}

	return ret;
}

int xGraphicsGetTextures(const char* strFilePath, xTexture ***pTextures, float *fAnimationRate)
{
	int i;
	char strFile[256];
	xTexture *temptextures[256];
	xFileIO objectfile;
	xP2DFileHeader fileheader;
	xP2DSpriteHeader spriteheader;
	RECT temprect;

	sprintf(strFile, "%s0", strFilePath);

	temptextures[0] = xGraphicsTexturePoolNew(strFile);

	if(objectfile.OpenFile(strFilePath, true, true))
		assert(0);

	if(objectfile.ReadData(&fileheader, sizeof(xP2DFileHeader)))
		assert(0);

	if(objectfile.ReadData(&spriteheader, sizeof(xP2DSpriteHeader)))
		assert(0);

	if(!temptextures[0]->m_bLoaded)
	{
		*pTextures = (xTexture**)dlmalloc(sizeof(xTexture*) * spriteheader.m_iNumFrames);
		(*pTextures)[0] = temptextures[0];

		for(i = 0; i < spriteheader.m_iNumFrames; i++)
		{
			if(i > 0)
			{
				sprintf(strFile, "%s%i", strFilePath, i);
				(*pTextures)[i] = xGraphicsTexturePoolNew(strFile);
			}

			xTextureCreateTexture((*pTextures)[i], &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
			strcpy((*pTextures)[i]->m_strObjectPath, strFile);
			(*pTextures)[i]->m_bLoaded = true;

			objectfile.ReadData(&temprect, sizeof(RECT));
		}
	}
	else
	{
		i = 1;
		while(1)
		{
			sprintf(strFile, "%s%i", strFilePath, i);
			temptextures[i] = xGraphicsTexturePoolGet(strFile);

			if(!temptextures[i])
				break;

			i++;
		}

		*pTextures = (xTexture**)dlmalloc(sizeof(xTexture*) * i);

		for(spriteheader.m_iNumFrames = 0; spriteheader.m_iNumFrames < i; spriteheader.m_iNumFrames++)
			(*pTextures)[spriteheader.m_iNumFrames] = temptextures[spriteheader.m_iNumFrames];
	}

	objectfile.CloseFile();

	if(fAnimationRate)
		*fAnimationRate = 1.0f / (float)spriteheader.m_iFPS;

	return spriteheader.m_iNumFrames;
}

void xGraphicsReloadTexturePool()
{
	xTexture *texture;
	xFileIO objectfile;
	char texturepath[256], framestring[4];
	xP2DFileHeader fileheader;
	xP2DSpriteHeader spriteheader;
	RECT temprect;
	uint8_t extensiontype;
	int frame, i, j;

	texture = g_xGraphics.m_xTexturePool.m_pNext;

	while(texture)
	{
		strcpy(texturepath, texture->m_strObjectPath);

		for(i = strlen(texturepath) - 1; i > 0; i--)
		{
			if(texturepath[i] == '.')
			{
				framestring[0] = texturepath[i + 4];

				if(framestring[0] != '\0')
				{
					framestring[1] = texturepath[i + 5];

					if(framestring[1] != '\0')
					{
						framestring[2] = texturepath[i + 6];
						framestring[3] = '\0';
					}

					frame = atoi(framestring);
				}
				else
					frame = 0;

				texturepath[i + 4] = '\0';
				break;
			}
		}

		if(objectfile.OpenFile(texturepath, true, true))
			assert(0);

		if(objectfile.ReadData(&fileheader, sizeof(xP2DFileHeader)))
			assert(0);

		for(j = 0; j < fileheader.m_iNumSprites; j++)
		{
			if(objectfile.ReadData(&spriteheader, sizeof(xP2DSpriteHeader)))
				assert(0);

			while(spriteheader.m_iNumExtensions > 0)
			{
				objectfile.ReadData(&extensiontype, sizeof(uint8_t));

				SkipExtension(extensiontype, &objectfile);

				spriteheader.m_iNumExtensions--;
			}

			for(i = 0; i < spriteheader.m_iNumFrames; i++)
			{
				if(i + j == frame)
				{
					xTextureCreateTexture(texture, &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
				}
				else
				{
						xTextureSkipTexture(&objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
				}

				objectfile.ReadData(&temprect, sizeof(RECT));
			}
		}

		objectfile.CloseFile();

		texture = texture->m_pNext;
	}
}

FORCE_INLINE void SkipExtension(uint8_t u8ExtensionType, xFileIO *pFileIO)
{
	uint8_t tempint8array[3];
	uint16_t tempint16;

	switch(u8ExtensionType)
	{
		case PE_P2D_EXTTYPE_PARTICLE:
		{
			// Skip ahead to the colors(because they are variable sized)
			pFileIO->SkipAhead(73);

			// Get the number of colors
			pFileIO->ReadData(&tempint16, sizeof(uint16_t));

			// For each color, read in the R, G, and B values(1 byte each)
			while(tempint16 > 0)
			{
				pFileIO->ReadData((void*)&tempint8array, sizeof(uint8_t) * 3);
				tempint16--;
			}

			break;
		}
	}
}

void xGraphicsDeleteTexture(xTexture* pTexture)
{
	xTexture *current;
	
	DEBUGLOG("xGraphicsDeleteTexture: %s - %i\n", pTexture->m_strObjectPath, pTexture->m_iReferenceCount);

	if(pTexture->m_iReferenceCount == 0)
	{
		GLuint texids[1] = {pTexture->m_uiID};
		glDeleteTextures(1, texids);

		// Remove it from the Texture Pool
		current = g_xGraphics.m_xTexturePool.m_pNext;
		while(current)
		{
			if(current == pTexture)
			{
				if(current->m_pPrev)
					current->m_pPrev->m_pNext = current->m_pNext;

				if(current->m_pNext)
					current->m_pNext->m_pPrev = current->m_pPrev;

				dlfree(pTexture);

				break;
			}

			current = current->m_pNext;
		}
	}
	else
		pTexture->m_iReferenceCount--;
}

void xGraphicsDeleteTextures(xTexture **pTextures, int iCount)
{
	int i;
	
	DEBUGLOG("xGraphicsDeleteTextures\n");

	for(i = 0; i < iCount; i++)
		xGraphicsDeleteTexture(pTextures[i]);

	dlfree(pTextures);
}
