#ifndef XHTTPQUERY_H_
#define XHTTPQUERY_H_

#include "md5/md5.h"

enum xHTTPQueryType
{
	BB_HTTPQUERY_NONE = 0,
	BB_HTTPQUERY_UPDATE,
	BB_HTTPQUERY_REGISTER,
	BB_HTTPQUERY_LOGIN,
	BB_HTTPQUERY_GETSERVERLIST,
	BB_HTTPQUERY_ADDSERVERTOLIST,
	BB_HTTPQUERY_REMOVESERVERFROMLIST,
	BB_HTTPQUERY_OPPONENTCONNECT,
	BB_HTTPQUERY_BATTLEFINISHED,
	BB_HTTPQUERY_GETLEADERBOARDDATA,
	BB_HTTPQUERY_UNLOCKACHIEVEMENT,
	BB_HTTPQUERY_GETACHIEVEMENTS,
	BB_HTTPQUERY_GETSPLEADERBOARD,
	BB_HTTPQUERY_UPDATESPLEADERBOARD
};

class xHTTPQuery
{
public:
	xHTTPQuery() { ClearQuery(); }

	void CheckForUpdate();
	void Register(char *strUserName, char *strPassword, char *strCDKey);
	void Login(char *strUserName, char *strPassword, bool bPasswordEncrypted);
	void GetServerList(class xGUIColumnListBox *pServerListBox);
	void AddServerToList(int iServerPort, char *strUserName, char *strSongName, int iSongDifficulty);
	void RemoveServerFromList(int iServerPort, char *strUserName);
	void OpponentConnected(int iServerPort, char *strUserName, char *strOpponentName);
	void BattleFinished(char *strServerAddr, int iServerPort, int iIsServer, char *strUserName);
	void GetLeaderboardData(char* strUserName);
	void UnlockAchievement(char* strAchievement, char* strUserName, char* strPassword);
	void GetAchievements(char* strUserName);
	void GetSPLeaderboard(char* strUserName, int iDifficulty, int iStartIndex = 0);
	int  GetSPLeaderboardStartIndex() { return m_iStartIndex; }
	void UpdateSPLeaderboard(char* strUserName, char* strPassword, int iDifficulty, int iCompletionPercentage, int iAverageCrowdPercentage, int iTotalPlayTime);

	void Update();
	void ClearQuery() { m_bHasConnected = false; m_bFoundContentStart = false; m_iCurrentStep = 0; m_iQueryType = BB_HTTPQUERY_NONE; m_iOldQueryType = BB_HTTPQUERY_NONE; m_strBuffer[0] = '\0'; }
	void CancelQuery();

	bool UpdateQueryComplete();
	bool RegisterQueryComplete(int &iOutErrorCode);
	bool LoginQueryComplete(int &iOutRank, int &iOutPoints);
	bool GetAchievementsComplete(int* iOutAchievements, int iSize);

	void GenerateMD5(char *strInput, char strOutput[33]);

	xHTTPQueryType GetQueryType() { return m_iQueryType; }
	xHTTPQueryType GetOldQueryType() { return m_iOldQueryType; }

	// Update Helpers
	char	*m_strFileList;
	int		 m_iFileListOffset;
	int		 m_iPreviousVersion;
	int		 m_iCurrentVersion;
	char	 m_strDownloadLocation[64];
	char	 m_strDownloadHost[64];
	char	 m_strDownloaderHash[32];

protected:
	void StartQuery(xHTTPQueryType iQueryType);

private:
	bool			m_bHasConnected;
	bool			m_bFoundContentStart;
	int				m_iCurrentStep;
	xHTTPQueryType  m_iQueryType;
	xHTTPQueryType	m_iOldQueryType;
	char			m_strBuffer[MAX_NETWORK_BUFFERSIZE * 100];

	// Register Helpers
	char	*m_strUserName;		// Shared with Login, AddServerToList, RemoveServerFromList, OpponentConnected, BattleFinished, GetLeaderboardData, UnlockAchievement, GetAchievements, and SP Leaderboard
	char	 m_strPassword[33];	// Shared with Login, UnlockAchievement
	char	*m_strCDKey;
	int		 m_iReturnValue;	// Shared with Login

	// Login Helpers
	int	m_iPoints;

	// GetServerList Helpers
	class xGUIColumnListBox	*m_pServerListBox;

	// AddServerToList Helpers
	int		 m_iServerPort;		// Shared with RemoveServerFromList, OpponentConnected, and BattleFinished
	char	*m_strSongName;
	int		 m_iSongDifficulty;

	// OpponentConnected Helpers
	char	*m_strOpponentName;
	
	// BattleFinished Helpers
	char	*m_strServerAddress;
	int		 m_iIsServer;

	// GetLeaderboardData Helpers
	xGUIWindowList	*m_pWindow;		// Shared with SP Leaderboard
	int				 m_iIndex;		// Shared with SP Leaderboard
	int				 m_iStartIndex;	// Shared with SP Leaderboard

	// UnlockAchievement Helpers
	char	*m_strAchievement;

	// GetAchievements Helpers
	int	m_pAchievements[256];

	// GetSPLeaderboard Helpers
	int m_iDifficulty;	// Shared with Update

	// UpdateSPLeaderboard Helpers
	int m_iCompletion;
	int m_iCrowd;
	int m_iTime;
};

#endif