/////////////////////////////////////////////////////////////////
// Name: xKeyFrame.h
/////////////////////////////////////////////////////////////////
// Description:
// - Maintains information about a frame in a key-framed 
//   animation
/////////////////////////////////////////////////////////////////
// - 8.5.2004, Update, Greg Feldman
//   File Created
/////////////////////////////////////////////////////////////////
#ifndef XKEYFRAME_H_
#define XKEYFRAME_H_

#include "xEngineDefines.h"
#include "xList.h"
#include "xRenderable2D.h"

#define ANIMATION_FPS	30

class xGUIObject;

enum xAnimationType
{
	PE_ANIMTYPE_QUAD,
	PE_ANIMTYPE_QUAD9,
	PE_ANIMTYPE_UIOBJECT,
	PE_ANIMTYPE_OTHER
};

struct xKeyFrame
{
	int		m_iFrame;
	float	m_fX;
	float	m_fY;
	float	m_fWidth;
	float	m_fHeight;
	float	m_fAngle;
	float	m_fScaleX;
	float	m_fScaleY;
	float	m_fAlpha;

	xKeyFrame	*m_pNext;
	xKeyFrame	*m_pPrev;
};

struct xAnimation
{
	void (*pfnUpdateRenderables)(xGUIObject *pThis);

	xKeyFrame		 m_xKeyFrameList;
	xKeyFrame		*m_pCurKeyFrame;
	double			 m_dCurKeyFrameTime;
	xAnimationType	 m_iAnimType;
	void			*m_pObject;
	xRenderable2D	*m_pRenderable;

	bool	m_bGenerated;
	bool	m_bDone;
	bool	m_bLoop;
	int		m_iCurFrameChange;
	float	m_fCurXChange;
	float	m_fCurYChange;
	float	m_fCurWidthChange;
	float	m_fCurHeightChange;
	float	m_fCurAngleChange;
	float	m_fCurScaleXChange;
	float	m_fCurScaleYChange;
	float	m_fCurAlphaChange;

	xAnimation	*m_pNext;
	xAnimation	*m_pPrev;
};

FORCE_INLINE void xKeyFrameInit(xKeyFrame *pThis)
{
	pThis->m_iFrame		= 0;
	pThis->m_fX			= 0.0f;
	pThis->m_fY			= 0.0f;
	pThis->m_fWidth		= 0.0f;
	pThis->m_fHeight	= 0.0f;
	pThis->m_fAngle		= 0.0f;
	pThis->m_fScaleX	= 1.0f;
	pThis->m_fScaleY	= 1.0f;
	pThis->m_fAlpha		= 1.0f;
	pThis->m_pNext		= 0;
	pThis->m_pPrev		= 0;
}

void xAnimationInit(xAnimation *pThis, void* pObject, xRenderable2D *pRenderable, void (*pfnUpdateRenderablesCallback)(xGUIObject *pThis));
void xAnimationInitQuad(xAnimation *pThis, xSceneGraph *pSceneGraph, const char *strObjectFile, int iFrame, float fDepth);
void xAnimationUpdate(xAnimation *pThis, double dTime);
void xAnimationDelete(xAnimation *pThis, xSceneGraph *pSceneGraph, bool bDeleteTexture);
void xAnimationLoadedAllKeyFrames(xAnimation *pThis);

FORCE_INLINE void xAnimationAddKeyFrame(xAnimation *pThis, int iFrame, float fX, float fY, float fWidth, float fHeight, float fAngle, float fScaleX, float fScaleY, float fAlpha)
{
	xKeyFrame *curkeyframe = &pThis->m_xKeyFrameList;

	while(curkeyframe->m_pNext)
		curkeyframe = curkeyframe->m_pNext;

	curkeyframe->m_pNext = (xKeyFrame*)dlmalloc(sizeof(xKeyFrame));
	xKeyFrameInit(curkeyframe->m_pNext);

	curkeyframe->m_pNext->m_pPrev	= curkeyframe;
	curkeyframe						= curkeyframe->m_pNext;

	curkeyframe->m_iFrame	= iFrame;
	curkeyframe->m_fX		= fX;
	curkeyframe->m_fY		= fY;
	curkeyframe->m_fWidth	= fWidth;
	curkeyframe->m_fHeight	= fHeight;
	curkeyframe->m_fAngle	= fAngle;
	curkeyframe->m_fScaleX	= fScaleX;
	curkeyframe->m_fScaleY	= fScaleY;
	curkeyframe->m_fAlpha	= fAlpha;

	if(!pThis->m_pCurKeyFrame)
	{
		pThis->m_pCurKeyFrame = curkeyframe;
		pThis->m_bDone = 0;
		pThis->m_bLoop = 0;
	}
}

#endif