#include "peGame.h"

char *g_strDifficultyNames[5] = {"Simple", "Easy", "Normal", "Hard", "Insane"};
#ifdef DEMO_RELEASE
char *g_strTourBackgrounds[7] = {"BkgTutorial", "BkgStreet", "BkgCity", "BkgCity", "BkgCity", "BkgCity", "BkgCity"};
#else
char *g_strTourBackgrounds[7] = {"BkgTutorial", "BkgStreet", "BkgCity", "BkgState", "BkgRegional", "BkgNational", "BkgGlobal"};
#endif

FORCE_INLINE bool xMenuSinglePlayerLogin(xSinglePlayerLogin xSPLogin, char* strLoginMessageBox, char* strLoginMenu, char* strTargetMenu);
FORCE_INLINE void xMenuSetChallengeScrollValues(int iIndex, int iStartIndex, int iTargetIndex);
FORCE_INLINE void xMenuLaunchChallenge(int iChallenge, char *strEnemy, int iNumColors, bool bLoopSong, bool bShowComboCounter, bool bShowClearCounter);
FORCE_INLINE void xMenuSetFreePlaySongScrollValues(int iIndex, int iStartIndex, int iTargetIndex);
FORCE_INLINE void xMenuUpdateFreePlayRenderable(xGUIObject *pObject, xRenderable2D *pStart, xRenderable2D *pTarget, float fProgress);

bool xMenu::Init(const char *strFile)
{
	xColor tempcolor;
	int i;

	m_bUpdatingGame = false;

	m_xGUI.Init(&m_xSceneGraph);

	xSceneGraphInit(&m_xSceneGraph);

	if(!m_pAudioStream)
	{
		m_pAudioStream = g_xPaperAPI.m_xAudio.LoadMusic(BB_MUSICPATH_MENU, true);
		m_pAudioStream->SetVolume(g_xOptions.m_fMusicVolume);
	}

	m_pAudioStream->Play();

	char nextlevel[256] = BB_LEVELS_FIRSTLEVEL;
	for(int i = 0; i < BB_LEVELS_MAX; i++)
		g_xGame.GetPreloadGameOptions(nextlevel, 0, 0, 0, 0, nextlevel, &m_xPreloadGameOptions[i + 1]);

	xGraphicsSetSceneGraph(&m_xSceneGraph);

	AddMenuWindow(BB_WINDOW_MENU_INTRO);
	m_fKillMenuIntroTime = 3.0f;

	float p[8];
	xRenderable2D temp;
	xRenderable2DInit(&temp);
	temp.m_fWidth = 100.0;
	temp.m_fHeight = 100.0;
	temp.m_fRotation = 45.0;
	xRenderable2DRotateVerts(&temp, &p[0], &p[1], &p[2], &p[3], &p[4], &p[5], &p[6], &p[7]);

	temp.m_fWidth = 75.0;
	temp.m_fRotationCenterX = 50.0f;
	temp.m_fRotationCenterY = 50.0f;
	xRenderable2DRotateVerts(&temp, &p[0], &p[1], &p[2], &p[3], &p[4], &p[5], &p[6], &p[7]);

	if(!m_pAchievementIcon[0])
	{
		xGraphicsGetTextures(BB_IMAGE_ACHIEVEMENTICON, &m_pAchievementIconTextures);

		for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
		{
			m_pAchievementIcon[i] = xSceneGraphCreateQuad(BB_IMAGE_ACHIEVEMENTICON, 0);
			xSceneGraphInsertQuad(&m_xSceneGraph, m_pAchievementIcon[i], 1, BB_DEPTH_ACHIEVEMENTICON, 0, 1);
			xRenderable2DQuadSetPosition(m_pAchievementIcon[i], 1025, 769);
			xRenderable2DQuadSetSize(m_pAchievementIcon[i], BB_ACHIEVEMENT_ICONSIZE, BB_ACHIEVEMENT_ICONSIZE);
			xRenderable2DQuadUpdate(m_pAchievementIcon[i]);

			xRenderable2DTextInit(&m_xAchievementTitle[i], BB_DEPTH_ACHIEVEMENTTITLE);
			xRenderable2DTextSetFont(&m_xAchievementTitle[i], g_xGame.m_xGUI.GetFont(BB_FONT_ACHIEVEMENTTITLE));
			xRenderable2DTextSetPosition(&m_xAchievementTitle[i], 1025, 769);
			xRenderable2DTextSetScale(&m_xAchievementTitle[i], BB_ACHIEVEMENT_TITLESCALE, BB_ACHIEVEMENT_TITLESCALE);
			xRenderable2DTextSetColor(&m_xAchievementTitle[i], xColorInit(&tempcolor, 1.0f, 1.0f, 1.0f, 1.0f));
		}
	}

	return 1;
}

bool xMenu::PreUpdate(DWORD dwTime)
{
	xGUIWindowList *window;
	xButton *pressedkey;
	char temptext[64];
	int i;

	if(g_xMenu.m_iBindingKeyIndex != -1)
	{
		window = m_xGUI.GetWindow(BB_WINDOW_MENU_OPTIONS);

		pressedkey = g_xPaperAPI.m_xInput.GetKeyPressed();

		if(window && pressedkey)
		{
			g_xOptions.m_pButtons[g_xMenu.m_iBindingKeyIndex] = pressedkey;

			if(g_xMenu.m_iBindingKeyIndex < MAX_GAME_BUTTONS)
			{
				for(i = 0; i < MAX_GAME_BUTTONS; i++)
				{
					if(i != g_xMenu.m_iBindingKeyIndex && g_xOptions.m_pButtons[i] == g_xOptions.m_pButtons[g_xMenu.m_iBindingKeyIndex])
					{
						sprintf(temptext, BB_NAME_CONTROLSBUTTON, i + 1);
						((xGUIButton*)window->m_pData->GetChildByName(temptext))->SetText("---");
						g_xOptions.m_pButtons[i] = 0;
					}
				}
				
				sprintf(temptext, BB_NAME_CONTROLSBUTTON, g_xMenu.m_iBindingKeyIndex + 1);
			}
			else
				sprintf(temptext, BB_NAME_CONTROLSALTBUTTON, g_xMenu.m_iBindingKeyIndex - MAX_GAME_BUTTONS + 1);

			((xGUIButton*)window->m_pData->GetChildByName(temptext))->SetText(g_xPaperAPI.m_xInput.GetStringFromKey(pressedkey));

			g_xMenu.m_iBindingKeyIndex = -1;

			g_xOptions.SaveOptions(BB_SETTINGSPATH);

			pressedkey->Eaten();
		}
	}

	m_xGUI.PreUpdate(dwTime);

	m_pAudioStream->ContinuePlaying();

	if(m_xHTTPQuery.GetQueryType() != BB_HTTPQUERY_NONE)
		m_xHTTPQuery.Update();
	else if(m_xHTTPQuery.GetOldQueryType() != BB_HTTPQUERY_NONE)
	{
		int error;

		if(m_xHTTPQuery.UpdateQueryComplete())
		{
			if(GAME_VERSION == m_xHTTPQuery.m_iCurrentVersion)
				m_xGUI.DelayedRemoveWindow(BB_WINDOW_MENU_UPDATE);
			else
			{
				window = m_xGUI.GetWindow(BB_WINDOW_MENU_UPDATE);

				if(window)
				{
					((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_UPDATEAVAILABLE);
					window->m_pData->GetChildByName(BB_NAME_UPDATEBUTTON)->SetEnabled(1);
					window->m_pData->SetHidden(0);
					window->m_pData->UpdateRenderables();
				}
			}
		}
		else if(m_xHTTPQuery.LoginQueryComplete(g_xGame.m_xPlayer.m_iRank, g_xGame.m_xPlayer.m_iPoints))
		{
			if(g_xGame.m_xPlayer.m_iRank != 0)
			{
				m_xGUI.DelayedRemoveWindow(BB_WINDOW_MENU_LOGGINGIN);

				if(m_xSinglePlayerLogin == BB_SPLOGIN_SINGLEPLAYER)
				{
					g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

					if(g_xGame.m_xPlayer.m_strCDKey[0] == '+')
						g_xMenu.AddMenuWindow(xDecryptString(g_pEncryptedCDKeyWindow, BB_ENCRYPTED_LENGTH_CDKEYWINDOW, temptext, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3));
					else if(AddMenuWindow(BB_WINDOW_MENU_BATTLES))
						g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_BATTLES)->m_pData);
				}
				else if(m_xSinglePlayerLogin == BB_SPLOGIN_CHALLENGES)
				{
					if(AddMenuWindow(BB_WINDOW_MENU_CHALLENGES))
						g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_CHALLENGES)->m_pData);
				}
				else if(m_xSinglePlayerLogin == BB_SPLOGIN_FREEPLAY)
				{
					if(AddMenuWindow(BB_WINDOW_MENU_FREEPLAY))
						g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_FREEPLAY)->m_pData);
				}
				else if(m_xSinglePlayerLogin != BB_SPLOGIN_CHANGELOGIN)
					AddMenuWindow(BB_WINDOW_MENU_MULTIPLAYER);

				g_xGame.m_xPlayer.m_strCDKey[0] = '=';
			}
			else
			{
				if(m_xSinglePlayerLogin)
					memset(g_xCurrentProfile.m_strLoginName, 0, BB_MAXLOGINNAMELENGTH);

				window = m_xGUI.GetWindow(BB_WINDOW_MENU_LOGGINGIN);
				if(window)
					((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_LOGINFAILED);
			}
		}
		else if(m_xHTTPQuery.RegisterQueryComplete(error))
		{
			if(error == 0)
			{
				m_xGUI.DelayedRemoveWindow(BB_WINDOW_MENU_LOGGINGIN);

				if(m_xSinglePlayerLogin == BB_SPLOGIN_SINGLEPLAYER)
				{
					g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

					if(AddMenuWindow(BB_WINDOW_MENU_BATTLES))
						g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_BATTLES)->m_pData);
				}
				else if(m_xSinglePlayerLogin == BB_SPLOGIN_CHALLENGES)
				{
					if(AddMenuWindow(BB_WINDOW_MENU_CHALLENGES))
						g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_CHALLENGES)->m_pData);
				}
				else if(m_xSinglePlayerLogin == BB_SPLOGIN_FREEPLAY)
				{
					if(AddMenuWindow(BB_WINDOW_MENU_FREEPLAY))
						g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_FREEPLAY)->m_pData);
				}
				else if(m_xSinglePlayerLogin != BB_SPLOGIN_CHANGELOGIN)
					AddMenuWindow(BB_WINDOW_MENU_MULTIPLAYER);
			}
			else
			{
				window = m_xGUI.GetWindow(BB_WINDOW_MENU_LOGGINGIN);
				if(window)
				{
					if(error == 1)
					{
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDUSERNAME);
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDUSERNAME2);
					}
					else if(error == 2)
					{
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDPASSWORD);
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDPASSWORD2);
					}
					else if(error == 3)
					{
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDCDKEY);
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDCDKEY2);
					}
					else if(error == 4)
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_USERNAMETAKEN);
				}
			}
		}
	}

	if(m_pPingInfo && m_pServerListBox->m_iNumItems > 0)
	{
		if(m_pPingInfo[m_pServerListBox->m_iNumItems - 1].m_iRequestCount < 3)
			RepingServers();
		else if(m_pPingInfo[0].m_iRequestCount < 4 && g_xPaperAPI.m_xPlatform.GetTime() - m_pPingTimes[m_pServerListBox->m_iNumItems - 1] > 1000)
		{
			m_bPingingServers = 0;
			ClearUnresponsiveServers();
			m_pRefreshButton->SetText(BB_STRING_REFRESH);
		}
	}

	if(m_bPingingServers)
		HandlePingPackets();

	if(m_bUpdatingGame)
		ContinueUpdate();

	return 1;
}

bool xMenu::Update(double dTime)
{
	float tempfloat;
	char *params[1];
	int songindex, i, j;

	m_xGUI.Update(dTime);

	if(m_dStartGameCountdown > 0.0)
	{
		m_dStartGameCountdown -= dTime;

		if(m_dStartGameCountdown <= 0.0)
		{
			g_pCurGameState = &g_xGame;
			g_xGame.InitVersusScreen(m_strStoredLevel);
			m_dStartGameCountdown = 0.0;
			m_xGUI.RemoveAllWindows();
		}
	}

	if(m_bSlideTransitioning)
	{
		m_dSlideTransitionTime += dTime;

		if(m_dSlideTransitionTime < BB_TIME_MENUSLIDETRANSITION)
		{
			tempfloat = (m_dSlideTransitionTime / BB_TIME_MENUSLIDETRANSITION) * 1024.0f;

			if(m_bSlideTransitionRight)
			{
				m_pLeftWindow->SetX(-tempfloat);
				m_pRightWindow->SetX(1024.0f - tempfloat);
			}
			else
			{
				m_pLeftWindow->SetX(tempfloat - 1024.0f);
				m_pRightWindow->SetX(tempfloat);
			}

			m_pLeftWindow->UpdateRenderables();
			m_pRightWindow->UpdateRenderables();
		}
		else
		{
			m_bSlideTransitioning = false;

			if(m_bSlideTransitionRight)
			{
				m_pRightWindow->SetX(0.0);
				m_pRightWindow->UpdateRenderables();

				m_xGUI.RemoveWindow(m_pLeftWindow);
			}
			else
			{
				m_pLeftWindow->SetX(0.0);
				m_pLeftWindow->UpdateRenderables();

				m_xGUI.RemoveWindow(m_pRightWindow);
			}
		}
	}

	if(g_bCreditsRolling)
		UpdateCredits(dTime);

	if(m_fKillMenuIntroTime > 0.0)
	{
		m_fKillMenuIntroTime -= dTime;

		if(m_fKillMenuIntroTime < 0.0)
		{
			params[0] = BB_WINDOW_MENU_PROFILE;
			xMenuRunInputCommand("CheckedAddProfileWindow", params, m_xGUI.GetWindow(BB_WINDOW_MENU_INTRO), NULL);
		}
	}

	if(m_iFreePlayScrollAmount || m_fFreePlayScrollTime < BB_FREEPLAY_SCROLLTIME)
	{
		if((m_bChallenges && !m_xGUI.GetWindow(BB_WINDOW_MENU_CHALLENGES)) ||
		   (!m_bChallenges && !m_xGUI.GetWindow(BB_WINDOW_MENU_FREEPLAY)))
		{
			m_iFreePlayScrollAmount = 0;
			m_fFreePlayScrollTime = 0.0;
			m_fFreePlayCurrentScrollTime = 0.0;
		}

		if(m_iFreePlayScrollAmount)
		{
			m_fFreePlayScrollTime += dTime;

			m_fFreePlayCurrentScrollTime += dTime;

			// NOTE: Challenges use a lot of Free Play variables to reuse variables/memory
			if(m_bChallenges)
			{
				while(m_fFreePlayCurrentScrollTime >= fabs(BB_FREEPLAY_SCROLLTIME / (float)m_iFreePlayScrollAmount))
				{
					m_iChallengesTop -= m_iFreePlayScrollDirection;

					if(m_iChallengesTop < 0)
						m_iChallengesTop = m_iChallengeCount - 1;
					else if(m_iChallengesTop >= m_iChallengeCount)
						m_iChallengesTop -= m_iChallengeCount;

					songindex = m_iChallengesTop;

					for(i = 0; i <= BB_FREEPLAY_SONGCOUNT; i++)
					{
						if(i == BB_FREEPLAY_SONGCOUNT)
						{
							if(m_iFreePlayScrollDirection > 0)
							{
								xMenuSetChallengeScrollValues(BB_FREEPLAY_SONGCOUNT, 0, 1);
								songindex = m_iChallengesTop > 0 ? m_iChallengesTop - 1 : m_iChallengeCount - 1;
							}
							else
							{
								xMenuSetChallengeScrollValues(BB_FREEPLAY_SONGCOUNT, BB_FREEPLAY_SONGCOUNT + 1, BB_FREEPLAY_SONGCOUNT);
								songindex = (m_iChallengesTop + BB_FREEPLAY_SONGCOUNT) % m_iChallengeCount;
							}
						}
						else
							xMenuSetChallengeScrollValues(i, i + 1, i + 1 + m_iFreePlayScrollDirection);

						m_pFreePlaySongNames[i]->SetText(g_strChallengeTitle[songindex]);
						m_pChallengeCompleteImages[i]->SetHidden(!g_xCurrentProfile.m_byteAchievementStatus[songindex]);

						for(j = 0; j < 12; j++)
							m_pFreePlaySongDiffStars[i][j]->SetHidden(j >= g_iChallengeDifficulties[songindex]);

						songindex++;

						if(songindex == m_iChallengeCount)
							songindex = 0;
					}

					m_pFreePlaySongArtist[0]->SetText(m_strChallengeSongNames[(m_iChallengesTop + 4) % m_iChallengeCount]);
					m_pFreePlaySongArtist[1]->SetText(m_strChallengeSongNames[(m_iChallengesTop + 4) % m_iChallengeCount]);

					m_fFreePlayCurrentScrollTime -= fabs(BB_FREEPLAY_SCROLLTIME / (float)m_iFreePlayScrollAmount);

					m_iFreePlayScrollAmount -= m_iFreePlayScrollDirection;
				}

				if(m_iFreePlayScrollAmount == 0)
				{
					tempfloat = 0;
					m_pFreePlaySongBackgrounds[BB_FREEPLAY_SONGCOUNT]->SetHidden(1);
					m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT]->SetHidden(1);
					m_pChallengeCompleteImages[BB_FREEPLAY_SONGCOUNT]->SetHidden(1);

					for(j = 0; j < 12; j++)
						m_pFreePlaySongDiffStars[BB_FREEPLAY_SONGCOUNT][j]->SetHidden(1);

					j = (m_iChallengesTop + 4) % m_iChallengeCount;

					m_pChallengeAchievementIcon->SetTextureFrame(j);

					for(i = 0; i < 4; i++)
						m_pChallengeTextLines[i]->SetText(g_strChallengeDescription[j][i]);

					m_pFreePlayEnemyModel->SetModel(m_strChallengeEnemies[j]);
				}
				else
					tempfloat = m_fFreePlayCurrentScrollTime / fabs(BB_FREEPLAY_SCROLLTIME / (float)m_iFreePlayScrollAmount);

				//DEBUGLOG("m_iFreePlayScrollAmount=%i m_fFreePlayScrollTime=%f m_fFreePlayCurrentScrollTime=%f tempfloat=%f\n", m_iFreePlayScrollAmount, m_fFreePlayScrollTime, m_fFreePlayCurrentScrollTime, tempfloat);

				for(i = 0; i < BB_FREEPLAY_SONGCOUNT + 1; i++)
				{
					xMenuUpdateFreePlayRenderable(m_pFreePlaySongBackgrounds[i], &m_xFreePlaySongBGStart[i], &m_xFreePlaySongBGTarget[i], tempfloat);

					m_pFreePlaySongNames[i]->m_xNormalColor.R = m_xFreePlaySongNameStart[i].m_xColor.R + (m_xFreePlaySongNameTarget[i].m_xColor.R - m_xFreePlaySongNameStart[i].m_xColor.R) * tempfloat;
					m_pFreePlaySongNames[i]->m_xNormalColor.G = m_xFreePlaySongNameStart[i].m_xColor.G + (m_xFreePlaySongNameTarget[i].m_xColor.G - m_xFreePlaySongNameStart[i].m_xColor.G) * tempfloat;
					m_pFreePlaySongNames[i]->m_xNormalColor.B = m_xFreePlaySongNameStart[i].m_xColor.B + (m_xFreePlaySongNameTarget[i].m_xColor.B - m_xFreePlaySongNameStart[i].m_xColor.B) * tempfloat;
					xRenderable2DTextUpdateColor(&m_pFreePlaySongNames[i]->m_xText, &m_pFreePlaySongNames[i]->m_xNormalColor);
					xMenuUpdateFreePlayRenderable(m_pFreePlaySongNames[i], &m_xFreePlaySongNameStart[i], &m_xFreePlaySongNameTarget[i], tempfloat);

					for(j = 0; j < 12; j++)
					{
						m_pFreePlaySongDiffStars[i][j]->SetY(m_fFreePlaySongDiffStartY[i] + (m_fFreePlaySongDiffTargetY[i] - m_fFreePlaySongDiffStartY[i]) * tempfloat);
						m_pFreePlaySongDiffStars[i][j]->SetAlpha(m_pFreePlaySongNames[i]->GetAlpha());
						m_pFreePlaySongDiffStars[i][j]->UpdateRenderables();
					}

					xMenuUpdateFreePlayRenderable(m_pChallengeCompleteImages[i], &m_xChallengeCompleteStart[i], &m_xChallengeCompleteTarget[i], tempfloat);
				}

				if(m_iFreePlayScrollAmount == 0)
					tempfloat = 1.0;

				m_pFreePlaySongArtist[m_iCurrentArtistIndex]->SetY(m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex] + (m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex] - m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex]) * tempfloat);
				m_pFreePlaySongArtist[m_iCurrentArtistIndex]->SetAlpha(1.0 - tempfloat);
				m_pFreePlaySongArtist[m_iCurrentArtistIndex]->m_xNormalColor.A = 1.0 - tempfloat;
				m_pFreePlaySongArtist[m_iCurrentArtistIndex]->UpdateRenderables();

				m_pFreePlaySongArtist[1 ^ m_iCurrentArtistIndex]->SetY(m_fFreePlaySongArtistStartY[1 ^ m_iCurrentArtistIndex] + (m_fFreePlaySongArtistTargetY[1 ^ m_iCurrentArtistIndex] - m_fFreePlaySongArtistStartY[1 ^ m_iCurrentArtistIndex]) * tempfloat);
				m_pFreePlaySongArtist[1 ^ m_iCurrentArtistIndex]->SetAlpha(tempfloat);
				m_pFreePlaySongArtist[1 ^ m_iCurrentArtistIndex]->m_xNormalColor.A = tempfloat;
				m_pFreePlaySongArtist[1 ^ m_iCurrentArtistIndex]->UpdateRenderables();
			}
			else
			{
				while(m_fFreePlayCurrentScrollTime >= fabs(BB_FREEPLAY_SCROLLTIME / (float)m_iFreePlayScrollAmount))
				{
					m_iFreePlayTopSong -= m_iFreePlayScrollDirection;

					if(m_iFreePlayTopSong < 0)
						m_iFreePlayTopSong = m_iFreePlaySongCount - 1;
					else if(m_iFreePlayTopSong >= m_iFreePlaySongCount)
						m_iFreePlayTopSong -= m_iFreePlaySongCount;

					songindex = m_iFreePlayTopSong;

					for(i = 0; i <= BB_FREEPLAY_SONGCOUNT; i++)
					{
						if(i == BB_FREEPLAY_SONGCOUNT)
						{
							if(m_iFreePlayScrollDirection > 0)
							{
								xMenuSetFreePlaySongScrollValues(BB_FREEPLAY_SONGCOUNT, 0, 1);
								songindex = m_iFreePlayTopSong > 0 ? m_iFreePlayTopSong - 1 : m_iFreePlaySongCount - 1;
							}
							else
							{
								xMenuSetFreePlaySongScrollValues(BB_FREEPLAY_SONGCOUNT, BB_FREEPLAY_SONGCOUNT + 1, BB_FREEPLAY_SONGCOUNT);
								songindex = (m_iFreePlayTopSong + BB_FREEPLAY_SONGCOUNT) % m_iFreePlaySongCount;
							}
						}
						else
							xMenuSetFreePlaySongScrollValues(i, i + 1, i + 1 + m_iFreePlayScrollDirection);

						m_pFreePlaySongNames[i]->SetText(m_strFreePlaySongList[songindex]);

						for(j = 0; j < 12; j++)
							m_pFreePlaySongDiffStars[i][j]->SetHidden(j >= m_iFreePlayDifficulties[m_iFreePlayDifficulty][songindex]);

						songindex++;

						if(songindex == m_iFreePlaySongCount)
							songindex = 0;
					}

					m_pFreePlaySongArtist[0]->SetText(m_strFreePlayArtistList[(m_iFreePlayTopSong + 4) % m_iFreePlaySongCount]);
					m_pFreePlaySongArtist[1]->SetText(m_strFreePlayArtistList[(m_iFreePlayTopSong + 4) % m_iFreePlaySongCount]);

					m_fFreePlayCurrentScrollTime -= fabs(BB_FREEPLAY_SCROLLTIME / (float)m_iFreePlayScrollAmount);

					m_iFreePlayScrollAmount -= m_iFreePlayScrollDirection;
				}

				if(m_iFreePlayScrollAmount == 0)
				{
					tempfloat = 0;
					m_pFreePlaySongBackgrounds[BB_FREEPLAY_SONGCOUNT]->SetHidden(1);
					m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT]->SetHidden(1);

					for(j = 0; j < 12; j++)
						m_pFreePlaySongDiffStars[BB_FREEPLAY_SONGCOUNT][j]->SetHidden(1);

					if(m_strFreePlayDefaultEnemies[(m_iFreePlayTopSong + 4) % m_iFreePlaySongCount])
						m_pFreePlayEnemyModel->SetModel(m_strFreePlayDefaultEnemies[(m_iFreePlayTopSong + 4) % m_iFreePlaySongCount]);
				}
				else
					tempfloat = m_fFreePlayCurrentScrollTime / fabs(BB_FREEPLAY_SCROLLTIME / (float)m_iFreePlayScrollAmount);

				//DEBUGLOG("m_iFreePlayScrollAmount=%i m_fFreePlayScrollTime=%f m_fFreePlayCurrentScrollTime=%f tempfloat=%f\n", m_iFreePlayScrollAmount, m_fFreePlayScrollTime, m_fFreePlayCurrentScrollTime, tempfloat);

				for(i = 0; i < BB_FREEPLAY_SONGCOUNT + 1; i++)
				{
					xMenuUpdateFreePlayRenderable(m_pFreePlaySongBackgrounds[i], &m_xFreePlaySongBGStart[i], &m_xFreePlaySongBGTarget[i], tempfloat);

					m_pFreePlaySongNames[i]->m_xNormalColor.R = m_xFreePlaySongNameStart[i].m_xColor.R + (m_xFreePlaySongNameTarget[i].m_xColor.R - m_xFreePlaySongNameStart[i].m_xColor.R) * tempfloat;
					m_pFreePlaySongNames[i]->m_xNormalColor.G = m_xFreePlaySongNameStart[i].m_xColor.G + (m_xFreePlaySongNameTarget[i].m_xColor.G - m_xFreePlaySongNameStart[i].m_xColor.G) * tempfloat;
					m_pFreePlaySongNames[i]->m_xNormalColor.B = m_xFreePlaySongNameStart[i].m_xColor.B + (m_xFreePlaySongNameTarget[i].m_xColor.B - m_xFreePlaySongNameStart[i].m_xColor.B) * tempfloat;
					xRenderable2DTextUpdateColor(&m_pFreePlaySongNames[i]->m_xText, &m_pFreePlaySongNames[i]->m_xNormalColor);
					xMenuUpdateFreePlayRenderable(m_pFreePlaySongNames[i], &m_xFreePlaySongNameStart[i], &m_xFreePlaySongNameTarget[i], tempfloat);

					for(j = 0; j < 12; j++)
					{
						m_pFreePlaySongDiffStars[i][j]->SetY(m_fFreePlaySongDiffStartY[i] + (m_fFreePlaySongDiffTargetY[i] - m_fFreePlaySongDiffStartY[i]) * tempfloat);
						m_pFreePlaySongDiffStars[i][j]->SetAlpha(m_pFreePlaySongNames[i]->GetAlpha());
						m_pFreePlaySongDiffStars[i][j]->UpdateRenderables();
					}
				}

				if(m_iFreePlayScrollAmount == 0)
					tempfloat = 1.0;

				m_pFreePlaySongArtist[m_iCurrentArtistIndex]->SetY(m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex] + (m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex] - m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex]) * tempfloat);
				m_pFreePlaySongArtist[m_iCurrentArtistIndex]->SetAlpha(1.0 - tempfloat);
				m_pFreePlaySongArtist[m_iCurrentArtistIndex]->m_xNormalColor.A = (1.0 - tempfloat);
				m_pFreePlaySongArtist[m_iCurrentArtistIndex]->UpdateRenderables();

				m_pFreePlaySongArtist[1 ^ m_iCurrentArtistIndex]->SetY(m_fFreePlaySongArtistStartY[1 ^ m_iCurrentArtistIndex] + (m_fFreePlaySongArtistTargetY[1 ^ m_iCurrentArtistIndex] - m_fFreePlaySongArtistStartY[1 ^ m_iCurrentArtistIndex]) * tempfloat);
				m_pFreePlaySongArtist[1 ^ m_iCurrentArtistIndex]->SetAlpha(tempfloat);
				m_pFreePlaySongArtist[1 ^ m_iCurrentArtistIndex]->m_xNormalColor.A = tempfloat;
				m_pFreePlaySongArtist[1 ^ m_iCurrentArtistIndex]->UpdateRenderables();
			}
		}
	}

	if(m_pAchievementIcon[0])
	{
		for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
		{
			if(m_iAchievementsUnlocking[i])
			{
				tempfloat = g_xPaperAPI.m_dwCurrentTime - m_dwAchievementUnlockStart[i];

				if(tempfloat >= BB_ACHIEVEMENT_TOTALTIME)
				{
					for(j = i; j < BB_PENDINGACHIEVEMENTS_MAX - 1; j++)
					{
						m_iAchievementsUnlocking[j] = m_iAchievementsUnlocking[j + 1];
						m_dwAchievementUnlockStart[j] = m_dwAchievementUnlockStart[j + 1];

						if(m_iAchievementsUnlocking[j] != BB_ACH_NONE)
							xRenderable2DQuadUpdateTexture(m_pAchievementIcon[j], m_pAchievementIconTextures[m_iAchievementsUnlocking[j] - 1]);
					}

					m_iAchievementsUnlocking[BB_PENDINGACHIEVEMENTS_MAX - 1] = 0;
					i--;
					continue;
				}
				else if(tempfloat < BB_ACHIEVEMENT_SHOWTIME)
					tempfloat = tempfloat / BB_ACHIEVEMENT_SHOWTIME;
				else if(tempfloat > BB_ACHIEVEMENT_HIDESTARTTIME)
					tempfloat = 1.0 - ((tempfloat - BB_ACHIEVEMENT_HIDESTARTTIME) / BB_ACHIEVEMENT_HIDETIME);
				else
					tempfloat = 1.0;

				xRenderable2DQuadUpdateVisibility(m_pAchievementIcon[i], 1);
				xRenderable2DQuadSetPosition(m_pAchievementIcon[i], BB_ACHIEVEMENT_ICONX, 768 - BB_ACHIEVEMENT_HEIGHT * i - BB_ACHIEVEMENT_HEIGHT * tempfloat);
				xRenderable2DQuadUpdate(m_pAchievementIcon[i]);

				xRenderable2DTextSetPosition(&m_xAchievementTitle[i], BB_ACHIEVEMENT_TITLEX - xFontGetWidth(m_xAchievementTitle[i].m_pFont, g_strAchievementTitle[m_iAchievementsUnlocking[i] - 1]) * BB_ACHIEVEMENT_TITLESCALE, m_pAchievementIcon[i]->m_xRenderable2D.m_fY + BB_ACHIEVEMENT_TITLEY);
				xRenderable2DTextSetText(&m_xAchievementTitle[i], &m_xSceneGraph, g_strAchievementTitle[m_iAchievementsUnlocking[i] - 1]);
			}
			else
			{
				xRenderable2DQuadUpdateVisibility(m_pAchievementIcon[i], 0);
				xRenderable2DTextSetText(&m_xAchievementTitle[i], &m_xSceneGraph, "");
			}
		}
	}

	return 1;
}

void xMenu::RefreshAllRenderables()
{
	m_xGUI.RefreshAllRenderables();
}

void xMenu::Shutdown()
{
	RemoveAllMenuWindows();
}

bool xMenu::AddMenuWindow(const char *strWindowName)
{
	if(m_xGUI.AddWindow(strWindowName))
	{
		char temp[256];
		sprintf(temp, "Unable to open Menu Window: %s", strWindowName);
		//StandardAlert(kAlertNoteAlert, (unsigned char*)temp, NULL, NULL, NULL);
		return false;
	}
	
	return true;
}

void xMenu::AddMainMenu()
{
	AddMenuWindow(BB_WINDOW_MENU_MAIN);
}

void xMenu::SlideTransition(bool bToRightWindow, xGUIWindow *pLeftWindow, xGUIWindow *pRightWindow)
{
	xGUIWindowList *window;

	if(m_bSlideTransitioning)
		return;

	m_bSlideTransitioning = true;
	m_dSlideTransitionTime = 0.0;
	m_bSlideTransitionRight = bToRightWindow;
	
	m_pLeftWindow = pLeftWindow;
	m_pRightWindow = pRightWindow;

	if(bToRightWindow && pRightWindow)
		pRightWindow->m_bHasFocus = true;
	else if(!bToRightWindow && pLeftWindow)
		pLeftWindow->m_bHasFocus = true;

	if(g_bCreditsRolling)
	{
		g_bCreditsRolling = false;

		window = g_xMenu.m_xGUI.GetWindow("Credits");

		if(window)
			g_xMenu.m_xGUI.RemoveWindow(window);
	}
}

void xMenu::RemoveAllMenuWindows()
{
	m_xGUI.RemoveAllWindows();
}

void xMenuRunPostLoadCommand(char *strCommand, char** strParams, xGUIWindowList *pWindow)
{
	xGUIText *textobject;
	xColor tempcolor;
	char temptext[64], temptext2[10];
	float tempfloat, tempfloat2;
	int tempint, tempint2, i, j;

	if(!stricmp(strCommand, "InitializeProfilesWindow"))
	{
		xGUIListBox *profileslistbox = (xGUIListBox*)pWindow->m_pData->GetChildByName(strParams[0]);

		if(profileslistbox->m_iNumItems == 0)
		{
			xList<xPlayerProfile>* curprofile = g_xOptions.GetAllAvailableProfiles();
			while(curprofile)
			{
				profileslistbox->AddItem(curprofile->m_pData->m_strProfileName);

				if(!curprofile->m_pNext)
				{
					delete curprofile->m_pData;
					delete curprofile;
					curprofile = NULL;
				}
				else
				{
					curprofile = curprofile->m_pNext;
					delete curprofile->m_pPrev->m_pData;
					delete curprofile->m_pPrev;
					curprofile->m_pPrev = NULL;
				}
			}
		}
	}
	else if(!stricmp(strCommand, "InitializeMainMenu"))
	{
		//((xGUIText*)pWindow->m_pData->GetChildByName("ProfileName"))->SetText(g_xCurrentProfile.m_strProfileName);
		
#ifdef DEMO_RELEASE
		pWindow->m_pData->GetChildByName("SinglePlayer")->SetHidden(1);
		pWindow->m_pData->GetChildByName("Challenge")->SetHidden(1);
		pWindow->m_pData->GetChildByName("Freeplay")->SetHidden(1);
		pWindow->m_pData->GetChildByName("ChallengePhone")->SetHidden(1);
		pWindow->m_pData->GetChildByName("FreeplayPhone")->SetHidden(1);
#else
		if(g_xPaperAPI.m_xPlatform.m_bIsPhone)
		{
			pWindow->m_pData->GetChildByName("SinglePlayer")->SetHidden(1);
			pWindow->m_pData->GetChildByName("Challenge")->SetHidden(1);
			pWindow->m_pData->GetChildByName("Freeplay")->SetHidden(1);
		}
		else
		{
			pWindow->m_pData->GetChildByName("SinglePlayerPhone")->SetHidden(1);
			pWindow->m_pData->GetChildByName("ChallengePhone")->SetHidden(1);
			pWindow->m_pData->GetChildByName("FreeplayPhone")->SetHidden(1);
		}
#endif		
		pWindow->m_pData->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "InitializeDifficultyWindow"))
	{
		g_xMenu.m_pDifficultyArrow = (xGUIImage*)pWindow->m_pData->GetChildByName(strParams[0]);

		if(g_xCurrentProfile.m_byteSelectedDifficulty < BB_NUMDIFFICULTIES)
		{
			g_xMenu.m_iSelectedDifficulty = g_xCurrentProfile.m_byteSelectedDifficulty;
			g_xMenu.m_pDifficultyArrow->SetY(atof(strParams[g_xCurrentProfile.m_byteSelectedDifficulty + 1]));
			g_xMenu.m_pDifficultyArrow->UpdateRenderables();
		}
		else
			g_xMenu.m_iSelectedDifficulty = 2;
	}
	else if(!stricmp(strCommand, "InitializeChangeDifficultyWindow"))
	{
		g_xMenu.m_pDifficultyArrow = (xGUIImage*)pWindow->m_pData->GetChildByName(strParams[0]);

		if(g_xCurrentProfile.m_byteSelectedDifficulty < BB_NUMDIFFICULTIES)
		{
			g_xMenu.m_iSelectedDifficulty = g_xCurrentProfile.m_byteSelectedDifficulty;
			g_xMenu.m_pDifficultyArrow->SetY(atof(strParams[g_xCurrentProfile.m_byteSelectedDifficulty + 1]));
			g_xMenu.m_pDifficultyArrow->UpdateRenderables();
		}
		else
			g_xMenu.m_iSelectedDifficulty = 2;

		tempint = strlen(strParams[6]) - 1;
		tempint2 = strlen(strParams[7]) - 1;

		for(i = 0; i < BB_NUMDIFFICULTIES; i++)
		{
			tempfloat = 0.0;
			tempfloat2 = 0.0;
			for(j = 1; j <= BB_LEVELS_MAX; j++)
			{
				if(g_xCurrentProfile.m_u32HighScores[i][j] > 0)
					tempfloat += 1.0;

				if(g_xCurrentProfile.m_u32HighScores[i][j] >= 100)
					tempfloat2 += 1.0;
			}

			sprintf(temptext2, "%i%%", (int)(tempfloat * 100.0 / 44.0));
			strParams[6][tempint] = '1' + i;
			textobject = (xGUIText*)pWindow->m_pData->GetChildByName(strParams[6]);
			textobject->SetText(temptext2);

			if(tempfloat >= 22.0)
			{
				tempfloat = (tempfloat - 22.0f) / 22.0f;
				textobject->UpdateColor(xColorInit(&tempcolor, 1.0f + (0.004f - 1.0f) * tempfloat, 0.635f + (1.0f - 0.635f) * tempfloat, 0.192f * tempfloat, 0.9f));
			}
			else
			{
				tempfloat = tempfloat / 22.0f;
				textobject->UpdateColor(xColorInit(&tempcolor, 1.0f, 0.988f + (0.635f - 0.988f) * tempfloat, 0.055f - 0.055f * tempfloat, 0.9f));
			}

			sprintf(temptext2, "%i%%", (int)(tempfloat2 * 100.0 / 44.0));
			strParams[7][tempint2] = '1' + i;
			textobject = (xGUIText*)pWindow->m_pData->GetChildByName(strParams[7]);
			textobject->SetText(temptext2);

			if(tempfloat2 >= 22.0)
			{
				tempfloat = (tempfloat2 - 22.0f) / 22.0f;
				textobject->UpdateColor(xColorInit(&tempcolor, 1.0f + (0.004f - 1.0f) * tempfloat, 0.635f + (1.0f - 0.635f) * tempfloat, 0.192f * tempfloat, 0.9f));
			}
			else
			{
				tempfloat = tempfloat2 / 22.0f;
				textobject->UpdateColor(xColorInit(&tempcolor, 1.0f, 0.988f + (0.635f - 0.988f) * tempfloat, 0.055f - 0.055f * tempfloat, 0.9f));
			}
		}
	}
	else if(!stricmp(strCommand, "InitSPLeaderboard"))
	{
		g_xMenu.m_iSPLeaderboardDifficulty = g_xCurrentProfile.m_byteSelectedDifficulty;

		g_xMenu.m_xHTTPQuery.GetSPLeaderboard(g_xCurrentProfile.m_strLoginName, g_xMenu.m_iSPLeaderboardDifficulty, g_xCurrentProfile.m_strLoginName[0] ? 0 : 1);

		g_xMenu.m_iSPLeaderboardStartIndex = -1;
	}
	else if(!stricmp(strCommand, "InitializeUpdateWindow"))
		g_xMenu.m_xHTTPQuery.CheckForUpdate();
	else if(!stricmp(strCommand, "InitializeBattlesWindow"))
		g_xMenu.InitializeBattlesWindow(pWindow);
	else if(!stricmp(strCommand, "InitializeChallengesWindow"))
		g_xMenu.InitializeChallengesWindow(pWindow);
	else if(!stricmp(strCommand, "InitializeFreePlayWindow"))
		g_xMenu.InitializeFreePlayWindow(pWindow);
	else if(!stricmp(strCommand, "InitializeOptionsWindow"))
		g_xMenu.InitializeOptionsWindow(pWindow);
	else if(!stricmp(strCommand, "InitLoginMenu"))
	{
		if(g_xCurrentProfile.m_strLoginName[0])
		{
			((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[0]))->SetText(g_xCurrentProfile.m_strLoginName);
			((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[1]))->SetText(BB_STRING_USESTOREDPASSWORD);
			pWindow->m_pData->UpdateRenderables();
		}
	}
	else if(!stricmp(strCommand, "InitRegisterMenu"))
	{
		if(g_xGame.m_xPlayer.m_strCDKey[0] != '+' && g_xGame.m_xPlayer.m_strCDKey[0] != '=')
		{
			((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[0]))->SetText(g_xGame.m_xPlayer.m_strCDKey);
			pWindow->m_pData->UpdateRenderables();
		}
	}
	else if(!stricmp(strCommand, "GetServerList"))
	{
		g_xMenu.m_pServerListBox = (xGUIColumnListBox*)pWindow->m_pData->GetChildByName(strParams[0]);
		g_xMenu.m_pRefreshButton = (xGUIButton*)pWindow->m_pData->GetChildByName(strParams[1]);
		g_xMenu.RefreshServerList();
	}
	else if(!stricmp(strCommand, "InitializeHostServerWindow"))
	{
		g_xMenu.m_pSongListBox = (xGUIListBox*)pWindow->m_pData->GetChildByName(strParams[0]);
		g_xMenu.InitializeHostServerWindow(pWindow);
	}
	else if(!stricmp(strCommand, "RollCredits"))
	{
		g_bCreditsRolling = true;

		if(g_pCreditTextArray)
			delete [] g_pCreditTextArray;

		g_iMaxCreditLines = atoi(strParams[0]);

		g_pCreditTextArray = new xGUIText*[g_iMaxCreditLines];

		for(i = 0; i < g_iMaxCreditLines; i++)
		{
			sprintf(temptext, BB_NAME_CREDITSTEXT, i + 1);
			g_pCreditTextArray[i] = (xGUIText*)pWindow->m_pData->GetChildByName(temptext);
			g_pCreditTextArray[i]->SetText(g_strCredits[i]);
		}

		g_iCurTextLine = 0;
		g_iCurCreditLine = g_iMaxCreditLines;

		pWindow->m_pData->GetChildByName(BB_NAME_CREDITSBG)->SetHidden(g_pCurGameState == &g_xMenu);
		pWindow->m_pData->GetChildByName(BB_NAME_CREDITSYOUWIN)->SetHidden(g_pCurGameState == &g_xMenu);
	}
}

void xMenuRunInputCommand(char *strCommand, char** strParams, xGUIWindowList *pWindow, xGUIObject *pObject)
{
	xFileIO fileio;
	char temptext[256], temptext2[16], *tempstring;
	xGUIObject *tempobject;
	xGUIImage *imageobject;
	xGUITextBox *textboxobject;
	xGUIListBox *listboxobject;
	xGUIWindowList *window;
	bool tempbool;
	int i, j;

	if(!stricmp(strCommand, "OpenMenu"))
		g_xMenu.AddMenuWindow(strParams[0]);
	else if(!stricmp(strCommand, "Close"))
		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);
	else if(!stricmp(strCommand, "OpenMenuAndClose"))
	{
		g_xMenu.AddMenuWindow(strParams[0]);
		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);
	}
	else if(!stricmp(strCommand, "SlideTransitionRight"))
	{
		if(g_xMenu.AddMenuWindow(strParams[0]))
			g_xMenu.SlideTransition(true, pWindow->m_pData, g_xMenu.m_xGUI.GetWindow(strParams[0])->m_pData);
	}
	else if(!stricmp(strCommand, "SlideTransitionLeft"))
	{
		if(g_xMenu.AddMenuWindow(strParams[0]))
			g_xMenu.SlideTransition(false, g_xMenu.m_xGUI.GetWindow(strParams[0])->m_pData, pWindow->m_pData);
	}
	else if(!stricmp(strCommand, "CheckedAddProfileWindow"))
	{
		g_xOptions.LoadPlayerProfile("B-Boi", &g_xCurrentProfile);
		g_xMenu.m_fKillMenuIntroTime = 0.0;
		g_xMenu.AddMainMenu();
		g_xMenu.SlideTransition(true, pWindow->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData);
	}
	else if(!stricmp(strCommand, "Exit"))
	{
		g_xPaperAPI.m_bRunning = 0;
		//QuitApplicationEventLoop();
	}
	else if(!stricmp(strCommand, g_strSingleplayer)) // "Singleplayer"
	{
		if(xMenuSinglePlayerLogin(BB_SPLOGIN_SINGLEPLAYER, strParams[0], strParams[1], strParams[2]))
		{
			if(g_xMenu.AddMenuWindow(strParams[2]))
				g_xMenu.SlideTransition(true, pWindow->m_pData, g_xMenu.m_xGUI.GetWindow(strParams[2])->m_pData);
		}
	}
	else if(!stricmp(strCommand, g_strChallenges)) // "Challenges"
	{
		if(xMenuSinglePlayerLogin(BB_SPLOGIN_CHALLENGES, strParams[0], strParams[1], strParams[2]))
		{
			if(g_xMenu.AddMenuWindow(strParams[2]))
				g_xMenu.SlideTransition(true, pWindow->m_pData, g_xMenu.m_xGUI.GetWindow(strParams[2])->m_pData);
		}
	}
	else if(!stricmp(strCommand, g_strFreePlay)) // "FreePlay"
	{
		if(xMenuSinglePlayerLogin(BB_SPLOGIN_FREEPLAY, strParams[0], strParams[1], strParams[2]))
		{
			if(g_xMenu.AddMenuWindow(strParams[2]))
				g_xMenu.SlideTransition(true, pWindow->m_pData, g_xMenu.m_xGUI.GetWindow(strParams[2])->m_pData);
		}
	}
	else if(!stricmp(strCommand, "ChangeLogin"))
	{
		g_xMenu.m_xSinglePlayerLogin = BB_SPLOGIN_CHANGELOGIN;
		g_xMenu.AddMenuWindow(strParams[0]);
	}
	else if(!stricmp(strCommand, "ClearLogin"))
	{
		memset(g_xCurrentProfile.m_strLoginName, 0, BB_MAXLOGINNAMELENGTH);
		memset(g_xCurrentProfile.m_strLoginPassword, 0, BB_MAXLOGINPASSLENGTH);

		g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);
	}
	else if(!stricmp(strCommand, "SaveLogin"))
	{
		strcpy(g_xGame.m_xPlayer.m_strAccountName, ((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[0]))->GetText());
		tempstring = ((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[1]))->GetText();

		if(!stricmp(tempstring, BB_STRING_USESTOREDPASSWORD))
			tempstring = g_xCurrentProfile.m_strLoginPassword;

		strcpy(g_xCurrentProfile.m_strLoginName, g_xGame.m_xPlayer.m_strAccountName);

		if(tempstring != g_xCurrentProfile.m_strLoginPassword)
			g_xMenu.m_xHTTPQuery.GenerateMD5(tempstring, g_xCurrentProfile.m_strLoginPassword);

		g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);
	}
	else if(!stricmp(strCommand, g_strContinueSingleplayer)) // "ContinueSingleplayer"
	{
		g_xOptions.SavePlayerProfile(&g_xCurrentProfile);
		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);

		if(g_xGame.m_xPlayer.m_strCDKey[0] == '+')
			g_xMenu.AddMenuWindow(strParams[0]);
		else if(g_xMenu.AddMenuWindow(g_xMenu.m_strSPLoginTarget))
			g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(g_xMenu.m_strSPLoginTarget)->m_pData);
	}
	else if(!stricmp(strCommand, "CancelRegistration"))
	{
		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);

		if(g_xMenu.m_xSinglePlayerLogin == BB_SPLOGIN_CHANGELOGIN)
			g_xMenu.AddMenuWindow(strParams[2]);
		else if(g_xMenu.m_xSinglePlayerLogin)
			g_xMenu.AddMenuWindow(strParams[0]);
		else
			g_xMenu.AddMenuWindow(strParams[1]);
	}
	else if(!stricmp(strCommand, "Multiplayer"))
	{
		g_xMenu.LoadAndValidateCDKey();
		g_xMenu.m_xSinglePlayerLogin = BB_SPLOGIN_MULTIPLAYER;
		g_xMenu.AddMenuWindow(strParams[0]);
	}
	else if(!stricmp(strCommand, "SelectDifficulty"))
	{
		g_xMenu.m_iSelectedDifficulty = strParams[0][0] - '1';
		g_xMenu.m_pDifficultyArrow->SetY(atof(strParams[1]));
		g_xMenu.m_pDifficultyArrow->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "DifficultyContinue"))
	{
		g_xCurrentProfile.m_byteSelectedDifficulty = g_xMenu.m_iSelectedDifficulty;
		g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

#ifdef DEMO_RELEASE
		g_xCurrentProfile.m_iCurrentLevel = 5;
		for(i = 1; i < 5; i++)
#else
		for(i = 1; i < BB_MAXLEVELS; i++)
#endif
		{
			if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][i] == 0)
			{
				g_xCurrentProfile.m_iCurrentLevel = i;
				break;
			}
		}

		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);

		if(g_xMenu.AddMenuWindow(strParams[0]))
			g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(strParams[0])->m_pData);
	}
	else if(!stricmp(strCommand, "ChangeDifficulty"))
	{
		g_xCurrentProfile.m_byteSelectedDifficulty = g_xMenu.m_iSelectedDifficulty;
		g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

#ifdef DEMO_RELEASE
		g_xCurrentProfile.m_iCurrentLevel = 5;
		for(i = 1; i < 5; i++)
#else
		for(i = 1; i < BB_MAXLEVELS; i++)
#endif
		{
			if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][i] == 0)
			{
				g_xCurrentProfile.m_iCurrentLevel = i;
				break;
			}
		}

		g_xMenu.InitializeBattlesWindow(g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_BATTLES));

		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);
	}
	else if(!stricmp(strCommand, "BattleMenuSelectTour"))
		g_xMenu.SelectTour(pWindow, strParams[0], g_strTourBackgrounds[strParams[0][0] - '1']);
	else if(!stricmp(strCommand, "BattleMenuSelectBattle"))
	{
		if(g_xMenu.m_iTourIndex == 0)
			sprintf(g_xMenu.m_strStoredLevel, "Tutorial%s", strParams[0]);
		else
			sprintf(g_xMenu.m_strStoredLevel, "Tour%iBattle%s", g_xMenu.m_iTourIndex, strParams[0]);

		g_xMenu.StartGame(g_xMenu.m_strStoredLevel);
	}
	else if(!stricmp(strCommand, "PlayLevel"))
		g_xMenu.StartGame(strParams[0]);
	else if(!stricmp(strCommand, "Ready"))
		g_xMenu.StartGame(g_xMenu.m_strStoredLevel);
	else if(!stricmp(strCommand, "ScrollChallenges"))
		g_xMenu.ChallengesScroll(strParams[0][0] == '0' ? 1 : -1);
	else if(!stricmp(strCommand, "ScrollToChallenge"))
		g_xMenu.ChallengesScroll((BB_FREEPLAY_SONGCOUNT - 1 - (strParams[0][0] - '0')) - 4);
	else if(!stricmp(strCommand, "StartChallenge"))
	{
		i = (g_xMenu.m_iChallengesTop + 4) % g_xMenu.m_iChallengeCount;
		if(g_xCurrentProfile.m_byteAchievementStatus[i] == 0)
			g_xMenu.ChallengeStart(i);
	}
	else if(!stricmp(strCommand, "ScrollFreePlay"))
		g_xMenu.FreePlayScrollSongs(strParams[0][0] == '0' ? 1 : -1);
	else if(!stricmp(strCommand, "ScrollToSong"))
		g_xMenu.FreePlayScrollSongs((BB_FREEPLAY_SONGCOUNT - 1 - (strParams[0][0] - '0')) - 4);
	else if(!stricmp(strCommand, "CycleFreePlayDifficulty"))
	{
		g_xMenu.m_iFreePlayDifficulty++;

		if(g_xMenu.m_iFreePlayDifficulty == 3)
			g_xMenu.m_iFreePlayDifficulty = 0;

		for(i = 0; i < 3; i++)
		{
			sprintf(temptext, BB_NAME_FREEPLAYDIFFICULTY, i + 1);
			pWindow->m_pData->GetChildByName(temptext)->SetHidden(i != g_xMenu.m_iFreePlayDifficulty);

			sprintf(temptext, BB_NAME_FREEPLAYDIFFICULTYOVER, i + 1);
			pWindow->m_pData->GetChildByName(temptext)->SetHidden(i != g_xMenu.m_iFreePlayDifficulty);
		}

		for(i = 0; i < BB_FREEPLAY_SONGCOUNT; i++)
		{
			for(j = 0; j < 12; j++)
				g_xMenu.m_pFreePlaySongDiffStars[i][j]->SetHidden(j >= g_xMenu.m_iFreePlayDifficulties[g_xMenu.m_iFreePlayDifficulty][(g_xMenu.m_iFreePlayTopSong + i) % g_xMenu.m_iFreePlaySongCount]);
		}

		pWindow->m_pData->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "SetFreePlayExtraBlocks"))
	{
		if(((xGUICheckBox*)pObject)->m_bChecked)
		{
			g_xMenu.m_iFreePlayExtraBlockCount = strParams[0][0] - '0';

			for(i = 0; i < 3; i++)
			{
				sprintf(temptext, BB_NAME_FREEPLAYEXTRABLOCKS, i + 1);
				((xGUICheckBox*)pWindow->m_pData->GetChildByName(temptext))->m_bChecked = i >= strParams[0][0] - '0';
			}
		}
		else
		{
			g_xMenu.m_iFreePlayExtraBlockCount = 1 + (strParams[0][0] - '0');

			for(i = 0; i < 3; i++)
			{
				sprintf(temptext, BB_NAME_FREEPLAYEXTRABLOCKS, i + 1);
				((xGUICheckBox*)pWindow->m_pData->GetChildByName(temptext))->m_bChecked = i > strParams[0][0] - '0';
			}
		}

		pWindow->m_pData->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "ChangeEnemy"))
	{
		g_xMenu.m_iFreePlayEnemy += strParams[0][0] == '0' ? -1 : 1;
		if(g_xMenu.m_iFreePlayEnemy < 0)
			g_xMenu.m_iFreePlayEnemy = g_xMenu.m_iFreePlayEnemyCount - 1;
		else if(g_xMenu.m_iFreePlayEnemy >= g_xMenu.m_iFreePlayEnemyCount)
			g_xMenu.m_iFreePlayEnemy -= g_xMenu.m_iFreePlayEnemyCount;

		g_xMenu.m_pFreePlayEnemyModel->SetModel(g_xMenu.m_strFreePlayEnemies[g_xMenu.m_iFreePlayEnemy]);
	}
	else if(!stricmp(strCommand, "StartFreePlay"))
	{
		g_xGame.SetupGameOptionsForMultiplayer(g_xMenu.m_strFreePlayEnemies[g_xMenu.m_iFreePlayEnemy], g_strTourBackgrounds[rand() % 7], g_xMenu.m_strFreePlaySongDirs[(g_xMenu.m_iFreePlayTopSong + 4) % g_xMenu.m_iFreePlaySongCount], (xGameDifficulty)(g_xMenu.m_iFreePlayDifficulty + 1));
		g_xGameOptions.m_iNumColors = 3 + g_xMenu.m_iFreePlayExtraBlockCount;
		g_xGameOptions.m_iNumRandomColors = g_xGameOptions.m_iNumColors;
		g_xGameOptions.m_bSkipPrePopup = 1;

		g_xGame.m_xGUI.RemoveAllWindows();
		g_pCurGameState = &g_xGame;
		g_xGame.Init("\0");

		g_xMenu.m_xGUI.DelayedRemoveAllWindows();
	}
	else if(!stricmp(strCommand, "CreateNewProfile"))
	{
		textboxobject = (xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[0]);

		memset(&g_xCurrentProfile, 0, BB_PLAYERPROFILESIZE);

		strcpy(g_xCurrentProfile.m_strProfileName, textboxobject->GetText());

		g_xCurrentProfile.m_u16Version = BB_PLAYERPROFILEVERSION;
		g_xCurrentProfile.m_iCurrentLevel = 1;
		g_xCurrentProfile.m_byteSelectedDifficulty = 2;

		//g_xCurrentProfile.m_iCurrentLevel = 44;
		//for(i = 0; i < BB_NUMDIFFICULTIES; i++)
		//	for(j = 0; j < BB_MAXLEVELS; j++)
		//		g_xCurrentProfile.m_u32HighScores[i][j] = 1;

		if(g_xCurrentProfile.m_strProfileName[0] != '\0')
		{
			//Go ahead and create the Profile file
			g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

			g_xMenu.AddMainMenu();

			g_xMenu.SlideTransition(true, pWindow->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData);

#if WITH_BALANCESTATS
			g_xBalanceStats.SetPlayerName(g_xCurrentProfile.m_strProfileName);
#endif
		}
	}
	else if(!stricmp(strCommand, "SelectProfile"))
	{
		listboxobject = (xGUIListBox*)pWindow->m_pData->GetChildByName(strParams[0]);

		char* profilename = listboxobject->GetSelectedString();
		if(profilename)
		{
			g_xOptions.LoadPlayerProfile(profilename, &g_xCurrentProfile);

			g_xMenu.AddMainMenu();

			g_xMenu.SlideTransition(true, pWindow->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData);

#if WITH_BALANCESTATS
			g_xBalanceStats.SetPlayerName(profilename);
#endif
		}
	}
	else if(!stricmp(strCommand, "ProfileMenuContinue"))
	{
		textboxobject = (xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[0]);

		if(textboxobject->GetText()[0] != '\0')
			xMenuRunInputCommand("CreateNewProfile", strParams, pWindow, textboxobject);
		else
			xMenuRunInputCommand("SelectProfile", &(strParams[1]), pWindow, pObject);
	}
	else if(!stricmp(strCommand, "UnhideAndRestartAnimation"))
	{
		tempobject = pWindow->m_pData->GetChildByName(strParams[0]);
		tempobject->SetHidden(false);
		tempobject->m_xAnimation.m_pCurKeyFrame = tempobject->m_xAnimation.m_xKeyFrameList.m_pNext;
		xAnimationLoadedAllKeyFrames(&tempobject->m_xAnimation);
		tempobject->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "MoreSPLeaderboardEntries"))
	{
		if(g_xMenu.m_iSPLeaderboardStartIndex < 0)
			g_xMenu.m_iSPLeaderboardStartIndex = g_xMenu.m_xHTTPQuery.GetSPLeaderboardStartIndex();

		if(g_xMenu.m_iSPLeaderboardStartIndex > 0)
		{
			if(strParams[1][0] == '0')
			{
				g_xMenu.m_iSPLeaderboardStartIndex -= 10;

				if(g_xMenu.m_iSPLeaderboardStartIndex < 1)
					g_xMenu.m_iSPLeaderboardStartIndex = 1;

				g_xMenu.m_xHTTPQuery.GetSPLeaderboard(g_xCurrentProfile.m_strLoginName, g_xMenu.m_iSPLeaderboardDifficulty, g_xMenu.m_iSPLeaderboardStartIndex);
			}
			else
			{
				sprintf(temptext, BB_NAME_RANK, 9);
				if(((xGUIText*)pWindow->m_pData->GetChildByName(temptext))->m_strBuffer[0])
				{
					g_xMenu.m_iSPLeaderboardStartIndex += 10;
					g_xMenu.m_xHTTPQuery.GetSPLeaderboard(g_xCurrentProfile.m_strLoginName, g_xMenu.m_iSPLeaderboardDifficulty, g_xMenu.m_iSPLeaderboardStartIndex);
				}
			}
		}
	}
	else if(!stricmp(strCommand, "SPLeaderboardTop10"))
	{
		g_xMenu.m_iSPLeaderboardStartIndex = 1;

		g_xMenu.m_xHTTPQuery.GetSPLeaderboard(g_xCurrentProfile.m_strLoginName, g_xMenu.m_iSPLeaderboardDifficulty, g_xMenu.m_iSPLeaderboardStartIndex);
	}
	else if(!stricmp(strCommand, "SPLeaderboardMyRank"))
	{
		g_xMenu.m_iSPLeaderboardStartIndex = -1;

		g_xMenu.m_xHTTPQuery.GetSPLeaderboard(g_xCurrentProfile.m_strLoginName, g_xMenu.m_iSPLeaderboardDifficulty);
	}
	else if(!stricmp(strCommand, "ToggleObjectVisibility"))
	{
		tempobject = pWindow->m_pData->GetChildByName(strParams[0]);
		tempobject->SetHidden(!tempobject->GetHidden());
		pWindow->m_pData->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "LeaderboardSelectDifficulty"))
	{
		if(g_xMenu.m_iSPLeaderboardStartIndex < 0)
			g_xMenu.m_iSPLeaderboardStartIndex = g_xMenu.m_xHTTPQuery.GetSPLeaderboardStartIndex();

		imageobject = (xGUIImage*)pWindow->m_pData->GetChildByName(strParams[1]);
		imageobject->SetY(g_xMenu.m_fSPDifficultyOriginalY + strParams[3][0] - '0');
		imageobject->SetWidth(((xGUIImage*)pObject)->GetWidth());
		imageobject->SetHeight(((xGUIImage*)pObject)->GetHeight());
		imageobject->SetTexture(((xGUIImage*)pObject)->m_pTextures, ((xGUIImage*)pObject)->m_iFrameCount, 0);
		pWindow->m_pData->GetChildByName(strParams[0])->SetHidden(true);
		pWindow->m_pData->UpdateRenderables();

		g_xMenu.m_iSPLeaderboardDifficulty = strParams[2][0] - '0';

		if(g_xMenu.m_iSPLeaderboardStartIndex > 0)
			g_xMenu.m_xHTTPQuery.GetSPLeaderboard(g_xCurrentProfile.m_strLoginName, g_xMenu.m_iSPLeaderboardDifficulty, g_xMenu.m_iSPLeaderboardStartIndex);
	}
	else if(!stricmp(strCommand, "ToggleHostServerDifficulty"))
	{
		if(strParams[0][0] == '0')
		{
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0))->m_bToggled = 1;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1))->m_bToggled = 0;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY2))->m_bToggled = 0;
		}
		else if(strParams[0][0] == '1')
		{
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0))->m_bToggled = 0;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1))->m_bToggled = 1;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY2))->m_bToggled = 0;
		}
		else
		{
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0))->m_bToggled = 0;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1))->m_bToggled = 0;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY2))->m_bToggled = 1;
		}

		g_xMenu.m_iSongDifficulty = (strParams[0][0] - '0') + 1;
	}
	else if(!stricmp(strCommand, "StartServer"))
	{
		char *selectedsong = g_xMenu.m_pSongListBox->GetSelectedString();

		if(selectedsong)
			g_xMenu.m_xHTTPQuery.AddServerToList(1984, g_xGame.m_xPlayer.m_strAccountName, selectedsong, g_xMenu.m_iSongDifficulty);
		else
		{
			g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX);

			window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);

			if(window)
				((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INITLISTENSERVERSELECTSONG);
		}
	}
	else if(!stricmp(strCommand, "JoinServer"))
	{
		g_xMenu.m_xHTTPQuery.CancelQuery();

		if(g_xMenu.m_pServerListBox->m_iNumItems >= 1)
		{
			char *server = g_xMenu.m_pServerListBox->GetSelectedData();
			if(server)
			{
				strcpy(temptext, server);

				int index = -1;
				for(int i = 0; i < strlen(temptext); i++)
				{
					if(temptext[i] == ':')
					{
						index = i;
						break;
					}
				}

				if(index != -1)
				{
					char *port = &temptext[index + 1];
					temptext[index] = '\0';
					g_xMenu.m_pAudioStream->Stop();
					g_xGame.InitClient(temptext, atoi(port));
					g_pCurGameState = &g_xGame;
					xGraphicsSetSceneGraph(&g_xGame.m_xSceneGraph);
				}
			}
		}
	}
	else if(!stricmp(strCommand, "ConnectToIP"))
	{
		g_xMenu.m_xHTTPQuery.CancelQuery();

		textboxobject = (xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[0]);

		strcpy(temptext, textboxobject->GetText());

		int index = -1;
		for(int i = 0; i < strlen(temptext); i++)
		{
			if(temptext[i] == ':')
			{
				index = i;
				break;
			}
		}

		if(index != -1)
		{
			char *port = &temptext[index + 1];
			temptext[index] = '\0';
			g_xMenu.m_pAudioStream->Stop();
			g_xGame.InitClient(temptext, atoi(port));
			g_pCurGameState = &g_xGame;
			xGraphicsSetSceneGraph(&g_xGame.m_xSceneGraph);
		}
		else
		{
			g_xMenu.m_pAudioStream->Stop();
			g_xGame.InitClient(temptext, 1984);
			g_pCurGameState = &g_xGame;
			xGraphicsSetSceneGraph(&g_xGame.m_xSceneGraph);
		}
	}
	else if(!stricmp(strCommand, "RefreshServerList"))
	{
		if(g_xMenu.m_pRefreshButton->GetText()[0] == 'S')
		{
			for(int i = 0; i < g_xMenu.m_pServerListBox->m_iNumItems; i++)
			{
				g_xMenu.m_pPingInfo[i].m_iRequestCount = 3;
				g_xMenu.m_pPingTimes[i] = 0;
			}

			g_xMenu.m_pRefreshButton->SetText(BB_STRING_REFRESH);
		}
		else
			g_xMenu.RefreshServerList();
	}
	else if(!stricmp(strCommand, "Login"))
	{
		g_xMenu.Login(((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[0]))->GetText(), ((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[1]))->GetText(), 0, pWindow);
	}
	else if(!stricmp(strCommand, "CancelLogin"))
	{
		g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);

		window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_LOGIN);
		if(!window)
		{
			window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_REGISTER);
			if(!window)
				window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_LOGINSP);
		}

		if(window)
		{
			g_xMenu.m_xGUI.SetFocus(window, 1);
			window->m_pData->SetHidden(0);
			window->m_pData->UpdateRenderables();
		}
	}
	else if(!stricmp(strCommand, "Register"))
	{
		if(!strcmp(((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[1]))->GetText(), ((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[2]))->GetText()))
		{
			strcpy(g_xGame.m_xPlayer.m_strAccountName, ((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[0]))->GetText());

			int length = strlen(g_xGame.m_xPlayer.m_strAccountName);
			for(int i = 0; i < length; i++)
			{
				if(g_xGame.m_xPlayer.m_strAccountName[i] < '0' || (g_xGame.m_xPlayer.m_strAccountName[i] > '9' && g_xGame.m_xPlayer.m_strAccountName[i] < 'A') ||
				  (g_xGame.m_xPlayer.m_strAccountName[i] > 'Z' && g_xGame.m_xPlayer.m_strAccountName[i] < 'a') || g_xGame.m_xPlayer.m_strAccountName[i] > 'z')
				{
					g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX);
					window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);
					if(window)
					{
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDUSERNAME);
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDUSERNAME2);
					}

					return;
				}
			}

			if(strlen(g_xGame.m_xPlayer.m_strAccountName) < 4 || strlen(g_xGame.m_xPlayer.m_strAccountName) > 20)
			{
				g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX);
				window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);
				if(window)
				{
					((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDUSERNAME);
					((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDUSERNAME2);
				}

				return;
			}

			char *password = ((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[1]))->GetText();
			if(strlen(password) < 8 || strlen(password) > 32)
			{
				g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX);
				window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);
				if(window)
				{
					((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDPASSWORD);
					((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDPASSWORD2);
				}

				return;
			}

			g_xMenu.m_xHTTPQuery.Register(g_xGame.m_xPlayer.m_strAccountName, password, ((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[3]))->GetText());

			if(g_xGame.m_xPlayer.m_strCDKey[0] == '+')
			{
				strcpy(g_xGame.m_xPlayer.m_strCDKey, ((xGUITextBox*)pWindow->m_pData->GetChildByName(strParams[3]))->GetText());
				g_xMenu.StoreCDKey();
			}

			strcpy(g_xCurrentProfile.m_strLoginName, g_xGame.m_xPlayer.m_strAccountName);
			g_xMenu.m_xHTTPQuery.GenerateMD5(password, g_xCurrentProfile.m_strLoginPassword);
			g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

			pWindow->m_pData->SetHidden(1);
			pWindow->m_pData->UpdateRenderables();

			//g_xMenu.AddMenuWindow(BB_WINDOW_MENU_LOGGINGIN);
		}
		else
		{
			g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX);
			window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);
			if(window)
				((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_PASSWORDMISMATCH);
		}
	}
	else if(!stricmp(strCommand, "ShowLeaderboard"))
	{
		g_xMenu.m_xHTTPQuery.GetLeaderboardData(g_xGame.m_xPlayer.m_strAccountName);
	}
	else if(!stricmp(strCommand, g_strStoreKey))
	{
		j = 0;
		for(i = 0; i < 5; i++)
		{
			sprintf(temptext, "%s%i", strParams[0], i + 1);
			textboxobject = (xGUITextBox*)pWindow->m_pData->GetChildByName(temptext);
			g_xGame.m_xPlayer.m_strCDKey[j++] = textboxobject->m_strBuffer[0];
			g_xGame.m_xPlayer.m_strCDKey[j++] = textboxobject->m_strBuffer[1];
			g_xGame.m_xPlayer.m_strCDKey[j++] = textboxobject->m_strBuffer[2];
			g_xGame.m_xPlayer.m_strCDKey[j++] = textboxobject->m_strBuffer[3];
			g_xGame.m_xPlayer.m_strCDKey[j++] = textboxobject->m_strBuffer[4];

			if(i < 4)
				g_xGame.m_xPlayer.m_strCDKey[j++] = '-';
		}

		g_xGame.m_xPlayer.m_strCDKey[j] = '\0';

		g_xPaperAPI.ValidateCDKey(g_xGame.m_xPlayer.m_strCDKey);

		if(g_xGame.m_xPlayer.m_strCDKey[0] == '+')
		{
			g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX);
			window = g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);
			if(window)
			{
				((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(xDecryptString(g_pEncryptedCDKeyInvalid1, BB_ENCRYPTED_LENGTH_CDKEYINVALID1, temptext, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3));
				((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(xDecryptString(g_pEncryptedCDKeyInvalid2, BB_ENCRYPTED_LENGTH_CDKEYINVALID2, temptext, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3));
			}
		}
		else
		{
			g_xMenu.StoreCDKey();

			if (g_xMenu.AddMenuWindow(BB_WINDOW_MENU_BATTLES))
				g_xMenu.SlideTransition(true, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_MAIN)->m_pData, g_xMenu.m_xGUI.GetWindow(BB_WINDOW_MENU_BATTLES)->m_pData);

			g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);
		}
	}
	else if(!stricmp(strCommand, "OptionsMenuSelectOption"))
	{
		temptext2[1] = '\0';

		for(i = 0, temptext2[0] = '1'; i < 3; i++, temptext2[0]++)
		{
			sprintf(temptext, BB_NAME_MENUOPTIONBG, temptext2);
			pWindow->m_pData->GetChildByName(temptext)->SetHidden(temptext2[0] != strParams[0][0]);

			sprintf(temptext, BB_NAME_MENUOPTIONTEXT, temptext2);
			imageobject = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
			imageobject->SetAlpha(temptext2[0] == strParams[0][0] ? 2.0 : 0.75);
			imageobject->UpdateRenderables();

			sprintf(temptext, BB_NAME_OPTIONSPANEL, i + 1);
			pWindow->m_pData->GetChildByName(temptext)->SetHidden(i != strParams[0][0] - '1');
		}

		if(strParams[0][0] == '3')
		{
			g_xMenu.AddMenuWindow("Credits");
			g_xMenu.m_xGUI.GetWindow("Credits")->m_pData->m_bHasFocus = 0;
			pWindow->m_pData->m_bHasFocus = 1;
		}
		else
		{
			g_bCreditsRolling = false;

			window = g_xMenu.m_xGUI.GetWindow("Credits");

			if(window)
				g_xMenu.m_xGUI.RemoveWindow(window);
		}

		pWindow->m_pData->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "BindKey"))
	{
		if(strParams[1][0] == 't')
		{
			g_xMenu.m_iBindingKeyIndex = atoi(strParams[0]) - 1;
			sprintf(temptext, BB_NAME_CONTROLSALTBUTTON, g_xMenu.m_iBindingKeyIndex + 1);
			g_xMenu.m_iBindingKeyIndex += MAX_GAME_BUTTONS;
		}
		else
		{
			g_xMenu.m_iBindingKeyIndex = atoi(strParams[0]) - 1;
			sprintf(temptext, BB_NAME_CONTROLSBUTTON, g_xMenu.m_iBindingKeyIndex + 1);
		}

		((xGUIButton*)pWindow->m_pData->GetChildByName(temptext))->SetText("???");
	}
	else if(!stricmp(strCommand, "ApplyVideoOptions"))
	{
		tempbool = g_xOptions.m_bFullScreen;
		g_xOptions.m_bFullScreen = ((xGUICheckBox*)pWindow->m_pData->GetChildByName(BB_NAME_FULLSCREENCHECK))->m_bChecked;

		g_xPaperAPI.m_bOutlineModels = ((xGUICheckBox*)pWindow->m_pData->GetChildByName(BB_NAME_CHARACTEROUTLINECHECK))->m_bChecked;
		g_xOptions.m_bCharacterOutlines = g_xPaperAPI.m_bOutlineModels;

		g_xPaperAPI.m_bLimitParticles = ((xGUICheckBox*)pWindow->m_pData->GetChildByName(BB_NAME_REDUCEDPARTICLESCHECK))->m_bChecked;
		g_xOptions.m_bLimitedParticles = g_xPaperAPI.m_bLimitParticles;
			
		strcpy(temptext, ((xGUIComboBox*)pWindow->m_pData->GetChildByName(BB_NAME_RESOLUTIONCOMBO))->GetSelectedString());

		for(i = 0; i < strlen(temptext); i++)
			if(temptext[i] == 'x')
				break;

		temptext[i] = '\0';

		g_xOptions.m_iScreenWidth = atoi(temptext);
		g_xOptions.m_iScreenHeight = atoi(temptext + i + 1);

		if(g_xOptions.m_iScreenWidth != g_xGraphics.m_iScreenWidth || g_xOptions.m_iScreenHeight != g_xGraphics.m_iScreenHeight || tempbool != g_xOptions.m_bFullScreen)
			g_xPaperAPI.ChangeScreenResolution(g_xOptions.m_iScreenWidth, g_xOptions.m_iScreenHeight, g_xOptions.m_bFullScreen);

		g_xOptions.SaveOptions(BB_SETTINGSPATH);
	}
	else if(!stricmp(strCommand, "CancelDownload"))
	{
		g_xMenu.m_xGUI.DelayedRemoveWindow(BB_WINDOW_MENU_UPDATE);
	}
	else if(!stricmp(strCommand, "StartDownload"))
	{
#ifndef DEMO_RELEASE
		g_xMenu.m_pUpdateWindowStatusText = (xGUIText*)pWindow->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1);

		g_xMenu.StartUpdate();
#endif
	}
}

void xMenuRunKeyCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject)
{
	if(g_xPaperAPI.m_xInput.m_keyUp.JustPressed(true) || g_xPaperAPI.m_xInput.m_keyUp.ShouldRepeat(g_xPaperAPI.m_dwCurrentTime) ||
		g_xPaperAPI.m_xInput.m_keyLeft.JustPressed(true) || g_xPaperAPI.m_xInput.m_keyLeft.ShouldRepeat(g_xPaperAPI.m_dwCurrentTime))
	{
		if(!stricmp(strCommand, "FreePlayScroll"))
		{
			g_xPaperAPI.m_xInput.m_keyUp.Eaten();
			g_xPaperAPI.m_xInput.m_keyLeft.Eaten();
			g_xMenu.FreePlayScrollSongs(1);
		}
		else if(!stricmp(strCommand, "ChallengesScroll"))
		{
			g_xPaperAPI.m_xInput.m_keyUp.Eaten();
			g_xPaperAPI.m_xInput.m_keyLeft.Eaten();
			g_xMenu.ChallengesScroll(1);
		}
	}
	else if(g_xPaperAPI.m_xInput.m_keyDown.JustPressed(true) || g_xPaperAPI.m_xInput.m_keyDown.ShouldRepeat(g_xPaperAPI.m_dwCurrentTime) ||
		g_xPaperAPI.m_xInput.m_keyRight.JustPressed(true) || g_xPaperAPI.m_xInput.m_keyRight.ShouldRepeat(g_xPaperAPI.m_dwCurrentTime))
	{
		if(!stricmp(strCommand, "FreePlayScroll"))
		{
			g_xPaperAPI.m_xInput.m_keyDown.Eaten();
			g_xPaperAPI.m_xInput.m_keyRight.Eaten();
			g_xMenu.FreePlayScrollSongs(-1);
		}
		else if(!stricmp(strCommand, "ChallengesScroll"))
		{
			g_xPaperAPI.m_xInput.m_keyDown.Eaten();
			g_xPaperAPI.m_xInput.m_keyRight.Eaten();
			g_xMenu.ChallengesScroll(-1);
		}
	}
	else if(g_xPaperAPI.m_xInput.m_buttonMouseExtra3.JustPressed(true)) // Swipe Up
	{
		if(!stricmp(strCommand, "FreePlayScroll"))
		{
			g_xPaperAPI.m_xInput.m_buttonMouseExtra3.Eaten();
			g_xMenu.FreePlayScrollSongs(-5);
		}
		else if(!stricmp(strCommand, "ChallengesScroll"))
		{
			g_xPaperAPI.m_xInput.m_buttonMouseExtra3.Eaten();
			g_xMenu.ChallengesScroll(-5);
		}
	}
	else if(g_xPaperAPI.m_xInput.m_buttonMouseExtra4.JustPressed(true)) // Swipe Down
	{
		if(!stricmp(strCommand, "FreePlayScroll"))
		{
			g_xPaperAPI.m_xInput.m_buttonMouseExtra4.Eaten();
			g_xMenu.FreePlayScrollSongs(5);
		}
		else if(!stricmp(strCommand, "ChallengesScroll"))
		{
			g_xPaperAPI.m_xInput.m_buttonMouseExtra4.Eaten();
			g_xMenu.ChallengesScroll(5);
		}
	}
	else if(g_xPaperAPI.m_xInput.m_keyEscape.JustPressed(true))
	{
		if(!stricmp(strCommand, "CloseOnEscape"))
		{
			g_xPaperAPI.m_xInput.m_keyEscape.Eaten();
			g_xMenu.m_xGUI.DelayedRemoveWindow(pWindow);
		}
	}
}

void xMenuRunHoverCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xGUIObject* &pOutHoverObject)
{
	xGUIObject *tempobject;

	if(!stricmp(strCommand, "ShowObject"))
	{
		tempobject = pWindow->m_pData->GetChildByName(strParams[0]);
		tempobject->SetHidden(0);
		tempobject->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "MouseOverAnimation"))
	{
		tempobject = pWindow->m_pData->GetChildByName(strParams[0]);
		tempobject->SetHidden(0);
		tempobject->UpdateRenderables();

		tempobject = pWindow->m_pData->GetChildByName(strParams[1]);
		tempobject->SetHidden(0);
		tempobject->m_xAnimation.m_pCurKeyFrame = tempobject->m_xAnimation.m_xKeyFrameList.m_pNext;
		xAnimationLoadedAllKeyFrames(&tempobject->m_xAnimation);
		tempobject->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "SetFrame"))
		((xGUIImage*)pObject)->SetTextureFrame(atoi(strParams[0]));
}

void xMenuRunUnHoverCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject)
{
	xGUIObject *tempobject;

	if(!stricmp(strCommand, "HideObject"))
	{
		tempobject = pWindow->m_pData->GetChildByName(strParams[0]);
		tempobject->SetHidden(1);
		tempobject->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "EndMouseOverAnimation"))
	{
		tempobject = pWindow->m_pData->GetChildByName(strParams[0]);
		tempobject->SetHidden(true);
		tempobject->UpdateRenderables();

		tempobject = pWindow->m_pData->GetChildByName(strParams[1]);
		tempobject->SetHidden(true);
		tempobject->UpdateRenderables();
	}
	else if(!stricmp(strCommand, "SetFrame"))
		((xGUIImage*)pObject)->SetTextureFrame(atoi(strParams[0]));
}

void xMenuRunDragCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xRenderable2DQuad* &pOutDragObject)
{
	if(!stricmp(strCommand, "UpdateMusicVolume"))
	{
		g_xOptions.m_fMusicVolume = ((xGUISlider*)pObject)->GetValue() / 0.88f;
		g_xMenu.m_pAudioStream->SetVolume(g_xOptions.m_fMusicVolume);

		if(g_xGame.m_xSong.m_pTrack)
			g_xGame.m_xSong.m_pTrack->SetVolume(g_xOptions.m_fMusicVolume);

		g_xOptions.SaveOptions(BB_SETTINGSPATH);
	}
	else if(!stricmp(strCommand, "UpdateSoundVolume"))
	{
		g_xPaperAPI.m_fVolume = ((xGUISlider*)pObject)->GetValue() / 0.88f;
		g_xOptions.m_fSoundVolume = g_xPaperAPI.m_fVolume;
		g_xOptions.SaveOptions(BB_SETTINGSPATH);
	}
	else if(!stricmp(strCommand, "UpdateVoiceOverVolume"))
	{
		g_xOptions.m_fVoiceOverVolume = ((xGUISlider*)pObject)->GetValue() / 0.88f;
		g_xOptions.SaveOptions(BB_SETTINGSPATH);
	}
}

void xMenuRunDropCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject)
{
}

void xMenuHandlePaste(char *strContents, xGUIWindowList *pWindow, xGUIObject *pObject)
{
	xGUIObjectList *temp;
	int i, j;

	if(strstr(pObject->m_strName, "CDKey"))
	{
		temp = pWindow->m_pData->m_xChildObjectList.m_pNext;
		while(temp && temp->m_pData != pObject)
			temp = temp->m_pNext;

		if(temp)
		{
			for(i = 0, j = 0; i < 5; i++)
			{
				j += ((xGUITextBox*)temp->m_pData)->HandlePaste(strContents + j, i > 0);

				if(temp->m_pData->m_strName[strlen(temp->m_pData->m_strName) - 1] == '5')
					break;

				if(strContents[j] == '-')
					j++;

				temp = pWindow->m_pData->GetNextTabIndexObject(temp);
			}
		}
	}
}

void xMenu::StartGame(char* strLevel)
{
	m_pAudioStream->Stop();

	if(m_dStartGameCountdown <= 0.0)
	{
		m_dStartGameCountdown = 0.2;

		if(strLevel != g_xMenu.m_strStoredLevel)
			strcpy(g_xMenu.m_strStoredLevel, strLevel);
	}
}

void xMenu::InitializeBattlesWindow(xGUIWindowList *pWindow)
{
	xGUIText *textobject;
	xColor tempcolor;
	int currentlevel, tourstart = 0, tour = 6, total, percentage, i, j;
	float tempfloat;
	char unlockedtour[2] = "1", temptext[32];

	//((xGUIText*)pWindow->m_pData->GetChildByName("ProfileName"))->SetText(g_xCurrentProfile.m_strProfileName);

	currentlevel = g_xCurrentProfile.m_iCurrentLevel;
	for(i = 0; i < 7; i++)
	{
		if(currentlevel <= g_iTourLevelCount[i])
		{
			tour = i;
			currentlevel = 999999999;
		}

		total = 0;

		for(j = 1; j <= g_iTourLevelCount[i]; j++)
			total += g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][tourstart + j];

		percentage = ((float)total / ((float)g_iTourLevelCount[i] * 100.0f)) * 100.0;

		sprintf(temptext, BB_NAME_TOURTHUMBSUPIMAGE, i + 1);
		pWindow->m_pData->GetChildByName(temptext)->SetHidden(percentage <= 0 || percentage == 100);

		sprintf(temptext, BB_NAME_TOURPERCENTAGETEXT, i + 1);
		textobject = (xGUIText*)pWindow->m_pData->GetChildByName(temptext);
		sprintf(temptext, "%i%%", percentage);
		textobject->SetText(temptext);
		textobject->SetHidden(percentage <= 0 || percentage == 100);

		if(percentage >= 50)
		{
			tempfloat = ((float)percentage - 50.0f) / 50.0f;
			textobject->UpdateColor(xColorInit(&tempcolor, 1.0f + (0.004f - 1.0f) * tempfloat, 0.635f + (1.0f - 0.635f) * tempfloat, 0.192f * tempfloat, 0.9f));
		}
		else
		{
			tempfloat = (float)percentage / 50.0f;
			textobject->UpdateColor(xColorInit(&tempcolor, 1.0f, 0.988f + (0.635f - 0.988f) * tempfloat, 0.055f - 0.055f * tempfloat, 0.9f));
		}

		sprintf(temptext, BB_NAME_TOURDOMINATEDIMAGE, i + 1);
		pWindow->m_pData->GetChildByName(temptext)->SetHidden(percentage != 100);

		if(currentlevel < 1000 && unlockedtour[0] < '7')
		{
			currentlevel -= g_iTourLevelCount[i];
			unlockedtour[0]++;
		}

		tourstart += g_iTourLevelCount[i];
	}

	SelectTour(pWindow, unlockedtour, g_strTourBackgrounds[tour]);
}

void xMenu::SelectTour(xGUIWindowList *pWindow, char *strTour, char* strTourBackground)
{
	char temptext[256], temptext2[16] = {0};
	xGUIText *textobject;
	xGUIImage *imageobject, *imageobject2;
	xGUIButton *buttonobject;
	xTexture **pTextures;
	xColor tempcolor;
	float tempfloat;
	int i;

	m_iTourIndex = strTour[0] - '1';

	pWindow->m_pData->GetChildByName(BB_NAME_TUTORIALARROW)->SetHidden(m_iTourIndex != 0 || g_xCurrentProfile.m_iCurrentLevel != 1);

	m_iTourBattleIndexStart = 0;
	for(i = 0; i < m_iTourIndex; i++)
		m_iTourBattleIndexStart += g_iTourLevelCount[i];

	for(temptext2[0] = '1'; temptext2[0] <= '7'; temptext2[0]++)
	{
		sprintf(temptext, BB_NAME_MENUOPTIONBG, temptext2);
		imageobject = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
		imageobject->SetHidden(temptext2[0] != strTour[0]);

		sprintf(temptext, BB_NAME_MENUOPTIONTEXT, temptext2);
		imageobject = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
		imageobject->SetAlpha(temptext2[0] == strTour[0] ? 2.0 : 0.75);
		imageobject->UpdateRenderables();
	}

	for(i = 1; i <= 12; i++)
	{
		sprintf(temptext, BB_NAME_BATTLEMENUENEMYBG, i);
		buttonobject = (xGUIButton*)pWindow->m_pData->GetChildByName(temptext);

		sprintf(temptext, BB_NAME_BATTLETHUMBSUPIMAGE, i);
		imageobject = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);

		sprintf(temptext, BB_NAME_BATTLEPERCENTAGETEXT, i);
		textobject = (xGUIText*)pWindow->m_pData->GetChildByName(temptext);

		sprintf(temptext, BB_NAME_BATTLEDOMINATEDIMAGE, i);
		imageobject2 = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);

		// Bosses only occupy slots 8, 10, 11, and 12
		if(i == 8 || i > 9)
		{
			sprintf(temptext, BB_NAME_BATTLEBOSSIMAGE, i);
			pWindow->m_pData->GetChildByName(temptext)->SetHidden(g_iTourEnemyIndices[m_iTourIndex][i - 1] < 9);
		}

		if(i <= g_iTourLevelCount[m_iTourIndex])
		{
			buttonobject->SetHidden(false);

			if(m_iTourBattleIndexStart + i > g_xCurrentProfile.m_iCurrentLevel)
			{
				buttonobject->SetIconFrame(g_iTourEnemyIndices[m_iTourIndex][i - 1]);
				buttonobject->SetEnabled(false);

				imageobject->SetHidden(true);
				textobject->SetHidden(true);
				imageobject2->SetHidden(true);
			}
			else
			{
				buttonobject->SetIconFrame(BB_ENEMIES_MAX + g_iTourEnemyIndices[m_iTourIndex][i - 1]);
				buttonobject->SetEnabled(true);

				if(m_iTourBattleIndexStart + i == g_xCurrentProfile.m_iCurrentLevel)
				{
					imageobject->SetHidden(true);
					textobject->SetHidden(true);
					imageobject2->SetHidden(true);
				}
				else
				{
					imageobject->SetHidden(false);
					textobject->SetHidden(false);

					if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][m_iTourBattleIndexStart + i] >= 100)
					{
						strcpy(temptext2, "100%");
						imageobject2->SetHidden(false);
						textobject->UpdateColor(xColorInit(&tempcolor, 0.004f, 1.0f, 0.192f, 0.9f));
					}
					else
					{
						sprintf(temptext2, "%i%%", g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][m_iTourBattleIndexStart + i]);
						imageobject2->SetHidden(true);

						if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][m_iTourBattleIndexStart + i] >= 50)
						{
							tempfloat = ((float)g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][m_iTourBattleIndexStart + i] - 50.0f) / 50.0f;
							textobject->UpdateColor(xColorInit(&tempcolor, 1.0f + (0.004f - 1.0f) * tempfloat, 0.635f + (1.0f - 0.635f) * tempfloat, 0.192f * tempfloat, 0.9f));
						}
						else
						{
							tempfloat = (float)g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][m_iTourBattleIndexStart + i] / 50.0f;
							textobject->UpdateColor(xColorInit(&tempcolor, 1.0f, 0.988f + (0.635f - 0.988f) * tempfloat, 0.055f - 0.055f * tempfloat, 0.9f));
						}
					}
				}

				textobject->SetText(temptext2);
			}
		}
		else
		{
			buttonobject->SetHidden(true);
			imageobject->SetHidden(true);
			textobject->SetHidden(true);
			imageobject2->SetHidden(true);
		}
	}

	if(strTourBackground)
	{
		sprintf(temptext, BB_IMAGE_PATH, strTourBackground);
		int count = xGraphicsGetTextures(temptext, &pTextures);
		((xGUIImage*)pWindow->m_pData->GetChildByName(BB_NAME_TOURBACKGROUND))->SetTexture(pTextures, count, 0);
	}

	pWindow->m_pData->UpdateRenderables();
}

FORCE_INLINE bool xMenuSinglePlayerLogin(xSinglePlayerLogin xSPLogin, char* strLoginMessageBox, char* strLoginMenu, char* strTargetMenu)
{
	if(g_xCurrentProfile.m_byteSelectedDifficulty >= BB_NUMDIFFICULTIES)
	{
		return false;
	}

	return true;
}
		
void xMenu::InitializeChallengesWindow(xGUIWindowList *pWindow)
{
	xFileIO fileio;
	xParser parser;
	char temptext[256], temptext2[256], *tempstring;
	int challengeindex, i, j;

	m_bChallenges = 1;
	m_iChallengeCount = BB_ACHIEVEMENTS_MAX - 1;
	m_iFreePlayScrollAmount = 0;

	challengeindex = m_iChallengesTop;

	if(!m_strChallengeSongNames[0])
	{
		for(i = 0; i < m_iChallengeCount; i++)
		{
			if(g_strChallengeSongList[i][0] != '\0')
			{
				sprintf(temptext2, "./music/%s/Song.bbm", g_strChallengeSongList[i]);
				parser.OpenScript(temptext2);
				parser.ReadLine(temptext, temptext, temptext, temptext);

				tempstring = parser.GetVariable("strName");
				m_strChallengeSongNames[i] = new char[strlen(tempstring) + 6 + 1];
				sprintf(m_strChallengeSongNames[i], "Song: %s", tempstring);

				tempstring = parser.GetVariable("strDefaultEnemy");
				if(tempstring)
				{
					m_strChallengeEnemies[i] = new char[strlen(tempstring) + 1];
					strcpy(m_strChallengeEnemies[i], tempstring);
				}
				else
				{
					j = rand() % i;
					m_strChallengeEnemies[i] = new char[strlen(m_strChallengeEnemies[j]) + 1];
					strcpy(m_strChallengeEnemies[i], m_strChallengeEnemies[j]);
				}

				m_bChallengeLoopSong[i] = parser.ToBool("bLoopChallenge");

				parser.EndScript();
			}
			else
			{
				m_strChallengeSongNames[i] = new char[16];
				strcpy(m_strChallengeSongNames[i], "Various Songs");

				j = rand() % BB_ENEMIES_MAX;
				m_strChallengeEnemies[i] = new char[strlen(g_strAllEnemies[j]) + 1];
				strcpy(m_strChallengeEnemies[i], g_strAllEnemies[j]);
			}
		}
	}

	for(i = 0; i < BB_FREEPLAY_SONGCOUNT; i++)
	{
		sprintf(temptext, BB_NAME_FREEPLAYSONGBG, i + 1);
		m_pFreePlaySongBackgrounds[i] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
		xRenderable2DCopy(&m_xFreePlaySongBGOriginal[i + 1], &m_pFreePlaySongBackgrounds[i]->m_xRenderable);

		sprintf(temptext, BB_NAME_FREEPLAYSONGNAME, i + 1);
		m_pFreePlaySongNames[i] = (xGUIText*)pWindow->m_pData->GetChildByName(temptext);
		xRenderable2DCopy(&m_xFreePlaySongNameOriginal[i + 1], &m_pFreePlaySongNames[i]->m_xRenderable);
		m_xFreePlaySongNameOriginal[i + 1].m_xColor.R = m_pFreePlaySongNames[i]->m_xNormalColor.R;
		m_xFreePlaySongNameOriginal[i + 1].m_xColor.G = m_pFreePlaySongNames[i]->m_xNormalColor.G;
		m_xFreePlaySongNameOriginal[i + 1].m_xColor.B = m_pFreePlaySongNames[i]->m_xNormalColor.B;

		m_pFreePlaySongNames[i]->SetText(g_strChallengeTitle[challengeindex]);

		for(j = 0; j < 12; j++)
		{
			sprintf(temptext, BB_NAME_FREEPLAYDIFFSTAR, i + 1, j + 1);
			m_pFreePlaySongDiffStars[i][j] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
			m_pFreePlaySongDiffStars[i][j]->SetHidden(j >= g_iChallengeDifficulties[challengeindex]);
		}
		
		m_fFreePlaySongDiffOriginalY[i + 1] = m_pFreePlaySongDiffStars[i][0]->m_xRenderable.m_fY;

		sprintf(temptext, BB_NAME_CHALLENGECOMPLETE, i + 1);
		m_pChallengeCompleteImages[i] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
		xRenderable2DCopy(&m_xChallengeCompleteOriginal[i + 1], &m_pChallengeCompleteImages[i]->m_xRenderable);
		m_pChallengeCompleteImages[i]->SetHidden(!g_xCurrentProfile.m_byteAchievementStatus[challengeindex]);

		challengeindex++;

		if(challengeindex == m_iChallengeCount)
			challengeindex = 0;
	}

	challengeindex = (m_iChallengesTop + 4) % m_iChallengeCount;

	m_xFreePlaySongBGOriginal[0].m_fX = m_xFreePlaySongBGOriginal[1].m_fX;
	m_xFreePlaySongBGOriginal[0].m_fY = m_xFreePlaySongBGOriginal[1].m_fY - m_xFreePlaySongBGOriginal[1].m_fHeight;
	m_xFreePlaySongBGOriginal[0].m_fWidth = m_xFreePlaySongBGOriginal[1].m_fWidth;
	m_xFreePlaySongBGOriginal[0].m_fHeight = m_xFreePlaySongBGOriginal[1].m_fHeight;
	m_xFreePlaySongBGOriginal[0].m_fScaleX = m_xFreePlaySongBGOriginal[1].m_fScaleX;
	m_xFreePlaySongBGOriginal[0].m_fScaleY = m_xFreePlaySongBGOriginal[1].m_fScaleY;
	m_xFreePlaySongBGOriginal[0].m_xColor.A = 0.0;

	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fX = m_xFreePlaySongBGOriginal[0].m_fX;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fY = m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT].m_fY + m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT].m_fHeight;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fWidth = m_xFreePlaySongBGOriginal[0].m_fWidth;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fHeight = m_xFreePlaySongBGOriginal[0].m_fHeight;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleX = m_xFreePlaySongBGOriginal[0].m_fScaleX;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleY = m_xFreePlaySongBGOriginal[0].m_fScaleY;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_xColor.A = 0.0;

	m_xFreePlaySongNameOriginal[0].m_fX = m_xFreePlaySongNameOriginal[1].m_fX;
	m_xFreePlaySongNameOriginal[0].m_fY = m_xFreePlaySongNameOriginal[1].m_fY - m_xFreePlaySongBGOriginal[1].m_fHeight;
	m_xFreePlaySongNameOriginal[0].m_fScaleX = m_xFreePlaySongNameOriginal[1].m_fScaleX - 0.05;
	m_xFreePlaySongNameOriginal[0].m_fScaleY = m_xFreePlaySongNameOriginal[1].m_fScaleY - 0.05;
	m_xFreePlaySongNameOriginal[0].m_xColor.A = 0.0;

	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fX = m_xFreePlaySongNameOriginal[0].m_fX;
	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fY = m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT].m_fY + m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT].m_fHeight;
	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleX = m_xFreePlaySongNameOriginal[0].m_fScaleX;
	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleY = m_xFreePlaySongNameOriginal[0].m_fScaleY;
	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_xColor.A = 0.0;

	m_fFreePlaySongDiffOriginalY[0] = m_fFreePlaySongDiffOriginalY[1] - m_xFreePlaySongBGOriginal[1].m_fHeight;
	m_fFreePlaySongDiffOriginalY[BB_FREEPLAY_SONGCOUNT + 1] = m_fFreePlaySongDiffOriginalY[BB_FREEPLAY_SONGCOUNT] + m_xFreePlaySongBGOriginal[1].m_fHeight;

	m_xChallengeCompleteOriginal[0].m_fX = m_xChallengeCompleteOriginal[1].m_fX;
	m_xChallengeCompleteOriginal[0].m_fY = m_xChallengeCompleteOriginal[1].m_fY - m_xChallengeCompleteOriginal[1].m_fHeight;
	m_xChallengeCompleteOriginal[0].m_fWidth = m_xChallengeCompleteOriginal[1].m_fWidth;
	m_xChallengeCompleteOriginal[0].m_fHeight = m_xChallengeCompleteOriginal[1].m_fHeight;
	m_xChallengeCompleteOriginal[0].m_fScaleX = m_xChallengeCompleteOriginal[1].m_fScaleX - 0.05;
	m_xChallengeCompleteOriginal[0].m_fScaleY = m_xChallengeCompleteOriginal[1].m_fScaleY - 0.05;
	m_xChallengeCompleteOriginal[0].m_xColor.A = 0.0;

	m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fX = m_xChallengeCompleteOriginal[0].m_fX;
	m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fY = m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT].m_fY + m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT].m_fHeight;
	m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fWidth = m_xChallengeCompleteOriginal[0].m_fWidth;
	m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fHeight = m_xChallengeCompleteOriginal[0].m_fHeight;
	m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleX = m_xChallengeCompleteOriginal[0].m_fScaleX;
	m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleY = m_xChallengeCompleteOriginal[0].m_fScaleY;
	m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_xColor.A = 0.0;

	sprintf(temptext, BB_NAME_FREEPLAYSONGBG, 69);
	m_pFreePlaySongBackgrounds[BB_FREEPLAY_SONGCOUNT] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
	sprintf(temptext, BB_NAME_FREEPLAYSONGNAME, 69);
	m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT] = (xGUIText*)pWindow->m_pData->GetChildByName(temptext);
	sprintf(temptext, BB_NAME_CHALLENGECOMPLETE, 69);
	m_pChallengeCompleteImages[BB_FREEPLAY_SONGCOUNT] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);

	for(j = 0; j < 12; j++)
	{
		sprintf(temptext, BB_NAME_FREEPLAYDIFFSTAR, 69, j + 1);
		m_pFreePlaySongDiffStars[BB_FREEPLAY_SONGCOUNT][j] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
	}

	m_pFreePlaySongArtist[0] = (xGUIText*)pWindow->m_pData->GetChildByName(BB_NAME_FREEPLAYARTIST1);
	m_pFreePlaySongArtist[1] = (xGUIText*)pWindow->m_pData->GetChildByName(BB_NAME_FREEPLAYARTIST2);
	m_pFreePlaySongArtist[0]->SetText(m_strChallengeSongNames[challengeindex]);
	m_fFreePlaySongArtistOriginalY = m_pFreePlaySongArtist[0]->m_xRenderable.m_fY;
	m_iCurrentArtistIndex = 0;

	for(i = 0; i < 4; i++)
	{
		sprintf(temptext, BB_NAME_CHALLENGELINE, i + 1);
		m_pChallengeTextLines[i] = (xGUIText*)pWindow->m_pData->GetChildByName(temptext);
		m_pChallengeTextLines[i]->SetText(g_strChallengeDescription[challengeindex][i]);
	}

	m_pChallengeAchievementIcon = (xGUIImage*)pWindow->m_pData->GetChildByName(BB_NAME_CHALLENGEACHIEVEMENT);
	m_pChallengeAchievementIcon->SetTextureFrame(challengeindex);

	m_pFreePlayEnemyModel = (xGUIModel*)pWindow->m_pData->GetChildByName(BB_NAME_FREEPLAYENEMYMODEL);
	m_pFreePlayEnemyModel->SetModel(m_strChallengeEnemies[challengeindex]);

	pWindow->m_pData->UpdateRenderables();
}

void xMenu::ChallengesScroll(int iAmount)
{
	float oldamount;
	int challengeindex, i;

	oldamount = m_iFreePlayScrollAmount;

	m_iFreePlayScrollAmount += iAmount;

	if(m_iFreePlayScrollAmount == 0)
		m_iFreePlayScrollAmount += iAmount;

	if(oldamount == 0)
	{
		m_fFreePlayScrollTime = 0.0;
		m_fFreePlayCurrentScrollTime = 0.0;
	}
	else
	{
		m_fFreePlayScrollTime = BB_FREEPLAY_SCROLLTIME * (((m_fFreePlayScrollTime / BB_FREEPLAY_SCROLLTIME) * oldamount) / (float)m_iFreePlayScrollAmount);
		m_fFreePlayCurrentScrollTime = m_fFreePlayCurrentScrollTime * (oldamount / (float)m_iFreePlayScrollAmount);
	}

	m_iFreePlayScrollDirection = m_iFreePlayScrollAmount > 0 ? 1 : -1;

	for(i = 0; i < BB_FREEPLAY_SONGCOUNT; i++)
		xMenuSetChallengeScrollValues(i, i + 1, i + 1 + m_iFreePlayScrollDirection);

	if(m_iFreePlayScrollDirection > 0)
	{
		xMenuSetChallengeScrollValues(BB_FREEPLAY_SONGCOUNT, 0, 1);

		challengeindex = m_iChallengesTop > 0 ? m_iChallengesTop - 1 : m_iChallengeCount - 1;
	}
	else
	{
		xMenuSetChallengeScrollValues(BB_FREEPLAY_SONGCOUNT, BB_FREEPLAY_SONGCOUNT + 1, BB_FREEPLAY_SONGCOUNT);

		challengeindex = (m_iChallengesTop + BB_FREEPLAY_SONGCOUNT) % m_iChallengeCount;
	}

	m_pFreePlaySongBackgrounds[BB_FREEPLAY_SONGCOUNT]->SetHidden(0);
	m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT]->SetHidden(0);
	m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT]->SetText(g_strChallengeTitle[challengeindex]);
	m_pChallengeCompleteImages[BB_FREEPLAY_SONGCOUNT]->SetHidden(!g_xCurrentProfile.m_byteAchievementStatus[challengeindex]);

	for(i = 0; i < 12; i++)
		m_pFreePlaySongDiffStars[BB_FREEPLAY_SONGCOUNT][i]->SetHidden(i >= g_iChallengeDifficulties[challengeindex]);

	m_pFreePlaySongArtist[m_iCurrentArtistIndex]->SetText(m_strChallengeSongNames[(m_iChallengesTop + 4 - m_iFreePlayScrollDirection) % m_iChallengeCount]);

	m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex] = m_fFreePlaySongArtistOriginalY - m_xFreePlaySongBGOriginal[4].m_fHeight * m_iFreePlayScrollDirection;
	m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex] = m_fFreePlaySongArtistOriginalY;
	DEBUGLOG("%i %f %f\n", m_iCurrentArtistIndex, m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex], m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex]);

	m_iCurrentArtistIndex = 1 ^ m_iCurrentArtistIndex;
	m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex] = m_fFreePlaySongArtistOriginalY;
	m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex] = m_fFreePlaySongArtistOriginalY + m_xFreePlaySongBGOriginal[5].m_fHeight * m_iFreePlayScrollDirection;
	DEBUGLOG("%i %f %f\n", m_iCurrentArtistIndex, m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex], m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex]);
}

FORCE_INLINE void xMenuSetChallengeScrollValues(int iIndex, int iStartIndex, int iTargetIndex)
{
	xRenderable2DCopy(&g_xMenu.m_xFreePlaySongBGStart[iIndex], &g_xMenu.m_xFreePlaySongBGOriginal[iStartIndex]);
	xRenderable2DCopy(&g_xMenu.m_xFreePlaySongBGTarget[iIndex], &g_xMenu.m_xFreePlaySongBGOriginal[iTargetIndex]);

	xRenderable2DCopy(&g_xMenu.m_xFreePlaySongNameStart[iIndex], &g_xMenu.m_xFreePlaySongNameOriginal[iStartIndex]);
	xRenderable2DCopy(&g_xMenu.m_xFreePlaySongNameTarget[iIndex], &g_xMenu.m_xFreePlaySongNameOriginal[iTargetIndex]);

	g_xMenu.m_fFreePlaySongDiffStartY[iIndex] = g_xMenu.m_fFreePlaySongDiffOriginalY[iStartIndex];
	g_xMenu.m_fFreePlaySongDiffTargetY[iIndex] = g_xMenu.m_fFreePlaySongDiffOriginalY[iTargetIndex];

	xRenderable2DCopy(&g_xMenu.m_xChallengeCompleteStart[iIndex], &g_xMenu.m_xChallengeCompleteOriginal[iStartIndex]);
	xRenderable2DCopy(&g_xMenu.m_xChallengeCompleteTarget[iIndex], &g_xMenu.m_xChallengeCompleteOriginal[iTargetIndex]);
}

void xMenu::ChallengeStart(int iChallenge)
{
	int achievement, tempint, i;

	// Achievements are offset by 1 due to the BB_ACH_NONE
	achievement = iChallenge + 1;

	switch(achievement)
	{
		case BB_ACH_DOMINATE_TRAINING:
		{
			if(g_xCurrentProfile.m_u32HighScores[0][i] < 100)
				g_xMenu.StartGame("Tutorial1");
			else
				g_xMenu.StartGame("Tutorial2");

			break;
		}

		case BB_ACH_WIN_NORMAL:
		{
			m_iSelectedDifficulty = 2;
			for(i = 1; i < BB_MAXLEVELS; i++)
			{
				if(g_xCurrentProfile.m_u32HighScores[m_iSelectedDifficulty][i] <= 0)
				{
					tempint = i;
					break;
				}
			}

			for(m_iTourIndex = 0; tempint > g_iTourLevelCount[m_iTourIndex]; m_iTourIndex++)
				tempint -= g_iTourLevelCount[m_iTourIndex];

			if(m_iTourIndex <= 6)
			{
				if(m_iTourIndex == 0)
					sprintf(g_xMenu.m_strStoredLevel, "Tutorial%i", tempint);
				else
					sprintf(g_xMenu.m_strStoredLevel, "Tour%iBattle%i", m_iTourIndex, tempint);

				StartGame(g_xMenu.m_strStoredLevel);
			}
			else
				UnlockAchievement(achievement);

			break;
		}

		case BB_ACH_DOMINATE_NORMAL:
		{
			m_iSelectedDifficulty = 2;
			for(i = 1; i < BB_MAXLEVELS; i++)
			{
				if(g_xCurrentProfile.m_u32HighScores[m_iSelectedDifficulty][i] < 100)
				{
					tempint = i;
					break;
				}
			}

			for(m_iTourIndex = 0; tempint > g_iTourLevelCount[m_iTourIndex]; m_iTourIndex++)
				tempint -= g_iTourLevelCount[m_iTourIndex];

			if(m_iTourIndex <= 6)
			{
				if(m_iTourIndex == 0)
					sprintf(g_xMenu.m_strStoredLevel, "Tutorial%i", tempint);
				else
					sprintf(g_xMenu.m_strStoredLevel, "Tour%iBattle%i", m_iTourIndex, tempint);

				StartGame(g_xMenu.m_strStoredLevel);
			}
			else
				UnlockAchievement(achievement);

			break;
		}

		case BB_ACH_COMBO_10:
		case BB_ACH_COMBO_100:
		case BB_ACH_COMBO_500:
		case BB_ACH_COMBO_1000:
		{
			xMenuLaunchChallenge(iChallenge, m_strChallengeEnemies[iChallenge], 3 + g_iChallengeSongDifficulties[iChallenge], m_bChallengeLoopSong[iChallenge], 1, 0);
			break;
		}

		case BB_ACH_BLOCKS_200:
		case BB_ACH_BLOCKS_250:
		case BB_ACH_BLOCKS_300:
		{
			xMenuLaunchChallenge(iChallenge, m_strChallengeEnemies[iChallenge], 3 + g_iChallengeSongDifficulties[iChallenge], m_bChallengeLoopSong[iChallenge], 0, 1);
			break;
		}

		case BB_ACH_DOM_1:
		{
			g_xMenu.StartGame("Tour1Battle1");
			break;
		}

		case BB_ACH_DOM_2:
		{
			g_xMenu.StartGame("Tour2Battle1");
			break;
		}

		case BB_ACH_DOM_3:
		{
			g_xMenu.StartGame("Tour3Battle6");
			break;
		}

		case BB_ACH_DOM_4:
		{
			g_xMenu.StartGame("Tour4Battle8");
			break;
		}

		case BB_ACH_DOM_5:
		{
			g_xMenu.StartGame("Tour5Battle8");
			break;
		}

		case BB_ACH_DOM_6:
		{
			g_xMenu.StartGame("Tour6Battle12");
			break;
		}

		default:
		{
			xMenuLaunchChallenge(iChallenge, m_strChallengeEnemies[iChallenge], 3 + g_iChallengeSongDifficulties[iChallenge], m_bChallengeLoopSong[iChallenge], 0, 0);
			break;
		}
	}
}

FORCE_INLINE void xMenuLaunchChallenge(int iChallenge, char *strEnemy, int iNumColors, bool bLoopSong, bool bShowComboCounter, bool bShowClearCounter)
{
	g_xGame.SetupGameOptionsForMultiplayer(strEnemy, g_strTourBackgrounds[rand() % 7], g_strChallengeSongList[iChallenge], (xGameDifficulty)g_iChallengeSongDifficulties[iChallenge]);
	g_xGameOptions.m_iNumColors = iNumColors;
	g_xGameOptions.m_iNumRandomColors = g_xGameOptions.m_iNumColors;
	g_xGameOptions.m_bSkipPrePopup = 1;
	g_xGameOptions.m_bLoopSong = bLoopSong;
	g_xGameOptions.m_bShowComboCounter = bShowComboCounter;
	g_xGameOptions.m_bShowClearCounter = bShowClearCounter;

	g_xGame.m_xGUI.RemoveAllWindows();
	g_pCurGameState = &g_xGame;
	g_xGame.Init("\0");

	g_xMenu.m_xGUI.DelayedRemoveAllWindows();
}

void xMenu::InitializeFreePlayWindow(xGUIWindowList *pWindow)
{
	xFileIO fileio;
	xParser parser;
	char temptext[256], temptext2[256], *tempstring;
	int songindex, i, j;

	m_bChallenges = 0;
	m_iFreePlayScrollAmount = 0;

	if(!m_strFreePlaySongList)
	{
		m_iFreePlaySongCount = 0;

		fileio.StartFindFiles("./music/*", 1);

		while(fileio.GetNextFile(temptext))
			m_iFreePlaySongCount++;

		fileio.EndFindFiles();

		m_strFreePlaySongDirs = new char*[m_iFreePlaySongCount];
		m_strFreePlaySongList = new char*[m_iFreePlaySongCount];
		m_strFreePlayArtistList = new char*[m_iFreePlaySongCount];
		m_iFreePlayDifficulties[0] = new int[m_iFreePlaySongCount];
		m_iFreePlayDifficulties[1] = new int[m_iFreePlaySongCount];
		m_iFreePlayDifficulties[2] = new int[m_iFreePlaySongCount];
		m_strFreePlayDefaultEnemies = new char*[m_iFreePlaySongCount];

		m_iFreePlaySongCount = 0;

		fileio.StartFindFiles("./music/*", 1);

		memset(temptext, 0, 256);
		while(fileio.GetNextFile(temptext))
		{
			m_strFreePlaySongDirs[m_iFreePlaySongCount] = new char[strlen(temptext) + 1];
			strcpy(m_strFreePlaySongDirs[m_iFreePlaySongCount], temptext);

			sprintf(temptext2, "./music/%s/Song.bbm", temptext);
			parser.OpenScript(temptext2);
			parser.ReadLine(temptext, temptext, temptext, temptext);

			tempstring = parser.GetVariable("strName");
			m_strFreePlaySongList[m_iFreePlaySongCount] = new char[strlen(tempstring) + 1];
			strcpy(m_strFreePlaySongList[m_iFreePlaySongCount], tempstring);

			tempstring = parser.GetVariable("strSongArtist");
			m_strFreePlayArtistList[m_iFreePlaySongCount] = new char[strlen(tempstring) + 1];
			strcpy(m_strFreePlayArtistList[m_iFreePlaySongCount], tempstring);

			m_iFreePlayDifficulties[0][m_iFreePlaySongCount] = parser.ToInt("iDifficultyNovice");
			m_iFreePlayDifficulties[1][m_iFreePlaySongCount] = parser.ToInt("iDifficultyAdvanced");
			m_iFreePlayDifficulties[2][m_iFreePlaySongCount] = parser.ToInt("iDifficultyExpert");

			tempstring = parser.GetVariable("strDefaultEnemy");
			if(tempstring)
			{
				m_strFreePlayDefaultEnemies[m_iFreePlaySongCount] = new char[strlen(tempstring) + 1];
				strcpy(m_strFreePlayDefaultEnemies[m_iFreePlaySongCount], tempstring);
			}
			else
				m_strFreePlayDefaultEnemies[m_iFreePlaySongCount] = 0;

			parser.EndScript();
			
			memset(temptext, 0, 256);
			m_iFreePlaySongCount++;
		}

		fileio.EndFindFiles();
	}

	songindex = m_iFreePlayTopSong;

	for(i = 0; i < BB_FREEPLAY_SONGCOUNT; i++)
	{
		sprintf(temptext, BB_NAME_FREEPLAYSONGBG, i + 1);
		m_pFreePlaySongBackgrounds[i] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
		xRenderable2DCopy(&m_xFreePlaySongBGOriginal[i + 1], &m_pFreePlaySongBackgrounds[i]->m_xRenderable);

		sprintf(temptext, BB_NAME_FREEPLAYSONGNAME, i + 1);
		m_pFreePlaySongNames[i] = (xGUIText*)pWindow->m_pData->GetChildByName(temptext);
		xRenderable2DCopy(&m_xFreePlaySongNameOriginal[i + 1], &m_pFreePlaySongNames[i]->m_xRenderable);
		m_xFreePlaySongNameOriginal[i + 1].m_xColor.R = m_pFreePlaySongNames[i]->m_xNormalColor.R;
		m_xFreePlaySongNameOriginal[i + 1].m_xColor.G = m_pFreePlaySongNames[i]->m_xNormalColor.G;
		m_xFreePlaySongNameOriginal[i + 1].m_xColor.B = m_pFreePlaySongNames[i]->m_xNormalColor.B;

		m_pFreePlaySongNames[i]->SetText(m_strFreePlaySongList[songindex]);

		for(j = 0; j < 12; j++)
		{
			sprintf(temptext, BB_NAME_FREEPLAYDIFFSTAR, i + 1, j + 1);
			m_pFreePlaySongDiffStars[i][j] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
			m_pFreePlaySongDiffStars[i][j]->SetHidden(j >= m_iFreePlayDifficulties[m_iFreePlayDifficulty][songindex]);
		}
		
		m_fFreePlaySongDiffOriginalY[i + 1] = m_pFreePlaySongDiffStars[i][0]->m_xRenderable.m_fY;

		songindex++;

		if(songindex == m_iFreePlaySongCount)
			songindex = 0;
	}

	songindex = (m_iFreePlayTopSong + 4) % m_iFreePlaySongCount;

	m_xFreePlaySongBGOriginal[0].m_fX = m_xFreePlaySongBGOriginal[1].m_fX;
	m_xFreePlaySongBGOriginal[0].m_fY = m_xFreePlaySongBGOriginal[1].m_fY - m_xFreePlaySongBGOriginal[1].m_fHeight;
	m_xFreePlaySongBGOriginal[0].m_fWidth = m_xFreePlaySongBGOriginal[1].m_fWidth;
	m_xFreePlaySongBGOriginal[0].m_fHeight = m_xFreePlaySongBGOriginal[1].m_fHeight;
	m_xFreePlaySongBGOriginal[0].m_fScaleX = m_xFreePlaySongBGOriginal[1].m_fScaleX;
	m_xFreePlaySongBGOriginal[0].m_fScaleY = m_xFreePlaySongBGOriginal[1].m_fScaleY;
	m_xFreePlaySongBGOriginal[0].m_xColor.A = 0.0;

	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fX = m_xFreePlaySongBGOriginal[0].m_fX;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fY = m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT].m_fY + m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT].m_fHeight;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fWidth = m_xFreePlaySongBGOriginal[0].m_fWidth;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fHeight = m_xFreePlaySongBGOriginal[0].m_fHeight;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleX = m_xFreePlaySongBGOriginal[0].m_fScaleX;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleY = m_xFreePlaySongBGOriginal[0].m_fScaleY;
	m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_xColor.A = 0.0;

	m_xFreePlaySongNameOriginal[0].m_fX = m_xFreePlaySongNameOriginal[1].m_fX;
	m_xFreePlaySongNameOriginal[0].m_fY = m_xFreePlaySongNameOriginal[1].m_fY - m_xFreePlaySongBGOriginal[1].m_fHeight;
	m_xFreePlaySongNameOriginal[0].m_fScaleX = m_xFreePlaySongNameOriginal[1].m_fScaleX - 0.05;
	m_xFreePlaySongNameOriginal[0].m_fScaleY = m_xFreePlaySongNameOriginal[1].m_fScaleY - 0.05;
	m_xFreePlaySongNameOriginal[0].m_xColor.A = 0.0;

	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fX = m_xFreePlaySongNameOriginal[0].m_fX;
	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fY = m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT].m_fY + m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT].m_fHeight;
	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleX = m_xFreePlaySongNameOriginal[0].m_fScaleX;
	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_fScaleY = m_xFreePlaySongNameOriginal[0].m_fScaleY;
	m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 1].m_xColor.A = 0.0;

	m_fFreePlaySongDiffOriginalY[0] = m_fFreePlaySongDiffOriginalY[1] - m_xFreePlaySongBGOriginal[1].m_fHeight;
	m_fFreePlaySongDiffOriginalY[BB_FREEPLAY_SONGCOUNT + 1] = m_fFreePlaySongDiffOriginalY[BB_FREEPLAY_SONGCOUNT] + m_xFreePlaySongBGOriginal[1].m_fHeight;

	sprintf(temptext, BB_NAME_FREEPLAYSONGBG, 69);
	m_pFreePlaySongBackgrounds[BB_FREEPLAY_SONGCOUNT] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
	sprintf(temptext, BB_NAME_FREEPLAYSONGNAME, 69);
	m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT] = (xGUIText*)pWindow->m_pData->GetChildByName(temptext);

	for(j = 0; j < 12; j++)
	{
		sprintf(temptext, BB_NAME_FREEPLAYDIFFSTAR, 69, j + 1);
		m_pFreePlaySongDiffStars[BB_FREEPLAY_SONGCOUNT][j] = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
	}

	m_pFreePlaySongArtist[0] = (xGUIText*)pWindow->m_pData->GetChildByName(BB_NAME_FREEPLAYARTIST1);
	m_pFreePlaySongArtist[1] = (xGUIText*)pWindow->m_pData->GetChildByName(BB_NAME_FREEPLAYARTIST2);
	m_pFreePlaySongArtist[0]->SetText(m_strFreePlayArtistList[songindex]);
	m_fFreePlaySongArtistOriginalY = m_pFreePlaySongArtist[0]->m_xRenderable.m_fY;
	m_iCurrentArtistIndex = 0;

	if(m_iFreePlayEnemyCount == 0)
	{
		fileio.StartFindFiles("./models/*", 1);

		while(fileio.GetNextFile(temptext))
		{
			if(stricmp(temptext, "Player"))
				m_iFreePlayEnemyCount++;
		}

		fileio.EndFindFiles();

		m_strFreePlayEnemies = new char*[m_iFreePlayEnemyCount];

		m_iFreePlayEnemyCount = 0;

		fileio.StartFindFiles("./models/*", 1);

		memset(temptext, 0, 256);
		while(fileio.GetNextFile(temptext))
		{
			if(stricmp(temptext, "Player"))
			{
				if(m_strFreePlayDefaultEnemies[songindex] && !stricmp(temptext, m_strFreePlayDefaultEnemies[songindex]))
					m_iFreePlayEnemy = m_iFreePlayEnemyCount;

				m_strFreePlayEnemies[m_iFreePlayEnemyCount] = new char[strlen(temptext) + 1];
				strcpy(m_strFreePlayEnemies[m_iFreePlayEnemyCount], temptext);
				m_iFreePlayEnemyCount++;
			}

			memset(temptext, 0, 256);
		}
	}

	m_pFreePlayEnemyModel = (xGUIModel*)pWindow->m_pData->GetChildByName(BB_NAME_FREEPLAYENEMYMODEL);

	if(!m_strFreePlayDefaultEnemies[songindex])
	{
		m_iFreePlayEnemy = 0;
		m_pFreePlayEnemyModel->SetModel(m_strFreePlayEnemies[0]);
	}
	else
		m_pFreePlayEnemyModel->SetModel(m_strFreePlayDefaultEnemies[songindex]);

	pWindow->m_pData->UpdateRenderables();
}

void xMenu::FreePlayScrollSongs(int iAmount)
{
	float oldamount;
	int songindex, i;

	oldamount = m_iFreePlayScrollAmount;

	m_iFreePlayScrollAmount += iAmount;

	if(m_iFreePlayScrollAmount == 0)
		m_iFreePlayScrollAmount += iAmount;

	if(oldamount == 0)
	{
		m_fFreePlayScrollTime = 0.0;
		m_fFreePlayCurrentScrollTime = 0.0;
	}
	else
	{
		m_fFreePlayScrollTime = BB_FREEPLAY_SCROLLTIME * (((m_fFreePlayScrollTime / BB_FREEPLAY_SCROLLTIME) * oldamount) / (float)m_iFreePlayScrollAmount);
		m_fFreePlayCurrentScrollTime = m_fFreePlayCurrentScrollTime * (oldamount / (float)m_iFreePlayScrollAmount);
	}

	m_iFreePlayScrollDirection = m_iFreePlayScrollAmount > 0 ? 1 : -1;

	for(i = 0; i < BB_FREEPLAY_SONGCOUNT; i++)
		xMenuSetFreePlaySongScrollValues(i, i + 1, i + 1 + m_iFreePlayScrollDirection);

	if(m_iFreePlayScrollDirection > 0)
	{
		xMenuSetFreePlaySongScrollValues(BB_FREEPLAY_SONGCOUNT, 0, 1);

		songindex = m_iFreePlayTopSong > 0 ? m_iFreePlayTopSong - 1 : m_iFreePlaySongCount - 1;
	}
	else
	{
		xMenuSetFreePlaySongScrollValues(BB_FREEPLAY_SONGCOUNT, BB_FREEPLAY_SONGCOUNT + 1, BB_FREEPLAY_SONGCOUNT);

		songindex = (m_iFreePlayTopSong + BB_FREEPLAY_SONGCOUNT) % m_iFreePlaySongCount;
	}

	m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT]->SetText(m_strFreePlaySongList[songindex]);

	for(i = 0; i < 12; i++)
		m_pFreePlaySongDiffStars[BB_FREEPLAY_SONGCOUNT][i]->SetHidden(i >= m_iFreePlayDifficulties[m_iFreePlayDifficulty][songindex]);

	m_pFreePlaySongBackgrounds[BB_FREEPLAY_SONGCOUNT]->SetHidden(0);
	m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT]->SetHidden(0);

	m_pFreePlaySongArtist[m_iCurrentArtistIndex]->SetText(m_strFreePlayArtistList[(m_iFreePlayTopSong + 4 - m_iFreePlayScrollDirection) % m_iFreePlaySongCount]);

	m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex] = m_fFreePlaySongArtistOriginalY - m_xFreePlaySongBGOriginal[4].m_fHeight * m_iFreePlayScrollDirection;
	m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex] = m_fFreePlaySongArtistOriginalY;
	DEBUGLOG("%i %f %f\n", m_iCurrentArtistIndex, m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex], m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex]);

	m_iCurrentArtistIndex = 1 ^ m_iCurrentArtistIndex;
	m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex] = m_fFreePlaySongArtistOriginalY;
	m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex] = m_fFreePlaySongArtistOriginalY + m_xFreePlaySongBGOriginal[5].m_fHeight * m_iFreePlayScrollDirection;
	DEBUGLOG("%i %f %f\n", m_iCurrentArtistIndex, m_fFreePlaySongArtistStartY[m_iCurrentArtistIndex], m_fFreePlaySongArtistTargetY[m_iCurrentArtistIndex]);
}

FORCE_INLINE void xMenuSetFreePlaySongScrollValues(int iIndex, int iStartIndex, int iTargetIndex)
{
	xRenderable2DCopy(&g_xMenu.m_xFreePlaySongBGStart[iIndex], &g_xMenu.m_xFreePlaySongBGOriginal[iStartIndex]);
	xRenderable2DCopy(&g_xMenu.m_xFreePlaySongBGTarget[iIndex], &g_xMenu.m_xFreePlaySongBGOriginal[iTargetIndex]);

	xRenderable2DCopy(&g_xMenu.m_xFreePlaySongNameStart[iIndex], &g_xMenu.m_xFreePlaySongNameOriginal[iStartIndex]);
	xRenderable2DCopy(&g_xMenu.m_xFreePlaySongNameTarget[iIndex], &g_xMenu.m_xFreePlaySongNameOriginal[iTargetIndex]);

	g_xMenu.m_fFreePlaySongDiffStartY[iIndex] = g_xMenu.m_fFreePlaySongDiffOriginalY[iStartIndex];
	g_xMenu.m_fFreePlaySongDiffTargetY[iIndex] = g_xMenu.m_fFreePlaySongDiffOriginalY[iTargetIndex];
}

FORCE_INLINE void xMenuUpdateFreePlayRenderable(xGUIObject *pObject, xRenderable2D *pStart, xRenderable2D *pTarget, float fProgress)
{
	pObject->SetX(pStart->m_fX + (pTarget->m_fX - pStart->m_fX) * fProgress);
	pObject->SetY(pStart->m_fY + (pTarget->m_fY - pStart->m_fY) * fProgress);
	pObject->SetWidth(pStart->m_fWidth + (pTarget->m_fWidth - pStart->m_fWidth) * fProgress);
	pObject->SetHeight(pStart->m_fHeight + (pTarget->m_fHeight - pStart->m_fHeight) * fProgress);
	pObject->SetScale(pStart->m_fScaleX + (pTarget->m_fScaleX - pStart->m_fScaleX) * fProgress, pStart->m_fScaleY + (pTarget->m_fScaleY - pStart->m_fScaleY) * fProgress);
	pObject->SetAlpha(pStart->m_xColor.A + (pTarget->m_xColor.A - pStart->m_xColor.A) * fProgress);
	pObject->UpdateRenderables();
}

void xMenu::InitializeOptionsWindow(xGUIWindowList *pWindow)
{
	xGUIComboBox *comboboxobject;
	xGUIImage *imageobject;
	int screenwidth, screenheight, i;
	char temptext[20], temptext2[16];

	temptext2[1] = '\0';

	for(i = 0, temptext2[0] = '1'; i < 3; i++, temptext2[0]++)
	{
		sprintf(temptext, BB_NAME_MENUOPTIONBG, temptext2);
		pWindow->m_pData->GetChildByName(temptext)->SetHidden(i != 0);

		sprintf(temptext, BB_NAME_MENUOPTIONTEXT, temptext2);
		imageobject = (xGUIImage*)pWindow->m_pData->GetChildByName(temptext);
		imageobject->SetAlpha(i == 0 ? 2.0 : 0.75);
		imageobject->UpdateRenderables();

		sprintf(temptext, BB_NAME_OPTIONSPANEL, i + 1);
		pWindow->m_pData->GetChildByName(temptext)->SetHidden(i != 0);
	}

	comboboxobject = (xGUIComboBox*)pWindow->m_pData->GetChildByName(BB_NAME_RESOLUTIONCOMBO);
	
	xGraphicsStartGettingAllDesktopResolutions();

	while(xGraphicsGetNextDesktopResolution(&screenwidth, &screenheight))
	{
		sprintf(temptext, "%ix%i", screenwidth, screenheight);
		comboboxobject->AddItem(temptext);
	}

	sprintf(temptext, "%ix%i", g_xPaperAPI.m_xEngineSettings.m_iScreenWidth, g_xPaperAPI.m_xEngineSettings.m_iScreenHeight);
	comboboxobject->SetSelectedByString(temptext);

	((xGUICheckBox*)pWindow->m_pData->GetChildByName(BB_NAME_FULLSCREENCHECK))->SetValue(g_xPaperAPI.m_xEngineSettings.m_bFullScreen);

	((xGUISlider*)pWindow->m_pData->GetChildByName(BB_NAME_MUSICVOLUMESLIDER))->SetValue(g_xOptions.m_fMusicVolume * 0.88f);
	((xGUISlider*)pWindow->m_pData->GetChildByName(BB_NAME_SOUNDVOLUMESLIDER))->SetValue(g_xOptions.m_fSoundVolume * 0.88f);
	((xGUISlider*)pWindow->m_pData->GetChildByName(BB_NAME_VOICEOVERVOLUMESLIDER))->SetValue(g_xOptions.m_fVoiceOverVolume * 0.88f);

	((xGUICheckBox*)pWindow->m_pData->GetChildByName(BB_NAME_CHARACTEROUTLINECHECK))->SetValue(g_xPaperAPI.m_bOutlineModels);
	((xGUICheckBox*)pWindow->m_pData->GetChildByName(BB_NAME_REDUCEDPARTICLESCHECK))->SetValue(g_xPaperAPI.m_bLimitParticles);

	for(i = 0; i < MAX_GAME_BUTTONS; i++)
	{
		sprintf(temptext, BB_NAME_CONTROLSBUTTON, i + 1);
		((xGUIButton*)pWindow->m_pData->GetChildByName(temptext))->SetText(g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[i]));

		sprintf(temptext, BB_NAME_CONTROLSALTBUTTON, i + 1);
		((xGUIButton*)pWindow->m_pData->GetChildByName(temptext))->SetText(g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[i + MAX_GAME_BUTTONS]));
	}

	pWindow->m_pData->UpdateRenderables();
}

void xMenu::Login(char *strUser, char *strPass, bool bPassEncrypted, xGUIWindowList *pWindow)
{
	xGUIWindowList *window;

	strcpy(g_xGame.m_xPlayer.m_strAccountName, strUser);

	int length = strlen(g_xGame.m_xPlayer.m_strAccountName);
	for(int i = 0; i < length; i++)
	{
		if(g_xGame.m_xPlayer.m_strAccountName[i] < '0' || (g_xGame.m_xPlayer.m_strAccountName[i] > '9' && g_xGame.m_xPlayer.m_strAccountName[i] < 'A') ||
		  (g_xGame.m_xPlayer.m_strAccountName[i] > 'Z' && g_xGame.m_xPlayer.m_strAccountName[i] < 'a') || g_xGame.m_xPlayer.m_strAccountName[i] > 'z')
		{
			if(AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX))
			{
				window = m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);
				((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDUSERNAME);
				((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDUSERNAME2);
			}

			return;
		}
	}

	if(strlen(g_xGame.m_xPlayer.m_strAccountName) < 4 || strlen(g_xGame.m_xPlayer.m_strAccountName) > 20)
	{
		if(AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX))
		{
			window = m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);
			((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDUSERNAME);
			((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDUSERNAME2);
		}

		return;
	}

	if(!stricmp(strPass, BB_STRING_USESTOREDPASSWORD))
	{
		strPass = g_xCurrentProfile.m_strLoginPassword;
		bPassEncrypted = 1;
	}
	else if(strlen(strPass) < 8 || strlen(strPass) > 32)
	{
		if(AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX))
		{
			window = m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);
			((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(BB_STRING_INVALIDPASSWORD);
			((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_INVALIDPASSWORD2);
		}

		return;
	}

	m_xHTTPQuery.Login(g_xGame.m_xPlayer.m_strAccountName, strPass, bPassEncrypted);

	if(m_xSinglePlayerLogin)
	{
		strcpy(g_xCurrentProfile.m_strLoginName, g_xGame.m_xPlayer.m_strAccountName);

		if(!bPassEncrypted)
			m_xHTTPQuery.GenerateMD5(strPass, g_xCurrentProfile.m_strLoginPassword);
		else
			strcpy(g_xCurrentProfile.m_strLoginPassword, strPass);
	}

	if(!pWindow && AddMenuWindow(BB_WINDOW_MENU_LOGIN))
		pWindow = m_xGUI.GetWindow(BB_WINDOW_MENU_LOGIN);

	if(pWindow)
	{
		pWindow->m_pData->SetHidden(1);
		pWindow->m_pData->UpdateRenderables();
	}

	//AddMenuWindow(BB_WINDOW_MENU_LOGGINGIN);
}

void xMenu::InitializeHostServerWindow(xGUIWindowList *pWindow)
{
	char curfilename[256] = {0};
	xFileIO fileio;

	fileio.StartFindFiles("./music/*", 1);

	while(fileio.GetNextFile(curfilename))
	{
		m_pSongListBox->AddItem(curfilename);
		memset(curfilename, 0, 255);
	}

	fileio.EndFindFiles();

	m_iSongDifficulty = 1;

	xGUIButton* tempbutton;
	tempbutton = (xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0);
	tempbutton->m_bToggled = 1;
	tempbutton = (xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1);
	tempbutton->m_bToggled = 0;
	tempbutton = (xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY2);
	tempbutton->m_bToggled = 0;
}

void xMenu::RefreshServerList()
{
	m_iLastServerPinged = 0;
	m_bPingingServers = 1;

	if(m_pServerListBox->m_iNumItems > 0)
		m_pServerListBox->RemoveAllItems();

	if(m_pPingInfo)
	{
		delete [] m_pPingInfo;
		m_pPingInfo = NULL;
	}

	m_pRefreshButton->SetText(BB_STRING_STOP);

	m_xHTTPQuery.GetServerList(m_pServerListBox);
}

void xMenu::OnFailedToInitListenServer(int iPort)
{
	AddMenuWindow(BB_WINDOW_MENU_MESSAGEBOX);

	xGUIWindowList *window = m_xGUI.GetWindow(BB_WINDOW_MENU_MESSAGEBOX);

	if(window)
	{
		char temp[256] = {0};
		sprintf(temp, BB_STRING_FAILEDTOINITSERVER1, iPort);
		((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE1))->SetText(temp);
		sprintf(temp, BB_STRING_FAILEDTOINITSERVER2);
		((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(temp);
	}
}

void xMenu::InitPingInfo()
{
	m_pPingInfo = new xPingInfo[m_pServerListBox->m_iNumItems];

	for(int i = 0; i < m_pServerListBox->m_iNumItems; i++)
	{
		m_pPingInfo[i].m_iRequestCount = 1;
		m_pPingInfo[i].m_dwPingTotal = atoi(m_pServerListBox->GetItemValue(5, i));

		if(m_pPingInfo[i].m_dwPingTotal == 999)
		{
			m_pPingInfo[i].m_dwPingTotal = 0;
			m_pPingInfo[i].m_iResponseCount = 0;
		}
		else
			m_pPingInfo[i].m_iResponseCount = 1;
	}

	m_iLastServerPinged = 0;
}

void xMenu::PingServer(char* strAddress, int iPort, int iListNumber)
{
	if(m_pPingTimes == NULL)
		m_pPingTimes = new DWORD[m_iPingTimesLength];
	else if(iListNumber >= m_iPingTimesLength)
	{
		DWORD *temparray = m_pPingTimes;
		m_pPingTimes = new DWORD[m_iPingTimesLength * 2];
		memcpy(m_pPingTimes, temparray, m_iPingTimesLength * sizeof(DWORD));
		m_iPingTimesLength *= 2;
		delete [] temparray;
	}

	g_xPaperAPI.m_xPingNetwork.Connect(strAddress, iPort);
	g_xPaperAPI.m_xPingNetwork.Send((char*)&iListNumber, 4);

	m_pPingTimes[iListNumber] = g_xPaperAPI.m_xPlatform.GetTime();
}

void xMenu::RepingServers()
{
	char temp[32];
	int stringlength;
	int port = 0;

	m_bPingingServers = 1;

	if(g_xPaperAPI.m_xPlatform.GetTime() - m_pPingTimes[m_iLastServerPinged] < 1000)
		return;

	for(int i = 0; i < 5 && i < m_pServerListBox->m_iNumItems; i++)
	{
		strcpy(temp, m_pServerListBox->GetData(m_iLastServerPinged));

		stringlength = strlen(temp);

		for(int j = 0; j < stringlength; j++)
		{
			if(temp[j] == ':')
			{
				temp[j] = '\0';
				port = atoi(&temp[j + 1]);
				break;
			}
		}

		if(port > 0)
		{
			PingServer(temp, port + 1, m_iLastServerPinged);
			port = 0;
		}

		m_pPingInfo[m_iLastServerPinged].m_iRequestCount++;

		m_iLastServerPinged++;
		
		if(m_iLastServerPinged >= m_pServerListBox->m_iNumItems)
		{
			m_iLastServerPinged = 0;
			break;
		}
	}
}

void xMenu::HandlePingPackets()
{
	char temp[MAX_NETWORK_BUFFERSIZE + 1] = {0};
	DWORD timereceived;
	int listindex;

	while(g_xPaperAPI.m_xPingNetwork.Receive(temp, timereceived))
	{
		listindex = *((int*)temp);

		if(temp[4] == 'c')
			m_pServerListBox->RemoveItem(listindex);
		else if(temp[4] == 's')
		{
			if(m_pPingInfo)
			{
				m_pPingInfo[listindex].m_iResponseCount++;
				m_pPingInfo[listindex].m_dwPingTotal += timereceived - m_pPingTimes[listindex];

				int ping = m_pPingInfo[listindex].m_dwPingTotal / m_pPingInfo[listindex].m_iResponseCount;

				itoa(ping, temp, 10);
			}
			else
				itoa(timereceived - m_pPingTimes[listindex], temp, 10);

			m_pServerListBox->SetItemValue(5, listindex, temp);
		}

		memset(temp, 0, MAX_NETWORK_PACKETSIZE);
	}
}

void xMenu::ClearUnresponsiveServers()
{
	int removaloffset = 0;
	int temp = m_pServerListBox->m_iNumItems;

	for(int i = 0; i < temp; i++)
	{
		m_pPingInfo[i].m_iRequestCount++;

		if(m_pPingInfo[i].m_iResponseCount == 0)
		{
			m_pServerListBox->RemoveItem(i - removaloffset);
			removaloffset++;
		}
	}
}

void xMenu::LoadAndValidateCDKey()
{
	char temp[6][100];
	uint8_t tempbytes[8], readbytes[8], extrachar;
	uint32_t privatekey[2], publickey[4];
	xFileIO file;

	file.OpenFile(xDecryptString(g_pEncryptedCDKeyFile, BB_ENCRYPTED_LENGTH_CDKEYFILE, temp[0], PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3), true, true);

	file.ReadData(&readbytes[0], sizeof(uint8_t));
	file.ReadData(&publickey[0], sizeof(uint32_t));
	file.ReadData(&readbytes[1], sizeof(uint8_t));
	file.ReadData(&privatekey[0], sizeof(uint32_t));
	file.ReadData(&readbytes[2], sizeof(uint8_t));
	file.ReadData(&publickey[1], sizeof(uint32_t));
	file.ReadData(&readbytes[3], sizeof(uint8_t));
	file.ReadData(&publickey[2], sizeof(uint32_t));
	file.ReadData(&readbytes[4], sizeof(uint8_t));
	file.ReadData(&privatekey[1], sizeof(uint32_t));
	file.ReadData(&readbytes[5], sizeof(uint8_t));
	file.ReadData(&publickey[3], sizeof(uint32_t));
	file.ReadData(&readbytes[6], sizeof(uint8_t));
	file.ReadData(&extrachar, sizeof(uint8_t));
	file.ReadData(&readbytes[7], sizeof(uint8_t));

	memcpy(tempbytes, readbytes, 8);

	g_xPaperAPI.m_xNetwork.GetMacAddress(tempbytes);

	if(readbytes[0] != tempbytes[0] || readbytes[1] != tempbytes[1] || readbytes[2] != tempbytes[2] || readbytes[3] != tempbytes[3] ||
	   readbytes[4] != tempbytes[4] || readbytes[5] != tempbytes[5] || readbytes[6] != tempbytes[6] || readbytes[7] != tempbytes[7])
	{
		g_xGame.m_xPlayer.m_strCDKey[0] = '+';
		return;
	}

	xToBase62(publickey[0], temp[0]);
	xToBase62(publickey[1], temp[1]);
	xToBase62(publickey[2], temp[2]);
	xToBase62(publickey[3], temp[3]);
	xToBase62(privatekey[0], temp[4]);
	xToBase62(privatekey[1], temp[5]);

	g_xGame.m_xPlayer.m_strCDKey[0] = temp[4][0];
	g_xGame.m_xPlayer.m_strCDKey[1] = temp[0][3];
	g_xGame.m_xPlayer.m_strCDKey[2] = temp[5][0];
	g_xGame.m_xPlayer.m_strCDKey[3] = temp[3][3];
	g_xGame.m_xPlayer.m_strCDKey[4] = temp[4][1];
	g_xGame.m_xPlayer.m_strCDKey[5] = '-';
	g_xGame.m_xPlayer.m_strCDKey[6] = temp[2][3];
	g_xGame.m_xPlayer.m_strCDKey[7] = temp[5][1];
	g_xGame.m_xPlayer.m_strCDKey[8] = temp[1][3];
	g_xGame.m_xPlayer.m_strCDKey[9] = temp[4][2];
	g_xGame.m_xPlayer.m_strCDKey[10] = temp[0][4];
	g_xGame.m_xPlayer.m_strCDKey[11] = '-';
	g_xGame.m_xPlayer.m_strCDKey[12] = temp[5][2];
	g_xGame.m_xPlayer.m_strCDKey[13] = temp[3][4];
	g_xGame.m_xPlayer.m_strCDKey[14] = extrachar;
	g_xGame.m_xPlayer.m_strCDKey[15] = temp[4][3];
	g_xGame.m_xPlayer.m_strCDKey[16] = temp[2][4];
	g_xGame.m_xPlayer.m_strCDKey[17] = '-';
	g_xGame.m_xPlayer.m_strCDKey[18] = temp[5][3];
	g_xGame.m_xPlayer.m_strCDKey[19] = temp[1][4];
	g_xGame.m_xPlayer.m_strCDKey[20] = temp[4][4];
	g_xGame.m_xPlayer.m_strCDKey[21] = temp[0][5];
	g_xGame.m_xPlayer.m_strCDKey[22] = temp[5][4];
	g_xGame.m_xPlayer.m_strCDKey[23] = '-';
	g_xGame.m_xPlayer.m_strCDKey[24] = temp[3][5];
	g_xGame.m_xPlayer.m_strCDKey[25] = temp[4][5];
	g_xGame.m_xPlayer.m_strCDKey[26] = temp[2][5];
	g_xGame.m_xPlayer.m_strCDKey[27] = temp[5][5];
	g_xGame.m_xPlayer.m_strCDKey[28] = temp[1][5];
	g_xGame.m_xPlayer.m_strCDKey[29] = '\0';

	g_xPaperAPI.ValidateCDKey(g_xGame.m_xPlayer.m_strCDKey);

	file.CloseFile();
}

void xMenu::StoreCDKey()
{
	char temp[6][100];
	uint8_t tempbytes[8], extrachar;
	uint32_t privatekey[2], publickey[4];
	xFileIO file;

	file.OpenFile(xDecryptString(g_pEncryptedCDKeyFile, BB_ENCRYPTED_LENGTH_CDKEYFILE, temp[0], PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3), false, true);

	memset(temp[0], 'a', 6); temp[0][6] = '\0';
	memset(temp[1], 'a', 6); temp[1][6] = '\0';
	memset(temp[2], 'a', 6); temp[2][6] = '\0';
	memset(temp[3], 'a', 6); temp[3][6] = '\0';
	memset(temp[4], 'a', 6); temp[4][6] = '\0';
	memset(temp[5], 'a', 6); temp[5][6] = '\0';

	temp[4][0] = g_xGame.m_xPlayer.m_strCDKey[0];
	temp[0][3] = g_xGame.m_xPlayer.m_strCDKey[1];
	temp[5][0] = g_xGame.m_xPlayer.m_strCDKey[2];
	temp[3][3] = g_xGame.m_xPlayer.m_strCDKey[3];
	temp[4][1] = g_xGame.m_xPlayer.m_strCDKey[4];
	temp[2][3] = g_xGame.m_xPlayer.m_strCDKey[6];
	temp[5][1] = g_xGame.m_xPlayer.m_strCDKey[7];
	temp[1][3] = g_xGame.m_xPlayer.m_strCDKey[8];
	temp[4][2] = g_xGame.m_xPlayer.m_strCDKey[9];
	temp[0][4] = g_xGame.m_xPlayer.m_strCDKey[10];
	temp[5][2] = g_xGame.m_xPlayer.m_strCDKey[12];
	temp[3][4] = g_xGame.m_xPlayer.m_strCDKey[13];
	extrachar = g_xGame.m_xPlayer.m_strCDKey[14];
	temp[4][3] = g_xGame.m_xPlayer.m_strCDKey[15];
	temp[2][4] = g_xGame.m_xPlayer.m_strCDKey[16];
	temp[5][3] = g_xGame.m_xPlayer.m_strCDKey[18];
	temp[1][4] = g_xGame.m_xPlayer.m_strCDKey[19];
	temp[4][4] = g_xGame.m_xPlayer.m_strCDKey[20];
	temp[0][5] = g_xGame.m_xPlayer.m_strCDKey[21];
	temp[5][4] = g_xGame.m_xPlayer.m_strCDKey[22];
	temp[3][5] = g_xGame.m_xPlayer.m_strCDKey[24];
	temp[4][5] = g_xGame.m_xPlayer.m_strCDKey[25];
	temp[2][5] = g_xGame.m_xPlayer.m_strCDKey[26];
	temp[5][5] = g_xGame.m_xPlayer.m_strCDKey[27];
	temp[1][5] = g_xGame.m_xPlayer.m_strCDKey[28];

	publickey[0] = xFromBase62(temp[0]);
	publickey[1] = xFromBase62(temp[1]);
	publickey[2] = xFromBase62(temp[2]);
	publickey[3] = xFromBase62(temp[3]);
	privatekey[0] = xFromBase62(temp[4]);
	privatekey[1] = xFromBase62(temp[5]);

	g_xPaperAPI.m_xNetwork.GetMacAddress(tempbytes);

	file.WriteData(&tempbytes[0], sizeof(uint8_t));
	file.WriteData(&publickey[0], sizeof(uint32_t));
	file.WriteData(&tempbytes[1], sizeof(uint8_t));
	file.WriteData(&privatekey[0], sizeof(uint32_t));
	file.WriteData(&tempbytes[2], sizeof(uint8_t));
	file.WriteData(&publickey[1], sizeof(uint32_t));
	file.WriteData(&tempbytes[3], sizeof(uint8_t));
	file.WriteData(&publickey[2], sizeof(uint32_t));
	file.WriteData(&tempbytes[4], sizeof(uint8_t));
	file.WriteData(&privatekey[1], sizeof(uint32_t));
	file.WriteData(&tempbytes[5], sizeof(uint8_t));
	file.WriteData(&publickey[3], sizeof(uint32_t));
	file.WriteData(&tempbytes[6], sizeof(uint8_t));
	file.WriteData(&extrachar, sizeof(uint8_t));
	file.WriteData(&tempbytes[7], sizeof(uint8_t));

	g_xPaperAPI.ValidateCDKey(g_xGame.m_xPlayer.m_strCDKey);

	file.CloseFile();
}

void xMenu::StartUpdate()
{
	char temp[256];

	m_bUpdatingGame = true;

	m_iUpdateFileListIndex = 0;
	m_iUpdateFileStartIndex = 0;
	m_iUpdateHashStartIndex = -1;
	m_iOutOfDateFiles = 0;

	m_xUpdateBatchFile.OpenFile(BB_FILE_UPDATEBATCH, false, false);

	// Write out the Ping Delay trick first:
	m_xUpdateBatchFile.WriteString("@echo off\n");
	m_xUpdateBatchFile.WriteString("ping 1.1.1.1 -n 1 -w 3000 >NUL\n");

	if(FileHashMismatch(BB_FILE_DOWNLOADEXE, m_xHTTPQuery.m_strDownloaderHash))
	{
		sprintf(temp, "move /Y %s Temp.exe\n", BB_FILE_DOWNLOADEXE);
		m_xUpdateBatchFile.WriteString(temp);

		m_xUpdateBatchFile.WriteString("ping 1.1.1.1 -n 1 -w 3000 >NUL\n");

		sprintf(temp, "Temp.exe %s %s %s\n", m_xHTTPQuery.m_strDownloadLocation, m_xHTTPQuery.m_strDownloadHost, BB_FILE_DOWNLOADEXE);
		m_xUpdateBatchFile.WriteString(temp);

		m_xUpdateBatchFile.WriteString("ping 1.1.1.1 -n 1 -w 3000 >NUL\n");

		m_iOutOfDateFiles++;
	}
}

void xMenu::ContinueUpdate()
{
	char temp[256];
	int len, offset, i;

	for(m_iUpdateFileListIndex = 0; m_iUpdateFileListIndex < m_xHTTPQuery.m_iFileListOffset; m_iUpdateFileListIndex++)
	{
		// If we haven't found the beginning of the hash code
		if(m_iUpdateHashStartIndex < m_iUpdateFileStartIndex)
		{
			// A space marks the beginning of the has code
			if(m_xHTTPQuery.m_strFileList[m_iUpdateFileListIndex] == ' ')
			{
				m_iUpdateHashStartIndex = m_iUpdateFileListIndex + 1;
				m_xHTTPQuery.m_strFileList[m_iUpdateFileListIndex] = '\0';
			}
		}
		// If we've found the end of the hash code
		else if(m_xHTTPQuery.m_strFileList[m_iUpdateFileListIndex] == '\n')
		{
			// Update the UI Window
			sprintf(temp, BB_STRING_UPDATECHECK, m_xHTTPQuery.m_strFileList + m_iUpdateFileStartIndex);
			m_pUpdateWindowStatusText->SetText(temp);

			// Replace the new line character with an end of string marker
			m_xHTTPQuery.m_strFileList[m_iUpdateFileListIndex] = '\0';

			// If this file doesn't exist or it doesn't match the has code
			if(FileHashMismatch(m_xHTTPQuery.m_strFileList + m_iUpdateFileStartIndex, m_xHTTPQuery.m_strFileList + m_iUpdateHashStartIndex))
			{
				sprintf(temp, "echo Downloading %s\n", m_xHTTPQuery.m_strFileList + m_iUpdateFileStartIndex);
				m_xUpdateBatchFile.WriteString(temp);
				sprintf(temp, "%s %s %s %s\n", BB_FILE_DOWNLOADEXE, m_xHTTPQuery.m_strDownloadLocation, m_xHTTPQuery.m_strDownloadHost, m_xHTTPQuery.m_strFileList + m_iUpdateFileStartIndex);

				// Strip out any extraneous new line characters
				offset = 0;
				len = strlen(temp) - 1;
				for(i = 0; i < len; i++)
				{
					if(temp[i] == '\n')
						offset++;

					temp[i - offset] = temp[i];
				}

				m_xUpdateBatchFile.WriteString(temp);

				m_iOutOfDateFiles++;
			}

			// Mark the start of the next file as 1 beyond this one
			m_iUpdateFileStartIndex = m_iUpdateFileListIndex + 1;

			// Make sure there aren't any extraneous characters in our way
			while(m_xHTTPQuery.m_strFileList[m_iUpdateFileStartIndex] == '\r' || m_xHTTPQuery.m_strFileList[m_iUpdateFileStartIndex] == '\n' ||
				  m_xHTTPQuery.m_strFileList[m_iUpdateFileStartIndex] == ' ' || m_xHTTPQuery.m_strFileList[m_iUpdateFileStartIndex] == '\t')
			{
				m_iUpdateFileStartIndex++;
			}

			return;
		}
	}

	// Once the update is done, launch the game
	m_xUpdateBatchFile.WriteString("Launching Game\n");
	m_xUpdateBatchFile.WriteString("start BreakBlocks.exe");

	m_xUpdateBatchFile.CloseFile();

	// This is only really used in the Verify Files menu, because the Update window is hidden unless there are new files
	if(m_iOutOfDateFiles)
	{
		g_xPaperAPI.m_bRunning = false;
		
		sprintf(g_xPaperAPI.m_strExitCommand, "open %s", BB_FILE_UPDATEBATCH);
	}
	else
		m_pUpdateWindowStatusText->SetText(BB_STRING_FILESUPTODATE);
}

FORCE_INLINE void xToBase62(uint64_t u64Value, char *strValue)
{
	int i;
	uint64_t placevalue = 13537086546263552LLU, charvalue;

	for(i = 0; i < 9; i++, placevalue /= 62LLU)
	{
		if(placevalue <= u64Value)
		{
			charvalue = u64Value / placevalue;
			u64Value -= placevalue * charvalue;

			if(charvalue + 1 < 27)
				strValue[i] = 97 + charvalue;
			else if(charvalue + 1 < 53)
				strValue[i] = 39 + charvalue; //65 + charvalue - 26
			else
				strValue[i] = charvalue - 4; //48 + charvalue - 52
		}
		else
			strValue[i] = 97;
	}
}

bool xMenu::FileHashMismatch(char *strFile, char *strHash)
{
	xFileIO fileio;
	char MD5[11] = {0}, buffer[1024];
	uint64_t total = 283929936586834022LLU; //(62 ^ 10) / 10
	int read, i;

	fileio.OpenFile(strFile, true, true);

	read = 1;
	while(read > 0)
	{
		fileio.ReadData(buffer, 1024, &read);

		for(i = 0; i < read; i++)
			total = (total + (((uint64_t)i)<<24) * (uint64_t)buffer[i]) % 2839299365868340224LLU; //(62 ^ 10)
	}

	fileio.CloseFile();

	xToBase62(total, MD5);

	for(i = 0; i < 10 && strHash[i] != '\0'; i++)
	{
		if(MD5[i] != strHash[i])
			return true;
	}

	return false;
}

void xMenu::UpdateCredits(double dTime)
{
	int i;

	for(i = 0; i < g_iMaxCreditLines; i++)
	{
		g_pCreditTextArray[i]->SetY(g_pCreditTextArray[i]->GetY() - dTime * 50.0);
		g_pCreditTextArray[i]->UpdateRenderables();
	}

	if(g_pCreditTextArray[g_iCurTextLine]->GetY() + g_pCreditTextArray[g_iCurTextLine]->GetHeight() < 0)
	{
		if(g_iCurTextLine > 0)
			g_pCreditTextArray[g_iCurTextLine]->SetY(g_pCreditTextArray[g_iCurTextLine - 1]->GetY() + (g_pCreditTextArray[(g_iCurTextLine + 1) % g_iMaxCreditLines]->GetY() - g_pCreditTextArray[g_iCurTextLine]->GetY()));
		else
			g_pCreditTextArray[g_iCurTextLine]->SetY(g_pCreditTextArray[g_iMaxCreditLines - 1]->GetY() + (g_pCreditTextArray[(g_iCurTextLine + 1) % g_iMaxCreditLines]->GetY() - g_pCreditTextArray[g_iCurTextLine]->GetY()));

		if(g_iCurCreditLine < CREDIT_LINES)
			g_pCreditTextArray[g_iCurTextLine]->SetText(g_strCredits[g_iCurCreditLine]);
		else
			g_pCreditTextArray[g_iCurTextLine]->SetText("");

		g_iCurTextLine = (g_iCurTextLine + 1) % g_iMaxCreditLines;
		g_iCurCreditLine++;
	}
}

void xMenu::UnlockAchievement(int iAchievementID)
{
	bool wasunlocked;
	int i;

	if(g_xCurrentProfile.m_byteAchievementStatus[iAchievementID - 1])
		return;

	wasunlocked = g_xGame.m_iAchievementStatus[iAchievementID - 1] != 0;

	g_xCurrentProfile.m_byteAchievementStatus[iAchievementID - 1] = 1;
	g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

	g_xGame.m_iAchievementStatus[iAchievementID - 1] = 1;

	for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
	{
		if(!m_strAchievementsPending[i])
		{
			m_strAchievementsPending[i] = g_strAchievements[iAchievementID - 1];
			break;
		}
	}

	if(i < BB_PENDINGACHIEVEMENTS_MAX)
	{
		for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
		{
			if(!m_iAchievementsUnlocking[i])
			{
				m_iAchievementsUnlocking[i] = iAchievementID;
				m_dwAchievementUnlockStart[i] = g_xPaperAPI.m_dwCurrentTime;
				xRenderable2DQuadUpdateTexture(m_pAchievementIcon[i], m_pAchievementIconTextures[iAchievementID - 1]);
				break;
			}
		}
	}

	if(!g_xCurrentProfile.m_strLoginName[0] || !g_xCurrentProfile.m_strLoginPassword[0] || wasunlocked)
		return;

	if(m_xHTTPQuery.GetQueryType() == BB_HTTPQUERY_NONE && m_strAchievementsPending[0])
		m_xHTTPQuery.UnlockAchievement(m_strAchievementsPending[0], g_xCurrentProfile.m_strLoginName, g_xCurrentProfile.m_strLoginPassword);
}
