/////////////////////////////////////////////////////////////////
// Name: xNetworkClient.h
/////////////////////////////////////////////////////////////////
// Description:
// - An underdeveloped direct interface to Networking
/////////////////////////////////////////////////////////////////
// - 29.11.2004, Update, Arto Ruotsalainen (Arex)
//   File created and classes added.
//
// - 7.6.2004, Update, Arto Ruotsalainen(Arex)
//   Comment stuff added to begin of the file.
//   Add new updates/fixes above this, so the old updates
//   will be at the bottom.
/////////////////////////////////////////////////////////////////
#ifndef XNETWORKCLIENT_H_
#define XNETWORKCLIENT_H_

#include <sys/types.h>
#include <sys/errno.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "xNetworkInfo.h"
#include "xResults.h"
#include "xList.h"

#define SOCKET_ERROR	-1
#define INVALID_SOCKET	-1

#define WM_USER					914728
#define WM_PE_WSASYNC			(WM_USER + 5)
#define WM_PE_HTTPASYNC			(WM_USER + 6)
#define WM_PE_PINGASYNC			(WM_USER + 7)
#define WM_PE_FTPASYNC			(WM_USER + 8)
#define WM_PE_FTPTRANSFERASYNC	(WM_USER + 9)

struct xUDPPacketWinsock
{
	xUDPPacketWinsock	*m_pNext;
	char				 m_strData[MAX_NETWORK_PACKETSIZE];
	sockaddr_in			 m_sockAddrIn;
	DWORD				 m_dwTimeReceived;
};

class xNetworkClient : public xNetworkInfo
{
public:
	xNetworkClient()
	{
		m_bConnected = false;
		m_iReadBufferOffset = 0;
		m_iWriteBufferOffset = 0;

		m_sockConnection		= INVALID_SOCKET;
		m_sockListen			= INVALID_SOCKET;
		m_pNext					= NULL;
		m_pPrev					= NULL;
		m_bUDPClientBound		= false;
		m_xUDPPackets.m_pNext	= NULL;

		memset(m_strSendBuffer, 0, MAX_NETWORK_BUFFERSIZE);
		memset(m_strReceiveBuffer, 0, MAX_NETWORK_BUFFERSIZE);
	}

	int  Init(bool bTCP, int iWinProcMessage);
	int  Listen(int iPort);
	int  Connect(const char* strIP, int iPort);
	int  Accept();
	int  Read();
	int  Update();
	void Close();
	void Shutdown();

	int	  Send(const char* strData, int iSize);
	int	  SendResponse(const char* strData, int iSize);
	char* Receive(char strBuffer[MAX_NETWORK_BUFFERSIZE + 1], DWORD &dwTimeReceived);

	bool MessageReceived() { return m_strReceiveBuffer[0] != 0; }
	bool IsConnected() { return m_bConnected; }
	void SetConnected(bool bValue) { m_bConnected = bValue; }

	void GetMacAddress(uint8_t *pMAC);

	int  m_iReadBufferOffset;
	int  m_iWriteBufferOffset;

	int		 m_sockListen;
	fd_set	 m_setListen;
	int		 m_sockConnection;
	fd_set	 m_setConnection;
	sockaddr m_sockAddress;

	xNetworkClient *m_pNext;
	xNetworkClient *m_pPrev;

protected:
	bool m_bConnected;
	int  m_iWinProcMessage;
	bool m_bTCP;
	bool m_bUDPClientBound;

	sockaddr_in m_sockAddrIn;

	xUDPPacketWinsock	m_xUDPPackets;
	sockaddr_in			m_xRespondAddress;

	char m_strSendBuffer[MAX_NETWORK_BUFFERSIZE];
	char m_strReceiveBuffer[MAX_NETWORK_BUFFERSIZE];
};

#endif