#include "Paper.h"

int xParser::OpenScript(const char *strFile)
{
	if((m_iResult = m_xFile.OpenFile(strFile, true, false)))
		assert(0);

	m_bFileOpen = true;

	return PE_RES_NOERROR;
}

int xParser::ReadLine(char* strFunction, char* strParam1, char* strParam2, char* strParam3)
{
	if(!m_bFileOpen)
		return PE_RES_FILENOTOPEN;

	char strcurline[256]	= {0};
	char strleftside[256]	= {0};
	char strrightside[256]	= {0};
	bool bwhitespace		= false;
	bool bfoundpound		= false;
	bool bfoundinclude		= false;
	bool bfounddefine		= false;
	bool bfoundparams		= false;
	bool bfoundequals		= false;

	while(!m_xFile.ReadLine(strcurline, 256))
	{
		int j = 0;
		for(int i = 0; i < (int)strlen(strcurline); i++)
		{
			if((strcurline[i] == ' ' || strcurline[i] == '	') && !bfoundequals)
			{
				bwhitespace = true;
				continue;
			}
			
			if(strcurline[i] == '/' && strlen(strcurline) > i && strcurline[i + 1] == '/')
				break;
			
			if(strcurline[i] == '#')
				bfoundpound = true;
			else if(!bfoundequals && strcurline[i] == '(')
			{
				bfoundparams = true;
				j = 0;
			}
			else if(!bfoundequals && strcurline[i] == ')')
				continue;
			else if(!bfoundequals && strcurline[i] == '=')
			{
				bfoundequals = true;
				j = 0;
				while(strcurline[i + 1] == ' ' || strcurline[i + 1] == '	')
					i++;
			}
			else if(bfoundpound)
			{
				if(bwhitespace && !(bfounddefine || bfoundinclude))
				{
					if(!stricmp(strleftside, "define"))
					{
						memset(strleftside, 0, 255);
						bfounddefine = true;
						j = 0;
					}
					else if(!stricmp(strleftside, "include"))
					{
						memset(strleftside, 0, 255);
						bfoundinclude = true;
						j = 0;
					}

					bwhitespace = false;
				}

				if(bfounddefine || bfoundinclude)
				{
					static bool bfoundleft = false;
					if(bwhitespace)
					{
						bfoundleft = true;
						j = 0;
						bwhitespace = false;
					}

					if(!bfoundleft)
						strleftside[j++] = strcurline[i];
					else
						strrightside[j++] = strcurline[i];
				}
				else
					strleftside[j++] = strcurline[i];
			}
			else if(bfoundparams || bfoundequals)
			{
				if(strcurline[i] != 10 && strcurline[i] != 13)
					strrightside[j++] = strcurline[i];
			}
			else
				strleftside[j++] = strcurline[i];

			bwhitespace = false;
		}

		if(bfounddefine)
		{
			//This will be more useful if we
			//start compiling the scripts
			//Add new variable for now
		}
		else if(bfoundinclude)
		{
			//Doesn't do anything yet, because we
			//dont have scripted functions yet, but
			//we could use this to have a config.pes
			//which would be full of #defines

			//Create a new xParser and
			//return a StartScript() from it
		}
		else if(bfoundequals)
		{
			//Add new variable or edit existing one
			if(SetVariable(strleftside, strrightside))
			{
				xVariableList *curvar = &m_xVariableList;
				while(curvar->m_pNext)
					curvar = curvar->m_pNext;

				curvar->m_pNext				= new xVariableList;
				curvar->m_pNext->m_pPrev	= curvar;
				curvar						= curvar->m_pNext;
				curvar->m_pData				= new xVariable;

				curvar->m_pData->CreateVariable(strleftside, strrightside);
			}
		}
		else if(bfoundparams)
		{
			//Call callback function
			SplitParams(strrightside, strParam1, strParam2, strParam3);

			strcpy(strFunction, strleftside);

			return PE_RES_NOERROR;
		}

		memset(strcurline,   0, 255);
		memset(strleftside,  0, 255);
		memset(strrightside, 0, 255);
		bwhitespace		= false;
		bfoundpound		= false;
		bfoundinclude	= false;
		bfounddefine	= false;
		bfoundparams	= false;
		bfoundequals	= false;
	}

	//If we reach the end of the file, end the script
	return EndScript(false);
}

void xParser::SplitParams(char *strParams, char* strParam1, char* strParam2, char* strParam3)
{
	int numparams = 1;
	int i = 0, j = 0, k = 0;
	int paramslen = (int)strlen(strParams);

	if(paramslen <= 0)
		return;

	for(i = 0; i < paramslen; i++)
	{
		if(strParams[i] == ',')
			numparams++;
	}

	memset(strParam1, 0, 50);
	memset(strParam2, 0, 50);
	memset(strParam3, 0, 50);

	for(i = 0; i < paramslen; i++)
	{
		if(strParams[i] == ',')
		{
			j++;
			k = 0;
			continue;
		}

		if(j == 0)
			strParam1[k] = strParams[i];
		else if(j == 1)
			strParam2[k] = strParams[i];
		else if(j == 2)
			strParam3[k] = strParams[i];

		k++;
	}
}

int xParser::SetVariable(char *strName, char *strValue)
{
	xVariableList *curvar = &m_xVariableList;

	while(curvar->m_pNext)
	{
		curvar = curvar->m_pNext;

		if(!stricmp(curvar->m_pData->GetName(), strName))
		{
			curvar->m_pData->SetValue(strValue);
			return PE_RES_NOERROR;
		}
	}

	return PE_RES_VARNOTFOUND;
}

char* xParser::GetVariable(const char *strName)
{
	xVariableList *curvar = &m_xVariableList;

	while(curvar->m_pNext)
	{
		curvar = curvar->m_pNext;

		if(!stricmp(curvar->m_pData->GetName(), strName))
			return curvar->m_pData->GetValue();
	}

	return NULL;
}

int xParser::EndScript(bool bClearVariables)
{
	if(bClearVariables)
	{
		xVariableList *curvar = m_xVariableList.m_pNext;

		while(curvar)
		{
			if(!curvar->m_pNext)
			{
				delete curvar->m_pData;
				delete curvar;
				break;
			}
			else
			{
				curvar = curvar->m_pNext;
				delete curvar->m_pPrev->m_pData;
				delete curvar->m_pPrev;
			}
		}

		m_xVariableList.m_pNext = NULL;
	}

	if(!m_bFileOpen)
		return PE_RES_FILENOTOPEN;

	m_xFile.CloseFile();

	m_bFileOpen = false;

	return PE_RES_ENDOFSCRIPT;
}

void xParser::ToColor(const char *strValue, xColor *pColor, xColor *pDefaultColor)
{
	int curcolorindex = 0;
	char curcolor[5] = {0};
	int i, j = 0;

	strValue = GetVariable(strValue);

	if(!strValue)
	{
		if(!pDefaultColor)
		{
			pColor->R = 1.0;
			pColor->G = 1.0;
			pColor->B = 1.0;
			pColor->A = 1.0;
		}
		else
			xColorCopy(pColor, pDefaultColor);

		return;
	}

	for(i = 0; strValue[i] != '\0'; i++)
	{
		if(strValue[i] == ')')
		{
			pColor->A = ((float)atoi(curcolor)) / 255.0;
			break;
		}
		if(strValue[i] == '(' || strValue[i] == ' ')
			continue;
		else if(strValue[i] == ',')
		{
			if(curcolorindex == 0)
				pColor->R = ((float)atoi(curcolor)) / 255.0;
			else if(curcolorindex == 1)
				pColor->G = ((float)atoi(curcolor)) / 255.0;
			else if(curcolorindex == 2)
				pColor->B = ((float)atoi(curcolor)) / 255.0;
			else//if(curcolorindex == 3)
				pColor->A = ((float)atoi(curcolor)) / 255.0;

			curcolorindex++;
			j = 0;
			memset(curcolor, 0, 5);
		}
		else
			curcolor[j++] = strValue[i];
	}
}

int xParser::ToIntArray(const char *strValue, int **pOutArray)
{
	char curint[5] = {0};
	int i, j = 0, k = 0, count = 0;

	strValue = GetVariable(strValue);

	if(!strValue)
		return 0;

	for(i = 0; strValue[i] != '\0'; i++)
		if(strValue[i] == ',')
			count++;

	*pOutArray = new int[count];

	for(i = 0; strValue[i] != '\0'; i++)
	{
		if(strValue[i] == ',')
		{
			(*pOutArray)[k++] = atoi(curint);
			j = 0;
			memset(curint, 0, 5);
		}
		else
			curint[j++] = strValue[i];
	}

	return count;
}

/*int xParser::AddScriptFunction(void *pFunction, char *strName, int iNumParams, xParamTypes iParamType1, ...)
{
	va_list paramtypes;
	xScriptFunction     *newfunction	 = NULL;
	xScriptFunctionList *curfunctionlist = &m_xScriptFunctions;

	while(curfunctionlist->m_pNext)
		curfunctionlist = curfunctionlist->m_pNext;

	curfunctionlist->m_pNext			= new xScriptFunctionList;
	curfunctionlist->m_pNext->m_pPrev	= curfunctionlist;
	curfunctionlist						= curfunctionlist->m_pNext;
	curfunctionlist->m_pData			= new xScriptFunction;

	newfunction						= curfunctionlist->m_pData;
	newfunction->m_pFunction		= pFunction;
	newfunction->m_iNumParams		= iNumParams;
	newfunction->m_iParamTypes[0]	= iParamType1;

	strcpy(newfunction->m_strName, strName);

	va_start(paramtypes, iParamType1);
	{
		for(int i = 1; i < iNumParams; i++)
		{
			newfunction->m_iParamTypes[i] = va_arg(paramtypes, char*);
		}
	}
	va_end(paramtypes);

	return PE_RES_NOERROR;
}

int xParser::CallScriptFunction(char *strName, char *strParams)
{
	return PE_RES_NOERROR;
}*/