/////////////////////////////////////////////////////////////////
// Name: xRenderData.h
/////////////////////////////////////////////////////////////////
// Description:
// - Render data structures for all Renderable objects
/////////////////////////////////////////////////////////////////
// - 23.8.2010, Created, Dayle Flowers (Xienen)
//   Created this as base class for xObject2D and xGUIObject to
//   allow for generic Animation system(xKeyFrame)
/////////////////////////////////////////////////////////////////
#ifndef XRENDERDATA_H_
#define XRENDERDATA_H_

#include "xEngineDefines.h"
#include "xVector.h"
#include "xColor.h"

struct xRenderDataTexture;
struct xRenderDataShader;

struct xTexturedVert
{
	xVector		m_xVector;
	xVector2D	m_xTextureCoords;
};

struct xRenderDataVerts
{
	xVector				 m_xRotation;
	xColor				 m_xColor;
	xTexturedVert		*m_xVertices;
	int					 m_iVectorArrayLength;
	GLuint				 m_uiVertexBuffer;
	GLuint				 m_uiIndexBuffer;
	bool				 m_bRender;
	float				 m_fDepth;
	xRenderDataTexture	*m_pTextureData;
	xRenderDataShader	*m_pShaderData;

	xRenderDataVerts	*m_pNext;
	xRenderDataVerts	*m_pPrev;
};

struct xRenderDataTexture
{
	GLenum				 m_uiRenderType;
	xColor				*m_xShaderParameterArray;
	GLuint				*m_uiTextureIndexArray;
	int					 m_iTextureIndexArrayLength;
	float				 m_fMinDepth;
	float				 m_fMaxDepth;
	float				 m_fLineWidth;
	xRenderDataVerts	 m_xVertDataList;
	bool				 m_bRender;
	bool				 m_bForcedUnique;

	xRenderDataTexture	*m_pNext;
	xRenderDataTexture	*m_pPrev;
};

struct xRenderDataShader
{
	bool				m_bIs3D;
	int					m_iBlendMode;
	float				m_fMinDepth;
	float				m_fMaxDepth;
	xRenderDataTexture	m_xTextureDataList;
	bool				m_bRender;

	xRenderDataShader	*m_pNext;
	xRenderDataShader	*m_pPrev;
};

FORCE_INLINE void xRenderDataVertsInit(xRenderDataVerts *pThis)
{
	xColorInit(&pThis->m_xColor, 1.0, 1.0, 1.0, 1.0);
	xVectorInit(&pThis->m_xRotation, 0.0, 0.0, 0.0);

	pThis->m_xVertices = 0;
	pThis->m_iVectorArrayLength = 0;
	pThis->m_uiVertexBuffer = 0;
	pThis->m_uiIndexBuffer = 0;
	pThis->m_bRender = 1;
	pThis->m_fDepth = -99999999.0f;

	pThis->m_pTextureData	= 0;
	pThis->m_pShaderData	= 0;

	pThis->m_pNext = 0;
	pThis->m_pPrev = 0;
}

FORCE_INLINE void xRenderDataTextureInit(xRenderDataTexture *pThis)
{
	xRenderDataVertsInit(&pThis->m_xVertDataList);

	pThis->m_uiRenderType = 0;
	pThis->m_xShaderParameterArray = 0;
	pThis->m_uiTextureIndexArray = 0;
	pThis->m_iTextureIndexArrayLength = 0;
	pThis->m_fMinDepth =  99999999.0f;
	pThis->m_fMaxDepth = -99999999.0f;
	pThis->m_fLineWidth = 1.0f;
	pThis->m_bRender = 1;
	pThis->m_bForcedUnique = 0;

	pThis->m_pNext = 0;
	pThis->m_pPrev = 0;
}

FORCE_INLINE void xRenderDataTextureInit(xRenderDataTexture *pThis, int iBlendMode, GLuint uiRenderType, int iTextureCount, GLuint *pTextureIDs, float fDepth, bool bForceUnique)
{
	pThis->m_uiRenderType = uiRenderType;
	pThis->m_fMinDepth = fDepth;
	pThis->m_fMaxDepth = fDepth;
	pThis->m_bForcedUnique = bForceUnique;

	pThis->m_iTextureIndexArrayLength = iTextureCount;
	pThis->m_uiTextureIndexArray = (GLuint*)dlmalloc(sizeof(GLuint) * iTextureCount);
	pThis->m_uiTextureIndexArray[0] = pTextureIDs[0];

	if(iTextureCount > 1)
		pThis->m_uiTextureIndexArray[1] = pTextureIDs[1];

	switch(iBlendMode)
	{
		case PE_BLEND_MODE_MASKED:
			pThis->m_xShaderParameterArray = (xColor*)dlmalloc(sizeof(xColor) * 2);
			break;
		case PE_BLEND_MODE_2COLOR:
			pThis->m_xShaderParameterArray = (xColor*)dlmalloc(sizeof(xColor) * 2);
			break;
	}
}

FORCE_INLINE void xRenderDataShaderInit(xRenderDataShader *pThis)
{
	xRenderDataTextureInit(&pThis->m_xTextureDataList);

	pThis->m_bIs3D = 0;
	pThis->m_iBlendMode = 0;
	pThis->m_fMinDepth =  99999999.0f;
	pThis->m_fMaxDepth = -99999999.0f;
	pThis->m_bRender = 1;

	pThis->m_pNext = 0;
	pThis->m_pPrev = 0;
}

#endif