/////////////////////////////////////////////////////////////////
// Name: xRenderable2D.h
/////////////////////////////////////////////////////////////////
// Description:
// - A renderable object(either a sprite or GUI object)
/////////////////////////////////////////////////////////////////
// - 23.8.2010, Created, Dayle Flowers (Xienen)
//   Created this as base class for xObject2D and xGUIObject to
//   allow for generic Animation system(xKeyFrame)
// - 14.12.2010, Updated, Dayle Flowers (Xienen)
//   Switched to C-style data structures
/////////////////////////////////////////////////////////////////
#ifndef XRENDERABLE2D_H_
#define XRENDERABLE2D_H_

#include "xPaper.h"
#include "xEngineDefines.h"
#include "xRenderData.h"
#include "xColor.h"

extern xPaper g_xPaperAPI;

struct xSceneGraph;

struct xRenderable2D
{
	int					 m_iBlendMode;
	xColor				 m_xColor;
	float				 m_fX;
	float				 m_fY;
	float				 m_fDepth;
	float				 m_fWidth;
	float				 m_fHeight;
	float				 m_fRotation;
	float				 m_fRotationCenterX;
	float				 m_fRotationCenterY;
	bool				 m_bMirrored;
	float				 m_fScaleX;
	float				 m_fScaleY;
	bool				 m_bVisible;
	xRenderDataVerts	*m_pVertData;
};

FORCE_INLINE void xRenderable2DInit(xRenderable2D *pThis)
{
	xColorInit(&pThis->m_xColor);

	pThis->m_iBlendMode			= 0;
	pThis->m_fX					= 0.0f;
	pThis->m_fY					= 0.0f;
	pThis->m_fDepth				= 0.0f;
	pThis->m_fRotation			= 0.0f;
	pThis->m_fRotationCenterX	= 0.0f;
	pThis->m_fRotationCenterY	= 0.0f;
	pThis->m_bMirrored			= 0;
	pThis->m_fScaleX			= 1.0f;
	pThis->m_fScaleY			= 1.0f;
	pThis->m_pVertData			= 0;
	pThis->m_bVisible			= 1;
}

FORCE_INLINE void xRenderable2DCopy(xRenderable2D *pDest, xRenderable2D *pSource)
{
	xColorCopy(&pDest->m_xColor, &pSource->m_xColor);

	pDest->m_iBlendMode			= pSource->m_iBlendMode;
	pDest->m_fX					= pSource->m_fX;
	pDest->m_fY					= pSource->m_fY;
	pDest->m_fWidth				= pSource->m_fWidth;
	pDest->m_fHeight			= pSource->m_fHeight;
	pDest->m_fDepth				= pSource->m_fDepth;
	pDest->m_fRotation			= pSource->m_fRotation;
	pDest->m_fRotationCenterX	= pSource->m_fRotationCenterX;
	pDest->m_fRotationCenterY	= pSource->m_fRotationCenterY;
	pDest->m_bMirrored			= pSource->m_bMirrored;
	pDest->m_fScaleX			= pSource->m_fScaleX;
	pDest->m_fScaleY			= pSource->m_fScaleY;
	pDest->m_pVertData			= pSource->m_pVertData;
	pDest->m_bVisible			= pSource->m_bVisible;
}

FORCE_INLINE void xRenderable2DCopyTransform(xRenderable2D *pDest, xRenderable2D *pSource)
{
	xColorCopy(&pDest->m_xColor, &pSource->m_xColor);

	pDest->m_fX					= pSource->m_fX;
	pDest->m_fY					= pSource->m_fY;
	pDest->m_fDepth				= pSource->m_fDepth;
	pDest->m_fRotation			= pSource->m_fRotation;
	pDest->m_fRotationCenterX	= pSource->m_fRotationCenterX;
	pDest->m_fRotationCenterY	= pSource->m_fRotationCenterY;
	pDest->m_bMirrored			= pSource->m_bMirrored;
	pDest->m_fScaleX			= pSource->m_fScaleX;
	pDest->m_fScaleY			= pSource->m_fScaleY;
	pDest->m_bVisible			= pSource->m_bVisible;
}

FORCE_INLINE void xRenderable2DSetDepth(xRenderable2D *pThis, float fDepth) { pThis->m_fDepth = fDepth; }
FORCE_INLINE void xRenderable2DSetBlendMode(xRenderable2D *pThis, int iBlendMode) { pThis->m_iBlendMode = iBlendMode; }
FORCE_INLINE void xRenderable2DSetPosition(xRenderable2D *pThis, float fX, float fY) { pThis->m_fX = fX; pThis->m_fY = fY; }
FORCE_INLINE void xRenderable2DSetSize(xRenderable2D *pThis, float fWidth, float fHeight) { pThis->m_fWidth = fWidth; pThis->m_fHeight = fHeight; }
FORCE_INLINE void xRenderable2DSetRotation(xRenderable2D *pThis, float fRotation) { pThis->m_fRotation = fRotation; }
FORCE_INLINE void xRenderable2DSetRotationCenter(xRenderable2D *pThis, float fRotationCenterX, float fRotationCenterY) { pThis->m_fRotationCenterX = fRotationCenterX; pThis->m_fRotationCenterY = fRotationCenterY; }
FORCE_INLINE void xRenderable2DSetMirror(xRenderable2D *pThis, bool bMirrored) { pThis->m_bMirrored = bMirrored; }
FORCE_INLINE void xRenderable2DSetScale(xRenderable2D *pThis, float fScaleX, float fScaleY) { pThis->m_fScaleX = fScaleX; pThis->m_fScaleY = fScaleY; }
FORCE_INLINE void xRenderable2DSetColor(xRenderable2D *pThis, const xColor *pSource) { xColorCopy(&pThis->m_xColor, pSource); }
FORCE_INLINE void xRenderable2DSetAlpha(xRenderable2D *pThis, float fAlpha) { pThis->m_xColor.A = fAlpha; }

FORCE_INLINE void xRenderable2DRotateVerts(xRenderable2D *pThis, float *pX1, float *pY1, float *pX2, float *pY2, float *pX3, float *pY3, float *pX4, float *pY4)
{
	float halfwidth, halfheight, distance, angle, cosangle, sinangle;

	cosangle = cos(pThis->m_fRotation * (PI / 180.0f));
	sinangle = sin(pThis->m_fRotation * (PI / 180.0f));

	if(pThis->m_fRotationCenterX == 0.0 && pThis->m_fRotationCenterY == 0.0)
	{
		halfwidth  = pThis->m_fWidth * pThis->m_fScaleX / 2.0;
		halfheight = pThis->m_fHeight * pThis->m_fScaleY / 2.0;

		*pX1 = g_xPaperAPI.m_iScreenOffsetX + (pThis->m_fX + ((cosangle * -halfwidth) - (sinangle * -halfheight) + halfwidth)) * g_xPaperAPI.m_fScreenRatioX;
		*pY1 = g_xPaperAPI.m_iScreenOffsetY + (pThis->m_fY + ((sinangle * -halfwidth) + (cosangle * -halfheight) + halfheight)) * g_xPaperAPI.m_fScreenRatioY;

		*pX2 = g_xPaperAPI.m_iScreenOffsetX + (pThis->m_fX + ((cosangle * -halfwidth) - (sinangle * halfheight) + halfwidth)) * g_xPaperAPI.m_fScreenRatioX;
		*pY2 = g_xPaperAPI.m_iScreenOffsetY + (pThis->m_fY + ((sinangle * -halfwidth) + (cosangle * halfheight) + halfheight)) * g_xPaperAPI.m_fScreenRatioY;

		*pX3 = g_xPaperAPI.m_iScreenOffsetX + (pThis->m_fX + ((cosangle * halfwidth) - (sinangle * halfheight) + halfwidth)) * g_xPaperAPI.m_fScreenRatioX;
		*pY3 = g_xPaperAPI.m_iScreenOffsetY + (pThis->m_fY + ((sinangle * halfwidth) + (cosangle * halfheight) + halfheight)) * g_xPaperAPI.m_fScreenRatioY;

		*pX4 = g_xPaperAPI.m_iScreenOffsetX + (pThis->m_fX + ((cosangle * halfwidth) - (sinangle * -halfheight) + halfwidth)) * g_xPaperAPI.m_fScreenRatioX;
		*pY4 = g_xPaperAPI.m_iScreenOffsetY + (pThis->m_fY + ((sinangle * halfwidth) + (cosangle * -halfheight) + halfheight)) * g_xPaperAPI.m_fScreenRatioY;
	}
	else
	{
		halfwidth = pThis->m_fRotationCenterX;
		halfheight = pThis->m_fRotationCenterY;

		*pX1 = g_xPaperAPI.m_iScreenOffsetX + (pThis->m_fX + ((cosangle * -halfwidth) - (sinangle * -halfheight) + halfwidth)) * g_xPaperAPI.m_fScreenRatioX;
		*pY1 = g_xPaperAPI.m_iScreenOffsetY + (pThis->m_fY + ((sinangle * -halfwidth) + (cosangle * -halfheight) + halfheight)) * g_xPaperAPI.m_fScreenRatioY;

		halfwidth *= -1.0;
		halfheight = pThis->m_fRotationCenterY - pThis->m_fHeight;
		distance = sqrt(halfwidth * halfwidth + halfheight * halfheight);
		angle = atan(halfheight / halfwidth);

		if(halfwidth < 0)
			angle += PI - pThis->m_fRotation * (PI / 180.0f);
		else
			angle -= pThis->m_fRotation * (PI / 180.0f);

		*pX2 = g_xPaperAPI.m_iScreenOffsetX + (pThis->m_fX + pThis->m_fRotationCenterX + cos(angle) * distance) * g_xPaperAPI.m_fScreenRatioX;
		*pY2 = g_xPaperAPI.m_iScreenOffsetY + (pThis->m_fY + pThis->m_fRotationCenterY - sin(angle) * distance) * g_xPaperAPI.m_fScreenRatioY;

		halfwidth = pThis->m_fWidth - pThis->m_fRotationCenterX;
		distance = sqrt(halfwidth * halfwidth + halfheight * halfheight);
		angle = atan(halfheight / halfwidth);

		if(halfwidth < 0)
			angle += PI - pThis->m_fRotation * (PI / 180.0f);
		else
			angle -= pThis->m_fRotation * (PI / 180.0f);

		*pX3 = g_xPaperAPI.m_iScreenOffsetX + (pThis->m_fX + pThis->m_fRotationCenterX + cos(angle) * distance) * g_xPaperAPI.m_fScreenRatioX;
		*pY3 = g_xPaperAPI.m_iScreenOffsetY + (pThis->m_fY + pThis->m_fRotationCenterY - sin(angle) * distance) * g_xPaperAPI.m_fScreenRatioY;

		halfheight = pThis->m_fRotationCenterY;
		distance = sqrt(halfwidth * halfwidth + halfheight * halfheight);
		angle = atan(halfheight / halfwidth);

		if(halfwidth < 0)
			angle += PI - pThis->m_fRotation * (PI / 180.0f);
		else
			angle -= pThis->m_fRotation * (PI / 180.0f);

		*pX4 = g_xPaperAPI.m_iScreenOffsetX + (pThis->m_fX + pThis->m_fRotationCenterX + cos(angle) * distance) * g_xPaperAPI.m_fScreenRatioX;
		*pY4 = g_xPaperAPI.m_iScreenOffsetY + (pThis->m_fY + pThis->m_fRotationCenterY - sin(angle) * distance) * g_xPaperAPI.m_fScreenRatioY;

		//DEBUGLOG("xRenderable2DRotateVerts Centered: %f,%f %f,%f %f,%f %f,%f\n", *pX1, *pY1, *pX2, *pY2, *pX3, *pY3, *pX4, *pY4);
	}
}

FORCE_INLINE void xRenderable2DUpdateBlendMode(xRenderable2D *pThis, int iBlendMode)
{
	xRenderable2DSetBlendMode(pThis, iBlendMode);

	if(pThis->m_pVertData->m_pTextureData->m_xShaderParameterArray)
		dlfree(pThis->m_pVertData->m_pTextureData->m_xShaderParameterArray);

	switch(iBlendMode)
	{
		case PE_BLEND_MODE_MASKED:
			pThis->m_pVertData->m_pTextureData->m_xShaderParameterArray = (xColor*)dlmalloc(sizeof(xColor) * 2);
			break;
		case PE_BLEND_MODE_2COLOR:
			pThis->m_pVertData->m_pTextureData->m_xShaderParameterArray = (xColor*)dlmalloc(sizeof(xColor) * 2);
			break;
	}
}

FORCE_INLINE void xRenderable2DUpdateDepth(xRenderable2D *pThis, xSceneGraph *pSceneGraph, float fDepth)
{
	xRenderable2DSetDepth(pThis, fDepth);

	extern void xSceneGraphDepthChanged(xSceneGraph *pThis, xRenderDataVerts *pVertData, float fDepth);
	xSceneGraphDepthChanged(pSceneGraph, pThis->m_pVertData, fDepth);

	xRenderable2DUpdateBlendMode(pThis, pThis->m_iBlendMode);
}

FORCE_INLINE void xRenderable2DUpdateColor(xRenderable2D *pThis, const xColor *pSource)
{
	xRenderable2DSetColor(pThis, pSource);

	xColorCopy(&pThis->m_pVertData->m_xColor, pSource);
}

FORCE_INLINE void xRenderable2DUpdateRGB(xRenderable2D *pThis, const xColor *pSource)
{
	pThis->m_xColor.R = pSource->R;
	pThis->m_xColor.G = pSource->G;
	pThis->m_xColor.B = pSource->B;

	xColorCopy(&pThis->m_pVertData->m_xColor, &pThis->m_xColor);
}

FORCE_INLINE void xRenderable2DUpdateAlpha(xRenderable2D *pThis, float fAlpha)
{
	xRenderable2DSetAlpha(pThis, fAlpha);

	pThis->m_pVertData->m_xColor.A = fAlpha;
}

FORCE_INLINE void xRenderable2DUpdateVisibility(xRenderable2D *pThis, bool bVisible)
{
	pThis->m_bVisible = bVisible;

	if(!bVisible)
	{
		pThis->m_pVertData->m_bRender = 0;

		if(!pThis->m_pVertData->m_pNext && pThis->m_pVertData->m_pPrev == &pThis->m_pVertData->m_pTextureData->m_xVertDataList)
		{
			pThis->m_pVertData->m_pTextureData->m_bRender = 0;

			if(!pThis->m_pVertData->m_pTextureData->m_pNext && pThis->m_pVertData->m_pTextureData->m_pPrev == &pThis->m_pVertData->m_pShaderData->m_xTextureDataList)
				pThis->m_pVertData->m_pShaderData->m_bRender = 0;
		}
	}
	else
	{
		pThis->m_pVertData->m_pShaderData->m_bRender = 1;
		pThis->m_pVertData->m_pTextureData->m_bRender = 1;
		pThis->m_pVertData->m_bRender = 1;
	}
}

#endif