#ifndef _RENDERABLER2DTEXT_H_
#define _RENDERABLER2DTEXT_H_

#include "xEngineDefines.h"
#include "xRenderData.h"
#include "xPaper.h"

extern xPaper g_xPaperAPI;

enum xTextAlignment
{
	PE_TEXTALIGN_LEFT,
	PE_TEXTALIGN_CENTER,
	PE_TEXTALIGN_RIGHT,
	PE_TEXTALIGN_JUSTIFIED
};

struct xRenderable2DText
{
	xFont				*m_pFont;
	xRenderable2DQuad	 m_xRenderable;
	int					 m_iStringLength;
	xColor				 m_xColor2;
	xTextAlignment		 m_xAlignment;
};

void xRenderable2DTextSetText(xRenderable2DText *pThis, xSceneGraph *pSceneGraph, const char *strNewText, bool bForceUniqueColorData = 0);

FORCE_INLINE void xRenderable2DTextSetPosition(xRenderable2DText *pThis, float fX, float fY) { xRenderable2DQuadSetPosition(&pThis->m_xRenderable, fX, fY); }
FORCE_INLINE void xRenderable2DTextSetAlignment(xRenderable2DText *pThis, xTextAlignment xAlignment, float fJustifiedWidth = 0.0) { pThis->m_xAlignment = xAlignment; pThis->m_xRenderable.m_xRenderable2D.m_fWidth = fJustifiedWidth; }
FORCE_INLINE void xRenderable2DTextSetColor(xRenderable2DText *pThis, xColor *pColor) { xRenderable2DQuadSetColor(&pThis->m_xRenderable, pColor); }
FORCE_INLINE void xRenderable2DTextSetAlpha(xRenderable2DText *pThis, float fAlpha) { xRenderable2DQuadSetAlpha(&pThis->m_xRenderable, fAlpha); }
FORCE_INLINE void xRenderable2DTextSetFont(xRenderable2DText *pThis, xFont *pFont) { pThis->m_pFont = pFont; }
FORCE_INLINE void xRenderable2DTextSetVisibility(xRenderable2DText *pThis, bool bVisible) { pThis->m_xRenderable.m_xRenderable2D.m_bVisible = bVisible; }
FORCE_INLINE void xRenderable2DTextSetScale(xRenderable2DText *pThis, float fScaleX, float fScaleY) { xRenderable2DQuadSetScale(&pThis->m_xRenderable, fScaleX, fScaleY); }
FORCE_INLINE void xRenderable2DTextSetRotation(xRenderable2DText *pThis, float fRotation) { xRenderable2DQuadSetRotation(&pThis->m_xRenderable, fRotation); }

FORCE_INLINE void xRenderable2DTextInit(xRenderable2DText *pThis, float fDepth = 1.0)
{
	xRenderable2DQuadInit(&pThis->m_xRenderable);
	
	pThis->m_xRenderable.m_xRenderable2D.m_fDepth = fDepth;

	pThis->m_pFont			= 0;
	pThis->m_iStringLength	= 0;
	pThis->m_xAlignment		= PE_TEXTALIGN_LEFT;
}

FORCE_INLINE void xRenderable2DTextSetDualColor(xRenderable2DText *pThis, const xColor *clrColor1, const xColor *clrColor2)
{
	if(pThis->m_xRenderable.m_xRenderable2D.m_pVertData)
	{
		xRenderable2DQuadUpdateBlendMode(&pThis->m_xRenderable, PE_BLEND_MODE_2COLOR);
		xRenderable2DQuadUpdateColor(&pThis->m_xRenderable, clrColor1);
	}
	else
	{
		xRenderable2DQuadSetBlendMode(&pThis->m_xRenderable, PE_BLEND_MODE_2COLOR);
		xRenderable2DQuadSetColor(&pThis->m_xRenderable, clrColor1);
	}

	xRenderable2DQuadSetColor2(&pThis->m_xRenderable, clrColor2);
}

FORCE_INLINE void xRenderable2DTextUpdateColor(xRenderable2DText *pThis, xColor *pColor)
{
	if(pThis->m_xRenderable.m_xRenderable2D.m_pVertData)
		xRenderable2DQuadUpdateColor(&pThis->m_xRenderable, pColor);
	else
		xRenderable2DQuadSetColor(&pThis->m_xRenderable, pColor);
}

FORCE_INLINE void xRenderable2DTextUpdateAlpha(xRenderable2DText *pThis, float fAlpha)
{
	if(pThis->m_xRenderable.m_xRenderable2D.m_pVertData)
		xRenderable2DQuadUpdateAlpha(&pThis->m_xRenderable, fAlpha);
	else
		xRenderable2DQuadSetAlpha(&pThis->m_xRenderable, fAlpha);
}

FORCE_INLINE void xRenderable2DTextUpdateVisibility(xRenderable2DText *pThis, bool bVisible)
{
	if(pThis->m_xRenderable.m_xRenderable2D.m_pVertData)
		xRenderable2DQuadUpdateVisibility(&pThis->m_xRenderable, bVisible);
	else
		pThis->m_xRenderable.m_xRenderable2D.m_bVisible = bVisible;
}

FORCE_INLINE void xRenderable2DTextUpdateDepth(xRenderable2DText *pThis, xSceneGraph *pSceneGraph, float fDepth)
{
	if(pThis->m_xRenderable.m_xRenderable2D.m_pVertData)
		xRenderable2DQuadUpdateDepth(&pThis->m_xRenderable, pSceneGraph, fDepth);
	else
		pThis->m_xRenderable.m_xRenderable2D.m_fDepth = fDepth;
}

FORCE_INLINE void xRenderable2DTextRefreshTextures(xRenderable2DText *pThis)
{
	if(pThis->m_xRenderable.m_xRenderable2D.m_pVertData)
		xRenderable2DQuadUpdateTexture(&pThis->m_xRenderable, pThis->m_pFont->m_pTexture);
	else
		xRenderable2DQuadSetTexture(&pThis->m_xRenderable, pThis->m_pFont->m_pTexture);
}

FORCE_INLINE void xRenderable2DTextDelete(xRenderable2DText *pThis, xSceneGraph *pSceneGraph)
{
	xSceneGraphRemove2D(pSceneGraph, &pThis->m_xRenderable.m_xRenderable2D);
	pThis->m_iStringLength = 0;
}

#endif