#ifndef XSOUND_H_
#define XSOUND_H_

#include "openal/al.h"
#include "openal/alc.h"
#include "xAudio.h"

#define MAX_VARIANTS	8

enum xSoundState
{
	PE_SOUNDSTATE_NULL = 0,
	PE_SOUNDSTATE_LOADED,
	PE_SOUNDSTATE_STOPPED,
	PE_SOUNDSTATE_PLAYING,
	PE_SOUNDSTATE_PAUSED
};

enum xPanSetting
{
	PE_SOUNDPAN_FULL_LEFT = 0,
	PE_SOUNDPAN_CENTERED = 128,
	PE_SOUNDPAN_FULL_RIGHT = 255,
	PE_SOUNDPAN_FULL_VOLUME = 256
};

class xSound : public xPoolItem<xSound>
{
public:
	xSound()
	{
		m_iNumSounds	= 0;
		m_bDontRepeat	= 0;
	}

	int  Open(const char *strPathName);
	int  AddVariant(const char *strPathName);
	int  Play(float fVolume = 1.0, bool bLooping = false, int iPan = PE_SOUNDPAN_FULL_VOLUME);
	int  Stop();
	int  Pause();
	int  UnPause();
	void Release();

	bool IsPlaying() { ALint state; if(m_xSoundState != PE_SOUNDSTATE_PLAYING) return false; alGetSourcei(m_uiSource, AL_SOURCE_STATE, &state); if(state != AL_PLAYING) { m_xSoundState = PE_SOUNDSTATE_STOPPED; return false; } return true; }
	bool IsPaused() { return m_xSoundState == PE_SOUNDSTATE_PAUSED; }

	xSoundState m_xSoundState;
	ALuint		m_uiSource;
	int			m_iNumSounds;

	bool		 m_bDontRepeat;
	int			*m_pPlayedSounds;
	int			 m_iCurrentPlayedIndex;

protected:
	int  LoadWave(const char *strPathName, int iBufferIndex);
	
	ALuint	m_uiBuffers[MAX_VARIANTS];
};

enum WAVEFILETYPE
{
	WF_EX  = 1,
	WF_EXT = 2
};

#define	WAVE_FORMAT_PCM			1
#define	WAVE_FORMAT_EXTENSIBLE	0xFFFE

#define SPEAKER_FRONT_LEFT		0x1
#define SPEAKER_FRONT_RIGHT		0x2
#define SPEAKER_FRONT_CENTER	0x4
#define SPEAKER_LOW_FREQUENCY	0x8
#define SPEAKER_BACK_LEFT		0x10
#define SPEAKER_BACK_RIGHT		0x20
#define SPEAKER_BACK_CENTER		0x100
#define SPEAKER_SIDE_LEFT		0x200
#define SPEAKER_SIDE_RIGHT		0x400

struct GUID
{
	UInt32	Data1;
	UInt16	Data2;
	UInt16	Data3;
	UInt8	Data4[8];
};

#ifndef _WAVEFORMATEX_
#define _WAVEFORMATEX_
typedef struct tWAVEFORMATEX
{
    UInt16    wFormatTag;
    UInt16    nChannels;
    UInt32    nSamplesPerSec;
    UInt32    nAvgBytesPerSec;
    UInt16    nBlockAlign;
    UInt16    wBitsPerSample;
    UInt16    cbSize;
} WAVEFORMATEX;
#endif

#ifndef _WAVEFORMATEXTENSIBLE_
#define _WAVEFORMATEXTENSIBLE_
typedef struct
{
    WAVEFORMATEX    Format;
    union
	{
        UInt16 wValidBitsPerSample;       /* bits of precision  */
        UInt16 wSamplesPerBlock;          /* valid if wBitsPerSample==0 */
        UInt16 wReserved;                 /* If neither applies, set to zero. */
    } Samples;
    UInt32          dwChannelMask;      /* which channels are present in stream  */
    GUID            SubFormat;
} WAVEFORMATEXTENSIBLE, *PWAVEFORMATEXTENSIBLE;
#endif

struct WAVEFILEINFO
{
	WAVEFILETYPE	wfType;
	WAVEFORMATEXTENSIBLE wfEXT;		// For non-WAVEFORMATEXTENSIBLE wavefiles, the header is stored in the Format member of wfEXT
	char			*pData;
	unsigned long	ulDataSize;
	unsigned long	ulDataOffset;
};

#pragma pack(push, 4)

struct WAVEFILEHEADER
{
	char			szRIFF[4];
	unsigned long	ulRIFFSize;
	char			szWAVE[4];
};

struct RIFFCHUNK
{
	char			szChunkName[4];
	unsigned long	ulChunkSize;
};

struct WAVEFMT
{
	unsigned short	usFormatTag;
	unsigned short	usChannels;
	unsigned long	ulSamplesPerSec;
	unsigned long	ulAvgBytesPerSec;
	unsigned short	usBlockAlign;
	unsigned short	usBitsPerSample;
	unsigned short	usSize;
	unsigned short  usReserved;
	unsigned long	ulChannelMask;
    GUID            guidSubFormat;
};

struct WAVEFORMAT
{
	UInt16	wFormatTag;
	UInt16	nChannels;
	UInt32	nSamplesPerSec;
	UInt32	nAvgBytesPerSec;
	UInt16	nBlockAlign;
};

struct PCMWAVEFORMAT
{
	WAVEFORMAT	wf;
	UInt16		wBitsPerSample;
};

#pragma pack(pop)

#endif