/////////////////////////////////////////////////////////////////
// Name: xVector.h
/////////////////////////////////////////////////////////////////
// Description:
// - Mathematical 3D and 2D vectors
/////////////////////////////////////////////////////////////////
// - 15.11.2004, Update, Arto Ruotsalainen (Arex)
//   Added operator functions.
//
// - 7.6.2004, Update, Arto Ruotsalainen(Arex)
//   Comment stuff added to begin of the file.
//   Add new updates/fixes above this, so the old updates
//   will be at the bottom.
/////////////////////////////////////////////////////////////////
#ifndef XVECTOR_H_
#define XVECTOR_H_

#include <math.h>

class xVector
{
public:
	xVector()
	{
		x = y = z = 0;
	}

	xVector(float fX, float fY, float fZ)
	{
		x = fX;
		y = fY;
		z = fZ;
	}

	xVector operator+(const xVector &Other) const
	{
		xVector tmp(x + Other.x, y + Other.y, z + Other.z);
		return tmp;
	}

	xVector operator-() const
	{
		xVector tmp(-x, -y, -z);
		return tmp;
	}

	xVector operator-(const xVector &Other) const
	{
		xVector tmp(x-Other.x, y-Other.y, z-Other.z);
		return tmp;
	}

	xVector operator*(const float fValue) const
	{
		xVector tmp(x*fValue, y*fValue, z*fValue);
		return tmp;
	}

	xVector operator*(const xVector &Other) const
	{
		xVector tmp(x*Other.x, y*Other.y, z*Other.z);
		return tmp;
	}

	xVector operator/(const float fValue) const
	{
		xVector tmp(x/fValue, y/fValue, z/fValue);
		return tmp;
	}

	xVector operator/(const xVector &Other) const
	{
		xVector tmp(x/Other.x, y/Other.y, z/Other.z);
		return tmp;
	}

	xVector operator+=(const xVector &Other)
	{
		x += Other.x;
		y += Other.y;
		z += Other.z;

		return *this;
	}

	xVector operator-=(const xVector &Other)
	{
		x -= Other.x;
		y -= Other.y;
		z -= Other.z;

		return *this;
	}

	xVector operator*=(const float fValue)
	{
		x *= fValue;
		y *= fValue;
		z *= fValue;

		return *this;
	}

	xVector operator*=(const xVector &Other)
	{
		x *= Other.x;
		y *= Other.y;
		z *= Other.z;

		return *this;
	}

	xVector operator/=(const xVector &Other)
	{
		x /= Other.x;
		y /= Other.y;
		z /= Other.z;

		return *this;
	}

	float GetLength()
	{
		return sqrtf(x*x+y*y+z*z);
	}

	float GetDotProduct(const xVector &Other) 
	{
		return x*Other.x+y*Other.y+z*Other.z;
	}

	float x, y, z;
};

class xVector4 : public xVector
{
	float w;
};

class xVector2D
{
public:
	xVector2D()
	{
		x = y = 0;
	}
	xVector2D(float fX, float fY)
	{
		x = fX;
		y = fY;
	}

/*	void operator+(const xVector &Other) const
	{
		xVector(x+Other.x, y+Other.y, z+Other.z);
	}

	void operator-() const
	{
		xVector(-x, -y, -z);
	}

	void operator-(const xVector &Other) const
	{
		xVector(x-Other.x, y-Other.y, z-Other.z);
	}

	void operator*(const float fValue) const
	{
		xVector(x*fValue, y*fValue, z*fValue);
	}

	void operator*(const xVector &Other) const
	{
		xVector(x*Other.x, y*Other.y, z*Other.z);
	}

	void operator/(const float fValue) const
	{
		xVector(x/fValue, y/fValue, z/fValue);
	}

	void operator/(const xVector &Other) const
	{
		xVector(x/Other.x, y/Other.y, z/Other.z);
	}

	void operator+=(const xVector &Other)
	{
		x += Other.x;
		y += Other.y;
		z += Other.z;
	}

	void operator-=(const xVector &Other)
	{
		x -= Other.x;
		y -= Other.y;
		z -= Other.z;
	}

	void operator*=(const xVector &Other)
	{
		x *= Other.x;
		y *= Other.y;
		z *= Other.z;
	}

	void operator/=(const xVector &Other)
	{
		x /= Other.x;
		y /= Other.y;
		z /= Other.z;
	}

	float GetLength()
	{
		return sqrtf(x*x+y*y+z*z);
	}

	float GetDotProduct(const xVector &Other) 
	{
		return x*Other.x+y*Other.y+z*Other.z;
	}
*/
	float x, y;
};

FORCE_INLINE void xVectorInit(xVector *pThis)
{
	pThis->x = 0.0;
	pThis->y = 0.0;
	pThis->z = 0.0;
}

FORCE_INLINE void xVectorInit(xVector4 *pThis)
{
	pThis->x = 0.0;
	pThis->y = 0.0;
	pThis->z = 0.0;
}

FORCE_INLINE void xVectorInit(xVector *pThis, float fX, float fY, float fZ)
{
	pThis->x = fX;
	pThis->y = fY;
	pThis->z = fZ;
}

FORCE_INLINE void xVectorInit(xVector4 *pThis, float fX, float fY, float fZ)
{
	pThis->x = fX;
	pThis->y = fY;
	pThis->z = fZ;
}

FORCE_INLINE void xVectorCopy(xVector *pDest, const xVector *pSource)
{
	pDest->x = pSource->x;
	pDest->y = pSource->y;
	pDest->z = pSource->z;
}

FORCE_INLINE void xVector2DInit(xVector2D *pThis, float fX, float fY)
{
	pThis->x = fX;
	pThis->y = fY;
}

#endif