#include "peGame.h"

int g_iTourLevelCount[7] = {2, 4, 4, 6, 8, 8, 12};
int g_iTourEnemyIndices[7][12] = {{12, 12},
#ifdef DEMO_RELEASE
								  {0, 1, 1, 3},
								  {2, 4, 0, 5},
#else
								  {0, 1, 2, 3},
								  {1, 4, 0, 5},
#endif
								  {6, 1, 7, 4, 8, 5},
								  {0, 6, 1, 2, 7, 3, 8, 9},
								  {2, 6, 3, 7, 4, 5, 8, 10},
								  {0, 1, 2, 3, 6, 4, 7, 8, 5, 9, 10, 11}};

float g_fPlayerDamageMultipliers[BB_NUMDIFFICULTIES] = {1.5f, 1.25f, 1.0f, 1.25f, 1.5f};
float g_fEnemyDamageMultipliers[BB_NUMDIFFICULTIES] = {0.5f, 0.75f, 1.0f, 1.25f, 1.5f};

int g_iPlayerChargeClearableCount[5] = {0, 10, 20, 40, 60};
char *g_strPlayerChargeBars[4] = {BB_IMAGE_PLAYERCHARGEBAR1, BB_IMAGE_PLAYERCHARGEBAR2, BB_IMAGE_PLAYERCHARGEBAR3, BB_IMAGE_PLAYERCHARGEBAR4};
char *g_strPlayerChargeOverlays[4] = {BB_IMAGE_PLAYERCHARGEFLASH1, BB_IMAGE_PLAYERCHARGEFLASH2, BB_IMAGE_PLAYERCHARGEFLASH3, BB_IMAGE_PLAYERCHARGEFLASH4};
float g_fPlayerChargeBarLocations[4][2] = {{BB_PLAYER_CHARGE1_LEFT, BB_PLAYER_CHARGE1_TOP},
										   {BB_PLAYER_CHARGE2_LEFT, BB_PLAYER_CHARGE2_TOP},
										   {BB_PLAYER_CHARGE3_LEFT, BB_PLAYER_CHARGE3_TOP},
										   {BB_PLAYER_CHARGE4_LEFT, BB_PLAYER_CHARGE4_TOP}};

FORCE_INLINE void xGamePrecache();
FORCE_INLINE void xGameUpdateCrowdBarArrows(xRenderable2DQuad *pCrowdBarArrow1, xRenderable2DQuad *pCrowdBarArrow2, float fCrowdBarPercentage);
FORCE_INLINE void xGameStopSounds();

void xGame::Init()
{
	xRenderable2DTextInit(&m_xVersusPlayerName, BB_DEPTH_VERSUS_PLAYERNAME);
	xRenderable2DTextInit(&m_xVersusEnemyName, BB_DEPTH_VERSUS_ENEMYNAME);
	xRenderable2DTextInit(&m_xVersusSongName, BB_DEPTH_VERSUS_SONGNAME);
	xRenderable2DTextInit(&m_xVersusArtistName, BB_DEPTH_VERSUS_SONGNAME);
	xRenderable2DTextInit(&m_xTimer, BB_DEPTH_TIMER);
	xRenderable2DTextInit(&m_xPlayerName, BB_DEPTH_PLAYERNAME);
	xRenderable2DTextInit(&m_xPlayerRank, BB_DEPTH_PLAYERRANK);
	xRenderable2DTextInit(&m_xEnemyName, BB_DEPTH_ENEMYNAME);
	xRenderable2DTextInit(&m_xEnemyRank, BB_DEPTH_ENEMYRANK);

	m_xPlayer.Init();
}

bool xGame::InitListenServer(const char *strMyIP, int iPort, const char *strBackgroundPath, const char *strSongName, int iDifficultyLevel)
{
	ResetListenServerConnection();

	if(!strSongName)
		return false;

	if(g_xPaperAPI.m_xNetwork.Listen(iPort))
		return false;

	if(g_xPaperAPI.m_xPingNetwork.Listen(iPort + 1))
		return false;

	g_xPaperAPI.m_iNetMode	= PE_NETMODE_LISTENSERVER;
	m_bReceivedPing			= false;
	m_dwServerStartTime		= g_xPaperAPI.m_dwCurrentTime;

	SetupGameOptionsForMultiplayer(BB_STRING_PLAYERNAMEWAITING, strBackgroundPath, strSongName, (xGameDifficulty)iDifficultyLevel);

	strcpy(g_xGameOptions.m_strServerAddress, strMyIP);
	g_xGameOptions.m_iServerPort = iPort;

	Init("\0");

	CREATELOG();

	return true;
}

bool xGame::InitClient(const char *strConnectToAddress, int iPort)
{
	m_bPrePopup = 0;
	m_bSenseiPrePopup = 0;
	m_bReady = 0;
	m_bSentReady = 0;
	m_bFinishedConnecting = 0;
	m_iConnectingStatus = 0;

	strcpy(g_xGameOptions.m_strServerAddress, strConnectToAddress);
	g_xGameOptions.m_iServerPort = iPort;

	g_xPaperAPI.m_iNetMode = PE_NETMODE_CLIENT;

	CREATELOG();

	return !g_xPaperAPI.m_xNetwork.Connect(strConnectToAddress, iPort);
}

void xGame::SetupGameOptionsForMultiplayer(const char *strEnemyName, const char *strBackgroundName, const char *strSongName, xGameDifficulty iDifficultyLevel)
{
	strcpy(g_xGameOptions.m_strEnemyName, strEnemyName);
	strcpy(g_xGameOptions.m_strBGFile, strBackgroundName);			
	strcpy(g_xGameOptions.m_strSongName, strSongName);			
	g_xGameOptions.m_iDifficultyLevel = iDifficultyLevel;

	g_xGameOptions.m_iLevelIndex				= -1;
	g_xGameOptions.m_strLevelFile[0]			= '\0';
	g_xGameOptions.m_strNextLevelFile[0]		= '\0';
	g_xGameOptions.m_strPrePopup[0]				= '.';
	g_xGameOptions.m_strPrePopup[1]				= '\0';
	g_xGameOptions.m_strSenseiPrePopup[0]		= '\0';
	g_xGameOptions.m_strSenseiPrePopupVoice[0]	= '\0';
	g_xGameOptions.m_dwEnemyFirstMoveDelay		= 0;
	g_xGameOptions.m_dwEnemyMoveDelay			= 0;
	g_xGameOptions.m_dwMinEnemyMoveDelay		= 0;
	g_xGameOptions.m_iEnemyDamage				= 0;
	g_xGameOptions.m_bTutorial					= 0;
	g_xGameOptions.m_bBoss						= 0;
	g_xGameOptions.m_iBossIndex					= -1;
	g_xGameOptions.m_dwLevelCompletionTime		= -1;
	g_xGameOptions.m_dwBronzeMedalScore			= 0;
	g_xGameOptions.m_dwSilverMedalScore			= 0;
	g_xGameOptions.m_dwGoldMedalScore			= 0;
	g_xGameOptions.m_iMaxPieces					= 0;
	g_xGameOptions.m_bLoopSong					= 0;
	g_xGameOptions.m_bShowMissed				= 1;
	g_xGameOptions.m_bCombosRequired			= (iDifficultyLevel == 3);
	g_xGameOptions.m_bAlwaysAddBlock			= 0;
	g_xGameOptions.m_iBeatMarkerShowTime		= 350;
	g_xGameOptions.m_iHitTime					= 100;
	g_xGameOptions.m_iNumRandomColors			= 6;
	g_xGameOptions.m_bShowComboCounter			= 0;
	g_xGameOptions.m_bShowClearCounter			= 0;

	// Make sure we're not in a bad state from previous battles
	g_xGame.m_bSenseiPrePopup = 0;
}

void xGame::ResetListenServerConnection()
{
	g_xPaperAPI.m_xNetwork.Close();

	m_iConnectingStatus = 0;
	m_bFinishedConnecting = 0;
	m_bReady = 0;
	m_bSentReady = 0;

	strcpy(g_xGameOptions.m_strEnemyDisplayName, BB_STRING_PLAYERNAMEWAITING);

	if(m_pRemotePlayer)
		m_pRemotePlayer->m_iRank = 0;
}

void xGame::DelayedInit(const char *strLevelName, float fDelay)
{
	//DEBUGLOG("DelayedInit %f\n", m_dInitTime);

	if(m_dInitTime <= 0.0)
	{
		m_dInitTime = fDelay;

		if(strLevelName != m_strStoredLevel)
			strcpy(m_strStoredLevel, strLevelName);
	}
}

void xGame::InitVersusScreen(const char *strLevelName)
{
	xFont *font, *font2;
	xColor tempcolor;
	char temp[256], enemyname[64], enemydisplayname[64], songname[64], artistname[64];

	g_xPaperAPI.m_xInput.m_bIgnoreInput = true;

	if(!m_pVersusScreenSound)
		m_pVersusScreenSound = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_VERSUSSCREEN);

	g_bTourComplete = 0;
	m_bPlayedVersusScreenSound = 0;

	//DEBUGLOG("InitVersusScreen\n");
	DelayedInit(strLevelName, BB_VERSUS_ANIMATION_TIME + 0.2f);

	GetPreloadGameOptions(strLevelName, enemyname, enemydisplayname, songname, artistname);

	m_bVersusScreenAnimating = 1;
	m_fVersusAnimationTime = 0.0f;

	sprintf(temp, BB_IMAGE_PATH, "M4_Background");
	m_pVersusBackground = xSceneGraphCreateQuad(temp, 0);
	xSceneGraphInsertQuad(&m_xSceneGraph, m_pVersusBackground, 1, BB_DEPTH_VERSUS_BACKGROUND, 0, 1);
	xRenderable2DQuadSetPosition(m_pVersusBackground, -g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX, -g_xPaperAPI.m_iScreenOffsetY / g_xPaperAPI.m_fScreenRatioY);
	xRenderable2DQuadSetSize(m_pVersusBackground, 1024.0f + 2.0f * (float)g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX, 768.0f + 2.0f * (float)g_xPaperAPI.m_iScreenOffsetY / g_xPaperAPI.m_fScreenRatioY);
	xRenderable2DQuadUpdate(m_pVersusBackground);

	sprintf(temp, BB_VERSUSIMAGE_PATH, "Versus");
	m_pVersusVSImage = xSceneGraphCreateQuad(temp, 0);
	xSceneGraphInsertQuad(&m_xSceneGraph, m_pVersusVSImage, 1, BB_DEPTH_VERSUS_VS, 0, 1);
	xRenderable2DQuadSetPosition(m_pVersusVSImage, BB_VERSUS_VS_LEFT, BB_VERSUS_VS_START);
	xRenderable2DQuadUpdate(m_pVersusVSImage);

	sprintf(temp, BB_VERSUSIMAGE_PATH, "Player");
	m_pVersusPlayerPortrait = xSceneGraphCreateQuad(temp, 0);
	xSceneGraphInsertQuad(&m_xSceneGraph, m_pVersusPlayerPortrait, 1, BB_DEPTH_VERSUS_PLAYERPORTRAIT, 0, 1);
	xRenderable2DQuadSetPosition(m_pVersusPlayerPortrait, BB_VERSUS_PLAYERPORTRAIT_START, BB_VERSUS_PLAYERPORTRAIT_TOP);
	xRenderable2DQuadUpdate(m_pVersusPlayerPortrait);

	sprintf(temp, BB_VERSUSIMAGE_PATH, enemyname);
	m_pVersusEnemyPortrait = xSceneGraphCreateQuad(temp, 0);
	xSceneGraphInsertQuad(&m_xSceneGraph, m_pVersusEnemyPortrait, 1, BB_DEPTH_VERSUS_ENEMYPORTRAIT, 0, 1);
	xRenderable2DQuadSetPosition(m_pVersusEnemyPortrait, BB_VERSUS_ENEMYPORTRAIT_START, BB_VERSUS_ENEMYPORTRAIT_TOP);
	xRenderable2DQuadUpdate(m_pVersusEnemyPortrait);

	if(!m_xVersusPlayerName.m_iStringLength)
		xRenderable2DTextInit(&m_xVersusPlayerName, BB_DEPTH_VERSUS_PLAYERNAME);

	font = g_xGame.m_xGUI.GetFont(BB_FONT_VERSUSNAMES);
	font2 = g_xGame.m_xGUI.GetFont(BB_FONT_SMALL_VERSUSNAMES);

	xRenderable2DTextSetFont(&m_xVersusPlayerName, xFontGetWidth(font, g_xCurrentProfile.m_strProfileName) < BB_VERSUS_NAME_MAXWIDTH ? font : font2);
	xRenderable2DTextSetPosition(&m_xVersusPlayerName, BB_VERSUS_PLAYERNAME_CENTER - xFontGetWidth(m_xVersusPlayerName.m_pFont, g_xCurrentProfile.m_strProfileName) / 2.0f, BB_VERSUS_PLAYERNAME_TOP);
	xRenderable2DTextSetText(&m_xVersusPlayerName, &m_xSceneGraph, g_xCurrentProfile.m_strProfileName);
	xRenderable2DTextUpdateVisibility(&m_xVersusPlayerName, 0);

	if(!m_xVersusEnemyName.m_iStringLength)
		xRenderable2DTextInit(&m_xVersusEnemyName, BB_DEPTH_VERSUS_ENEMYNAME);

	xRenderable2DTextSetFont(&m_xVersusEnemyName, xFontGetWidth(font, enemydisplayname) < BB_VERSUS_NAME_MAXWIDTH ? font : font2);
	xRenderable2DTextSetPosition(&m_xVersusEnemyName, BB_VERSUS_ENEMYNAME_CENTER - xFontGetWidth(m_xVersusEnemyName.m_pFont, enemydisplayname) / 2.0f, BB_VERSUS_ENEMYNAME_TOP);
	xRenderable2DTextSetText(&m_xVersusEnemyName, &m_xSceneGraph, enemydisplayname);
	xRenderable2DTextUpdateVisibility(&m_xVersusEnemyName, 0);

	if(!m_xVersusSongName.m_iStringLength)
		xRenderable2DTextInit(&m_xVersusSongName, BB_DEPTH_VERSUS_SONGNAME);

	sprintf(temp, "Song: %s", songname);
	xRenderable2DTextSetFont(&m_xVersusSongName, g_xGame.m_xGUI.GetFont(BB_FONT_SONGNAME));
	xRenderable2DTextSetPosition(&m_xVersusSongName, BB_VERSUS_SONGNAME_RIGHT - xFontGetWidth(m_xVersusSongName.m_pFont, temp), BB_VERSUS_SONGNAME_TOP);
	xRenderable2DTextSetColor(&m_xVersusSongName, xColorInit(&tempcolor, 1.0, 1.0, 1.0, 0.0));
	xRenderable2DTextSetText(&m_xVersusSongName, &m_xSceneGraph, temp);

	if(!m_xVersusArtistName.m_iStringLength)
		xRenderable2DTextInit(&m_xVersusArtistName, BB_DEPTH_VERSUS_SONGNAME);

	sprintf(temp, "By: %s", artistname);
	xRenderable2DTextSetFont(&m_xVersusArtistName, m_xVersusSongName.m_pFont);
	xRenderable2DTextSetPosition(&m_xVersusArtistName, BB_VERSUS_SONGNAME_RIGHT - xFontGetWidth(m_xVersusArtistName.m_pFont, temp), BB_VERSUS_ARTISTNAME_TOP);
	xRenderable2DTextSetColor(&m_xVersusArtistName, xColorInit(&tempcolor, 1.0, 1.0, 1.0, 0.0));
	xRenderable2DTextSetText(&m_xVersusArtistName, &m_xSceneGraph, temp);

	m_dwVersusScreenStartTime = g_xPaperAPI.m_xPlatform.GetTime();

	xGraphicsSetSceneGraph(&m_xSceneGraph);
}

bool xGame::Init(const char *strLevelName)
{
	char levelfile[256] = {0};
	xFileIO fileio;
	char strline[25500] = {0};
	char path[256];
	xRenderable2DQuad *oldbg;
	xColor tempcolor1;
	DWORD songlength;
	float tempfloat;
	int width, i;

	g_xPaperAPI.m_xInput.m_bIgnoreInput = true;

	if(m_xPlayer.m_strCDKey[0] == '+')
		assert(0);

	CREATELOG();

	g_bCreditsRolling = !stricmp(strLevelName, "Credits");

	if(!g_bCreditsRolling)
	{
#if WITH_BALANCESTATS
		g_xBalanceStats.SetLevelName(strLevelName);
#endif

		CopyLevelPath(strLevelName, levelfile);
	}

	g_bTourComplete = 0;
	m_bFailed = 0;
	m_bPrePopupMoreLines = 0;
	m_pPrePopupPressEnterText = 0;
	m_iGameSyncFrameCount = 0;
	m_bFoundArrowStart = 0;
	m_bCountdown = 0;
	m_dEnemyIntroTime = 0;
	m_bStopSounds = 0;
	m_iTutorialStep = 0;
	m_bAllowFieldRotation = 1;
	m_xCrowdState = BB_CROWD_NONE;
	m_dwNextBossInteraction = g_xGameOptions.m_bBoss && (g_xGameOptions.m_iBossIndex == 2 || g_xGameOptions.m_iBossIndex == 4 || g_xGameOptions.m_iBossIndex == 5) ? 0 : 2000000000;
	memset(m_strAchievementsPending, 0, sizeof(char*) * BB_PENDINGACHIEVEMENTS_MAX);
	memset(m_iAchievementsUnlocking, 0, sizeof(int) * BB_PENDINGACHIEVEMENTS_MAX);
	m_iTutVoiceIndex = -1;
	m_iSenseiTutVoiceIndex = -1;
	m_xEnemyMoveState = BB_ENEMYMOVE_UNDECIDED;
	m_dwPlayerVoiceTime = 0;
	m_pVoiceSoundPlaying = 0;
	m_bShowTutorialDimmer = 0;

	if(!m_bDonePrecache)
	{
		xGamePrecache();
		m_bDonePrecache = 1;
	}

	if(strLevelName && strLevelName[0] != '\0')
	{
		g_xPaperAPI.m_iNetMode = PE_NETMODE_SINGLEPLAYER;
		g_xGameOptions.Delete();
	}

	m_xGUI.DelayedRemoveAllWindows();

	m_xGUI.Init(&m_xSceneGraph);

	m_xPlayer.m_xBeatManager.ResetBeats();

	HideAllKeys(1);

	if(!g_bCreditsRolling)
	{
		m_pParsedGameOptions = &g_xGameOptions;
		if(strLevelName && strLevelName[0] != '\0')
		{
			if(fileio.OpenFile(levelfile, true, false))
				return false;

			strcpy(g_xGameOptions.m_strLevelFile, levelfile);

			while(!fileio.ReadLine(strline, 25500))
				ParseLine(strline);

			fileio.CloseFile();

			g_xGameOptions.m_iDifficultyLevel = (xGameDifficulty)g_xGameOptions.m_iDifficultyLevels[g_xCurrentProfile.m_byteSelectedDifficulty];
		}

		sprintf(path, BB_MUSICFILE_PATH, g_xGameOptions.m_strSongName);
		if(fileio.OpenFile(path, 1, 0))
			return false;

		srand(g_xPaperAPI.m_xPlatform.GetTime());

		while(!fileio.ReadLine(strline, 25500))
			ParseLine(strline);

		fileio.CloseFile();

		if(g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER )
		{
			sprintf(path, BB_ENEMYFILE_PATH, g_xGameOptions.m_strEnemyName);

			if(fileio.OpenFile(path, 1, 0))
				return false;

			while(!fileio.ReadLine(strline, 25500))
				ParseLine(strline);

			fileio.CloseFile();
		}
	}
	else
		g_xGameOptions.m_fBPM = 100.0f;

	Pause(g_xPaperAPI.m_bHasFocus);
	m_bPrePopup = 1;

	if(m_pSenseiPrePopupVoice)
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pSenseiPrePopupVoice);
		m_pSenseiPrePopupVoice = 0;
	}

	if(!g_bCreditsRolling)
	{
		if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
		{
			m_bPlayedSenseiPrePopupSound = 1;

			if(!m_pRemotePlayer)
				m_pRemotePlayer = new xNetPlayer();

			if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
				AddServerPrePopupWindow();
			else
				AddClientPrePopupWindow();
		}
		else
		{
			if(g_xGameOptions.m_bTutorial || g_xGameOptions.m_strSenseiPrePopup[0] != '\0')
			{
				m_bRequireEnterPrePopup = 1;
				AddSenseiPrePopupWindow();

				if(!m_pSenseiPrePopupSound)
					m_pSenseiPrePopupSound = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_SENSEIPREPOPUP);

				if(g_xGameOptions.m_strSenseiPrePopupVoice)
					m_pSenseiPrePopupVoice = g_xPaperAPI.m_xAudio.LoadSound(g_xGameOptions.m_strSenseiPrePopupVoice);

				m_bPlayedSenseiPrePopupSound = g_xGameOptions.m_bTutorial;
			}
			else
			{
				m_bPlayedSenseiPrePopupSound = 1;
				m_bRequireEnterPrePopup = 1;

				if(g_xGameOptions.m_bSkipPrePopup)
					m_bPrePopup = 1;
				else
					AddPrePopupWindow();
			}
		}

		oldbg = m_pBackground;

		sprintf(path, BB_IMAGE_PATH, g_xGameOptions.m_strBGFile);

		m_pBackground = xSceneGraphCreateQuad(path, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pBackground, 1, BB_DEPTH_BACKGROUND, 0, 1);
		xRenderable2DQuadSetPosition(m_pBackground, -64, 0);
		xRenderable2DQuadSetSize(m_pBackground, 1152, 768);
		xRenderable2DQuadUpdate(m_pBackground);

		if(oldbg)
			xSceneGraphDeleteQuad(&m_xSceneGraph, oldbg, 1);
	}
	else
	{
		AddWindow("Credits");
		m_pVoiceSoundPlaying = g_xPaperAPI.m_xAudio.LoadSound("sounds/Sensei/YouWin.wav");
		m_pVoiceSoundPlaying->Play(g_xOptions.m_fVoiceOverVolume);
	}

	m_dwOldGameTime = 0;

	if(g_bCreditsRolling)
	{
		m_xSong.Load("Menu", 1);
		m_xSong.Play(0, g_xOptions.m_fMusicVolume);
	}
	else
		m_xSong.Load(g_xGameOptions.m_strSongName, g_xGameOptions.m_bLoopSong);

	if(g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER && g_xGameOptions.m_dwLevelCompletionTime == -1)
	{
		songlength = m_xSong.GetLength();

		if(songlength > m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iNumBeats - 1])
			g_xGameOptions.m_dwLevelCompletionTime = m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iNumBeats - 1] - 1000;
		else
			g_xGameOptions.m_dwLevelCompletionTime = songlength - 1000;
	}

	m_xPlayer.Reset();

	if(!m_pCountdown)
	{
		m_pCountdown = xSceneGraphCreateQuad(BB_IMAGE_COUNTDOWN, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pCountdown, 1, BB_DEPTH_COUNTDOWN, 0, 1);
		xGraphicsGetTextures(BB_IMAGE_COUNTDOWN, &m_pCountdownTextures);
		xRenderable2DQuadUpdate(m_pCountdown);
		for(i = 0; i < 3; i++)
			m_pCountdownTextures[i]->m_iReferenceCount--;
	}

	xRenderable2DQuadUpdateVisibility(m_pCountdown, false);

	if(!m_pCrowdBarArrow1)
	{
		m_pCrowdBarArrow1 = xSceneGraphCreateQuad(BB_IMAGE_CROWDBARARROW, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pCrowdBarArrow1, 1, BB_DEPTH_CROWDBARARROW, 0, 1);
		xRenderable2DQuadUpdate(m_pCrowdBarArrow1);

		m_pCrowdBarArrow2 = xSceneGraphCreateQuad(BB_IMAGE_CROWDBARARROW, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pCrowdBarArrow2, 1, BB_DEPTH_CROWDBARARROW, 0, 1);
		xRenderable2DQuadUpdate(m_pCrowdBarArrow2);

		m_fCrowdBarWidth = m_pCrowdBarArrow1->m_xRenderable2D.m_fWidth;
		m_fCrowdBarHeight = m_pCrowdBarArrow1->m_xRenderable2D.m_fHeight;
	}

	if(!g_bCreditsRolling)
	{
		tempfloat = (float)-g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX;
		
		m_fCrowdBarPercentage = 0.50f;
		m_fCrowdBarDisplayedPercentage = 0.49f;
		m_fCrowdBarMomentum = 0.0f;

		xGameUpdateCrowdBarArrows(m_pCrowdBarArrow1, m_pCrowdBarArrow2, m_fCrowdBarDisplayedPercentage);

		if(!m_xPlayerName.m_pFont)
		{
			m_pPlayerChargeBG = xSceneGraphCreateQuad(BB_IMAGE_PLAYERCHARGEBG, 0);
			xSceneGraphInsertQuad(&m_xSceneGraph, m_pPlayerChargeBG, 1, BB_DEPTH_PLAYERCHARGE_BG, 0, 1);
			xRenderable2DQuadSetPosition(m_pPlayerChargeBG, tempfloat, BB_PLAYER_CHARGE_BG_TOP);
			xRenderable2DQuadUpdate(m_pPlayerChargeBG);

			for(i = 0; i < 4; i++)
			{
				m_pPlayerCharges[i] = xSceneGraphCreateQuad(g_strPlayerChargeBars[i], 0);
				xSceneGraphInsertQuad(&m_xSceneGraph, m_pPlayerCharges[i], 1, BB_DEPTH_PLAYERCHARGE, 0, 1);
				xRenderable2DQuadSetPosition(m_pPlayerCharges[i], g_fPlayerChargeBarLocations[i][0] + tempfloat, g_fPlayerChargeBarLocations[i][1]);
				xRenderable2DQuadUpdatePartialY(m_pPlayerCharges[i], 0.0);

				m_pPlayerChargeOverlays[i] = xSceneGraphCreateQuad(g_strPlayerChargeOverlays[i], 0);
				xSceneGraphInsertQuad(&m_xSceneGraph, m_pPlayerChargeOverlays[i], 1, BB_DEPTH_PLAYERCHARGE_OVERLAY, 0, 1);
				xRenderable2DQuadSetPosition(m_pPlayerChargeOverlays[i], m_pPlayerCharges[i]->m_xRenderable2D.m_fX, m_pPlayerCharges[i]->m_xRenderable2D.m_fY);
				xRenderable2DQuadUpdatePartialY(m_pPlayerChargeOverlays[i], 0.0);
			}

			m_pPlayerNameBG = xSceneGraphCreateQuad(BB_IMAGE_PLAYERNAMEBG, 0);
			xSceneGraphInsertQuad(&m_xSceneGraph, m_pPlayerNameBG, 9, BB_DEPTH_PLAYERNAME_BG, 0, 1);

			xRenderable2DTextInit(&m_xPlayerName, BB_DEPTH_PLAYERNAME);
			xRenderable2DTextSetFont(&m_xPlayerName, g_xGame.m_xGUI.GetFont(BB_FONT_NAMES));
			xRenderable2DTextSetColor(&m_xPlayerName, xColorInit(&tempcolor1, 1.0f, 1.0f, 1.0f, 0.8f));
		}
		else
		{
			for(i = 0; i < 4; i++)
			{
				xRenderable2DQuadUpdatePartialY(m_pPlayerCharges[i], 0.0);
				xRenderable2DQuadUpdatePartialY(m_pPlayerChargeOverlays[i], 0.0);
			}
		}

		if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER && m_pRemotePlayer->m_iRank > 0)
		{
			xRenderable2DTextSetFont(&m_xPlayerRank, g_xGame.m_xGUI.GetFont(BB_FONT_RANK));
			itoa(m_xPlayer.m_iRank, m_strPlayerRank, 10);
			xRenderable2DTextSetText(&m_xPlayerRank, &m_xSceneGraph, m_strPlayerRank);
		}

		if((g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER && g_xCurrentProfile.m_strProfileName[0] != '\0') || (g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER && m_xPlayer.m_strAccountName[0] != '\0'))
		{
			if(g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
			{
				width = xFontGetWidth(m_xPlayerName.m_pFont, g_xCurrentProfile.m_strProfileName);
				xRenderable2DTextSetPosition(&m_xPlayerName, BB_PLAYER_NAME_LEFT + tempfloat, BB_PLAYER_NAME_TOP);
				xRenderable2DTextSetText(&m_xPlayerName, &m_xSceneGraph, g_xCurrentProfile.m_strProfileName);
			}
			else
			{
				width = xFontGetWidth(m_xPlayerName.m_pFont, m_xPlayer.m_strAccountName) + xFontGetWidth(m_xPlayerRank.m_pFont, m_strPlayerRank);
				xRenderable2DTextSetPosition(&m_xPlayerName, BB_PLAYER_NAME_LEFT + tempfloat, BB_PLAYER_NAME_TOP);
				xRenderable2DTextSetText(&m_xPlayerName, &m_xSceneGraph, m_xPlayer.m_strAccountName);
			}

			xRenderable2DQuadSetPosition(m_pPlayerNameBG, BB_PLAYER_NAME_BG_LEFT + tempfloat, BB_PLAYER_NAME_BG_TOP);
			xRenderable2DQuadSetSize(m_pPlayerNameBG, (width < 50 ? 50 : width) + 14, m_pPlayerNameBG->m_xRenderable2D.m_fHeight);
			xRenderable2DQuad9Update(m_pPlayerNameBG);

			InitializeEnemyName();
		}

		if(!m_pPlayerModel)
		{
			m_pPlayerModel = new xMD3Model();
			m_pPlayerModel->Init("Player", BB_PLAYERANIMATION_MAX, &m_xSceneGraph, BB_DEPTH_PLAYERMODEL);
			m_pPlayerModel->pfnOnAnimationCompleted = xGameOnPlayerAnimationCompleted;
		}

		m_pPlayerModel->SetAnimation(BB_PLAYERANIMATION_INTRO, g_xGameOptions.m_fBPM);
		m_pPlayerModel->SetTranslation(20.0, 68.0, -179.0);
		m_pPlayerModel->SetRotation(0.0, 0.0, 210.0);
	}

	m_fEnemyDamage = ((float)g_xGameOptions.m_iEnemyDamage / 1000.0f) * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
	m_dwNextEnemyMoveTime = 0;
	m_iEnemyMovesExecuted = 0;
	m_dwUsedEnemy3MoveTime = 0;

#ifdef _DEBUG
	for(i = 0; i < BB_PLAYERANIMATION_MAX; i++)
		LOG(temp, "ANIMATION: AnimIndex=%i Start=%i End=%i NumBeats=%i\n", i, m_pPlayerModel->m_pAnimations[i].m_iFirstFrame, m_pPlayerModel->m_pAnimations[i].m_iLastFrame, m_pPlayerModel->m_pAnimations[i].m_iNumBeats);
#endif

	if(!m_pCrowdLoopSound)
	{
		m_pCrowdLoopSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_CROWDLOOP);
		m_pCrowdLoopExcitedSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_CROWDLOOPEXCITED);
		m_pCrowdPositiveSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_CROWDPOSITIVE);
		m_pCrowdNegativeSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_CROWDNEGATIVE);
		m_pCrowdGameWonSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_CROWDGAMEWON);
		m_pCrowdGameLostSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_CROWDGAMELOST);
	}

	if(m_pEnemyModel)
	{
		m_pEnemyModel->Delete();
		delete m_pEnemyModel;
		m_pEnemyModel = 0;
	}

	if(m_pEnemyIntroSound)
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyIntroSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyIntroVoiceSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyPreMoveSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyMoveSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyPostMoveSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyReactionSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pWonVoiceSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pFailureVoiceSound);

		m_pEnemyIntroSound = 0;
		m_pEnemyIntroVoiceSound = 0;
		m_pEnemyPreMoveSound = 0;
		m_pEnemyMoveSound = 0;
		m_pEnemyPostMoveSound = 0;
		m_pEnemyReactionSound = 0;
		m_pWonVoiceSound = 0;
		m_pFailureVoiceSound = 0;
	}

	if(!g_bCreditsRolling && g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER && g_xGameOptions.m_strEnemyName[0] != '\0')
	{
		m_pEnemyModel = new xMD3Model();
		m_pEnemyModel->Init(g_xGameOptions.m_strEnemyName, BB_ENEMYANIMATION_MAX, &m_xSceneGraph, BB_DEPTH_ENEMYMODEL);
		m_pEnemyModel->pfnOnAnimationCompleted = xGameOnEnemyAnimationCompleted;

		m_pEnemyModel->SetAnimation(BB_ENEMYANIMATION_INTRO, g_xGameOptions.m_fBPM);
		m_pEnemyModel->SetTranslation(20.0, 70.0, -179.0);
		m_pEnemyModel->SetRotation(0.0, 0.0, -160.0);

		sprintf(path, BB_SOUND_ENEMYINTRO, g_xGameOptions.m_strEnemyName);
		m_pEnemyIntroSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path);

		sprintf(path, BB_SOUND_ENEMYMOVE, g_xGameOptions.m_strEnemyName);
		m_pEnemyMoveSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path);

		sprintf(path, BB_SOUND_ENEMYINTROVOICE, g_xGameOptions.m_strEnemyName);
		m_pEnemyIntroVoiceSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_ENEMYPREMOVE, g_xGameOptions.m_strEnemyName);
		m_pEnemyPreMoveSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_ENEMYPOSTMOVE, g_xGameOptions.m_strEnemyName);
		m_pEnemyPostMoveSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_ENEMYREACTION, g_xGameOptions.m_strEnemyName);
		m_pEnemyReactionSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_WON, g_xGameOptions.m_strEnemyName);
		m_pWonVoiceSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_FAILURE, g_xGameOptions.m_strEnemyName);
		m_pFailureVoiceSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);
	}
	else if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER && !g_bCreditsRolling)
	{
		m_pEnemyModel = new xMD3Model();
		m_pEnemyModel->Init("Player", BB_ENEMYANIMATION_MAX, &m_xSceneGraph, BB_DEPTH_ENEMYMODEL);
		m_pEnemyModel->pfnOnAnimationCompleted = xGameOnEnemyAnimationCompleted;

		m_pEnemyModel->SetAnimation(BB_ENEMYANIMATION_INTRO, g_xGameOptions.m_fBPM);
		m_pEnemyModel->SetTranslation(20.0, 70.0, -179.0);
		m_pEnemyModel->SetRotation(0.0, 0.0, -160.0);

		sprintf(path, BB_SOUND_ENEMYINTRO, "Sensei");
		m_pEnemyIntroSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path);

		sprintf(path, BB_SOUND_ENEMYMOVE, "Sensei");
		m_pEnemyMoveSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path);

		sprintf(path, BB_SOUND_ENEMYINTROVOICE, "Sensei");
		m_pEnemyIntroVoiceSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_ENEMYPREMOVE, "Sensei");
		m_pEnemyPreMoveSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_ENEMYPOSTMOVE, "Sensei");
		m_pEnemyPostMoveSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_ENEMYREACTION, "Sensei");
		m_pEnemyReactionSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_WON, "Sensei");
		m_pWonVoiceSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);

		sprintf(path, BB_SOUND_FAILURE, "Sensei");
		m_pFailureVoiceSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(path, 1);
	}

	if(!m_pMessageFont)
		m_pMessageFont = g_xGame.m_xGUI.GetFont(BB_FONT_TEXTMESSAGES);

	if(!m_pCountdownSounds[0])
	{
		m_pCountdownSounds[0] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_COUNTDOWN1);
		m_pCountdownSounds[1] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_COUNTDOWN2);
		m_pCountdownSounds[2] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_COUNTDOWN3);
	}

	if(!m_pAchievementIcon[0])
	{
		xGraphicsGetTextures(BB_IMAGE_ACHIEVEMENTICON, &m_pAchievementIconTextures);

		for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
		{
			m_pAchievementIcon[i] = xSceneGraphCreateQuad(BB_IMAGE_ACHIEVEMENTICON, 0);
			xSceneGraphInsertQuad(&m_xSceneGraph, m_pAchievementIcon[i], 1, BB_DEPTH_ACHIEVEMENTICON, 0, 1);
			xRenderable2DQuadSetPosition(m_pAchievementIcon[i], 1025, 769);
			xRenderable2DQuadSetSize(m_pAchievementIcon[i], BB_ACHIEVEMENT_ICONSIZE, BB_ACHIEVEMENT_ICONSIZE);
			xRenderable2DQuadUpdate(m_pAchievementIcon[i]);

			xRenderable2DTextInit(&m_xAchievementTitle[i], BB_DEPTH_ACHIEVEMENTTITLE);
			xRenderable2DTextSetFont(&m_xAchievementTitle[i], g_xGame.m_xGUI.GetFont(BB_FONT_ACHIEVEMENTTITLE));
			xRenderable2DTextSetPosition(&m_xAchievementTitle[i], 1025, 769);
			xRenderable2DTextSetScale(&m_xAchievementTitle[i], BB_ACHIEVEMENT_TITLESCALE, BB_ACHIEVEMENT_TITLESCALE);
			xRenderable2DTextSetColor(&m_xAchievementTitle[i], xColorInit(&tempcolor1, 1.0f, 1.0f, 1.0f, 1.0f));
		}
	}
	
	xGUIWindowList *window = m_xGUI.GetWindow(BB_WINDOW_GAMEPAUSED);
	if(window)
		m_xGUI.DelayedRemoveWindow(window);

	songlength = m_xSong.GetLength();

	if(songlength > m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iNumBeats - 1])
		InitTimeString((m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iNumBeats - 1] / 1000) + 1);
	else
		InitTimeString(songlength / 1000);
		
	if(!m_pPauseButton)
	{
		m_pPauseButton = xSceneGraphCreateQuad(BB_IMAGE_PAUSE_BUTTON, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pPauseButton, 1, BB_DEPTH_PAUSE_BUTTON, 0, 1);
		xRenderable2DQuadSetPosition(m_pPauseButton, 8 - (float)g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX, 8);
		xRenderable2DQuadUpdate(m_pPauseButton);
	}

	xGraphicsSetSceneGraph(&m_xSceneGraph);

	return true;
}

bool xGame::PreUpdate(DWORD dwTime)
{
	xColor tempcolor;
	float timerscale, colorscale, tempfloat;
	bool timechanged = 0;
	DWORD tempdword;
	int x, y, i;

	if(m_bVersusScreenAnimating)
		return 1;

	if(m_bPrePopup && g_xGameOptions.m_bSkipPrePopup)
	{
		m_bPrePopup = 0;
		m_dCountdownTime = 2.9999999;
		m_bCountdown = 1;

		UnPause();
	}

	if(!m_bPlayedSenseiPrePopupSound && m_pSenseiPrePopupSound)
	{
		m_pSenseiPrePopupSound->Play(g_xOptions.m_fSoundVolume);

		if(m_pSenseiPrePopupVoice)
			PlayVoiceSound(m_pSenseiPrePopupVoice);

		m_bPlayedSenseiPrePopupSound = 1;
	}

	if(m_xCrowdState == BB_CROWD_NONE)
	{
		m_pCrowdLoopSound->Play(g_xOptions.m_fSoundVolume, 1);
		m_xCrowdState = BB_CROWD_NORMAL;
	}

	g_xPaperAPI.m_xInput.m_bIgnoreInput = 0;

#ifdef DEMO_RELEASE
	if(m_pBuyItBackground)
	{
		if(g_xPaperAPI.m_xInput.m_buttonMouseLeft.JustPressed())
		{
			if(g_xPaperAPI.m_xInput.m_lMouseX >= m_pBuyItCloseButton->m_xRenderable2D.m_fX && g_xPaperAPI.m_xInput.m_lMouseY >= m_pBuyItCloseButton->m_xRenderable2D.m_fY &&
				g_xPaperAPI.m_xInput.m_lMouseX <= m_pBuyItCloseButton->m_xRenderable2D.m_fX + m_pBuyItCloseButton->m_xRenderable2D.m_fWidth &&
				g_xPaperAPI.m_xInput.m_lMouseY <= m_pBuyItCloseButton->m_xRenderable2D.m_fY + m_pBuyItCloseButton->m_xRenderable2D.m_fHeight)
			{
				g_xGame.Shutdown();
				g_xMenu.PlayMusic();

				g_xMenu.m_xGUI.DelayedRemoveAllWindows();
				g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MAIN);

				g_pCurGameState = &g_xMenu;
				xGraphicsSetSceneGraph(&g_xMenu.m_xSceneGraph);

				g_xGame.m_xGUI.RemoveAllWindows();
			}
			else
			{
				extern void LoadAppStore();
				LoadAppStore();
			}
		}
	}
#endif

	//DEBUGBOSS("Checking Next Boss Interaction Time: %i %i %u > %u\n", g_xGameOptions.m_bBoss, g_xGameOptions.m_iBossIndex, dwTime, m_dwNextBossInteraction);

	if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
	{
		HandleNetworkPackets();

		if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
		{
			if(!m_bFinishedConnecting)
			{
				HandlePingPackets();
			
				if(!m_bReceivedPing && g_xPaperAPI.m_dwCurrentTime - m_dwServerStartTime > 60000)
				{
					//Add the Server Port Warning window if the player hasn't seen it
					if(g_xCurrentProfile.m_byteNoServerPortWarning == 0)
						AddWindow(BB_WINDOW_MESSAGEBOX_SERVERPORTWARNING);

					m_bReceivedPing = true;
				}
			}
		}

		if(m_bFinishedConnecting)
		{
			m_xSong.ContinuePlaying();

			if(m_xPlayer.m_bViewingPostPopup && !g_xPaperAPI.m_xNetwork.IsConnected())
			{
				xGUIWindowList *window;

				if(g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
					window = m_xGUI.GetWindow(BB_WINDOW_POSTPOPUPCLIENT);
				else
					window = m_xGUI.GetWindow(BB_WINDOW_POSTPOPUPSERVER);

				if(window)
				{
					if(g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_POSTPOPUP_CLIENTSERVERQUITLINE2);
					else
					{
						ResetListenServerConnection();

						if(((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_REMATCHBUTTON))->m_bDisabled)
						{
							m_xGUI.RemoveAllWindows();

							Init("");
							return true;
						}
						else
						{
							((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_POSTPOPUP_SERVERCLIENTQUITLINE2);
							((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE3))->SetText(BB_STRING_POSTPOPUP_SERVERCLIENTQUITLINE3);
						}
					}
				}
			}

			if(m_bPrePopup)
			{
				if(g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
					m_xGUI.DelayedRemoveWindow(BB_WINDOW_PREPOPUPCLIENT);
				else
				{
					m_xGUI.DelayedRemoveWindow(BB_WINDOW_PREPOPUPSERVER);
					m_xHTTPQuery.OpponentConnected(1984, m_xPlayer.m_strAccountName, g_xGameOptions.m_strEnemyDisplayName);
				}

				m_bPrePopup = 0;

				m_dCountdownTime = 2.9999999;
				m_bCountdown = 1;

				UnPause();
			}
		}
		else if(!g_xPaperAPI.m_xNetwork.IsConnected())
		{
			if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
			{
				if(m_iConnectingStatus > 0)
				{
					m_xGUI.RemoveAllWindows();

					ResetListenServerConnection();

					Init("");
				}
			}
			else
			{
				if(m_iConnectingStatus < 6)
				{
					g_xPaperAPI.m_xNetwork.Close();

					Shutdown();
					g_xMenu.PlayMusic();

					//Show the Player Moves window
					g_xMenu.RemoveAllMenuWindows();
					g_xMenu.AddMainMenu();
					g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MULTIPLAYER);

					g_pCurGameState = &g_xMenu;
					xGraphicsSetSceneGraph(&g_xMenu.m_xSceneGraph);

					m_xGUI.RemoveAllWindows();
					return 1;
				}
				else
				{
					g_xPaperAPI.m_xNetwork.Close();

					g_xGameOptions.m_strEnemyDisplayName[0] = ' ';
					g_xGameOptions.m_strEnemyDisplayName[1] = '\0';

					xGUIWindowList *window = m_xGUI.GetWindow(BB_WINDOW_PREPOPUPCLIENT);
					if(window)
						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_PREPOPUP_SERVERQUIT);
				}
			}
		}
	}
	else
	{
		m_xSong.ContinuePlaying();

		if(g_xGameOptions.m_bBoss && dwTime > m_dwNextBossInteraction && (g_xGameOptions.m_iBossIndex == 2 || g_xGameOptions.m_iBossIndex == 4 || g_xGameOptions.m_iBossIndex == 5))
		{
			DEBUGBOSS("Attempting to execute Boss Move 2/4/5\n");

			for(y = 0; y < BB_FIELDHEIGHT; y++)
			{
				for(x = 0; x < BB_FIELDWIDTH; x++)
				{
					if(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_iColorIndex > 0 && rand() % 5 == 0)
					{
						do
						{
							m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_iNewColorIndex = rand() % g_xGameOptions.m_iNumRandomColors;
						} while(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_iNewColorIndex == m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_iColorIndex);

						DEBUGBOSS("Boss Move 2/4/5 executed\n");
						m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_dwColorAnimStartTime = dwTime;
						y = BB_FIELDHEIGHT;
						break;
					}
				}
			}

			m_dwNextBossInteraction = dwTime + g_xGameOptions.m_dwBossMoveMinTime + rand() % g_xGameOptions.m_dwBossMoveRandTime;
		}
	}

	if(g_xPaperAPI.m_bHasFocus)
	{
		if(m_bFinishedConnecting)
		{
			if(m_bPaused)
			{
				if(g_xPaperAPI.m_xInput.m_buttonMouseLeft.JustPressed(0) &&
					g_xPaperAPI.m_xInput.m_lMouseX >= m_pPauseButton->m_xRenderable2D.m_fX && g_xPaperAPI.m_xInput.m_lMouseY >= m_pPauseButton->m_xRenderable2D.m_fY &&
						g_xPaperAPI.m_xInput.m_lMouseX <= m_pPauseButton->m_xRenderable2D.m_fX + m_pPauseButton->m_xRenderable2D.m_fWidth &&
						g_xPaperAPI.m_xInput.m_lMouseY <= m_pPauseButton->m_xRenderable2D.m_fY + m_pPauseButton->m_xRenderable2D.m_fHeight)
				{
					if(!m_xGUI.GetWindow(BB_WINDOW_GAMEPAUSED))
					{
						if(!m_bVersusScreenAnimating)
						{
							g_xPaperAPI.m_xInput.m_buttonMouseLeft.Eaten();
							Pause(g_xPaperAPI.m_bHasFocus);
							m_xGUI.AddWindow(BB_WINDOW_GAMEPAUSED);
						}
					}
					else if(g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
					{
						g_xPaperAPI.m_xInput.m_buttonMouseLeft.Eaten();

						m_xGUI.DelayedRemoveWindow(BB_WINDOW_GAMEPAUSED);
						UnPause();
					}
				}
				else if(m_xPlayer.m_bDead)
					m_xPlayer.PreUpdate(dwTime);
				else if(m_bPrePopup && g_xPaperAPI.m_xInput.m_buttonMouseLeft.JustPressed(0) &&
						(!stricmp(m_xGUI.m_pWindowList->m_pData->m_strName, BB_WINDOW_SENSEIPREPOPUP) ||
						 !stricmp(m_xGUI.m_pWindowList->m_pData->m_strName, BB_WINDOW_PREPOPUP)))
				{
					g_xPaperAPI.m_xInput.m_buttonMouseLeft.Eaten();
					Ready();
				}
			}
			else
			{
				if(g_xPaperAPI.m_xInput.m_buttonMouseLeft.JustPressed(1))
				{
					if(g_xPaperAPI.m_xInput.m_lMouseX >= m_pPauseButton->m_xRenderable2D.m_fX && g_xPaperAPI.m_xInput.m_lMouseY >= m_pPauseButton->m_xRenderable2D.m_fY &&
						g_xPaperAPI.m_xInput.m_lMouseX <= m_pPauseButton->m_xRenderable2D.m_fX + m_pPauseButton->m_xRenderable2D.m_fWidth &&
						g_xPaperAPI.m_xInput.m_lMouseY <= m_pPauseButton->m_xRenderable2D.m_fY + m_pPauseButton->m_xRenderable2D.m_fHeight)
					{
						g_xPaperAPI.m_xInput.m_buttonMouseLeft.Eaten();

						if(!m_bVersusScreenAnimating)
						{
							Pause(g_xPaperAPI.m_bHasFocus);
							m_xGUI.AddWindow(BB_WINDOW_GAMEPAUSED);
						}
					}
				}
			
				if(!m_bCountdown)
				{
					m_iGameSyncFrameCount++;
					if(m_iGameSyncFrameCount > 10)
					{
						m_dwGameStartTime = dwTime - m_xSong.GetTime();
						m_iGameSyncFrameCount = 0;
					}

					if(m_pEnemyModel && g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
					{
						if(m_dwNextEnemyMoveTime == 0)
						{
							m_dwLastEnemyMoveTime = dwTime - m_dwGameStartTime;
							m_dwNextEnemyMoveTime = m_dwLastEnemyMoveTime + g_xGameOptions.m_dwEnemyFirstMoveDelay;
							m_dwUsedEnemy3MoveTime = 0;
							m_dwEnemyMoveFlashTime = 0;
						}
						else
						{
							if(m_dwEnemyMoveFlashTime > 0)
							{
								if(!m_pEnemyChargeOverlay->m_xRenderable2D.m_bVisible)
								{
									xRenderable2DQuadUpdatePartialY(m_pEnemyCharge, 1.0);
									xRenderable2DQuadUpdatePartialY(m_pEnemyChargeOverlay, 1.0);
									xRenderable2DQuadUpdateVisibility(m_pEnemyChargeOverlay, 1);
								}

								tempdword = (dwTime - m_dwGameStartTime) - m_dwEnemyMoveFlashTime;
								if(tempdword < BB_CHARGE_FLASH_TOTAL)
								{
									if(tempdword < BB_CHARGE_FLASH1)
										xRenderable2DQuadUpdateAlpha(m_pEnemyChargeOverlay, (float)tempdword / BB_CHARGE_FLASH1);
									else if(tempdword < BB_CHARGE_UNFLASH1)
										xRenderable2DQuadUpdateAlpha(m_pEnemyChargeOverlay, 1.0 - ((float)(tempdword - BB_CHARGE_UNFLASH1) / (BB_CHARGE_UNFLASH1 - BB_CHARGE_FLASH1)));
									else if(tempdword < BB_CHARGE_FLASH2)
										xRenderable2DQuadUpdateAlpha(m_pEnemyChargeOverlay, (float)(tempdword - BB_CHARGE_FLASH2) / (BB_CHARGE_FLASH2 - BB_CHARGE_UNFLASH1));
									else if(tempdword < BB_CHARGE_UNFLASH2)
										xRenderable2DQuadUpdateAlpha(m_pEnemyChargeOverlay, 1.0 - ((float)(tempdword - BB_CHARGE_UNFLASH2) / (BB_CHARGE_UNFLASH2 - BB_CHARGE_FLASH2)));
									else if(tempdword < BB_CHARGE_FLASH3)
										xRenderable2DQuadUpdateAlpha(m_pEnemyChargeOverlay, (float)(tempdword - BB_CHARGE_FLASH3) / (BB_CHARGE_FLASH3 - BB_CHARGE_UNFLASH2));
									else
										xRenderable2DQuadUpdateAlpha(m_pEnemyChargeOverlay, 1.0 - ((float)(tempdword - BB_CHARGE_UNFLASH3) / (BB_CHARGE_UNFLASH3 - BB_CHARGE_FLASH3)));
								}
								else
								{
									xRenderable2DQuadUpdatePartialY(m_pEnemyChargeOverlay, 0.0);
									xRenderable2DQuadUpdateVisibility(m_pEnemyChargeOverlay, 0);
									m_dwEnemyMoveFlashTime = 0;
								}
							}
							else
							{
								xRenderable2DQuadUpdatePartialY(m_pEnemyCharge, (float)((dwTime - m_dwGameStartTime) - m_dwLastEnemyMoveTime) / (float)(m_dwNextEnemyMoveTime - m_dwLastEnemyMoveTime));
								xRenderable2DQuadUpdatePartialY(m_pEnemyChargeOverlay, (float)((dwTime - m_dwGameStartTime) - m_dwLastEnemyMoveTime) / (float)(m_dwNextEnemyMoveTime - m_dwLastEnemyMoveTime));
							}

							if(m_xEnemyMoveState == BB_ENEMYMOVE_UNDECIDED)
							{
								if(!g_xGameOptions.m_bTutorial && dwTime - m_dwGameStartTime >= m_dwNextEnemyMoveTime - 1500)
								{
									if(rand() % 2 == 0)
									{
										if(dwTime > m_dwPlayerVoiceTime)
											PlayVoiceSound(m_pEnemyPreMoveSound);

										m_xEnemyMoveState = BB_ENEMYMOVE_PREMOVE;
									}
									else
									{
										m_xEnemyMoveState = BB_ENEMYMOVE_POSTMOVE;
										m_dwEnemyPostMoveSoundTime = 2999999999;
									}

									DEBUGVOICE("ENEMY VOICE: %u Decided %i\n", dwTime, (int)m_xEnemyMoveState);
								}
							}
							else if(dwTime - m_dwGameStartTime >= m_dwNextEnemyMoveTime)
							{
	#if _DEBUG && 0
								int randnum = BB_ENEMYANIMATION_ENEMYMOVE1 + rand() % 2;
								DEBUGLOG("Setting Enemy Animation: %i\n", randnum);
								m_pEnemyModel->SetAnimation(randnum, g_xGameOptions.m_fBPM);
	#else
								m_xPlayer.m_dEnemyMoveApplauseDelay = 0.0;
								m_pEnemyModel->SetAnimation(BB_ENEMYANIMATION_ENEMYMOVE1 + rand() % 2, g_xGameOptions.m_fBPM);
	#endif
								if(!m_bStopSounds)
									m_pEnemyMoveSound->Play(g_xOptions.m_fSoundVolume);

								AdjustCrowdBar(m_fEnemyDamage);

								if(m_xEnemyMoveState == BB_ENEMYMOVE_POSTMOVE)
								{
									m_dwEnemyPostMoveSoundTime = dwTime + 2000;
									DEBUGVOICE("ENEMY VOICE: %u Post Move Sound Time set to %u\n", dwTime, m_dwEnemyPostMoveSoundTime);
								}
								else
								{
									DEBUGVOICE("ENEMY VOICE: %u from Pre Move to Undecided\n", dwTime);
									m_xEnemyMoveState = BB_ENEMYMOVE_UNDECIDED;
								}

								m_iEnemyMovesExecuted++;

								switch(m_iEnemyMovesExecuted)
								{
									case 3:
										m_iEnemyMovesExecuted = 0;
										m_dwUsedEnemy3MoveTime = 0;
									case 1:
									{
										DWORD randomtime = g_xGameOptions.m_dwMinEnemyMoveDelay + rand() % (g_xGameOptions.m_dwEnemyMoveDelay - g_xGameOptions.m_dwMinEnemyMoveDelay);
										m_dwNextEnemyMoveTime = (dwTime - m_dwGameStartTime) + randomtime;
										m_dwUsedEnemy3MoveTime += randomtime;
										break;
									}
									case 2:
										m_dwNextEnemyMoveTime = (dwTime - m_dwGameStartTime) + (g_xGameOptions.m_dwEnemyMoveDelay * 3 - m_dwUsedEnemy3MoveTime);
										break;
								}

								m_dwLastEnemyMoveTime = dwTime - m_dwGameStartTime;
								m_dwEnemyMoveFlashTime = m_dwLastEnemyMoveTime;
							}
						}

						if(m_xEnemyMoveState == BB_ENEMYMOVE_POSTMOVE && dwTime > m_dwEnemyPostMoveSoundTime)
						{
							DEBUGVOICE("ENEMY VOICE: %u Playing Post Move %u\n", dwTime, m_dwPlayerVoiceTime);
							if(dwTime > m_dwPlayerVoiceTime)
								PlayVoiceSound(m_pEnemyPostMoveSound);

							m_xEnemyMoveState = BB_ENEMYMOVE_POSTMOVEDONE;
						}
						else if(m_xEnemyMoveState == BB_ENEMYMOVE_POSTMOVEDONE && dwTime - m_dwEnemyPostMoveSoundTime > 2000)
						{
							DEBUGVOICE("ENEMY VOICE: %u from Post Move Done to Undecided %u\n", dwTime, m_dwEnemyPostMoveSoundTime);
							m_xEnemyMoveState = BB_ENEMYMOVE_UNDECIDED;
						}
					}

					m_xPlayer.PreUpdate(dwTime);
				}
				else
				{
					if(!m_bPrePopup && (g_xOptions.m_pButtons[KEY_GOTOMENU]->JustPressed(true) || g_xOptions.m_pButtons[KEY_PAUSE]->JustPressed(true)))
					{
						g_xOptions.m_pButtons[KEY_GOTOMENU]->Eaten();
						g_xOptions.m_pButtons[KEY_PAUSE]->Eaten();

						if(!m_bVersusScreenAnimating)
						{
							Pause(g_xPaperAPI.m_bHasFocus);
							m_xGUI.AddWindow(BB_WINDOW_GAMEPAUSED);
						}
					}
				}
			}

			if(!m_bPaused && m_xSong.GetTime() - m_dwCurrentTime >= 1000 && (m_strTime[0] > 32/*'0'*/ || m_strTime[2] > 32/*'0'*/ || m_strTime[3] > 32/*'0'*/))
			{
				DEBUGTIMER("Timer - Decrementing - %s\n", m_strTime);
				if(m_strTime[3] > 32) // '0'
					m_strTime[3]--;
				else
				{
					m_strTime[3] = 41; // '9'

					if(m_strTime[2] > 32) // '0'
						m_strTime[2]--;
					else
					{
						m_strTime[2] = 37; // '5'
						m_strTime[0]--;
					}
				}

				m_dwCurrentTime += 1000;

				timechanged = 1;

				// Every minute
				if(m_strTime[2] == 32/*'0'*/ && m_strTime[3] == 33/*'1'*/)
				{
					m_dwNextTimerGrowTime = dwTime + 750;
					m_bColorizeTimer = false;
				}
				// 30 seconds remaining or 10 seconds or less
				else if(m_strTime[0] == 32/*'0'*/ && ((m_strTime[2] == 35/*'3'*/ && m_strTime[3] == 33/*'1'*/) || (m_strTime[2] == 33/*'1'*/ && m_strTime[3] < 33/*'1'*/) || m_strTime[2] == 32/*'0'*/))
				{
					m_dwNextTimerGrowTime = dwTime + 750;
					m_bColorizeTimer = true;
				}

				if(m_dwTimerGrowTime == 0 && m_dwNextTimerGrowTime)
				{
					m_dwTimerGrowTime = m_dwNextTimerGrowTime;
					m_dwNextTimerGrowTime = 0;
				}

				DEBUGTIMER("Timer - After Decrementing - %s\n", m_strTime);
			}

			if(m_dwTimerGrowTime && m_dwTimerGrowTime <= dwTime)
			{
				if(dwTime - m_dwTimerGrowTime >= 750)
				{
					m_dwTimerGrowTime = m_dwNextTimerGrowTime;
					m_dwNextTimerGrowTime = 0;
					colorscale = 0.0f;
				}
				else
				{
					if(dwTime - m_dwTimerGrowTime <= 250)
						colorscale = (float)(dwTime - m_dwTimerGrowTime) / 250.0f;
					else
						colorscale = 1.0f - (float)((dwTime - m_dwTimerGrowTime) - 250) / 500.0f;

					colorscale = (colorscale * colorscale * colorscale * colorscale);
				}

				timerscale = colorscale / 2.0f + 1.0f;
					
				xRenderable2DTextSetScale(&m_xTimer, timerscale, timerscale);
				m_xTimer.m_xRenderable.m_xRenderable2D.m_fX = BB_TIME_CENTER - xFontGetWidth(m_xTimer.m_pFont, m_strTime) * timerscale / 2.0f;
				m_xTimer.m_xRenderable.m_xRenderable2D.m_fY = BB_TIME_TOP - (m_xTimer.m_pFont->m_iSize * (timerscale - 1.0f)) / 2.0f;
				DEBUGTIMER("Timer - Growing - %s\n", m_strTime);
				xRenderable2DTextSetText(&m_xTimer, &m_xSceneGraph, m_strTime);

				if(m_bColorizeTimer)
					xRenderable2DTextUpdateColor(&m_xTimer, xColorInit(&tempcolor, 1.0f - colorscale * 0.15f, 1.0f - colorscale * 0.75f, 1.0f - colorscale * 0.75f, 0.8f));
			}
			else if(timechanged)
			{
				DEBUGTIMER("Timer - timechanged - %s\n", m_strTime);
				xRenderable2DTextSetColor(&m_xTimer, xColorInit(&tempcolor, 1.0f, 1.0f, 1.0f, 0.8f));
				xRenderable2DTextSetScale(&m_xTimer, 1.0f, 1.0f);
				m_xTimer.m_xRenderable.m_xRenderable2D.m_fX = BB_TIME_CENTER - xFontGetWidth(m_xTimer.m_pFont, m_strTime) / 2.0f;
				m_xTimer.m_xRenderable.m_xRenderable2D.m_fY = BB_TIME_TOP;
				xRenderable2DTextSetText(&m_xTimer, &m_xSceneGraph, m_strTime);
			}

			if(g_xGameOptions.m_bTutorial && (m_bPrePopup || m_bSenseiPrePopup))
				m_fBeatPercentage = 0.5f;
			else if(m_xPlayer.m_xPiece.m_dwCurTime > m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iCurBeatIndex])
				m_fBeatPercentage = (float)(m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iCurBeatIndex + 1] - m_xPlayer.m_xPiece.m_dwCurTime) / (float)(m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iCurBeatIndex + 1] - m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iCurBeatIndex]);
			else if(m_xPlayer.m_xBeatManager.m_iCurBeatIndex > 0)
				m_fBeatPercentage = (float)(m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iCurBeatIndex] - m_xPlayer.m_xPiece.m_dwCurTime) / (float)(m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iCurBeatIndex] - m_xPlayer.m_xBeatManager.m_dwBeatTimes[m_xPlayer.m_xBeatManager.m_iCurBeatIndex - 1]);
			else
				m_fBeatPercentage = (float)(m_xPlayer.m_xBeatManager.m_dwBeatTimes[0] - m_xPlayer.m_xPiece.m_dwCurTime) / (float)m_xPlayer.m_xBeatManager.m_dwBeatTimes[0];

			if(m_fBeatPercentage <= 0.50f)
				m_fBeatPercentage = (0.5f - m_fBeatPercentage) * 2.0f;
			else
				m_fBeatPercentage = (m_fBeatPercentage - 0.5f) * 2.0f;

			m_fBeatPercentageLinear = m_fBeatPercentage;

			m_fBeatPercentage = m_fBeatPercentage <= 1.0f ? (m_fBeatPercentage * m_fBeatPercentage * m_fBeatPercentage * m_fBeatPercentage) : 1.0f;

			for(i = 0; i < 4; i++)
			{
				if(m_dwPlayerMoveFlashTime != 0)
				{
					tempdword = (dwTime - m_dwGameStartTime) - m_dwPlayerMoveFlashTime;
					//DEBUGLOG("%u Player Move Flash Time %u\n", tempdword);
					if(tempdword < BB_CHARGE_FLASH_TOTAL)
					{
						if(tempdword < BB_CHARGE_FLASH1)
							xRenderable2DQuadUpdateAlpha(m_pPlayerChargeOverlays[i], (float)tempdword / BB_CHARGE_FLASH1);
						else if(tempdword < BB_CHARGE_UNFLASH1)
							xRenderable2DQuadUpdateAlpha(m_pPlayerChargeOverlays[i], 1.0 - ((float)(tempdword - BB_CHARGE_UNFLASH1) / (BB_CHARGE_UNFLASH1 - BB_CHARGE_FLASH1)));
						else if(tempdword < BB_CHARGE_FLASH2)
							xRenderable2DQuadUpdateAlpha(m_pPlayerChargeOverlays[i], (float)(tempdword - BB_CHARGE_FLASH2) / (BB_CHARGE_FLASH2 - BB_CHARGE_UNFLASH1));
						else if(tempdword < BB_CHARGE_UNFLASH2)
							xRenderable2DQuadUpdateAlpha(m_pPlayerChargeOverlays[i], 1.0 - ((float)(tempdword - BB_CHARGE_UNFLASH2) / (BB_CHARGE_UNFLASH2 - BB_CHARGE_FLASH2)));
						else if(tempdword < BB_CHARGE_FLASH3)
							xRenderable2DQuadUpdateAlpha(m_pPlayerChargeOverlays[i], (float)(tempdword - BB_CHARGE_FLASH3) / (BB_CHARGE_FLASH3 - BB_CHARGE_UNFLASH2));
						else
							xRenderable2DQuadUpdateAlpha(m_pPlayerChargeOverlays[i], 1.0 - ((float)(tempdword - BB_CHARGE_UNFLASH3) / (BB_CHARGE_UNFLASH3 - BB_CHARGE_FLASH3)));
					}
					else
					{
						xRenderable2DQuadUpdatePartialY(m_pPlayerChargeOverlays[i], 0.0);
						m_dwPlayerMoveFlashTime = 0;
					}
				}
				else if(m_xPlayer.m_xField.m_iClearableBlockCount < g_iPlayerChargeClearableCount[i])
				{
					xRenderable2DQuadUpdatePartialY(m_pPlayerCharges[i], 0.0);
					xRenderable2DQuadUpdatePartialY(m_pPlayerChargeOverlays[i], 0.0);
				}
				else if(m_xPlayer.m_xField.m_iClearableBlockCount > g_iPlayerChargeClearableCount[i + 1])
				{
					xRenderable2DQuadUpdatePartialY(m_pPlayerCharges[i], 1.0);
					xRenderable2DQuadUpdateAlpha(m_pPlayerChargeOverlays[i], m_fBeatPercentage);
					xRenderable2DQuadUpdatePartialY(m_pPlayerChargeOverlays[i], 1.0);
				}
				else
				{
					tempfloat = (float)(m_xPlayer.m_xField.m_iClearableBlockCount - g_iPlayerChargeClearableCount[i]) / (float)(g_iPlayerChargeClearableCount[i + 1] - g_iPlayerChargeClearableCount[i]);
					xRenderable2DQuadUpdatePartialY(m_pPlayerCharges[i], tempfloat);
					xRenderable2DQuadUpdateAlpha(m_pPlayerChargeOverlays[i], 0.0);
					xRenderable2DQuadUpdatePartialY(m_pPlayerChargeOverlays[i], tempfloat);
				}
			}
		}
		else
		{
			if(!m_bTypingMessage && g_xPaperAPI.m_xInput.m_keyEnter.JustPressed(true))
			{
				g_xPaperAPI.m_xInput.m_keyEnter.Eaten();
				m_bReady = 1;
			}

			if(!m_bPrePopup && g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
				return 1;
		}

		if(m_bTypingMessage)
		{
			if(g_xPaperAPI.m_xInput.m_keyBackspace.IsDown() && m_iTypedMessageLength > 0)
			{
				if(g_xPaperAPI.m_xInput.m_keyBackspace.JustPressed() || g_xPaperAPI.m_xInput.m_keyBackspace.ShouldRepeat(dwTime))
					m_strMessageBuffer[m_iMessagePrefixLength + --m_iTypedMessageLength] = '\0';
			}
			else if(m_iTypedMessageLength < BB_PACKET_MESSAGESIZE - 1)
			{
				char pressed = g_xPaperAPI.m_xInput.GetCharacterPressed(dwTime, true);

				if(pressed != '\0')
				{
					m_strMessageBuffer[m_iMessagePrefixLength + m_iTypedMessageLength++] = pressed;
					m_strMessageBuffer[m_iMessagePrefixLength + m_iTypedMessageLength] = '\0';
				}
			}

			if(g_xPaperAPI.m_xInput.m_keyEnter.JustPressed(true) || g_xPaperAPI.m_xInput.m_keyNumEnter.JustPressed(true))
			{
				g_xPaperAPI.m_xInput.m_keyEnter.Eaten();
				g_xPaperAPI.m_xInput.m_keyNumEnter.Eaten();

				ShiftMessagesUp();
				m_strTextMessages[0] = new char[BB_PACKET_MESSAGESIZE + 30];
				sprintf(m_strTextMessages[0], "%s: %s", g_xCurrentProfile.m_strProfileName, m_strMessageBuffer + m_iMessagePrefixLength);

				xGameNetwork::SendTextMessage(m_strMessageBuffer + m_iMessagePrefixLength);
				m_iTypedMessageLength = 0;
				m_bTypingMessage = false;
			}
		}
		else if(g_xPaperAPI.m_bHasFocus && g_xOptions.m_pButtons[KEY_TYPEMESSAGE]->JustPressed(true))
		{
			g_xOptions.m_pButtons[KEY_TYPEMESSAGE]->Eaten();
			m_bTypingMessage = true;
			strcpy(m_strMessageBuffer, BB_STRING_MESSAGE);
			m_iMessagePrefixLength = strlen(BB_STRING_MESSAGE);
		}
	}

	if(!g_bCreditsRolling)
	{
		if((m_bPaused && !m_bPrePopup && !m_xPlayer.m_bViewingPostPopup) || m_bVersusScreenAnimating || m_bSenseiPrePopup || m_bCountdown)
		{
			DEBUGANIMATION("Updating Models without dwTime - %i %i %i %i %i %i %i\n", m_bPaused, m_bPrePopup, m_xPlayer.m_bViewingPostPopup, m_bVersusScreenAnimating, m_bSenseiPrePopup, m_bCountdown, m_pEnemyModel->m_iCurrentAnim);

			if(m_pEnemyModel)
				m_pEnemyModel->PreUpdate(m_bSenseiPrePopup || m_pEnemyModel->m_iCurrentAnim != BB_ENEMYANIMATION_INTRO ? 1 : dwTime);

			if(m_pPlayerModel)
				m_pPlayerModel->PreUpdate(m_bSenseiPrePopup || m_pPlayerModel->m_iCurrentAnim != BB_PLAYERANIMATION_INTRO ? 1 : dwTime);
		}
		else
		{
			if(m_pEnemyModel)
				m_pEnemyModel->PreUpdate(dwTime);

			if(m_pPlayerModel)
				m_pPlayerModel->PreUpdate(dwTime);
		}
	}

	//LOG("ANIMATION: CurAnim=%i CurFrame=%i NextFrame=%i\n", m_pPlayerModel->m_iCurrentAnim, m_pPlayerModel->m_iCurrentFrame, m_pPlayerModel->m_iNextFrame);

#if WITH_BALANCESTATS
	g_xBalanceStats.Update();
#endif

#if MOVINGPLAYERMODEL
	if(g_xPaperAPI.m_xInput.m_keyQ.JustPressed())
		m_pEnemyModel->m_vecTranslation.x += 1.0;
	if(g_xPaperAPI.m_xInput.m_keyA.JustPressed())
		m_pEnemyModel->m_vecTranslation.x -= 1.0;
	if(g_xPaperAPI.m_xInput.m_keyW.JustPressed())
		m_pEnemyModel->m_vecTranslation.y += 1.0;
	if(g_xPaperAPI.m_xInput.m_keyS.JustPressed())
		m_pEnemyModel->m_vecTranslation.y -= 1.0;
	if(g_xPaperAPI.m_xInput.m_keyE.JustPressed())
		m_pEnemyModel->m_vecTranslation.z += 1.0;
	if(g_xPaperAPI.m_xInput.m_keyD.JustPressed())
		m_pEnemyModel->m_vecTranslation.z -= 1.0;

	static xVector rotation(180, 188, 150);
	if(g_xPaperAPI.m_xInput.m_keyR.JustPressed())
		rotation.x += 1.0;
	if(g_xPaperAPI.m_xInput.m_keyF.JustPressed())
		rotation.x -= 1.0;
	if(g_xPaperAPI.m_xInput.m_keyT.JustPressed())
		rotation.y += 1.0;
	if(g_xPaperAPI.m_xInput.m_keyG.JustPressed())
		rotation.y -= 1.0;
	if(g_xPaperAPI.m_xInput.m_keyY.JustPressed())
		rotation.z += 1.0;
	if(g_xPaperAPI.m_xInput.m_keyH.JustPressed())
		rotation.z -= 1.0;
	m_pEnemyModel->SetRotation(rotation.x, rotation.y, rotation.z);
#endif

	m_xGUI.PreUpdate(dwTime);
	
	return true;
}

bool xGame::Update(double dTime)
{
	char temptext[256];
	xColor tempcolor;
	float tempfloat;
	int i, j;

	if(g_bTourComplete && !g_bCreditsRolling)
		return 1;
		
	if(m_dInitTime > 0.0)
	{
		m_dInitTime -= dTime;

		if(m_dInitTime <= 0.0)
		{
			//DEBUGLOG("Calling Init\n");
			g_pCurGameState->Init(m_strStoredLevel);
			m_dInitTime = 0.0;
		}
	}

	if(m_xHTTPQuery.GetQueryType() != BB_HTTPQUERY_NONE)
	{
		m_xHTTPQuery.Update();

		// If the current Unlock Achievement query completed, see if there are more pending
		if(m_xHTTPQuery.GetQueryType() == BB_HTTPQUERY_NONE && m_xHTTPQuery.GetOldQueryType() == BB_HTTPQUERY_UNLOCKACHIEVEMENT)
		{
			for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX - 1; i++)
				m_strAchievementsPending[i] = m_strAchievementsPending[i + 1];

			m_strAchievementsPending[BB_PENDINGACHIEVEMENTS_MAX - 1] = 0;

			if(m_strAchievementsPending[0])
				m_xHTTPQuery.UnlockAchievement(m_strAchievementsPending[0], g_xCurrentProfile.m_strLoginName, g_xCurrentProfile.m_strLoginPassword);
		}
	}

	if(m_pAchievementIcon[0])
	{
		for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
		{
			if(m_iAchievementsUnlocking[i])
			{
				tempfloat = g_xPaperAPI.m_dwCurrentTime - m_dwAchievementUnlockStart[i];

				if(tempfloat >= BB_ACHIEVEMENT_TOTALTIME)
				{
					for(j = i; j < BB_PENDINGACHIEVEMENTS_MAX - 1; j++)
					{
						m_iAchievementsUnlocking[j] = m_iAchievementsUnlocking[j + 1];
						m_dwAchievementUnlockStart[j] = m_dwAchievementUnlockStart[j + 1];

						if(m_iAchievementsUnlocking[j] != BB_ACH_NONE)
							xRenderable2DQuadUpdateTexture(m_pAchievementIcon[j], m_pAchievementIconTextures[m_iAchievementsUnlocking[j] - 1]);
					}

					m_iAchievementsUnlocking[BB_PENDINGACHIEVEMENTS_MAX - 1] = 0;
					i--;
					continue;
				}
				else if(tempfloat < BB_ACHIEVEMENT_SHOWTIME)
					tempfloat = tempfloat / BB_ACHIEVEMENT_SHOWTIME;
				else if(tempfloat > BB_ACHIEVEMENT_HIDESTARTTIME)
					tempfloat = 1.0 - ((tempfloat - BB_ACHIEVEMENT_HIDESTARTTIME) / BB_ACHIEVEMENT_HIDETIME);
				else
					tempfloat = 1.0;

				xRenderable2DQuadUpdateVisibility(m_pAchievementIcon[i], 1);
				xRenderable2DQuadSetPosition(m_pAchievementIcon[i], BB_ACHIEVEMENT_ICONX, 768 - BB_ACHIEVEMENT_HEIGHT * i - BB_ACHIEVEMENT_HEIGHT * tempfloat);
				xRenderable2DQuadUpdate(m_pAchievementIcon[i]);

				xRenderable2DTextSetPosition(&m_xAchievementTitle[i], BB_ACHIEVEMENT_TITLEX - xFontGetWidth(m_xAchievementTitle[i].m_pFont, g_strAchievementTitle[m_iAchievementsUnlocking[i] - 1]) * BB_ACHIEVEMENT_TITLESCALE, m_pAchievementIcon[i]->m_xRenderable2D.m_fY + BB_ACHIEVEMENT_TITLEY);
				xRenderable2DTextSetText(&m_xAchievementTitle[i], &m_xSceneGraph, g_strAchievementTitle[m_iAchievementsUnlocking[i] - 1]);
			}
			else
			{
				xRenderable2DQuadUpdateVisibility(m_pAchievementIcon[i], 0);
				xRenderable2DTextSetText(&m_xAchievementTitle[i], &m_xSceneGraph, "");
			}
		}
	}

	if(m_bVersusScreenAnimating)
	{
		if(dTime > 0.0)
		{
			//DEBUGLOG("m_bVersusScreenAnimating %f %f\n", m_fVersusAnimationTime, dTime);
			m_fVersusAnimationTime += dTime < 0.025 ? dTime : 0.025;

			if(!m_bPlayedVersusScreenSound && m_fVersusAnimationTime > 0.03)
			{
				m_pVersusScreenSound->Play(g_xOptions.m_fSoundVolume);
				m_bPlayedVersusScreenSound = 1;
			}

			if(m_fVersusAnimationTime >= BB_VERSUS_ANIMATION_TIME)
			{
				xRenderable2DTextUpdateVisibility(&m_xVersusPlayerName, 1);
				xRenderable2DTextUpdateVisibility(&m_xVersusEnemyName, 1);

				m_fVersusAnimationTime = BB_VERSUS_ANIMATION_TIME;
			}

			xRenderable2DQuadSetPosition(m_pVersusVSImage, BB_VERSUS_VS_LEFT, BB_VERSUS_VS_START + (m_fVersusAnimationTime / BB_VERSUS_ANIMATION_TIME) * (BB_VERSUS_VS_END - BB_VERSUS_VS_START));
			xRenderable2DQuadUpdate(m_pVersusVSImage);

			xRenderable2DQuadSetPosition(m_pVersusPlayerPortrait, BB_VERSUS_PLAYERPORTRAIT_START + (m_fVersusAnimationTime / BB_VERSUS_ANIMATION_TIME) * (BB_VERSUS_PLAYERPORTRAIT_END - BB_VERSUS_PLAYERPORTRAIT_START), BB_VERSUS_PLAYERPORTRAIT_TOP);
			xRenderable2DQuadUpdate(m_pVersusPlayerPortrait);

			xRenderable2DQuadSetPosition(m_pVersusEnemyPortrait, BB_VERSUS_ENEMYPORTRAIT_START + (m_fVersusAnimationTime / BB_VERSUS_ANIMATION_TIME) * (BB_VERSUS_ENEMYPORTRAIT_END - BB_VERSUS_ENEMYPORTRAIT_START), BB_VERSUS_ENEMYPORTRAIT_TOP);
			xRenderable2DQuadUpdate(m_pVersusEnemyPortrait);

			xRenderable2DTextUpdateColor(&m_xVersusSongName, xColorInit(&tempcolor, 1.0, 1.0, 1.0, m_fVersusAnimationTime / BB_VERSUS_ANIMATION_TIME));
			xRenderable2DTextUpdateColor(&m_xVersusArtistName, &tempcolor);

			if(g_xPaperAPI.m_xPlatform.GetTime() - m_dwVersusScreenStartTime >= BB_VERSUS_MINIMUM_TIME)
			{
				//DEBUGLOG("Done Animating\n");
				m_bVersusScreenAnimating = 0;

				xSceneGraphDeleteQuad(&m_xSceneGraph, m_pVersusBackground, 1);
				xSceneGraphDeleteQuad(&m_xSceneGraph, m_pVersusVSImage, 1);
				xSceneGraphDeleteQuad(&m_xSceneGraph, m_pVersusPlayerPortrait, 1);
				xSceneGraphDeleteQuad(&m_xSceneGraph, m_pVersusEnemyPortrait, 1);

				xRenderable2DTextDelete(&m_xVersusPlayerName, &m_xSceneGraph);
				xRenderable2DTextDelete(&m_xVersusEnemyName, &m_xSceneGraph);
				xRenderable2DTextDelete(&m_xVersusSongName, &m_xSceneGraph);
				xRenderable2DTextDelete(&m_xVersusArtistName, &m_xSceneGraph);

				m_pVersusBackground = 0;
				m_pVersusVSImage = 0;
				m_pVersusPlayerPortrait = 0;
				m_pVersusEnemyPortrait = 0;

				g_xGraphics.m_iFrameSkip = 3;
			}
		}

		return true;
	}

	if(!m_bPrePopup && !m_bFinishedConnecting && g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
		return true;

	if(m_bPaused)
	{
		g_xPaperAPI.m_bDragged = false;

		if(g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
		{
			if(!g_xPaperAPI.m_bHasFocus || g_xPaperAPI.m_bDragged)
			{
				if(m_pCrowdLoopSound->IsPlaying())
					Pause(g_xPaperAPI.m_bHasFocus);
			}
			else if(m_bPrePopup)
				UnPause();
		}
	}
	else if((!g_xPaperAPI.m_bHasFocus || g_xPaperAPI.m_bDragged) && g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
	{
		if(!m_bVersusScreenAnimating)
		{
			Pause(g_xPaperAPI.m_bHasFocus);

			if(!m_bPrePopup)
				AddWindow(BB_WINDOW_GAMEPAUSED);
		}

		g_xPaperAPI.m_bDragged = false;
	}
	else if(m_bCountdown)
	{
		m_dCountdownTime -= dTime;

		if(m_dCountdownTime <= 0.0)
		{
			m_bCountdown = 0;

			xSceneGraphDeleteQuad(&m_xSceneGraph, m_pCountdown, 0);
			m_pCountdown = 0;

			m_dwGameStartTime = g_xPaperAPI.m_xPlatform.GetTime();

			m_xSong.Play(1, g_xOptions.m_fMusicVolume);

			xGameOnPlayerAnimationCompleted(m_pPlayerModel);
			if(g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
				xGameOnEnemyAnimationCompleted(m_pEnemyModel);
			m_pPlayerModel->m_dwFrameStartTime = g_xPaperAPI.m_dwCurrentTime;
			if(g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
				m_pEnemyModel->m_dwFrameStartTime = g_xPaperAPI.m_dwCurrentTime;

			if(g_xGameOptions.m_bBoss && (g_xGameOptions.m_iBossIndex == 2 || g_xGameOptions.m_iBossIndex == 4 || g_xGameOptions.m_iBossIndex == 5))
			{
				m_dwNextBossInteraction = g_xPaperAPI.m_xPlatform.GetTime() + g_xGameOptions.m_dwBossMoveMinTime + rand() % g_xGameOptions.m_dwBossMoveRandTime;
				DEBUGBOSS("Setting Next Boss Interaction Time to %u\n", m_dwNextBossInteraction);
			}
		}
	}

	m_xPlayer.m_xField.Update(dTime);

	if(!g_bCreditsRolling)
	{
		if(m_pEnemyModel)
			m_pEnemyModel->Update(dTime);

		if(m_pPlayerModel)
			m_pPlayerModel->Update(dTime);
	}
	else
		g_xMenu.UpdateCredits(dTime);

	m_xPlayer.Update(dTime);

	if(g_xGameOptions.m_bTutorial)
		UpdateTutorial(dTime);

	if(m_xPlayer.m_bWonGame)
	{
		Pause(g_xPaperAPI.m_bHasFocus);

		if(!m_xPlayer.m_bViewingPostPopup)
		{
			if(m_xSong.GetTime() >= m_xPlayer.m_xPiece.m_dwSongLength - 1000)
				m_dwOldGameTime = m_xPlayer.m_xPiece.m_dwSongLength + 1000;
			else
				m_dwOldGameTime = g_xPaperAPI.m_xPlatform.GetTime() - m_dwGameStartTime;

			m_xHTTPQuery.BattleFinished(g_xGameOptions.m_strServerAddress, g_xGameOptions.m_iServerPort, g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER, m_xPlayer.m_strAccountName);

			if(g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
				AddClientPostPopupWindow();
			else if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
				AddServerPostPopupWindow();
			else
				AddPostPopupWindow();

#if WITH_BALANCESTATS
			g_xBalanceStats.SetScore(((0.5f - m_fCrowdBarPercentage) / 0.5f) * 100.0f, g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][g_xGameOptions.m_iLevelIndex]);
			g_xBalanceStats.SetWonGame(true, g_xGame.m_xSong.GetLength() - g_xGame.m_xSong.GetTime());

			g_xBalanceStats.Commit();
#endif

			m_xPlayer.LevelComplete(g_xGameOptions.m_iLevelIndex, m_dwOldGameTime);

			m_xPlayer.m_bViewingPostPopup = true;
			m_xPlayer.m_bDead = true;
		}
		else if(g_xGameOptions.m_strNextLevelFile[0] && m_xPlayer.m_bReadyToMoveOn)
		{
			if(!stricmp(g_xGameOptions.m_strNextLevelFile, "Credits"))
			{
#ifdef DEMO_RELEASE
				sprintf(temptext, BB_IMAGE_PATH, "upgradebackground");
				m_pBuyItBackground = xSceneGraphCreateQuad(temptext, 0);
				xSceneGraphInsertQuad(&m_xSceneGraph, m_pBuyItBackground, 1, BB_DEPTH_BUYIT_BACKGROUND, 0, 1);
				xRenderable2DQuadSetPosition(m_pBuyItBackground, -g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX, -g_xPaperAPI.m_iScreenOffsetY / g_xPaperAPI.m_fScreenRatioY);
				xRenderable2DQuadSetSize(m_pBuyItBackground, 1024.0f + 2.0f * (float)g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX, 768.0f + 2.0f * (float)g_xPaperAPI.m_iScreenOffsetY / g_xPaperAPI.m_fScreenRatioY);
				xRenderable2DQuadUpdate(m_pBuyItBackground);
				
				sprintf(temptext, BB_IMAGE_PATH, "upgradetext");
				m_pBuyItText = xSceneGraphCreateQuad(temptext, 0);
				xSceneGraphInsertQuad(&m_xSceneGraph, m_pBuyItText, 1, BB_DEPTH_BUYIT_TEXT, 0, 1);
				xRenderable2DQuadSetPosition(m_pBuyItText, 170, 16);
				xRenderable2DQuadUpdate(m_pBuyItText);

				sprintf(temptext, BB_IMAGE_PATH, "CloseButton");
				m_pBuyItCloseButton = xSceneGraphCreateQuad(temptext, 0);
				xSceneGraphInsertQuad(&m_xSceneGraph, m_pBuyItCloseButton, 1, BB_DEPTH_BUYIT_CLOSE, 0, 1);
				xRenderable2DQuadSetPosition(m_pBuyItCloseButton, 1024.0f + (float)g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX - 8.0f - m_pBuyItCloseButton->m_xRenderable2D.m_fWidth, 8.0f);
				xRenderable2DQuadUpdate(m_pBuyItCloseButton);
#else
				char *strtemp = new char[strlen(g_xGameOptions.m_strNextLevelFile) + 1];
				strcpy(strtemp, g_xGameOptions.m_strNextLevelFile);
				DelayedInit(strtemp, 0.03f);
				delete [] strtemp;
#endif
			}
			else
				InitVersusScreen(g_xGameOptions.m_strNextLevelFile);

			return 1;
		}
		//else
			//Goto Song Select
	}

	if(m_bCountdown)
	{
		float countdownpct = m_dCountdownTime - floor(m_dCountdownTime);
		float offset = m_pCountdown->m_xRenderable2D.m_fWidth - m_pCountdown->m_xRenderable2D.m_fWidth * countdownpct / 2.0;

		if(countdownpct > m_fOldCountdownPercent)
		{
			m_pCountdownSounds[(int)m_dCountdownTime]->Play(g_xOptions.m_fSoundVolume);
			xRenderable2DQuadUpdateTexture(m_pCountdown, m_pCountdownTextures[(int)m_dCountdownTime]);
			xRenderable2DQuadUpdateVisibility(m_pCountdown, true);
		}
		
		xRenderable2DQuadUpdateAlpha(m_pCountdown, countdownpct);
		xRenderable2DQuadSetScale(m_pCountdown, countdownpct, countdownpct);
		xRenderable2DQuadSetPosition(m_pCountdown, BB_COUNTDOWN_LEFT + offset, BB_COUNTDOWN_TOP + offset);
		xRenderable2DQuadUpdate(m_pCountdown);

		m_fOldCountdownPercent = countdownpct;
	}

	if(m_bSenseiPrePopup || m_bPrePopup)
	{
		m_dPrePopupPressEnterTime += dTime;
		if(m_pPrePopupPressEnterText)
		{
			if(m_dPrePopupPressEnterTime > 0.5)
			{
				m_pPrePopupPressEnterText->SetAlpha(0.8f);

				if(m_dPrePopupPressEnterTime > 1.0)
					m_dPrePopupPressEnterTime = 0.0;
			}
			else if(m_dPrePopupPressEnterTime < 0.25)
			{
				tempfloat = (0.25 - m_dPrePopupPressEnterTime) / 0.25;
				m_pPrePopupPressEnterText->SetAlpha(0.8f * tempfloat * tempfloat * tempfloat * tempfloat);
			}
			else
			{
				tempfloat = (m_dPrePopupPressEnterTime - 0.25) / 0.25;
				m_pPrePopupPressEnterText->SetAlpha(0.8f * tempfloat * tempfloat * tempfloat * tempfloat);
			}
		}

		if(m_bSenseiPrePopup)
		{
			if(m_bSenseiPrePopupMoreLines)
			{
				if(dTime > BB_PREPOPUP_MORELINESTIME)
					m_fSenseiPrePopupMoreLinesTime += BB_PREPOPUP_MORELINESTIME;
				else
					m_fSenseiPrePopupMoreLinesTime += dTime;

				if(m_fSenseiPrePopupMoreLinesTime > BB_PREPOPUP_MORELINESTIME)
				{
					m_fSenseiPrePopupMoreLinesTime -= BB_PREPOPUP_MORELINESTIME;
					if(m_pSenseiPrePopupMoreLinesText->m_strBuffer[0] != '.')
					{
						m_pSenseiPrePopupMoreLinesText->m_strBuffer[0] = '.';
						m_pSenseiPrePopupMoreLinesText->m_strBuffer[1] = '\0';
					}
					else if(m_pSenseiPrePopupMoreLinesText->m_strBuffer[1] != '.')
					{
						m_pSenseiPrePopupMoreLinesText->m_strBuffer[1] = '.';
						m_pSenseiPrePopupMoreLinesText->m_strBuffer[2] = '\0';
					}
					else if(m_pSenseiPrePopupMoreLinesText->m_strBuffer[2] != '.')
					{
						m_pSenseiPrePopupMoreLinesText->m_strBuffer[2] = '.';
						m_pSenseiPrePopupMoreLinesText->m_strBuffer[3] = '\0';
					}
					else
						m_pSenseiPrePopupMoreLinesText->m_strBuffer[0] = '\0';

					m_pSenseiPrePopupMoreLinesText->UpdateRenderable();
				}
			}
		}
		else if(m_bPrePopup && m_pPrePopupPressEnterText)
		{
			if(!g_xGameOptions.m_bTutorial && m_pEnemyIntroSound)
			{
				if(m_dEnemyIntroTime < BB_PREPOPUP_INTROSOUNDDELAY)
				{
					m_dEnemyIntroTime += dTime;

					if(m_dEnemyIntroTime >= BB_PREPOPUP_INTROSOUNDDELAY)
						m_pEnemyIntroSound->Play(g_xOptions.m_fSoundVolume);
				}
				else if(m_dEnemyIntroTime < BB_PREPOPUP_INTROVOICESOUNDDELAY)
				{
					m_dEnemyIntroTime += dTime;

					if(m_dEnemyIntroTime >= BB_PREPOPUP_INTROVOICESOUNDDELAY)
						PlayVoiceSound(m_pEnemyIntroVoiceSound);
				}
			}

			m_pPrePopupPressEnterText->UpdateRenderables();
		}
	}
	
	if(m_fCrowdBarDisplayedPercentage < m_fCrowdBarPercentage)
		m_fCrowdBarMomentum += (m_fCrowdBarPercentage - m_fCrowdBarDisplayedPercentage) / 0.01f * 0.002f * (dTime < 0.1f ? dTime : 0.1f);
	else
		m_fCrowdBarMomentum -= (m_fCrowdBarDisplayedPercentage - m_fCrowdBarPercentage) / 0.01f * 0.002f * (dTime < 0.1f ? dTime : 0.1f);

	m_fCrowdBarDisplayedPercentage += m_fCrowdBarMomentum;

	m_fCrowdBarMomentum *= 0.999f * (1.0f - (dTime < 0.1f ? dTime : 0.1f));

	xRenderable2DQuadSetSize(m_pCrowdBarArrow1, (m_fBeatPercentage / 2.0f + 1.0f) * m_fCrowdBarWidth, (m_fBeatPercentage / 2.0f + 1.0f) * m_fCrowdBarHeight);
	xRenderable2DQuadSetSize(m_pCrowdBarArrow2, (m_fBeatPercentage / 2.0f + 1.0f) * m_fCrowdBarWidth, (m_fBeatPercentage / 2.0f + 1.0f) * m_fCrowdBarHeight);
	xGameUpdateCrowdBarArrows(m_pCrowdBarArrow1, m_pCrowdBarArrow2, m_fCrowdBarDisplayedPercentage);

	//if(m_bTypingMessage)
	//	m_pMessageFont->Render(m_strMessageBuffer, BB_TEXTMESSAGES_BUFFERX, BB_TEXTMESSAGES_BUFFERY);

	int y = BB_TEXTMESSAGES_OUTPUTY;
	for(i = 0; i < BB_TEXTMESSAGES_MAX; i++)
	{
		if(m_strTextMessages[i])
		{
			//m_pMessageFont->Render(m_strTextMessages[i], BB_TEXTMESSAGES_OUTPUTX, y);
			y -= BB_TEXTMESSAGES_OFFSETY;
		}
	}

	if(!m_xPlayer.m_bReadyToMoveOn && m_xPlayer.m_bViewingPostPopup && m_xPlayer.m_bWonGame)
		UpdatePostPopup(dTime);

	m_xGUI.Update(dTime);

	return true;
}

void xGame::RefreshAllRenderables()
{
	int i;

	m_xGUI.RefreshAllRenderables();

	if(m_xTimer.m_pFont && m_xTimer.m_xRenderable.m_xRenderable2D.m_pVertData)
	{
		xRenderable2DTextRefreshTextures(&m_xTimer);
		xRenderable2DTextSetText(&m_xTimer, &m_xSceneGraph, m_strTime);

		xRenderable2DQuadUpdateTexture(m_pTimerBG, m_pTimerBG->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pTimerBG);
	}

	if(m_xPlayerName.m_pFont && m_xPlayerName.m_xRenderable.m_xRenderable2D.m_pVertData)
	{
		xRenderable2DTextRefreshTextures(&m_xPlayerName);
		xRenderable2DTextSetText(&m_xPlayerName, &m_xSceneGraph, g_xCurrentProfile.m_strProfileName);
	}

	if(m_xPlayerRank.m_pFont && m_xPlayerRank.m_xRenderable.m_xRenderable2D.m_pVertData)
	{
		xRenderable2DTextRefreshTextures(&m_xPlayerRank);
		xRenderable2DTextSetText(&m_xPlayerRank, &m_xSceneGraph, m_strPlayerRank);
	}

	if(m_xEnemyName.m_pFont && m_xEnemyName.m_xRenderable.m_xRenderable2D.m_pVertData)
	{
		xRenderable2DTextRefreshTextures(&m_xEnemyName);
		xRenderable2DTextSetText(&m_xEnemyName, &m_xSceneGraph, g_xGameOptions.m_strEnemyDisplayName);
	}

	if(m_pRemotePlayer && m_pRemotePlayer->m_iRank > 0 && m_xEnemyRank.m_pFont && m_xEnemyRank.m_xRenderable.m_xRenderable2D.m_pVertData)
	{
		xRenderable2DTextRefreshTextures(&m_xEnemyRank);
		xRenderable2DTextSetText(&m_xEnemyRank, &m_xSceneGraph, m_strRemotePlayerRank);
	}

	if(m_pBackground)
	{
		xRenderable2DQuadUpdateTexture(m_pBackground, m_pBackground->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pBackground);
	}

	if(m_pCountdown)
	{
		xRenderable2DQuadUpdateTexture(m_pCountdown, m_pCountdown->m_pTextures[0]);
		xGraphicsGetTextures(BB_IMAGE_COUNTDOWN, &m_pCountdownTextures);
		xRenderable2DQuadUpdate(m_pCountdown);
	}

	if(m_pCrowdBarArrow1)
	{
		xRenderable2DQuadUpdateTexture(m_pCrowdBarArrow1, m_pCrowdBarArrow1->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pCrowdBarArrow1);
		xRenderable2DQuadUpdateTexture(m_pCrowdBarArrow2, m_pCrowdBarArrow2->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pCrowdBarArrow2);
	}

	if(m_pPlayerNameBG)
	{
		xRenderable2DQuadUpdateTexture(m_pPlayerChargeBG, m_pPlayerChargeBG->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pPlayerChargeBG);

		for(i = 0; i < 4; i++)
		{
			xRenderable2DQuadUpdateTexture(m_pPlayerCharges[i], m_pPlayerCharges[i]->m_pTextures[0]);
			xRenderable2DQuadUpdate(m_pPlayerCharges[i]);

			xRenderable2DQuadUpdateTexture(m_pPlayerChargeOverlays[i], m_pPlayerChargeOverlays[i]->m_pTextures[0]);
			xRenderable2DQuadUpdate(m_pPlayerChargeOverlays[i]);
		}

		xRenderable2DQuadUpdateTexture(m_pPlayerNameBG, m_pPlayerNameBG->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pPlayerNameBG);
	}

	if(m_pEnemyNameBG)
	{
		xRenderable2DQuadUpdateTexture(m_pEnemyChargeBG, m_pEnemyChargeBG->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pEnemyChargeBG);

		xRenderable2DQuadUpdateTexture(m_pEnemyCharge, m_pEnemyCharge->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pEnemyCharge);

		xRenderable2DQuadUpdateTexture(m_pEnemyChargeOverlay, m_pEnemyChargeOverlay->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pEnemyChargeOverlay);

		xRenderable2DQuadUpdateTexture(m_pEnemyNameBG, m_pEnemyNameBG->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pEnemyNameBG);
	}

	if(m_pTutorialDimmer)
	{
		xRenderable2DQuadUpdateTexture(m_pTutorialDimmer, m_pTutorialDimmer->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pTutorialDimmer);
		xRenderable2DQuadUpdateTexture(m_pSwipeGesture, m_pSwipeGesture->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pSwipeGesture);
		xRenderable2DQuadUpdateTexture(m_pDoubleTapGesture, m_pDoubleTapGesture->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pDoubleTapGesture);
	}

	if(m_pAchievementIcon[0])
	{
		for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
		{
			xRenderable2DQuadUpdateTexture(m_pAchievementIcon[i], m_pAchievementIcon[i]->m_pTextures[0]);
			xRenderable2DQuadUpdate(m_pAchievementIcon[i]);

			xRenderable2DTextRefreshTextures(&m_xAchievementTitle[i]);
			
			if(m_iAchievementsUnlocking[i] > 0)
				xRenderable2DTextSetText(&m_xAchievementTitle[i], &m_xSceneGraph, g_strAchievementTitle[m_iAchievementsUnlocking[i] - 1]);
		}
	}

	if(m_pEnemyModel)
		m_pEnemyModel->RefreshTextures();

	if(m_pPlayerModel)
		m_pPlayerModel->RefreshTextures();

	m_xPlayer.RefreshAllRenderables();
}

void xGame::Shutdown()
{
	int i;

	xGameStopSounds();
	
	m_xPlayer.Shutdown();

	if(m_pBackground)
	{
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pBackground, 1);
		m_pBackground = 0;
	}
	
	if(m_pEnemyNameBG)
	{
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pEnemyChargeBG, 1);
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pEnemyCharge, 1);
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pEnemyChargeOverlay, 1);
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pEnemyNameBG, 1);
		m_pEnemyNameBG = 0;
	}
	
	if(m_xTimer.m_pFont)
	{
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pTimerBG, 1);
		m_xTimer.m_pFont = 0;
	}

	if(m_pTutorialDimmer)
	{
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pTutorialDimmer, 1);
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pSwipeGesture, 1);
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pDoubleTapGesture, 1);
		m_pTutorialDimmer = 0;
		m_pSwipeGesture = 0;
	}
	
	if(m_pCountdown)
	{
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pCountdown, 0);
		xGraphicsDeleteTextures(m_pCountdownTextures, 3);
		m_pCountdown = 0;
	}
	
	if(m_pCrowdBarArrow1)
	{
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pCrowdBarArrow1, 1);
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pCrowdBarArrow2, 1);
		m_pCrowdBarArrow1 = 0;
	}
		
	if(m_xPlayerName.m_pFont)
	{
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pPlayerChargeBG, 1);
		xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pPlayerNameBG, 1);

		for(i = 0; i < 4; i++)
		{
			xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pPlayerCharges[i], 1);
			xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, m_pPlayerChargeOverlays[i], 1);
		}
		
		m_xPlayerName.m_pFont = 0;
	}

	if(m_pPlayerModel)
	{
		m_pPlayerModel->Delete();
		delete m_pPlayerModel;
		m_pPlayerModel = 0;
	}

	if(m_pEnemyModel)
	{
		m_pEnemyModel->Delete();
		delete m_pEnemyModel;
		m_pEnemyModel = 0;
	}
	
	if(m_pVersusScreenSound)
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pVersusScreenSound);
		m_pVersusScreenSound = 0;
	}
	
	if(m_pTutorialVoice)
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pTutorialVoice);
		m_pTutorialVoice = 0;
	}

	if(m_pScoreCountLongSound)
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pScoreCountLongSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pUnlockedSound);
		m_pScoreCountLongSound = 0;
	}
	
	if(m_pSenseiPrePopupVoice)
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pSenseiPrePopupVoice);
		m_pSenseiPrePopupVoice = 0;
	}
	
	if(m_pCrowdLoopSound)
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCrowdLoopSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCrowdLoopExcitedSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCrowdPositiveSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCrowdNegativeSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCrowdGameWonSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCrowdGameLostSound);
		m_pCrowdLoopSound = 0;
	}
	
	if(m_pEnemyIntroSound)
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyIntroSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyIntroVoiceSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyPreMoveSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyMoveSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyPostMoveSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pEnemyReactionSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pWonVoiceSound);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pFailureVoiceSound);
		m_pEnemyIntroSound = 0;
	}
	
	if(m_pCountdownSounds[0])
	{
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCountdownSounds[0]);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCountdownSounds[1]);
		g_xPaperAPI.m_xAudio.UnloadSound(m_pCountdownSounds[2]);
		m_pCountdownSounds[0] = 0;
	}
}

void xGame::CopyLevelPath(const char *strLevelName, char *strLevelPath)
{
	if(strLevelName && (strLevelName[0] != 'l' || strLevelName[1] != 'e' || strLevelName[2] != 'v' || strLevelName[3] != 'e' ||
						strLevelName[4] != 'l' || strLevelName[5] != 's' || (strLevelName[6] != '/' && strLevelName[6] != '\\' )))
	{
		sprintf(strLevelPath, "levels/%s.gbl", strLevelName);
	}
	else
	{
		strcpy(strLevelPath, strLevelName);
	}
}

void xGame::GetPreloadGameOptions(const char *strLevelName, char *strEnemyName, char *strEnemyDisplayName, char *strSongName, char *strArtistName, char *strNextLevel, xPreloadGameOptions *pDest)
{
	xFileIO fileio;
	char levelfile[256] = {0}, strline[10240] = {0}, songfile[256] = {0};

	CopyLevelPath(strLevelName, levelfile);

	if(levelfile[0] != '\0')
	{
		if(fileio.OpenFile(levelfile, true, false))
			return;

		xGameOptions tempgameoptions;
		m_pParsedGameOptions = &tempgameoptions;

		while(!fileio.ReadLine(strline, 25600))
			ParseLine(strline);

		fileio.CloseFile();

		if(strEnemyName)
			strcpy(strEnemyName, tempgameoptions.m_strEnemyName);

		if(strEnemyDisplayName)
		{
			sprintf(levelfile, BB_ENEMYFILE_PATH, tempgameoptions.m_strEnemyName);
			if(fileio.OpenFile(levelfile, 1, 0))
				return;

			while(!fileio.ReadLine(strline, 25600))
				ParseLine(strline);

			fileio.CloseFile();

			strcpy(strEnemyDisplayName, tempgameoptions.m_strEnemyDisplayName);
		}

		if(pDest)
		{
			pDest->m_dwBronzeMedalScore = tempgameoptions.m_dwBronzeMedalScore;
			pDest->m_dwSilverMedalScore = tempgameoptions.m_dwSilverMedalScore;
			pDest->m_dwGoldMedalScore = tempgameoptions.m_dwGoldMedalScore;
		}

		if(strNextLevel)
			strcpy(strNextLevel, tempgameoptions.m_strNextLevelFile);

		if(strSongName || strArtistName)
		{
			sprintf(songfile, BB_MUSICFILE_PATH, tempgameoptions.m_strSongName);
			if(fileio.OpenFile(songfile, 1, 0))
				return;

			while(!fileio.ReadLine(strline, 25600))
				ParseLine(strline);

			fileio.CloseFile();

			if(strSongName)
				strcpy(strSongName, tempgameoptions.m_strSongDisplayName);

			if(strArtistName)
				strcpy(strArtistName, tempgameoptions.m_strSongArtistName);
		}
	}
}

void xGame::InitializeEnemyName()
{
	xColor tempcolor;
	float width, tempfloat;

	if(g_xGameOptions.m_strEnemyDisplayName[0] != '\0')
	{
		tempfloat = (float)g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX;
		width = xFontGetWidth(m_xPlayerName.m_pFont, g_xGameOptions.m_strEnemyDisplayName);

		if(m_pRemotePlayer && m_pRemotePlayer->m_iRank > 0)
		{
			xRenderable2DTextSetFont(&m_xEnemyRank, g_xGame.m_xGUI.GetFont(BB_FONT_RANK));
			itoa(m_pRemotePlayer->m_iRank, m_strRemotePlayerRank, 10);
			xRenderable2DTextSetText(&m_xEnemyRank, &m_xSceneGraph, m_strRemotePlayerRank);

			width += m_iEnemyRankWidth;
		}

		if(width < 64)
			width = 64;

		if(!m_pEnemyNameBG)
		{
			m_pEnemyChargeBG = xSceneGraphCreateQuad(BB_IMAGE_ENEMYCHARGEBG, 0);
			xSceneGraphInsertQuad(&m_xSceneGraph, m_pEnemyChargeBG, 1, BB_DEPTH_ENEMYCHARGE_BG, 0, 1);
			xRenderable2DQuadSetPosition(m_pEnemyChargeBG, BB_ENEMY_CHARGE_BG_LEFT + tempfloat, 0);
			xRenderable2DQuadUpdate(m_pEnemyChargeBG);

			m_pEnemyCharge = xSceneGraphCreateQuad(BB_IMAGE_ENEMYCHARGEBAR, 0);
			xSceneGraphInsertQuad(&m_xSceneGraph, m_pEnemyCharge, 1, BB_DEPTH_ENEMYCHARGE, 0, 1);
			xRenderable2DQuadSetPosition(m_pEnemyCharge, BB_ENEMY_CHARGE_LEFT + tempfloat, BB_ENEMY_CHARGE_TOP);

			m_pEnemyChargeOverlay = xSceneGraphCreateQuad(BB_IMAGE_ENEMYCHARGEFLASH, 0);
			xSceneGraphInsertQuad(&m_xSceneGraph, m_pEnemyChargeOverlay, 1, BB_DEPTH_ENEMYCHARGE_OVERLAY, 0, 1);
			xRenderable2DQuadSetPosition(m_pEnemyChargeOverlay, m_pEnemyCharge->m_xRenderable2D.m_fX, m_pEnemyCharge->m_xRenderable2D.m_fY);

			m_pEnemyNameBG = xSceneGraphCreateQuad(BB_IMAGE_ENEMYNAMEBG, 0);
			xSceneGraphInsertQuad(&m_xSceneGraph, m_pEnemyNameBG, 9, BB_DEPTH_ENEMYNAME_BG, 0, 1);
		}

		xRenderable2DQuadUpdatePartialY(m_pEnemyCharge, 0.0);
		xRenderable2DQuadUpdateVisibility(m_pEnemyChargeOverlay, 0);
		xRenderable2DQuadUpdatePartialY(m_pEnemyChargeOverlay, 0.0);

		xRenderable2DQuadSetPosition(m_pEnemyNameBG, BB_ENEMY_CHARGE_BG_LEFT + tempfloat - width, 0);
		xRenderable2DQuadSetSize(m_pEnemyNameBG, width, m_pEnemyNameBG->m_xRenderable2D.m_fHeight);
		xRenderable2DQuad9Update(m_pEnemyNameBG);

		xRenderable2DTextSetFont(&m_xEnemyName, g_xGame.m_xGUI.GetFont(BB_FONT_NAMES));
		xRenderable2DTextSetColor(&m_xEnemyName, xColorInit(&tempcolor, 1.0f, 1.0f, 1.0f, 0.8f));
		xRenderable2DTextSetPosition(&m_xEnemyName, m_pEnemyNameBG->m_xRenderable2D.m_fX + BB_ENEMY_NAME_SPACING, BB_ENEMY_NAME_TOP);
		xRenderable2DTextSetText(&m_xEnemyName, &m_xSceneGraph, g_xGameOptions.m_strEnemyDisplayName);
	}
}

void xGame::InitTimeString(int iTime)
{
	xColor tempcolor;
	int minutes = iTime / 60;
	int secondstens = (iTime % 60) / 10;
	int secondsones = iTime % 10;

	if(g_xGameOptions.m_bLoopSong)
	{
		minutes = 0;
		secondstens = 0;
		secondsones = 0;
	}

	DEBUGTIMER("Timer - InitTimeString - %i %i:%i%i %x\n", iTime, minutes, secondstens, secondsones, m_xTimer.m_pFont);

	m_dwCurrentTime = 0;

	m_strTime[0] = 32 + minutes;
	m_strTime[1] = 42; //':'
	m_strTime[2] = 32 + secondstens;
	m_strTime[3] = 32 + secondsones;
	m_strTime[4] = '\0';

	if(!m_xTimer.m_pFont)
	{
		m_pTimerBG = xSceneGraphCreateQuad(BB_IMAGE_TIMERBG, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pTimerBG, 1, BB_DEPTH_TIMERBACKGROUND, 0, 1);
		xRenderable2DQuadSetPosition(m_pTimerBG, BB_TIME_CENTER - m_pTimerBG->m_xRenderable2D.m_fWidth / 2.0, 0);
		xRenderable2DQuadUpdate(m_pTimerBG);

		xRenderable2DTextSetFont(&m_xTimer, g_xGame.m_xGUI.GetFont(BB_FONT_TIMER));
	}

	xRenderable2DTextSetColor(&m_xTimer, xColorInit(&tempcolor, 1.0f, 1.0f, 1.0f, 0.8f));
	xRenderable2DTextSetScale(&m_xTimer, 1.0f, 1.0f);
	xRenderable2DTextSetPosition(&m_xTimer, BB_TIME_CENTER - xFontGetWidth(m_xTimer.m_pFont, m_strTime) / 2.0, BB_TIME_TOP);
	DEBUGTIMER("Timer - Init - %s\n", m_strTime);
	xRenderable2DTextSetText(&m_xTimer, &m_xSceneGraph, m_strTime);
}

void xGame::AdjustCrowdBar(float fShift)
{
	if(!g_xGameOptions.m_bLoopSong && (!g_xGameOptions.m_bTutorial || fShift < 0.0))
	{
		m_fCrowdBarPercentage += fShift;

		if(m_fCrowdBarPercentage <= 0.0f)
		{
			m_fCrowdBarPercentage = 0.0f;
			m_xPlayer.WonGame();

			if(g_xCurrentProfile.m_byteSelectedDifficulty >= 2)
			{
				if(!stricmp(g_xGameOptions.m_strLevelFile, "levels/Tour1Battle1.gbl"))
					UnlockAchievement(BB_ACH_DOM_1);
				else if(!stricmp(g_xGameOptions.m_strLevelFile, "levels/Tour2Battle1.gbl"))
					UnlockAchievement(BB_ACH_DOM_2);
				else if(!stricmp(g_xGameOptions.m_strLevelFile, "levels/Tour3Battle6.gbl"))
					UnlockAchievement(BB_ACH_DOM_3);
				else if(!stricmp(g_xGameOptions.m_strLevelFile, "levels/Tour4Battle8.gbl"))
					UnlockAchievement(BB_ACH_DOM_4);
				else if(!stricmp(g_xGameOptions.m_strLevelFile, "levels/Tour5Battle8.gbl"))
					UnlockAchievement(BB_ACH_DOM_5);
				else if(!stricmp(g_xGameOptions.m_strLevelFile, "levels/Tour6Battle12.gbl"))
					UnlockAchievement(BB_ACH_DOM_6);
			}
		}
		else if(m_fCrowdBarPercentage >= 1.0f)
		{
			m_fCrowdBarPercentage = 1.0f;

			if(!g_xGame.GameOver())
				g_xGame.Failed();
		}

		xGameUpdateCrowdBarArrows(m_pCrowdBarArrow1, m_pCrowdBarArrow2, m_fCrowdBarPercentage);

		if(m_fCrowdBarPercentage > 0.8f && m_xCrowdState != BB_CROWD_EXCITED)
		{
			g_xGame.m_pCrowdLoopSound->Stop();
			g_xGame.m_pCrowdLoopExcitedSound->Play(g_xOptions.m_fSoundVolume, 1);
			g_xGame.m_xCrowdState = BB_CROWD_EXCITED;
	}
		else if(m_fCrowdBarPercentage < 0.7f && m_xCrowdState != BB_CROWD_NORMAL)
		{
			g_xGame.m_pCrowdLoopSound->Play(g_xOptions.m_fSoundVolume, 1);
			g_xGame.m_pCrowdLoopExcitedSound->Stop();
			g_xGame.m_xCrowdState = BB_CROWD_NORMAL;
		}
	}
}

void xGame::HandleNetworkPackets()
{
	if(!g_xPaperAPI.m_xNetwork.IsConnected())
	{
		if(!m_xPlayer.m_bViewingPostPopup && m_bFinishedConnecting && !m_bPrePopup && !m_bCountdown)
		{
			m_xPlayer.WonGame();
			g_xPaperAPI.m_xNetwork.Close();
		}

		return;
	}

	g_xPaperAPI.m_xNetwork.Update();

	if(!m_bFinishedConnecting)
	{
		switch(m_iConnectingStatus)
		{
			case 0:
			{
				if(g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
				{
					if(m_xPlayer.m_iRank < 0)
						m_xPlayer.m_iRank = 1;

					xGameNetwork::SendClientHandshake(m_xPlayer.m_strAccountName, m_xPlayer.m_iRank);
					m_iConnectingStatus++;
					return;
				}

				break;
			}
			case 1:
			{
				if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
				{
					xGameNetwork::SendServerHandshake(m_xPlayer.m_strAccountName, g_xGameOptions.m_strBGFile, g_xGameOptions.m_strSongName, g_xGameOptions.m_iDifficultyLevel, m_xPlayer.m_iRank);
					m_iConnectingStatus++;
					return;
				}

				break;
			}
			case 2:
			case 3:
			{
				if((m_iConnectingStatus == 2 && g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT) || (m_iConnectingStatus == 3 && g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER))
				{
					// TODO: Fill this in with Real MD5s(try pulling EXE MD5 from memory instead of file)
					xGameNetwork::SendSecurity(g_xPaperAPI.m_xPlatform.GetTime(), "asdf4313ASDF4246safg4267AEFD6434", "asdf4313ASDF4246safg4267AEFD6434");
					m_iConnectingStatus++;
					return;
				}

				break;
			}
			case 4:
			case 5:
			{
				if((m_iConnectingStatus == 4 && g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT) || (m_iConnectingStatus == 5 && g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER))
				{
					// TODO: Fill in appearance info
					xGameNetwork::SendAppearance();
					m_iConnectingStatus++;
					return;
				}

				break;
			}
			case 6:
			case 7:
			{
				if(m_bReady && !m_bSentReady)
					SendReady();

				break;
			}
		}
	}

	int iPacketType;
	while(xGameNetwork::ReceivedNetPacket(iPacketType))
	{
		float newbarposition = -1.0;

		switch(iPacketType)
		{
			case BB_NETPACKETTYPE_CLIENTHANDSHAKE:
			{
				xGameNetwork::ReceiveClientHandshake(g_xGameOptions.m_strEnemyDisplayName, m_pRemotePlayer->m_iRank);
				itoa(m_pRemotePlayer->m_iRank, m_strRemotePlayerRank, 10);
				xRenderable2DTextSetFont(&m_xEnemyRank, g_xGame.m_xGUI.GetFont(BB_FONT_RANK));
				if(!m_xPlayerRank.m_pFont)
					xRenderable2DTextSetFont(&m_xPlayerRank, g_xGame.m_xGUI.GetFont(BB_FONT_RANK));
				m_iEnemyRankWidth = xFontGetWidth(m_xPlayerRank.m_pFont, m_strRemotePlayerRank) + BB_RANK_NAME_SPACING;
				m_iConnectingStatus++;

				InitializeEnemyName();

				break;
			}

			case BB_NETPACKETTYPE_SERVERHANDSHAKE:
			{
				if(!m_pRemotePlayer)
					m_pRemotePlayer = new xNetPlayer;

				char enemyname[BB_PACKET_NAMESIZE], background[BB_PACKET_BGSIZE], song[BB_PACKET_SONGSIZE];
				int beatlevel;
				xGameNetwork::ReceiveServerHandshake(enemyname, background, song, beatlevel, m_pRemotePlayer->m_iRank);
				itoa(m_pRemotePlayer->m_iRank, m_strRemotePlayerRank, 10);

				SetupGameOptionsForMultiplayer(enemyname, background, song, (xGameDifficulty)beatlevel);

				Init("");

				if(!m_xPlayerRank.m_pFont)
					xRenderable2DTextSetFont(&m_xPlayerRank, g_xGame.m_xGUI.GetFont(BB_FONT_RANK));

				m_iEnemyRankWidth = xFontGetWidth(m_xPlayerRank.m_pFont, m_strRemotePlayerRank) + BB_RANK_NAME_SPACING;

				m_iConnectingStatus++;
				break;
			}

			case BB_NETPACKETTYPE_SECURITY:
			{
				// TODO: Check MD5s against our own MD5s
				char exemd5[33], beatmd5[33];
				xGameNetwork::ReceiveSecurity(exemd5, beatmd5);
				m_iConnectingStatus++;
				break;
			}

			case BB_NETPACKETTYPE_APPEARANCE:
			{
				xGameNetwork::ReceiveAppearance();
				m_iConnectingStatus++;

				if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
				{
					xGUIWindowList *window = m_xGUI.GetWindow(BB_WINDOW_PREPOPUPSERVER);
					if(window)
					{
						((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_READYBUTTON))->SetEnabled(1);
						((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_KICKCLIENTBUTTON))->SetEnabled(1);

						((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_PREPOPUP_CLICKREADY);
					}
				}

				break;
			}

			case BB_NETPACKETTYPE_READY:
			{
				xGameNetwork::ReceiveReady();
				m_iConnectingStatus++;

				if(m_iConnectingStatus == 8)
					m_bFinishedConnecting = true;

				break;
			}

			case BB_NETPACKETTYPE_ADDEDBLOCK:
			{
				int combocount;
				float barposition;
				xGameNetwork::ReceiveAddedBlock(combocount, barposition);

				m_pRemotePlayer->m_iPieceBlockCount++;
				m_pRemotePlayer->m_iComboCount++;

				// TODO: Check replicated combo count against m_pRemotePlayer->m_iComboCount as anti-cheat

				break;
			}

			case BB_NETPACKETTYPE_FINISHEDPIECE:
			{
				int blocksinpiece;
				//float newbarposition;
				xGameNetwork::ReceiveFinishedPiece(blocksinpiece, newbarposition);

				// TODO: Check replicated piece count against m_pRemotePlayer->m_iPieceBlockCount as anti-cheat

				m_pRemotePlayer->m_iPieceBlockCount = 0;

				break;
			}

			case BB_NETPACKETTYPE_PUSHEDDOWNPIECE:
			{
				int dropdistance;
				//float newbarposition;
				xGameNetwork::ReceivePushedDownPiece(dropdistance, newbarposition);

				break;
			}

			case BB_NETPACKETTYPE_DROPPEDPIECE:
			{
				int dropdistance;
				//float newbarposition;
				xGameNetwork::ReceiveDroppedPiece(dropdistance, newbarposition);

				break;
			}

			case BB_NETPACKETTYPE_BLOCKSCLEARED:
			{
				int blockcount;
				//float newbarposition;
				xGameNetwork::ReceiveBlocksCleared(blockcount, newbarposition);

				AdjustCrowdBar(-1.0 * GetCrowdBarBlocksClearedAdjustment(blockcount));

				break;
			}

			case BB_NETPACKETTYPE_FAILEDTOADDBLOCK:
			{
				//float newbarposition;
				xGameNetwork::ReceiveFailedToAddBlock(newbarposition);

				AdjustCrowdBar(-1.0 * GetCrowdBarFailedToAddBlockAdjustment(g_xGameOptions.m_iDifficultyLevel));

				break;
			}

			case BB_NETPACKETTYPE_MISSEDBEAT:
			{
				int combocount;
				//float newbarposition;
				xGameNetwork::ReceiveMissedBeat(combocount, newbarposition);

				// TODO: Check replicated combo count against m_pRemotePlayer->m_iComboCount as anti-cheat

				AdjustCrowdBar(-1.0 * GetCrowdBarMissedBeatAdjustment(g_xGameOptions.m_iDifficultyLevel));

				break;
			}

			case BB_NETPACKETTYPE_MISSEDCOMBO:
			{
				int combocount;
				//float newbarposition;
				xGameNetwork::ReceiveMissedCombo(combocount, newbarposition);

				// TODO: Check replicated combo count against m_pRemotePlayer->m_iComboCount as anti-cheat

				AdjustCrowdBar(-1.0 * GetCrowdBarMissedComboAdjustment(g_xGameOptions.m_iDifficultyLevel));

				break;
			}

			case BB_NETPACKETTYPE_REMATCH:
			{
				xGameNetwork::ReceiveRematch();

				xGUIWindowList *window = m_xGUI.GetWindow(BB_WINDOW_POSTPOPUPCLIENT);
				if(window)
				{
					((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_POSTPOPUP_CLIENTREMATCHLINE2);

					((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_ACCEPTBUTTON))->SetEnabled(true);
				}

				break;
			}

			case BB_NETPACKETTYPE_NEXTSONG:
			{
				xGameNetwork::ReceiveNextSong(g_xGameOptions.m_strSongName, g_xGameOptions.m_iDifficultyLevel);

				xGUIWindowList *window = m_xGUI.GetWindow(BB_WINDOW_POSTPOPUPCLIENT);
				if(window)
				{
					char temp[256];
					sprintf(temp, BB_STRING_POSTPOPUP_CLIENTNEWSONGLINE2, g_xGameOptions.m_strSongName);
					((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(temp);

					((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_ACCEPTBUTTON))->SetEnabled(true);
				}

				break;
			}

			case BB_NETPACKETTYPE_ACCEPT:
			{
				xGameNetwork::ReceiveAccept();

				m_xGUI.RemoveAllWindows();

				m_iConnectingStatus = 6;
				m_bFinishedConnecting = 0;
				m_bReady = 0;
				m_bSentReady = 0;

				Init("");

				break;
			}

			case BB_NETPACKETTYPE_MESSAGE:
			{
				char tempmessage[BB_PACKET_MESSAGESIZE];
				xGameNetwork::ReceiveTextMessage(tempmessage);

				ShiftMessagesUp();
				m_strTextMessages[0] = new char[BB_PACKET_MESSAGESIZE + 30];
				sprintf(m_strTextMessages[0], "%s: %s", g_xGameOptions.m_strEnemyDisplayName, tempmessage);

				m_xGUI.RemoveAllWindows();

				m_iConnectingStatus = 6;
				m_bFinishedConnecting = 0;
				m_bReady = 0;
				m_bSentReady = 0;

				Init("");

				break;
			}
		}

		if(newbarposition != -1.0)
		{
			//if(abs(m_fCrowdBarPercentage - (1.0 - newbarposition)) > 0.05)
			//	break;

			LOG("NETWORK: Crowd Bar Position - %f - %f\n", m_fCrowdBarPercentage, (1.0 - newbarposition));
		}
	}
}

void xGame::HandlePingPackets()
{
	char temp[MAX_NETWORK_BUFFERSIZE + 1];
	DWORD timereceived;

	LOG("NETWORK: HandlePingPackets Called\n");
	while(g_xPaperAPI.m_xPingNetwork.Receive(temp, timereceived))
	{
		LOG("NETWORK: Received Ping Packet\n");

		if(!g_xPaperAPI.m_xNetwork.IsConnected())
			temp[4] = 's';
		else
			temp[4] = 'c';

		g_xPaperAPI.m_xPingNetwork.SendResponse(temp, 5);

		m_bReceivedPing = true;
	}
}

void xGame::ParseLine(char *strLine)
{
	int  length				= strlen(strLine);
	bool foundequals		= false;
	char typebuffer[256]	= {0};
	char valuebuffer[1024]	= {0};
	int  pos				= 0;

	for(int i = 0; i < length; i++)
	{
		//Ignore Tabs and Spaces
		if((strLine[i] == '	' || strLine[i] == ' ') && !foundequals)
			continue;

		if(strLine[i] == ':')
		{
			if(!stricmp(typebuffer, "Novice"))
			{
				if(m_pParsedGameOptions->m_iDifficultyLevel <= 1) //0 and 1 are Novice Difficulty
					m_bFoundArrowStart = true;
				else
					return;
			}

			if(!stricmp(typebuffer, "Advanced"))
			{
				if(m_pParsedGameOptions->m_iDifficultyLevel == 2) //2 is Advanced Difficulty
					m_bFoundArrowStart = true;
				else
					return;
			}

			if(!stricmp(typebuffer, "Expert"))
			{
				if(m_pParsedGameOptions->m_iDifficultyLevel == 3) //3 is Expert Difficulty
					m_bFoundArrowStart = true;
				else
					return;
			}
		}
		else if(strLine[i] == '=')
		{
			foundequals = true;
			pos = 0;

			while(strLine[i + 1] == ' ' || strLine[i + 1] == '	')
				i++;

			continue;
		}

		if(m_bFoundArrowStart)
		{
			if(strLine[i] == ';')
				m_bFoundArrowStart = false;
			else if((strLine[i] >= '0' && strLine[i] <= '9') || (strLine[i] >= 'a' && strLine[i] <= 'z') || (strLine[i] >= 'A' && strLine[i] <= 'Z'))
			{
				//DEBUGLOG("Adding Beat: %f %f %f\n", m_fCurArrowTime, m_pParsedGameOptions->m_fBPM, 15000.0 / m_pParsedGameOptions->m_fBPM);

				m_fCurArrowTime += m_xPlayer.m_xBeatManager.AddBeat(m_fCurArrowTime, strLine[i]) / m_pParsedGameOptions->m_fBPM;
			}
		}
		else if(!foundequals)
			typebuffer[pos++] = strLine[i];
		else
			valuebuffer[pos++] = strLine[i];
	}

	if(m_bFoundArrowStart)
		return;

	while(valuebuffer[pos - 1] == ' ')
		valuebuffer[--pos] = '\0';

	if(!stricmp(typebuffer, "iLevelIndex"))
		m_pParsedGameOptions->m_iLevelIndex = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "strBackground"))
		strcpy(m_pParsedGameOptions->m_strBGFile, valuebuffer);
	else if(!stricmp(typebuffer, "strSongName"))
		strcpy(m_pParsedGameOptions->m_strSongName, valuebuffer);
	else if(!stricmp(typebuffer, "strSenseiPrePopup"))
		ReplacePrePopupText(m_pParsedGameOptions->m_strSenseiPrePopup, valuebuffer);
	else if(!stricmp(typebuffer, "strSenseiVoice"))
		strcpy(m_pParsedGameOptions->m_strSenseiPrePopupVoice, valuebuffer);
	else if(!stricmp(typebuffer, "strEnemyName"))
		strcpy(m_pParsedGameOptions->m_strEnemyName, valuebuffer);
	else if(!stricmp(typebuffer, "strDisplayName"))
		strcpy(m_pParsedGameOptions->m_strEnemyDisplayName, valuebuffer);
	else if(!stricmp(typebuffer, "dwEnemyFirstMoveDelay"))
		m_pParsedGameOptions->m_dwEnemyFirstMoveDelay = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "dwEnemyMoveDelay"))
		m_pParsedGameOptions->m_dwEnemyMoveDelay = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "dwMinEnemyMoveDelay"))
		m_pParsedGameOptions->m_dwMinEnemyMoveDelay = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iEnemyDamage"))
		m_pParsedGameOptions->m_iEnemyDamage = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "bTutorial"))
	{
		if(!stricmp(valuebuffer, "true"))
			m_pParsedGameOptions->m_bTutorial = true;
		else
			m_pParsedGameOptions->m_bTutorial = false;
	}
	else if(!stricmp(typebuffer, "bBoss"))
	{
		if(!stricmp(valuebuffer, "true"))
			m_pParsedGameOptions->m_bBoss = true;
		else
			m_pParsedGameOptions->m_bBoss = false;
	}
	else if(!stricmp(typebuffer, "iBossIndex"))
		m_pParsedGameOptions->m_iBossIndex = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "dwBossMoveMinTime"))
		m_pParsedGameOptions->m_dwBossMoveMinTime = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "dwBossMoveRandTime"))
		m_pParsedGameOptions->m_dwBossMoveRandTime = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iCashPrize"))
		m_pParsedGameOptions->m_iCashPrize = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "dwLevelCompletionTime"))
		m_pParsedGameOptions->m_dwLevelCompletionTime = atoi(valuebuffer) * 1000;
	else if(!stricmp(typebuffer, "dwBronzeMedalScore"))
		m_pParsedGameOptions->m_dwBronzeMedalScore = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "dwSilverMedalScore"))
		m_pParsedGameOptions->m_dwSilverMedalScore = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "dwGoldMedalScore"))
		m_pParsedGameOptions->m_dwGoldMedalScore = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iDifficultyLevelSimple"))
		m_pParsedGameOptions->m_iDifficultyLevels[0] = (xGameDifficulty)atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iDifficultyLevelEasy"))
		m_pParsedGameOptions->m_iDifficultyLevels[1] = (xGameDifficulty)atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iDifficultyLevelNormal"))
		m_pParsedGameOptions->m_iDifficultyLevels[2] = (xGameDifficulty)atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iDifficultyLevelHard"))
		m_pParsedGameOptions->m_iDifficultyLevels[3] = (xGameDifficulty)atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iDifficultyLevelInsane"))
		m_pParsedGameOptions->m_iDifficultyLevels[4] = (xGameDifficulty)atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iMaxPieces"))
		m_pParsedGameOptions->m_iMaxPieces = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "bCombosRequired"))
	{
		if(!stricmp(valuebuffer, "true"))
			m_pParsedGameOptions->m_bCombosRequired = true;
		else
			m_pParsedGameOptions->m_bCombosRequired = false;
	}
	else if(!stricmp(typebuffer, "bAlwaysAddBlock"))
	{
		if(!stricmp(valuebuffer, "true"))
			m_pParsedGameOptions->m_bAlwaysAddBlock = true;
		else
			m_pParsedGameOptions->m_bAlwaysAddBlock = false;
	}
	else if(!stricmp(typebuffer, "bShowMissed"))
	{
		if(!stricmp(valuebuffer, "true"))
			m_pParsedGameOptions->m_bShowMissed = true;
		else
			m_pParsedGameOptions->m_bShowMissed = false;
	}
	else if(!stricmp(typebuffer, "iHitTime"))
		m_pParsedGameOptions->m_iHitTime = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "strNextLevel"))
		strcpy(m_pParsedGameOptions->m_strNextLevelFile, valuebuffer);
	else if(!stricmp(typebuffer, "strName"))
		strcpy(m_pParsedGameOptions->m_strSongDisplayName, valuebuffer);
	else if(!stricmp(typebuffer, "strSongArtist"))
		strcpy(m_pParsedGameOptions->m_strSongArtistName, valuebuffer);
	else if(!stricmp(typebuffer, "dwStartOffset"))
	{
 		m_pParsedGameOptions->m_dwStartOffset = atoi(valuebuffer);
		m_fCurArrowTime = m_pParsedGameOptions->m_dwStartOffset;
	}
	else if(!stricmp(typebuffer, "fBPM"))
	{
 		m_pParsedGameOptions->m_fBPM = atof(valuebuffer);
		m_pParsedGameOptions->m_iBeatMarkerShowTime = 90000 / m_pParsedGameOptions->m_fBPM;
	}
	else if(!stricmp(typebuffer, "iNumColors"))
		m_pParsedGameOptions->m_iNumColors = atoi(valuebuffer);
	else if(!stricmp(typebuffer, "iNumRandomColors"))
		m_pParsedGameOptions->m_iNumRandomColors = atoi(valuebuffer);
}

xGUIWindowList* xGame::AddWindow(const char *strWindowName)
{
	if(!m_xGUI.AddWindow(strWindowName))
		return m_xGUI.GetWindow(strWindowName);

	return NULL;
}

void xGame::AddPrePopupWindow()
{
	xGUIWindowList *window = m_xGUI.GetWindow(BB_WINDOW_PREPOPUP);

	if(!window)	
		window = AddWindow(BB_WINDOW_PREPOPUP);

	if(window)
	{
		window->m_pData->m_bPendingClose = 0;

		m_iPrePopupTextIndex = 0;
		m_dPrePopupPressEnterTime = 0.0;
		m_pPrePopupPressEnterText = (xGUIText*)window->m_pData->GetChildByName(BB_NAME_PRESSENTER);

		UpdatePrePopupText();
	}
	else
		assert(0);
}

void xGame::AddSenseiPrePopupWindow()
{
	m_bSenseiPrePopup = m_bPrePopup;

	if(m_pTutorialDimmer)
		xRenderable2DQuadUpdateVisibility(m_pTutorialDimmer, 0);
				
	xGUIWindowList *window = m_xGUI.GetWindow(BB_WINDOW_SENSEIPREPOPUP);

	if(!window)
		window = AddWindow(BB_WINDOW_SENSEIPREPOPUP);

	if(window)
	{
		window->m_pData->m_bPendingClose = 0;

		m_iSenseiPrePopupTextIndex = 0;
		m_fSenseiPrePopupMoreLinesTime = 0.0;
		m_pSenseiPrePopupMoreLinesText = (xGUIText*)window->m_pData->GetChildByName(BB_NAME_PREPOPUPMORELINES);
		m_dPrePopupPressEnterTime = 0.0;
		m_pPrePopupPressEnterText = (xGUIText*)window->m_pData->GetChildByName(BB_NAME_PRESSENTER);

		UpdateSenseiPrePopupText();
	}
	else
		assert(0);
}

void xGame::AddClientPrePopupWindow()
{
	xGUIWindowList *window = AddWindow(BB_WINDOW_PREPOPUPCLIENT);
	if(window)
	{
		xGUIObjectList *curobj = window->m_pData->m_xChildObjectList.m_pNext;
		while(curobj)
		{
			if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE1))
				((xGUIText*)curobj->m_pData)->SetText(BB_STRING_PREPOPUP_LINE1);
			else if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE2))
				((xGUIText*)curobj->m_pData)->SetText(BB_STRING_PREPOPUP_CLICKREADY);

			curobj = curobj->m_pNext;
		}
	}
	else
		assert(0);
}

void xGame::AddServerPrePopupWindow()
{
	xGUIWindowList *window = AddWindow(BB_WINDOW_PREPOPUPSERVER);
	if(window)
	{
		xGUIObjectList *curobj = window->m_pData->m_xChildObjectList.m_pNext;
		while(curobj)
		{
			if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE1))
				((xGUIText*)curobj->m_pData)->SetText(BB_STRING_PREPOPUP_LINE1);
			else if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE2))
				((xGUIText*)curobj->m_pData)->SetText(BB_STRING_PREPOPUP_SERVERLINE2);

			curobj = curobj->m_pNext;
		}
	}
	else
		assert(0);
}

void xGame::AddPostPopupWindow()
{
	xGUIWindowList *window;
	float highestdepth = -99999.0;
	xEmitter tempemitter;
	int tempint, i;
	bool endoftour;

	window = m_xGUI.GetWindow(BB_WINDOW_PREPOPUP);
	if(window)
		m_xGUI.DelayedRemoveWindow(window);

	tempint = g_xGameOptions.m_iLevelIndex;
	for(i = 0; i < 7; i++)
	{
		if(tempint <= g_iTourLevelCount[i])
		{
			endoftour = tempint == g_iTourLevelCount[i] || !stricmp(g_xGameOptions.m_strNextLevelFile, "Credits");
			break;
		}

		tempint -= g_iTourLevelCount[i];
	}

	window = AddWindow(BB_WINDOW_POSTPOPUP);
	if(window)
	{
		m_pPostPopupWindow = window->m_pData;

		if(!m_pScoreCountLongSound)
		{
			m_pScoreCountLongSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_SCORELONG);
			m_pUnlockedSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_UNLOCK);
		}

		xGUIObjectList *curobj = m_pPostPopupWindow->m_xChildObjectList.m_pNext;
		while(curobj)
		{
			if(!stricmp(curobj->m_pData->m_strName, "OptionText1"))
				curobj->m_pData->SetHidden(endoftour);
			else if(!stricmp(curobj->m_pData->m_strName, "OptionText2"))
				curobj->m_pData->SetHidden(!endoftour);
			else if(!stricmp(curobj->m_pData->m_strName, "ScoreText"))
			{
				m_pScoreText = (xGUIText*)curobj->m_pData;
				m_pScoreText->SetText("0");
				m_fStartingScoreTextY = m_pScoreText->GetY();
			}
			else if(!stricmp(curobj->m_pData->m_strName, "Dominated"))
			{
				m_pDominatedImage = (xGUIImage*)curobj->m_pData;
				m_pDominatedImage->SetHidden(true);
				m_pDominatedImage->UpdateRenderables();
			}
			else if(!stricmp(curobj->m_pData->m_strName, "NewRecord"))
			{
				m_pNewRecordImage = (xGUIImage*)curobj->m_pData;
				m_pNewRecordImage->SetHidden(true);
				m_pNewRecordImage->UpdateRenderables();
			}

			if(curobj->m_pData->GetAbsoluteDepth() > highestdepth)
				highestdepth = curobj->m_pData->GetAbsoluteDepth();

			curobj = curobj->m_pNext;
		}

		m_iPostPopupStep = 0;
		m_dPostPopupStepTime = 0.0;
		m_bPostPopupDominationBonus = m_fCrowdBarPercentage <= 0.0;
		m_dwPostPopupCrowdScore = ((0.5f - m_fCrowdBarPercentage) / 0.5f) * 100.0f;
		m_dwPostPopupCrowdScore = m_dwPostPopupCrowdScore == 0 ? 1 : m_dwPostPopupCrowdScore;
		m_iPostPopupOldHighScore = g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][g_xGameOptions.m_iLevelIndex];
		m_bPostPopupNewHighScore = m_dwPostPopupCrowdScore > m_iPostPopupOldHighScore;
		m_iPostPopupPrevAwardScore = 0;
		m_iPostPopupNextAwardScore = 0;

		m_pPostPopupWindow->UpdateRenderables();

		//DEBUGLOG("OldScore: %u NewScore: %u BaseScore: %u DominationBonus: %u NewHighScore: %i\n", g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][g_xGameOptions.m_iLevelIndex], m_xPlayer.m_dwScore + m_dwPostPopupDominationBonus, m_xPlayer.m_dwScore, m_dwPostPopupDominationBonus, m_bPostPopupNewHighScore);

		m_pScoreCountLongSound->Play(g_xOptions.m_fSoundVolume);
	}
	else
		assert(0);
}

void xGame::UpdatePostPopup(double dTime)
{
	char temp[30]="";
	int tempint;

	m_dPostPopupStepTime += dTime;

	if( g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
	{
	switch(m_iPostPopupStep)
	{
		case 0:
		{
			if(m_dPostPopupStepTime > BB_POSTPOPUP_SCORECOUNTUPTIME)
			{
				m_iPostPopupStep++;
				m_dPostPopupStepTime = 0.0;
				sprintf(temp, "%i%%", m_dwPostPopupCrowdScore);
				m_pScoreText->SetText(temp);

				if(m_bPostPopupNewHighScore && m_pNewRecordImage->GetHidden())
				{
					m_pNewRecordImage->SetHidden(false);
					m_pNewRecordImage->m_xAnimation.m_pCurKeyFrame = m_pNewRecordImage->m_xAnimation.m_xKeyFrameList.m_pNext;
					xAnimationLoadedAllKeyFrames(&m_pNewRecordImage->m_xAnimation);
					m_pNewRecordImage->UpdateRenderables();
				}

				if(!m_bPostPopupDominationBonus)
				m_iPostPopupStep++;
			}
			else
		{
				tempint = (int)((double)m_dwPostPopupCrowdScore * (m_dPostPopupStepTime / BB_POSTPOPUP_SCORECOUNTUPTIME));
				sprintf(temp, "%i%%", tempint);
				m_pScoreText->SetText(temp);

				if(m_bPostPopupNewHighScore && m_pNewRecordImage->GetHidden() && tempint >= m_iPostPopupOldHighScore)
		{
					m_pNewRecordImage->SetHidden(false);
					m_pNewRecordImage->m_xAnimation.m_pCurKeyFrame = m_pNewRecordImage->m_xAnimation.m_xKeyFrameList.m_pNext;
					xAnimationLoadedAllKeyFrames(&m_pNewRecordImage->m_xAnimation);
					m_pNewRecordImage->UpdateRenderables();
			}
			}

			break;
		}

		case 1:
		{
			if(m_dPostPopupStepTime > BB_POSTPOPUP_DOMINATEDDELAY)
			{
				m_iPostPopupStep++;
				m_dPostPopupStepTime = 0.0;
				m_pDominatedImage->SetHidden(false);
				m_pDominatedImage->UpdateRenderables();

				m_pUnlockedSound->Play(g_xOptions.m_fSoundVolume);
			}

			break;
		}

/*		case 2:
		{
			if(m_dPostPopupStepTime > BB_POSTPOPUP_AWARDSHOWDELAY)
			{
				m_dPostPopupStepTime = 0.0;

				switch(m_iPostPopupAwardsShown)
				{
					case 0:
						m_iPostPopupNextAwardScore = BB_CURRENCY_BRONZEMEDAL;
						break;
					case 1:
						m_iPostPopupPrevAwardScore += m_iPostPopupNextAwardScore;
						m_iPostPopupNextAwardScore = BB_CURRENCY_SILVERMEDAL;
						break;
					case 2:
						m_iPostPopupPrevAwardScore += m_iPostPopupNextAwardScore;
						m_iPostPopupNextAwardScore = BB_CURRENCY_GOLDMEDAL;
						break;
					case 3:
						m_iPostPopupPrevAwardScore += m_iPostPopupNextAwardScore;
						break;
				}

				if(m_iPostPopupAwardsShown < m_iPostPopupAwardsUnlocked)
				{
					switch(m_iPostPopupAwardsShown)
					{
						case 0:
							m_pBronzeMedalImage->SetHidden(false);
							m_pBronzeMedalImage->UpdateRenderables();
							m_pUnlockedSound->Play(g_xOptions.m_fSoundVolume);
							break;
						case 1:
							m_pSilverMedalImage->SetHidden(false);
							m_pSilverMedalImage->UpdateRenderables();
							m_pUnlockedSound->Play(g_xOptions.m_fSoundVolume);
							break;
						case 2:
							m_pGoldMedalImage->SetHidden(false);
							m_pGoldMedalImage->UpdateRenderables();
							m_pUnlockedSound->Play(g_xOptions.m_fSoundVolume);
							break;
					}

					m_iPostPopupAwardsShown++;
				}
				else
					m_iPostPopupStep += 2;
			}

			break;
		}
*/	}
	}
}

void xGame::AddFailedPostPopupWindow()
{
	AddWindow(BB_WINDOW_POSTPOPUPFAILED);
}

void xGame::AddClientPostPopupWindow()
{
	xGUIWindowList *window = AddWindow(BB_WINDOW_POSTPOPUPCLIENT);
	if(window)
	{
		xGUIObjectList *curobj = window->m_pData->m_xChildObjectList.m_pNext;
		while(curobj)
		{
			if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE1))
			{
				if(m_xPlayer.m_bWonGame)
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_MULTIPLAYERWONLINE1);
				else
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_MULTIPLAYERLOSTLINE1);
			}
			else if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE2))
			{
				if(!g_xPaperAPI.m_xNetwork.IsConnected())
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_CLIENTSERVERQUITLINE2);
				else
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_CLIENTLINE2);
			}

			curobj = curobj->m_pNext;
		}
	}
	else
		assert(0);
}

void xGame::AddServerPostPopupWindow()
{
	xGUIWindowList *window = AddWindow(BB_WINDOW_POSTPOPUPSERVER);
	if(window)
	{
		xGUIObjectList *curobj = window->m_pData->m_xChildObjectList.m_pNext;
		while(curobj)
		{
			if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE1))
			{
				if(m_xPlayer.m_bWonGame)
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_MULTIPLAYERWONLINE1);
				else
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_MULTIPLAYERLOSTLINE1);
			}
			else if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE2))
			{
				if(!g_xPaperAPI.m_xNetwork.IsConnected())
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_SERVERCLIENTQUITLINE2);
				else
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_SERVERLINE2);
			}
			else if(!stricmp(curobj->m_pData->m_strName, BB_NAME_MESSAGEBOXLINE3))
			{
				if(!g_xPaperAPI.m_xNetwork.IsConnected())
					((xGUIText*)curobj->m_pData)->SetText(BB_STRING_POSTPOPUP_SERVERCLIENTQUITLINE3);
			}

			curobj = curobj->m_pNext;
		}
	}
	else
		assert(0);
}

void xGameRunPostLoadCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow)
{
	xFileIO fileio;
	xGUIImage *tempimage;
	char temptext[128];
	int tour, tourstart = 0, tempint, tempint2, i;
	
	if(!stricmp(strCommand, "InitializeServerSelectSongWindow"))
	{
		xGUIListBox *songlist = (xGUIListBox*)pWindow->m_pData->GetChildByName(strParams[0]);
		char curfilename[256] = {0};

		fileio.StartFindFiles("./music/*", true);

		while(fileio.GetNextFile(curfilename))
		{
			songlist->AddItem(curfilename);
			memset(curfilename, 0, 255);
		}

		fileio.EndFindFiles();

		((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0))->m_bToggled = true;
		((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1))->m_bToggled = false;
		((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY2))->m_bToggled = false;
	}
	else if(!stricmp(strCommand, "InitializeTourComplete"))
	{
		tempint = g_xGameOptions.m_iLevelIndex;
		for(i = 0; i < 7; i++)
		{
			if(tempint <= g_iTourLevelCount[i])
			{
				tour = i;
				break;
			}

			tempint -= g_iTourLevelCount[i];
			tourstart += g_iTourLevelCount[i];
		}

		tempint = 0; tempint2 = 0;
		for(i = 1; i <= g_iTourLevelCount[tour]; i++)
		{
			tempint += g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][tourstart + i];

			if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][tourstart + i] == 100)
				tempint2++;
		}

		sprintf(temptext, BB_IMAGE_PATH, g_xGameOptions.m_strBGFile);
		tempimage = (xGUIImage*)pWindow->m_pData->GetChildByName("GameBackground");
		tempimage->SetTexture(temptext, 0);
		tempimage->SetX(-pWindow->m_pData->GetX() - g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX);
		tempimage->SetY(-pWindow->m_pData->GetY() - g_xPaperAPI.m_iScreenOffsetY / g_xPaperAPI.m_fScreenRatioY);
		tempimage->SetWidth(1024.0f + 2.0f * (float)g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX);
		tempimage->SetHeight(768.0f + 2.0f * (float)g_xPaperAPI.m_iScreenOffsetY / g_xPaperAPI.m_fScreenRatioY);

		sprintf(temptext, "%i%%", (int)(((float)tempint / ((float)g_iTourLevelCount[tour] * 100.0f)) * 100.0));
		((xGUIText*)pWindow->m_pData->GetChildByName("TourPercentText"))->SetText(temptext);

		sprintf(temptext, "%i%%", (int)(((float)tempint2 / (float)g_iTourLevelCount[tour]) * 100.0));
		((xGUIText*)pWindow->m_pData->GetChildByName("TourDominateText"))->SetText(temptext);

		g_xGame.m_pCrowdLoopExcitedSound->Play(g_xOptions.m_fSoundVolume, 1);
		g_xGame.m_pCrowdGameWonSound->Play(g_xOptions.m_fSoundVolume);
		g_xGame.m_pCrowdPositiveSound->Play(g_xOptions.m_fSoundVolume);
		
		pWindow->m_pData->UpdateRenderables();

		g_xGame.Shutdown();
		g_bTourComplete = 1;
	}
	else if(!stricmp(strCommand, "InitSenseiPrePopup"))
	{
		tempimage = (xGUIImage*)pWindow->m_pData->GetChildByName("Sensei");
		tempimage->m_xAnimation.m_pCurKeyFrame->m_pNext->m_fX = 1024.0f + (float)g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX - tempimage->GetWidth();
		tempimage->m_xAnimation.m_fCurXChange = tempimage->m_xAnimation.m_pCurKeyFrame->m_fX - tempimage->m_xAnimation.m_pCurKeyFrame->m_pNext->m_fX;
	}
	else
		xMenuRunPostLoadCommand(strCommand, strParams, pWindow);
}

void xGameRunInputCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject)
{
	xFileIO fileio;

	if(!stricmp(strCommand, "OpenMenu"))
	{
		g_xGame.Shutdown();
		g_xMenu.PlayMusic();

		g_xMenu.m_xGUI.DelayedRemoveAllWindows();
		g_xMenu.AddMenuWindow(strParams[0]);

		g_pCurGameState = &g_xMenu;
		xGraphicsSetSceneGraph(&g_xMenu.m_xSceneGraph);

		g_xGame.m_xGUI.DelayedRemoveAllWindows();
	}
	else if(!stricmp(strCommand, "ShowWindow"))
	{
		g_xGame.AddWindow(strParams[0]);
	}
	else if(!stricmp(strCommand, "ShowWindowAndClose"))
	{
		g_xGame.AddWindow(strParams[0]);
		g_xGame.m_xGUI.DelayedRemoveWindow(pWindow);
	}
	else if(!stricmp(strCommand, "GotoBattleSelect"))
	{
		g_xGame.Shutdown();
		g_xMenu.PlayMusic();

		g_xMenu.m_xGUI.DelayedRemoveAllWindows();
		g_xMenu.AddMenuWindow(BB_WINDOW_MENU_BATTLES);

		g_pCurGameState = &g_xMenu;
		xGraphicsSetSceneGraph(&g_xMenu.m_xSceneGraph);

		g_xGame.m_xGUI.DelayedRemoveAllWindows();
	}
	else if(!stricmp(strCommand, "PlayAgain"))
	{
		xGameStopSounds();

		g_xGame.m_xGUI.DelayedRemoveAllWindows();

		g_xGame.m_pPrePopupPressEnterText = 0;
		g_xGame.m_pSenseiPrePopupMoreLinesText = 0;

		g_xGame.DelayedInit(g_xGameOptions.m_strLevelFile, 0.1f);
	}
	else if(!stricmp(strCommand, "Continue"))
	{
		xGameStopSounds();

		g_xGame.m_xGUI.DelayedRemoveWindow(pWindow);

		g_xGame.m_xPlayer.m_bReadyToMoveOn = 1;
	}
	else if(!stricmp(strCommand, "TourCompleteContinue"))
	{
		if(stricmp(g_xGameOptions.m_strNextLevelFile, "Credits"))
		{
			g_xGame.Shutdown();
			g_xMenu.PlayMusic();

			g_xMenu.m_xGUI.DelayedRemoveAllWindows();
			g_xMenu.AddMenuWindow(strParams[0]);

			g_pCurGameState = &g_xMenu;
			xGraphicsSetSceneGraph(&g_xMenu.m_xSceneGraph);

			g_xGame.m_xGUI.DelayedRemoveAllWindows();
		}
		else
		{
			g_xGame.m_xGUI.DelayedRemoveWindow(pWindow);
			g_xGame.m_xPlayer.m_bReadyToMoveOn = 1;
		}
	}
	else if(!stricmp(strCommand, "Close"))
		g_xGame.m_xGUI.DelayedRemoveWindow(pWindow);
	else if(!stricmp(strCommand, "UnpauseAndClose"))
	{
		g_xGame.m_xGUI.DelayedRemoveWindow(pWindow);
		g_xGame.UnPause();
	}
	else if(!stricmp(strCommand, "Ready"))
		g_xGame.Ready();
	else if(!stricmp(strCommand, "SendReady"))
		g_xGame.SendReady();
	else if(!stricmp(strCommand, "ServerRematch"))
	{
		if(g_xPaperAPI.m_xNetwork.IsConnected())
		{
			xGameNetwork::SendRematch();
			g_xGame.m_bReady = false;
			g_xGame.m_bSentReady = false;

			xGUIWindowList *window = g_xGame.m_xGUI.GetWindow(BB_WINDOW_POSTPOPUPSERVER);
			if(window)
			{
				((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_POSTPOPUP_SERVERWAITING2);
				((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_REMATCHBUTTON))->SetEnabled(false);
				((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_SELECTSONGBUTTON))->SetEnabled(false);
			}
		}
		else
		{
			g_xGame.m_xGUI.DelayedRemoveAllWindows();

			g_xGame.ResetListenServerConnection();

			g_xGame.m_xHTTPQuery.AddServerToList(1984, g_xGame.m_xPlayer.m_strAccountName, g_xGameOptions.m_strSongName, g_xGameOptions.m_iDifficultyLevel);

			g_xGame.DelayedInit("", 0.1f);
		}
	}
	else if(!stricmp(strCommand, "ClientAccept"))
	{
		xGameNetwork::SendAccept();

		g_xGame.m_xGUI.DelayedRemoveAllWindows();

		g_xGame.m_iConnectingStatus = 6;
		g_xGame.m_bFinishedConnecting = false;
		g_xGame.m_bReady = false;
		g_xGame.m_bSentReady = false;

		g_xGame.m_xSong.Stop();
		g_xGame.DelayedInit("", 0.3f);
	}
	else if(!stricmp(strCommand, "KickClient"))
	{
		g_xGame.ResetListenServerConnection();

		g_xGame.DelayedInit("", 0.3f);
	}
	else if(!stricmp(strCommand, "Disconnect"))
	{
		g_xPaperAPI.m_xNetwork.Close();
		g_xPaperAPI.m_xPingNetwork.Close();

		g_xGame.Shutdown();
		g_xMenu.PlayMusic();

		//Show the Multiplayer window
		g_xMenu.m_xGUI.DelayedRemoveAllWindows();
		g_xMenu.AddMainMenu();
		g_xMenu.AddMenuWindow(BB_WINDOW_MENU_MULTIPLAYER);

		//Remove our server from the Master Server
		if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
			g_xMenu.m_xHTTPQuery.RemoveServerFromList(1984, g_xGame.m_xPlayer.m_strAccountName);

		g_pCurGameState = &g_xMenu;
		xGraphicsSetSceneGraph(&g_xMenu.m_xSceneGraph);

		g_xGame.m_xGUI.DelayedRemoveAllWindows();
	}
	else if(!stricmp(strCommand, "ServerChangeSong"))
	{
		g_xGame.m_xGUI.DelayedRemoveAllWindows();
		g_xGame.m_xGUI.AddWindow(BB_WINDOW_SERVERSELECTSONG);
	}
	else if(!stricmp(strCommand, "ToggleSelectSongDifficulty"))
	{
		if(strParams[0][0] == '0')
		{
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0))->m_bToggled = true;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1))->m_bToggled = false;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY2))->m_bToggled = false;
		}
		else if(strParams[0][0] == '1')
		{
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0))->m_bToggled = false;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1))->m_bToggled = true;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY2))->m_bToggled = false;
		}
		else
		{
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0))->m_bToggled = false;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1))->m_bToggled = false;
			((xGUIButton*)pWindow->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY2))->m_bToggled = true;
		}
	}
	else if(!stricmp(strCommand, "ServerSelectNextSong"))
	{
		xGUIWindowList *windowitem = g_xGame.m_xGUI.GetWindow(BB_WINDOW_SERVERSELECTSONG);
		if(windowitem)
		{
			strcpy(g_xGameOptions.m_strSongName, ((xGUIListBox*)windowitem->m_pData->GetChildByName(strParams[0]))->GetSelectedString());

			if(((xGUIButton*)windowitem->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY0))->m_bToggled)
				g_xGameOptions.m_iDifficultyLevel = BB_DIFFICULTY_NOVICE;
			else
			{
				if(((xGUIButton*)windowitem->m_pData->GetChildByName(BB_NAME_HOSTDIFFICULTY1))->m_bToggled)
					g_xGameOptions.m_iDifficultyLevel = BB_DIFFICULTY_ADVANCED;
				else
					g_xGameOptions.m_iDifficultyLevel = BB_DIFFICULTY_EXPERT;
			}

			if(g_xPaperAPI.m_xNetwork.IsConnected())
			{
				xGameNetwork::SendNextSong(g_xGameOptions.m_strSongName, g_xGameOptions.m_iDifficultyLevel);
				g_xGame.m_bReady = false;
				g_xGame.m_bSentReady = false;
			}
			else
			{
				g_xGame.m_xGUI.DelayedRemoveAllWindows();

				g_xGame.ResetListenServerConnection();

				g_xGame.m_xHTTPQuery.AddServerToList(1984, g_xGame.m_xPlayer.m_strAccountName, g_xGameOptions.m_strSongName, g_xGameOptions.m_iDifficultyLevel);

				g_xGame.DelayedInit("", 0.3f);
			}
		}
	}
	else if(!stricmp(strCommand, "NeverShowServerPortsMessage"))
	{
		g_xCurrentProfile.m_byteNoServerPortWarning = 1 - g_xCurrentProfile.m_byteNoServerPortWarning;
		g_xOptions.SavePlayerProfile(&g_xCurrentProfile);
	}
	else
		xMenuRunInputCommand(strCommand, strParams, pWindow, pObject);
}

void xGameRunKeyCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject)
{
	xMenuRunKeyCommand(strCommand, strParams, pWindow, pObject);
}

void xGameRunHoverCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xGUIObject* &pOutHoverObject)
{
	xMenuRunHoverCommand(strCommand, strParams, pWindow, pObject, pOutHoverObject);
}

void xGameRunUnHoverCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject)
{
	xMenuRunUnHoverCommand(strCommand, strParams, pWindow, pObject);
}

void xGameRunDragCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xRenderable2DQuad* &pOutDragObject)
{
	xMenuRunDragCommand(strCommand, strParams, pWindow, pObject, pOutDragObject);
}

void xGameRunDropCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject)
{
	xMenuRunDropCommand(strCommand, strParams, pWindow, pObject);
}

void xGame::ReplacePrePopupText(char *strDestination, const char *strSource)
{
	int length, length2, i, j, k;
	char temp[1024], *newstring;

	j = 0;
	length = strlen(strSource);

	strcpy(temp, strSource);

	for(i = 0; i < length; i++)
	{
		if(temp[i] == '%')
		{
			if(i > 0 && temp[i - 1] == '\\')
				strDestination[j - 1] = temp[i];
			else
			{
				*strchr(&temp[i + 1], '%') = '\0';

				if(!stricmp(&temp[i + 1], "red"))
					newstring = g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[KEY_RED]);
				else if(!stricmp(&temp[i + 1], "blue"))
					newstring = g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[KEY_BLUE]);
				else if(!stricmp(&temp[i + 1], "yellow"))
					newstring = g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[KEY_YELLOW]);
				else if(!stricmp(&temp[i + 1], "rotatecw"))
					newstring = g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[KEY_ROTATEBOARDCLOCKWISE_ALT]);
				else if(!stricmp(&temp[i + 1], "rotateccw"))
					newstring = g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[KEY_ROTATEBOARDCOUNTERCLOCKWISE_ALT]);
				else if(!stricmp(&temp[i + 1], "rotatepiece"))
					newstring = g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[KEY_ROTATEPIECE_ALT]);
				else if(!stricmp(&temp[i + 1], "droppiece"))
					newstring = g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[KEY_DROPPIECE_ALT]);
				else if(!stricmp(&temp[i + 1], "clear"))
					newstring = g_xPaperAPI.m_xInput.GetStringFromKey(g_xOptions.m_pButtons[KEY_CLEAR]);

				length2 = strlen(newstring);

				for(k = 0; k < length2; k++)
					strDestination[j++] = newstring[k];

				i += strlen(&temp[i]);
			}
		}
		else
			strDestination[j++] = temp[i];
	}

	strDestination[j] = '\0';
}

void xGame::UpdatePrePopupText()
{
	char temp[256];
	int length, i, j;

	xGUIWindow *window = m_xGUI.GetWindow(BB_WINDOW_PREPOPUP)->m_pData;

	length = strlen(g_xGameOptions.m_strPrePopup);

	for(i = 0; i < BB_PREPOPUP_LINESVISIBLE; i++)
	{
		sprintf(temp, BB_NAME_MESSAGEBOXLINEX, i + 1);
		xGUIText* text = (xGUIText*)window->GetChildByName(temp);

		memset(temp, 0, 256);

		j = 0;
		for(; m_iPrePopupTextIndex < length; m_iPrePopupTextIndex++)
		{
			temp[j++] = g_xGameOptions.m_strPrePopup[m_iPrePopupTextIndex];

			if(xFontGetWidth(text->m_pFont, temp) > text->GetWidth())
			{
				while(temp[j] != ' ')
				{
					m_iPrePopupTextIndex--;
					j--;
				}

				temp[j] = '\0';
				m_iPrePopupTextIndex += 2;

				break;
			}
		}

		text->SetText(temp);
	}

	m_bPrePopupMoreLines = m_iPrePopupTextIndex < length;

	window->UpdateRenderables();
}

void xGame::UpdateSenseiPrePopupText()
{
	char temp[256];
	int length, i, j;

	xGUIWindow *window = m_xGUI.GetWindow(BB_WINDOW_SENSEIPREPOPUP)->m_pData;

	length = strlen(g_xGameOptions.m_strSenseiPrePopup);

	for(i = 0; i < BB_SENSEIPREPOPUP_LINESVISIBLE; i++)
	{
		sprintf(temp, BB_NAME_MESSAGEBOXLINEX, i + 1);
		xGUIText* text = (xGUIText*)window->GetChildByName(temp);

		memset(temp, 0, 256);

		j = 0;
		for(; m_iSenseiPrePopupTextIndex < length; m_iSenseiPrePopupTextIndex++)
		{
			temp[j++] = g_xGameOptions.m_strSenseiPrePopup[m_iSenseiPrePopupTextIndex];

			if(xFontGetWidth(text->m_pFont, temp) > text->GetWidth())
			{
				while(temp[j] != ' ')
				{
					m_iSenseiPrePopupTextIndex--;
					j--;
				}

				temp[j] = '\0';
				m_iSenseiPrePopupTextIndex += 2;

				break;
			}
		}

		text->SetText(temp);
	}

	m_bSenseiPrePopupMoreLines = m_iSenseiPrePopupTextIndex < length;

	m_pSenseiPrePopupMoreLinesText->SetHidden(!m_bSenseiPrePopupMoreLines);

	window->UpdateRenderables();
}

void xGame::Ready()
{
	char temptext[256];
	
	if(m_pVoiceSoundPlaying && m_pVoiceSoundPlaying->IsPlaying())
	{
		m_pVoiceSoundPlaying->Stop();
		m_pVoiceSoundPlaying = 0;
	}

	if(m_bSenseiPrePopup)
	{
		if(!m_bSenseiPrePopupMoreLines)
		{
			m_xGUI.DelayedRemoveWindow(BB_WINDOW_SENSEIPREPOPUP);
			m_bSenseiPrePopup = 0;
			
			if(m_bShowTutorialDimmer && m_pTutorialDimmer)
				xRenderable2DQuadUpdateVisibility(m_pTutorialDimmer, 1);

			if(g_xGameOptions.m_strPrePopup[0] == '\0')
			{
				m_bPrePopup = 0;
				m_pPrePopupPressEnterText = 0;

				if(g_xGameOptions.m_bTutorial)
				{
					if(m_iTutorialStep == 1)
						m_dwGameStartTime = g_xPaperAPI.m_xPlatform.GetTime();

					xGameOnPlayerAnimationCompleted(m_pPlayerModel);
					xGameOnEnemyAnimationCompleted(m_pEnemyModel);

					m_iGameSyncFrameCount = 0;
				}
				else
				{
					m_dCountdownTime = 2.9999999;
					m_bCountdown = 1;
				}

				UnPause();
			}
			else
			{
				m_bPrePopup = m_bRequireEnterPrePopup;

				if(!m_bPrePopup && m_bPaused)
				{
					if(g_xGameOptions.m_bTutorial)
					{
						if(m_iTutorialStep == 1)
							m_dwGameStartTime = g_xPaperAPI.m_xPlatform.GetTime();

						xGameOnPlayerAnimationCompleted(m_pPlayerModel);
						xGameOnEnemyAnimationCompleted(m_pEnemyModel);

						m_iGameSyncFrameCount = 0;
					}
					else
					{
						m_dCountdownTime = 2.9999999;
						m_bCountdown = 1;
					}

					UnPause();
				}

				AddPrePopupWindow();

				if(m_iTutVoiceIndex != -1)
				{
					if(m_pTutorialVoice)
						g_xPaperAPI.m_xAudio.UnloadSound(m_pTutorialVoice);
					sprintf(temptext, BB_SOUND_TUTORIALVOICE, m_iTutVoiceIndex + 1);
					m_pTutorialVoice = g_xPaperAPI.m_xAudio.LoadSound(temptext);
					PlayVoiceSound(m_pTutorialVoice);
					m_iTutVoiceIndex = -1;
				}

				m_pPrePopupPressEnterText->SetHidden(!m_bPrePopup);
				m_pPrePopupPressEnterText->UpdateRenderables();
			}
		}
		else
		{
			UpdateSenseiPrePopupText();

			if(m_iSenseiTutVoiceIndex != -1)
			{
				if(m_pTutorialVoice)
					g_xPaperAPI.m_xAudio.UnloadSound(m_pTutorialVoice);
				sprintf(temptext, BB_SOUND_TUTORIALVOICE, m_iSenseiTutVoiceIndex + 1);
				m_pTutorialVoice = g_xPaperAPI.m_xAudio.LoadSound(temptext);
				PlayVoiceSound(m_pTutorialVoice);
				m_iSenseiTutVoiceIndex = -1;
			}
		}
	}
	else if(!m_bPrePopupMoreLines)
	{
		//DEBUGLOG("Closing PrePopup\n");

		m_bPrePopup = 0;
		m_pPrePopupPressEnterText = 0;

		m_xGUI.DelayedRemoveWindow(BB_WINDOW_PREPOPUP);

		if(g_xGameOptions.m_bTutorial)
		{
			HideAllKeys(0);

			if(m_iTutorialStep == 1)
				m_dwGameStartTime = g_xPaperAPI.m_xPlatform.GetTime();
			else
				m_dwGameStartTime = g_xPaperAPI.m_xPlatform.GetTime() - m_xSong.GetTime();

			xGameOnPlayerAnimationCompleted(m_pPlayerModel);
			xGameOnEnemyAnimationCompleted(m_pEnemyModel);

			m_iGameSyncFrameCount = 0;
		}
		else
		{
			m_dCountdownTime = 2.9999999;
			m_bCountdown = 1;
		}

		UnPause();
	}
	else
		UpdatePrePopupText();
}

void xGame::SendReady()
{
	if(!m_bSentReady)
	{
		xGameNetwork::SendReady();
		m_iConnectingStatus++;

		m_bSentReady = true;

		if(m_iConnectingStatus == 8)
			m_bFinishedConnecting = true;

		xGUIWindowList *window;
		if(g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
			window = m_xGUI.GetWindow(BB_WINDOW_PREPOPUPCLIENT);
		else
			window = m_xGUI.GetWindow(BB_WINDOW_PREPOPUPSERVER);

		if(window)
		{
			((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_READYBUTTON))->SetEnabled(false);

			if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
				((xGUIButton*)window->m_pData->GetChildByName(BB_NAME_KICKCLIENTBUTTON))->SetEnabled(false);

			((xGUIText*)window->m_pData->GetChildByName(BB_NAME_MESSAGEBOXLINE2))->SetText(BB_STRING_PREPOPUP_WAITINGREADY);
		}
	}
}

void xGame::Failed()
{
	if(m_pFailureVoiceSound)
		m_pFailureVoiceSound->Play(g_xOptions.m_fVoiceOverVolume);

	m_pPlayerModel->SetAnimation(BB_PLAYERANIMATION_LOST, g_xGameOptions.m_fBPM);
	m_pEnemyModel->SetAnimation(BB_ENEMYANIMATION_WON, g_xGameOptions.m_fBPM);
	m_pCrowdGameLostSound->Play(g_xOptions.m_fSoundVolume);
	m_bFailed = 1;
	Pause(g_xPaperAPI.m_bHasFocus);
}

void xGame::ShiftMessagesUp()
{
	char *deleteme = m_strTextMessages[BB_TEXTMESSAGES_MAX - 1];

	for(int i = BB_TEXTMESSAGES_MAX - 1; i >= 0 ; i--)
	{
		if(m_strTextMessages[i] == NULL)
			continue;

		m_strTextMessages[i + 1] = m_strTextMessages[i];
	}

	if(deleteme)
		delete [] deleteme;
}

FORCE_INLINE void UpdateFieldBlockColor(xBlock *pBlock, int iColorIndex)
{
	pBlock->m_iColorIndex = iColorIndex;
	xRenderable2DQuadUpdateColor(pBlock->m_pBlockObject, &g_colFilledBlockColors[iColorIndex]);
	xRenderable2DQuadUpdateVisibility(pBlock->m_pBlockObject, true);
	xRenderable2DQuadUpdateTexture(pBlock->m_pBlockObject, pBlock->m_pBlockTexture);

	xRenderable2DQuadUpdateVisibility(pBlock->m_pBlockGlowObject, true);
	xRenderable2DQuadUpdateColor(pBlock->m_pBlockGlowObject, &g_colBlockGlowColors[iColorIndex]);

	xRenderable2DQuadUpdateVisibility(pBlock->m_pBlockOverlayObject, true);
}

void xGame::InitTutorial()
{
	if(!m_pTutorialDimmer)
	{
		m_pTutorialDimmer = xSceneGraphCreateQuad(BB_IMAGE_DIMMER, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pTutorialDimmer, 1, BB_DEPTH_TUTORIALDIMMER, 0, 1);
		xRenderable2DQuadSetPosition(m_pTutorialDimmer, -g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX, -g_xPaperAPI.m_iScreenOffsetY / g_xPaperAPI.m_fScreenRatioY);
		xRenderable2DQuadSetSize(m_pTutorialDimmer, 1024.0f + 2.0f * (float)g_xPaperAPI.m_iScreenOffsetX / g_xPaperAPI.m_fScreenRatioX, 768.0f + 2.0f * (float)g_xPaperAPI.m_iScreenOffsetY / g_xPaperAPI.m_fScreenRatioY);
		xRenderable2DQuadUpdate(m_pTutorialDimmer);

		m_pSwipeGesture = xSceneGraphCreateQuad(BB_IMAGE_SWIPEGESTURE, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pSwipeGesture, 1, BB_DEPTH_TUTKEYS, 0, 1);
		xRenderable2DQuadSetPosition(m_pSwipeGesture, BB_TUT_GESTURE_LEFT, BB_TUT_GESTURE_TOP);
		xRenderable2DQuadSetSize(m_pSwipeGesture, 2.0f * m_pSwipeGesture->m_xRenderable2D.m_fWidth, 2.0f * m_pSwipeGesture->m_xRenderable2D.m_fHeight);
		
		m_pDoubleTapGesture = xSceneGraphCreateQuad(BB_IMAGE_DOUBLETAPGESTURE, 0);
		xSceneGraphInsertQuad(&m_xSceneGraph, m_pDoubleTapGesture, 1, BB_DEPTH_TUTKEYS, 0, 1);
		xRenderable2DQuadSetPosition(m_pDoubleTapGesture, BB_TUT_GESTURE_LEFT + (m_pSwipeGesture->m_xRenderable2D.m_fWidth - m_pDoubleTapGesture->m_xRenderable2D.m_fWidth) / 2.0f, BB_TUT_GESTURE_TOP);
		xRenderable2DQuadSetSize(m_pDoubleTapGesture, 2.0f * m_pDoubleTapGesture->m_xRenderable2D.m_fWidth, 2.0f * m_pDoubleTapGesture->m_xRenderable2D.m_fHeight);
	}

	xRenderable2DQuadSetRotation(m_pSwipeGesture, 0.0f);
	xRenderable2DQuadUpdate(m_pSwipeGesture);
	xRenderable2DQuadUpdate(m_pDoubleTapGesture);

	HideAllKeys(0);

	m_xPlayer.m_bAllowClearing = 0;

	m_bFinishedConnecting = 1;
}

void xGame::InitTutorialField()
{
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[0][5], 1);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[1][5], 1);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[2][5], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[2][4], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[2][3], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[3][5], 2);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[4][5], 2);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[5][5], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[5][4], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[6][5], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[6][4], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[7][5], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[10][5], 0);
	UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[11][5], 0);
	
	m_xPlayer.m_xField.UpdateGlowAlphas(0);
}

void xGame::UpdateTutorial(double dTime)
{
	xBlock *curblock;
	float tempfloat;
	bool filled, missed, success;
	int x, y, i;

	if(m_pSwipeGesture)
	{
		if(m_dSwipeGestureDelay <= 0.0)
		{
			xRenderable2DQuadUpdateAnimation(m_pSwipeGesture, dTime);
			
			if(m_pSwipeGesture->m_iCurrentTexture == 0 || m_pSwipeGesture->m_iCurrentTexture == m_pSwipeGesture->m_iNumTextures - 1)
				m_dSwipeGestureDelay = 0.75;
		}
		else
			m_dSwipeGestureDelay -= dTime;
			
		if(m_dDoubleTapDelay <= 0.0)
		{
			xRenderable2DQuadUpdateAnimation(m_pDoubleTapGesture, dTime);
			
			if(m_pDoubleTapGesture->m_iCurrentTexture == 0 || m_pDoubleTapGesture->m_iCurrentTexture == m_pDoubleTapGesture->m_iNumTextures - 1)
				m_dDoubleTapDelay = 0.5;
		}
		else
			m_dDoubleTapDelay -= dTime;
	}

	if(m_iTutorialStep == 0)
	{
		InitTutorial();

		ShowTutorialText(BB_STRING_TUTORIAL_RED, BB_TUTVOICE_RED, 0, BB_STRING_TUTORIAL_INTRO, BB_TUTVOICE_INTRO, BB_TUTVOICE_INTRO2);

		m_xPlayer.m_xPiece.m_iColorIndex = 0;

		m_xPlayer.m_xPiece.AddBlockDown(1);
		xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[1]);

		m_xPlayer.m_xPiece.AddBlockDown(2);
		xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[2]);

		m_xPlayer.m_xPiece.m_pCurBlock = m_xPlayer.m_xPiece.m_pBlockList;
		m_xPlayer.m_xPiece.m_iNumBlocks = 3;
		m_xPlayer.m_xPiece.m_iUnfilled = 3;

		xRenderable2DQuadSetPosition(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0f, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0f);
		xRenderable2DQuadSetSize(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, BB_LARGEBLOCKOFFSET, BB_LARGEBLOCKOFFSET);
		xRenderable2DQuadUpdate(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject);

		xRenderable2DQuadSetPosition(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockOverlayObject, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockOverlayObject, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight);
		xRenderable2DQuadUpdate(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockOverlayObject);

		m_xPlayer.m_xPiece.UpdateBeatMarkerLocation();

		InitTutorialField();

		m_xPlayer.m_xField.NewPiece();
		m_xPlayer.m_xField.UpdateShadow();

		m_dwTutorialPeteStartTime = g_xGame.m_dwCurrentTime;

		m_iTutorialStep++;
	}
	else if(m_iTutorialStep == 1)
	{
		if(!m_bSenseiPrePopup)
		{
			if(m_fBeatPercentage > 0.70f)
			{
				if(!m_bPlayedHitSound)
				{
					m_xPlayer.m_pHitTimeSounds[3 + rand() % 4]->Play(g_xOptions.m_fSoundVolume);
					m_bPlayedHitSound = 1;
				}
			}
			else
				m_bPlayedHitSound = 0;

			if(m_xPlayer.m_xPiece.m_iUnfilled < 3)
			{
				if(m_xPlayer.m_xPiece.m_pBlockList->m_iColorIndex != 0)
				{
					if(g_xOptions.m_pButtons[KEY_BLUE]->IsDown() || g_xOptions.m_pButtons[KEY_YELLOW]->IsDown())
						ShowTutorialText(BB_STRING_TUTORIAL_RED_WRONGKEY, BB_TUTVOICE_RED_WRONGKEY, 0);
					else
						ShowTutorialText(BB_STRING_TUTORIAL_RED_MISSEDBEAT, BB_TUTVOICE_RED_MISSEDBEAT, 0);

					m_xPlayer.m_xPiece.m_iUnfilled = 3;
					m_xPlayer.m_xPiece.m_pCurBlock = m_xPlayer.m_xPiece.m_pBlockList;
					m_xPlayer.m_xPiece.m_pPrevBlock = m_xPlayer.m_xPiece.m_pCurBlock->m_pPrev;
					m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = -1;
					m_xPlayer.m_xPiece.m_pCurBlock->m_iUnfilledColorIndex = 0;
					m_xPlayer.m_xPiece.m_fBuildAnimationTime = BB_PIECE_ANIMBUILDSUCCESSFUL + 0.1;
					m_xPlayer.m_xPiece.UpdateBeatMarkerLocation();
					xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[0]);
					m_xPlayer.m_xField.UpdateShadow();
				}
				else if(!m_bPaused)
				{
					ShowTutorialText(BB_STRING_TUTORIAL_BLUE, BB_TUTVOICE_BLUE, 0);
					m_iTutorialStep++;
					m_dwTutorialPeteStartTime = g_xGame.m_dwCurrentTime;
					m_bPlayedHitSound = 0;
				}
			}
		}
	}
	else if(m_iTutorialStep == 2)
	{
		if(!m_bSenseiPrePopup)
		{
			if(m_fBeatPercentage > 0.70f)
			{
				if(!m_bPlayedHitSound)
				{
					m_xPlayer.m_pHitTimeSounds[3 + rand() % 4]->Play(g_xOptions.m_fSoundVolume);
					m_bPlayedHitSound = 1;
				}
			}
			else
				m_bPlayedHitSound = 0;

			if(m_xPlayer.m_xPiece.m_iUnfilled < 2)
			{
				if(m_xPlayer.m_xPiece.m_pBlockList->m_pNext->m_iColorIndex != 0)
				{
					if(g_xOptions.m_pButtons[KEY_RED]->IsDown() || g_xOptions.m_pButtons[KEY_YELLOW]->IsDown())
						ShowTutorialText(BB_STRING_TUTORIAL_BLUE_WRONGKEY, BB_TUTVOICE_BLUE_WRONGKEY, 0);
					else
						ShowTutorialText(BB_STRING_TUTORIAL_BLUE_MISSEDBEAT, BB_TUTVOICE_BLUE_MISSEDBEAT, 0);

					m_xPlayer.m_xPiece.m_iUnfilled = 2;
					m_xPlayer.m_xPiece.m_pCurBlock = m_xPlayer.m_xPiece.m_pBlockList->m_pNext;
					m_xPlayer.m_xPiece.m_pPrevBlock = m_xPlayer.m_xPiece.m_pCurBlock->m_pPrev;
					m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = -1;
					m_xPlayer.m_xPiece.m_pCurBlock->m_iUnfilledColorIndex = 1;
					m_xPlayer.m_xPiece.m_fBuildAnimationTime = BB_PIECE_ANIMBUILDSUCCESSFUL + 0.1;
					m_xPlayer.m_xPiece.UpdateBeatMarkerLocation();
					xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[1]);
					m_xPlayer.m_xField.UpdateShadow();
				}
				else if(!m_bPaused)
				{
					ShowTutorialText(BB_STRING_TUTORIAL_YELLOW, BB_TUTVOICE_YELLOW, 0);
					m_iTutorialStep++;
					m_dwTutorialPeteStartTime = g_xGame.m_dwCurrentTime;
					m_bPlayedHitSound = 0;
				}
			}
			else
				m_xPlayer.m_xBeatManager.m_dwLastHitTime = -BB_HITTIME_SCALEOUT * 2;
		}
	}
	else if(m_iTutorialStep == 3)
	{
		if(!m_bSenseiPrePopup)
		{
			if(m_fBeatPercentage > 0.70f)
			{
				if(!m_bPlayedHitSound)
				{
					m_xPlayer.m_pHitTimeSounds[3 + rand() % 4]->Play(g_xOptions.m_fSoundVolume);
					m_bPlayedHitSound = 1;
				}
			}
			else
				m_bPlayedHitSound = 0;

			if(m_xPlayer.m_xPiece.m_iUnfilled < 1)
			{
				if(m_xPlayer.m_xPiece.m_pBlockList->m_pNext->m_pNext->m_iColorIndex != 0)
				{
					for (i = 0; i < 6; i++)
						xRenderable2DQuadUpdateVisibility(m_xPlayer.m_pColorButton[i], 1);
					
					if(g_xOptions.m_pButtons[KEY_RED]->IsDown() || g_xOptions.m_pButtons[KEY_BLUE]->IsDown())
						ShowTutorialText(BB_STRING_TUTORIAL_YELLOW_WRONGKEY, BB_TUTVOICE_YELLOW_WRONGKEY, 0);
					else
						ShowTutorialText(BB_STRING_TUTORIAL_YELLOW_MISSEDBEAT, BB_TUTVOICE_YELLOW_MISSEDBEAT, 0);

					m_xPlayer.m_xPiece.m_iUnfilled = 1;
					m_xPlayer.m_xPiece.m_pCurBlock = m_xPlayer.m_xPiece.m_pBlockList->m_pNext->m_pNext;
					m_xPlayer.m_xPiece.m_pPrevBlock = m_xPlayer.m_xPiece.m_pCurBlock->m_pPrev;
					m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = -1;
					m_xPlayer.m_xPiece.m_pCurBlock->m_iUnfilledColorIndex = 2;
					m_xPlayer.m_xPiece.m_fBuildAnimationTime = BB_PIECE_ANIMBUILDSUCCESSFUL + 0.1;
					m_xPlayer.m_xPiece.UpdateBeatMarkerLocation();
					xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[2]);
					m_xPlayer.m_xField.UpdateShadow();
				}
				else if(!m_bPaused)
				{
					Pause(g_xPaperAPI.m_bHasFocus);

					ShowRotateAndThrowKeys();
					ShowTutorialText(BB_STRING_TUTORIAL_ROTATEBOARD, BB_TUTVOICE_ROTATEBOARD, 0, BB_STRING_TUTORIAL_BUILDINGCOMPLETE, BB_TUTVOICE_BUILDINGCOMPLETE, BB_TUTVOICE_BUILDINGCOMPLETE2);
					m_iTutorialStep++;
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
					m_bPlayedHitSound = 0;
					m_iBoardRotateKeyPressed = 0;
				}
			}
			else
				m_xPlayer.m_xBeatManager.m_dwLastHitTime = -BB_HITTIME_SCALEOUT * 2;
		}
	}
	else if(m_iTutorialStep >= 4 && m_iTutorialStep <= 5)
	{
		if(m_iTutorialStep == 4)
		{
			if(!m_bSenseiPrePopup)
			{
				if(m_xPlayer.m_xPiece.m_bDropping)
				{
					if(m_xPlayer.m_xPiece.m_bTouchedDown)
						ResetTutorialOnDropFailure(BB_STRING_TUTORIAL_ROTATEBOARD_RESET, BB_TUTVOICE_ROTATEBOARD_RESET);
				}
				else if(g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 500)
				{
					if(m_xPlayer.m_xField.m_pFieldBlocks[3][5].m_iColorIndex >= 0 || g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 6000)
					{
						if(m_xPlayer.m_xField.m_pFieldBlocks[3][5].m_iColorIndex >= 0)
						{
							m_xPlayer.m_pSlideSound->Play(g_xOptions.m_fSoundVolume);
							m_xPlayer.m_xField.RotateFieldClockwise();
						}
						else
							ResetFieldRotation();

						m_xPlayer.m_xField.Update(0.0001f);

						m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
					}
				}

				if(m_iBoardRotateKeyPressed)
				{
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
					ResetFieldRotation();
					m_xPlayer.m_xBeatManager.m_dwLastHitTime = -BB_HITTIME_SCALEOUT * 2;
					m_iTutorialStep++;
				}
			}
		}
		else if(m_iTutorialStep == 5)
		{
			if(!m_bPrePopup)
			{
				if(m_xPlayer.m_xPiece.m_bDropping)
				{
					if(m_xPlayer.m_xPiece.m_bTouchedDown)
						ResetTutorialOnDropFailure(BB_STRING_TUTORIAL_ROTATEBOARD_RESET, BB_TUTVOICE_ROTATEBOARD_RESET);
				}
				else if(m_iBoardRotateKeyPressed < 3)
				{
					if(m_iBoardRotateKeyPressed == 1)
						m_xPlayer.m_xField.RotateFieldClockwise();
					else if(m_iBoardRotateKeyPressed == 2)
						m_xPlayer.m_xField.RotateFieldCounterClockwise();

					m_iBoardRotateKeyPressed = 3;
				}
				else if(m_xPlayer.m_xField.m_pFieldBlocks[3][5].m_iColorIndex < 0)
				{
					xRenderable2DQuadSetRotation(m_pSwipeGesture, 90.0f);
					xRenderable2DQuadUpdate(m_pSwipeGesture);
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
					ShowKeys(1, 0);
					ShowTutorialText(BB_STRING_TUTORIAL_ROTATEPIECE, BB_TUTVOICE_ROTATEPIECE, 0);
					m_iTutorialStep++;
					m_bAllowFieldRotation = 0;
					m_bPieceRotateKeyPressed = 0;
				}
				else if(!m_bSenseiPrePopup && g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 5000)
				{
					xRenderable2DQuadSetRotation(m_pSwipeGesture, 0.0f);
					xRenderable2DQuadUpdate(m_pSwipeGesture);
					ShowKeys(1, 0);
					ShowTutorialText(BB_STRING_TUTORIAL_ROTATEBOARD, BB_TUTVOICE_ROTATEBOARD, 0);
					m_iBoardRotateKeyPressed = 0;
					m_iTutorialStep--;
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
				}
			}
		}

		tempfloat = (float)((g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime) % 500) / 500.0f;
		tempfloat = tempfloat < 0.5f ? tempfloat / 0.5f : 1.0f - ((tempfloat - 0.5f) / 0.5f);
		for(i = 0; i < BB_FIELDWIDTH; i++)
		{
			if(m_xPlayer.m_xField.m_pFieldBlocks[i][5].m_iColorIndex < 0)
			{
				xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[i][5].m_pBlockGlowObject, m_iTutorialStep < 6);
				xRenderable2DQuadUpdateAlpha(m_xPlayer.m_xField.m_pFieldBlocks[i][5].m_pBlockGlowObject, tempfloat);
			}
			else
				xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[i][5].m_pBlockGlowObject, 0);
		}
	}
	else if(m_iTutorialStep == 6)
	{
		if(!m_bSenseiPrePopup)
		{
			if(m_xPlayer.m_xPiece.m_bDropping)
			{
				if(m_xPlayer.m_xPiece.m_bTouchedDown)
					ResetTutorialOnDropFailure(BB_STRING_TUTORIAL_ROTATEPIECE_RESET, BB_TUTVOICE_ROTATEBOARD_RESET);
			}
			else if(m_bPieceRotateKeyPressed)
			{
				if(m_xPlayer.m_xPiece.m_pBlockList->m_iRowID != m_xPlayer.m_xPiece.m_pCurBlock->m_iRowID)
				{
					m_xPlayer.m_xPiece.Rotate();
					m_xPlayer.m_xField.UpdateShadow();
				}

				xRenderable2DQuadSetRotation(m_pSwipeGesture, -90.0f);
				xRenderable2DQuadUpdate(m_pSwipeGesture);
				m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
				m_bPieceDropped = 0;
				m_bDropKeyPressed = 0;
				m_iTutorialStep += 2;
				ShowKeys(1, 0);
				ShowTutorialText(BB_STRING_TUTORIAL_DROPPIECE, BB_TUTVOICE_DROPPIECE, 0);
			}
			else
			{
				if(g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 1000)
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
			
				if(g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 5000)
				{
					xRenderable2DQuadSetRotation(m_pSwipeGesture, 90.0f);
					xRenderable2DQuadUpdate(m_pSwipeGesture);
					ShowKeys(1, 0);
					ShowTutorialText(BB_STRING_TUTORIAL_ROTATEPIECE, BB_TUTVOICE_ROTATEPIECE, 0);
					m_iTutorialStep--;
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
					m_bPieceRotateKeyPressed = 0;
				}
			}
		}
	}
	else if(m_iTutorialStep == 8)
	{
		if(!m_bSenseiPrePopup)
		{
			if(!m_bDropKeyPressed)
			{
				if(g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 1000)
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
			}
			else if(!m_bPieceDropped && m_dwTutorialTime > 0)
			{
				m_dwTutorialTime = 0;

				HideAllKeys(0);

				m_xPlayer.m_xField.DropPiece();
			}
			else if(m_bPieceDropped || m_xPlayer.m_xPiece.m_bDropping)
			{
				if(!m_bPieceDropped)
				{
					m_bPieceDropped = 1;
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
				}

				if(m_xPlayer.m_xPiece.m_bTouchedDown)
				{
					for(i = 0; i < BB_FIELDWIDTH; i++)
					{
						if(m_xPlayer.m_xField.m_pFieldBlocks[i][5].m_iColorIndex < 0)
						{
							ResetTutorialOnDropFailure(BB_STRING_TUTORIAL_DROPPIECE_RESET, BB_TUTVOICE_DROPPIECE_RESET);
							break;
						}
					}

					if(!m_bPaused && m_xPlayer.m_xPiece.m_bTouchedDown && g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 1200)
					{
						xRenderable2DQuadUpdateTexture(m_xPlayer.m_xField.m_pFieldBlocks[5][5].m_pBlockObject, m_xPlayer.m_xField.m_pFieldBlocks[5][7].m_pBlockTexture);

						m_xPlayer.m_xField.UpdateShadow();
						m_xPlayer.m_xPiece.Delete(1, 1, 1);
						m_xPlayer.m_xField.UpdateShadow();

						Pause(g_xPaperAPI.m_bHasFocus);

						m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;

						xRenderable2DQuadUpdateVisibility(m_pDoubleTapGesture, 1);
						m_bAllowFieldRotation = 1;
						m_xPlayer.m_bAllowClearing = 1;
						ShowTutorialText(BB_STRING_TUTORIAL_CLEAR, BB_TUTVOICE_CLEAR, 0, BB_STRING_TUTORIAL_POWERMOVES, BB_TUTVOICE_POWERMOVES);
						m_iTutorialStep++;
					}
				}
			}
		}
	}
	else if(m_iTutorialStep == 9)
	{
		if(m_xPlayer.m_iNumBlocksCleared > 0)
		{
			HideAllKeys(1);
			m_xPlayer.m_pBlocksClearedSound->Play(g_xOptions.m_fSoundVolume);
			m_iTutorialStep++;
		}
	}
	else if(m_iTutorialStep == 10)
	{
		if(!m_bPrePopup)
		{
			m_pPlayerModel->SetAnimation(BB_PLAYERANIMATION_MOVE1, g_xGameOptions.m_fBPM);
			m_pCrowdPositiveSound->Play(g_xOptions.m_fSoundVolume);

			m_xPlayer.m_dPlayerMoveApplauseDelay = 0.0;

			m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
			m_iTutorialStep++;
		}
	}
	else if(m_iTutorialStep == 11)
	{
		if(!m_bPaused && !m_bSenseiPrePopup && g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 3000)
		{
			ShowTutorialText(BB_STRING_TUTORIAL_CROWDBAR, BB_TUTVOICE_CROWDBAR, 0);
			m_iTutorialStep++;
		}
	}
	else if(m_iTutorialStep == 12)
	{
		if(!m_bPrePopup)
		{
			AdjustCrowdBar(-0.1f);

			m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
			m_iTutorialStep++;
		}
	}
	else if(m_iTutorialStep == 13)
	{
		if(!m_bPaused && !m_bSenseiPrePopup && g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 8000)
		{
			m_xPlayer.m_xMultiplier.MultiplierAddTime(10.0f);

			m_xPlayer.m_xPiece.Init(BB_PIECESTARTX, BB_PIECESTARTY, 3, false, true);

			for (i = 0; i < 6; i++)
				xRenderable2DQuadUpdateVisibility(m_xPlayer.m_pColorButton[i], 1);

			m_xPlayer.m_xPiece.AddBlockDown(2);
			xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[2]);

			m_xPlayer.m_xPiece.AddBlockDown(1);
			xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[1]);

			m_xPlayer.m_xPiece.m_pCurBlock = m_xPlayer.m_xPiece.m_pBlockList;
			m_xPlayer.m_xPiece.m_iNumBlocks = 3;
			m_xPlayer.m_xPiece.m_iUnfilled = 3;

			xRenderable2DQuadSetPosition(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0f, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0f);
			xRenderable2DQuadSetSize(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, BB_LARGEBLOCKOFFSET, BB_LARGEBLOCKOFFSET);
			xRenderable2DQuadUpdate(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject);

			xRenderable2DQuadSetPosition(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockOverlayObject, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
			xRenderable2DQuadSetSize(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockOverlayObject, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth, m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight);
			xRenderable2DQuadUpdate(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockOverlayObject);

			m_xPlayer.m_xPiece.UpdateBeatMarkerLocation();

			m_xPlayer.m_xField.NewPiece();
			m_xPlayer.m_xField.UpdateShadow();

			Pause(g_xPaperAPI.m_bHasFocus);

			ShowTutorialText(BB_STRING_TUTORIAL_WHITEBLOCKS_BUILD, BB_TUTVOICE_WHITEBLOCKS_BUILD, 0, BB_STRING_TUTORIAL_WHITEBLOCKS, BB_TUTVOICE_WHITEBLOCKS);
			m_iTutorialStep++;
		}
	}
	else if(m_iTutorialStep == 14)
	{
		if(!m_bPrePopup)
		{
			m_iTutorialStep++;
		}
	}
	else if(m_iTutorialStep == 15)
	{
		filled = 1;
		missed = 0;

		for(curblock = m_xPlayer.m_xPiece.m_pBlockList; curblock; curblock = curblock->m_pNext)
		{
			if(curblock->m_iColorIndex == -1)
			{
				filled = 0;
				break;
			}
			else if(curblock->m_iColorIndex != BB_COLOR_WHITE)
			{
				missed = 1;
				break;
			}
		}

		if(missed)
		{
			Pause(g_xPaperAPI.m_bHasFocus);

			curblock->m_iColorIndex = -1;
			curblock->m_iUnfilledColorIndex = (m_xPlayer.m_xPiece.m_iUnfilled == 2 ? 0 : (m_xPlayer.m_xPiece.m_iUnfilled == 1 ? 2 : 1));
			m_xPlayer.m_xPiece.m_iUnfilled++;
			m_xPlayer.m_xPiece.m_pCurBlock = curblock;
			m_xPlayer.m_xPiece.m_pPrevBlock = curblock->m_pPrev;
			m_xPlayer.m_xPiece.m_fBuildAnimationTime = BB_PIECE_ANIMBUILDSUCCESSFUL + 0.1;
			m_xPlayer.m_xPiece.UpdateBeatMarkerLocation();
			xRenderable2DQuadUpdateColor(curblock->m_pBlockObject, &g_colUnfilledBlockColors[curblock->m_iUnfilledColorIndex]);
			m_xPlayer.m_xField.UpdateShadow();

			for (i = 0; i < 6; i++)
				xRenderable2DQuadUpdateVisibility(m_xPlayer.m_pColorButton[i], 1);

			ShowTutorialText(BB_STRING_TUTORIAL_WHITEBLOCKS_MISSED, BB_TUTVOICE_WHITEBLOCKS_MISSED);
		}
		else if(filled)
		{
			ShowTutorialText(BB_STRING_TUTORIAL_WHITEBLOCKS_PLACE, BB_TUTVOICE_WHITEBLOCKS_PLACE, 0);
			m_bPieceDropped = 0;
			m_iTutorialStep++;
		}
		else if(!m_xPlayer.m_xMultiplier.TimerRunning())
		{
			Pause(g_xPaperAPI.m_bHasFocus);

			m_xPlayer.m_xMultiplier.MultiplierAddTime(10.0f);
			m_xPlayer.m_xPiece.m_iColorIndex = BB_COLOR_WHITE;

			for (i = 0; i < 6; i++)
				xRenderable2DQuadUpdateVisibility(m_xPlayer.m_pColorButton[i], 1);
				
			ShowTutorialText(BB_STRING_TUTORIAL_WHITEBLOCKS_TIMEUP, BB_TUTVOICE_WHITEBLOCKS_TIMEUP);
		}
	}
	else if(m_iTutorialStep == 16)
	{
		if(m_bPieceDropped || m_xPlayer.m_xPiece.m_bDropping)
		{
			m_bPieceDropped = 1;

			if(m_xPlayer.m_xPiece.m_bTouchedDown)
			{
				success = 0;

				for(i = 0; i < BB_FIELDWIDTH; i++)
				{
					if(m_xPlayer.m_xField.m_pFieldBlocks[i][4].m_iColorIndex == BB_COLOR_WHITE &&
					   m_xPlayer.m_xField.m_pFieldBlocks[(i - 2) % BB_FIELDWIDTH][4].m_iColorIndex == BB_COLOR_WHITE)
					{
						success = 1;
						break;
					}
				}

				if(success)
				{
					m_xPlayer.m_xField.UpdateShadow();
					m_xPlayer.m_xPiece.Delete(true, true, true);
					m_xPlayer.m_xField.UpdateShadow();
					m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
					ShowTutorialText(BB_STRING_TUTORIAL_WHITEBLOCKS_CLEAR, BB_TUTVOICE_WHITEBLOCKS_CLEAR, 0);
					m_iTutorialStep++;
				}
				else
				{
					Pause(g_xPaperAPI.m_bHasFocus);

					for(y = 0; y < BB_FIELDHEIGHT; y++)
					{
						for(x = 0; x < BB_FIELDWIDTH; x++)
						{
							m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_iColorIndex = -1;
							xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockObject, false);
							xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockGlowObject, false);
							xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockOverlayObject, false);
						}
					}

					UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[0][5], 1);
					UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[1][5], 1);
					UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[2][5], 0);
					UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[2][4], 0);
					UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[2][3], 0);
					UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[3][5], 2);
					UpdateFieldBlockColor(&m_xPlayer.m_xField.m_pFieldBlocks[4][5], 2);

					m_xPlayer.m_xPiece.Init(BB_PIECESTARTX, BB_PIECESTARTY, 3, false, true);

					m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = BB_COLOR_WHITE;
					xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colFilledBlockColors[BB_COLOR_WHITE]);
					xRenderable2DQuadUpdateTexture(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, m_xPlayer.m_xPiece.m_pBlockTextures[0]);

					m_xPlayer.m_xPiece.AddBlockDown(0);
					m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = BB_COLOR_WHITE;
					xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colFilledBlockColors[BB_COLOR_WHITE]);
					xRenderable2DQuadUpdateTexture(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, m_xPlayer.m_xPiece.m_pBlockTextures[0]);

					m_xPlayer.m_xPiece.AddBlockDown(0);
					m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = BB_COLOR_WHITE;
					xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colFilledBlockColors[BB_COLOR_WHITE]);
					xRenderable2DQuadUpdateTexture(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, m_xPlayer.m_xPiece.m_pBlockTextures[0]);

					m_xPlayer.m_xPiece.m_pPrevBlock = m_xPlayer.m_xPiece.m_pCurBlock;

					m_xPlayer.m_xPiece.m_iUnfilled = 0;

					m_xPlayer.m_xField.UpdateShadow();

					for (i = 0; i < 6; i++)
						xRenderable2DQuadUpdateVisibility(m_xPlayer.m_pColorButton[i], 1);
					
					ShowTutorialText(BB_STRING_TUTORIAL_WHITEBLOCKS_RESET, BB_TUTVOICE_WHITEBLOCKS_RESET);
				}
			}
		}
	}
	else if(m_iTutorialStep == 17)
	{
		if(m_xPlayer.m_iNumBlocksCleared > 15)
		{
			m_xPlayer.BlocksCleared(16, 0, 0);
			AdjustCrowdBar(-1.0f);
			m_iTutorialStep++;

			if(m_pVoiceSoundPlaying == m_pTutorialVoice)
				m_pVoiceSoundPlaying = 0;
			
			if(m_pTutorialVoice)
				g_xPaperAPI.m_xAudio.UnloadSound(m_pTutorialVoice);
			
			m_pTutorialVoice = 0;
		}
		else if(g_xPaperAPI.m_dwCurrentTime - m_dwTutorialTime > 5000)
		{
			ShowTutorialText(BB_STRING_TUTORIAL_CLEAR_REMINDER, BB_TUTVOICE_CLEAR_REMINDER, 0);
			m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
		}
		else if(m_bPrePopup)
			m_dwTutorialTime = g_xPaperAPI.m_dwCurrentTime;
	}
}

void xGame::ShowTutorialText(const char *strText, int iTutVoiceIndex, bool bRequireEnterPress, const char *strSenseiText, int iSenseiTutVoiceIndex, int iSenseiTutVoiceIndex2)
{
	char temptext[256];
	
	if(m_pVoiceSoundPlaying && m_pVoiceSoundPlaying->IsPlaying())
	{
		m_pVoiceSoundPlaying->Stop();
		m_pVoiceSoundPlaying = 0;
	}

	m_bPrePopup = bRequireEnterPress || strSenseiText != 0;
	m_bRequireEnterPrePopup = bRequireEnterPress;

	//DEBUGLOG("ShowTutorialText %s ||||| %s\n", strText, strSenseiText);

	g_xGameOptions.m_strPrePopup[0] = '.';
	g_xGameOptions.m_strPrePopup[1] = '\0';

	if(strSenseiText)
	{
		ReplacePrePopupText(g_xGameOptions.m_strSenseiPrePopup, strSenseiText);

		AddSenseiPrePopupWindow();

		m_pSenseiPrePopupSound->Play(g_xOptions.m_fSoundVolume);

		if(m_pTutorialVoice)
			g_xPaperAPI.m_xAudio.UnloadSound(m_pTutorialVoice);
		sprintf(temptext, BB_SOUND_TUTORIALVOICE, iSenseiTutVoiceIndex + 1);
		m_pTutorialVoice = g_xPaperAPI.m_xAudio.LoadSound(temptext);
		PlayVoiceSound(m_pTutorialVoice);
		m_iSenseiTutVoiceIndex = iSenseiTutVoiceIndex2;
		m_iTutVoiceIndex = iTutVoiceIndex;
	}
	else if(strText)
	{
		AddPrePopupWindow();
		m_pPrePopupPressEnterText->SetHidden(!m_bPrePopup);
		m_pPrePopupPressEnterText->UpdateRenderables();
		
		if(m_pTutorialVoice)
			g_xPaperAPI.m_xAudio.UnloadSound(m_pTutorialVoice);
		sprintf(temptext, BB_SOUND_TUTORIALVOICE, iTutVoiceIndex + 1);
		m_pTutorialVoice = g_xPaperAPI.m_xAudio.LoadSound(temptext);
		PlayVoiceSound(m_pTutorialVoice);
	}
}

void xGame::ShowRotateAndThrowKeys()
{
	ShowKeys(1, 0);
}

void xGame::ShowKeys(bool bShowBottomKey, bool bShowTutorialDimmer)
{
	m_bShowTutorialDimmer = bShowTutorialDimmer;
	
	xRenderable2DQuadUpdateVisibility(m_pTutorialDimmer, m_xGUI.GetWindow(BB_WINDOW_SENSEIPREPOPUP) == 0 && m_bShowTutorialDimmer);
	xRenderable2DQuadUpdateVisibility(m_pSwipeGesture, 1);
}

void xGame::HideAllKeys(bool bDelete)
{
	//DEBUGLOG("HideAllKeys %i\n", bDelete);
	if(m_pTutorialDimmer)
	{
		xRenderable2DQuadUpdateVisibility(m_pTutorialDimmer, 0);
		xRenderable2DQuadUpdateVisibility(m_pSwipeGesture, 0);
		xRenderable2DQuadUpdateVisibility(m_pDoubleTapGesture, 0);

		if(bDelete)
		{
			xSceneGraphDeleteQuad(&m_xSceneGraph, m_pTutorialDimmer, 1);
			xSceneGraphDeleteQuad(&m_xSceneGraph, m_pSwipeGesture, 1);
			xSceneGraphDeleteQuad(&m_xSceneGraph, m_pDoubleTapGesture, 1);

			m_pTutorialDimmer = 0;
			m_pSwipeGesture = 0;
		}
	}
}

void xGame::ResetFieldRotation()
{
	int x, y;

	for(y = 0; y < BB_FIELDHEIGHT; y++)
	{
		for(x = 0; x < BB_FIELDWIDTH; x++)
		{
			m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_iColorIndex = -1;
			xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockObject, false);
			xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockGlowObject, false);
			xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockOverlayObject, false);
		}
	}

	InitTutorialField();

	m_xPlayer.m_xField.UpdateShadow();

	m_xPlayer.m_xField.RotateFieldClockwise();
	m_xPlayer.m_xField.RotateFieldCounterClockwise();
}

void xGame::ResetTutorialOnDropFailure(const char *strText, int iTutVoiceIndex)
{
	int x, y, i;

	m_xPlayer.m_xPiece.m_bDropping = 0;
	m_xPlayer.m_xPiece.m_bTouchedDown = 0;
	m_bAllowFieldRotation = 1;
	m_iBoardRotateKeyPressed = 0;
	m_bPieceDropped = 0;
	m_iTutorialStep = 5;

	Pause(g_xPaperAPI.m_bHasFocus);

	for(y = 0; y < BB_FIELDHEIGHT; y++)
	{
		for(x = 0; x < BB_FIELDWIDTH; x++)
		{
			m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_iColorIndex = -1;
			xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockObject, 0);
			xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockGlowObject, 0);
			xRenderable2DQuadUpdateVisibility(m_xPlayer.m_xField.m_pFieldBlocks[x][y].m_pBlockOverlayObject, 0);
		}
	}

	InitTutorialField();

	m_xPlayer.m_xPiece.Init(BB_PIECESTARTX, BB_PIECESTARTY, 3, 0, 1);

	m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = 0;
	xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colFilledBlockColors[0]);
	xRenderable2DQuadUpdateTexture(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, m_xPlayer.m_xPiece.m_pBlockTextures[0]);

	m_xPlayer.m_xPiece.AddBlockDown(0);
	m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = 0;
	xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colFilledBlockColors[0]);
	xRenderable2DQuadUpdateTexture(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, m_xPlayer.m_xPiece.m_pBlockTextures[0]);

	m_xPlayer.m_xPiece.AddBlockDown(0);
	m_xPlayer.m_xPiece.m_pCurBlock->m_iColorIndex = 0;
	xRenderable2DQuadUpdateColor(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, &g_colFilledBlockColors[0]);
	xRenderable2DQuadUpdateTexture(m_xPlayer.m_xPiece.m_pCurBlock->m_pBlockObject, m_xPlayer.m_xPiece.m_pBlockTextures[0]);

	m_xPlayer.m_xPiece.m_pPrevBlock = m_xPlayer.m_xPiece.m_pCurBlock;

	m_xPlayer.m_xPiece.m_iUnfilled = 0;

	m_xPlayer.m_xField.UpdateShadow();
	
	for (i = 0; i < 6; i++)
		xRenderable2DQuadUpdateVisibility(m_xPlayer.m_pColorButton[i], 1);
	
	ShowTutorialText(strText, iTutVoiceIndex);
}

void xGameOnPlayerAnimationCompleted(xMD3Model *pModel, void *pCallbackObject)
{
	DEBUGANIMATION("xGameOnPlayerAnimationCompleted %i %i\n", pModel->m_iCurrentAnim, g_xGame.m_bFailed);

	if(g_bCreditsRolling)
		pModel->SetAnimation(BB_PLAYERANIMATION_IDLE1 + rand() % 12, g_xGameOptions.m_fBPM);
	else if(g_xGame.m_bPrePopup || g_xGame.m_bCountdown)
		pModel->SetAnimation(BB_PLAYERANIMATION_INTROLOOP, g_xGameOptions.m_fBPM);
	else if(g_xGame.m_xPlayer.m_bWonGame)
		pModel->SetAnimation(BB_PLAYERANIMATION_WONLOOP, g_xGameOptions.m_fBPM);
	else if(g_xGame.m_bFailed)
		pModel->SetAnimation(BB_PLAYERANIMATION_LOSTLOOP, g_xGameOptions.m_fBPM);
	else if(g_xGame.m_fCrowdBarPercentage < 0.25f)
		pModel->SetAnimation(BB_PLAYERANIMATION_IDLE1 + rand() % 2, g_xGameOptions.m_fBPM);
	else if(g_xGame.m_fCrowdBarPercentage < 0.50f)
		pModel->SetAnimation(BB_PLAYERANIMATION_IDLE2 + rand() % 2, g_xGameOptions.m_fBPM);
	else if(g_xGame.m_fCrowdBarPercentage < 0.75f)
		pModel->SetAnimation(BB_PLAYERANIMATION_IDLE3 + rand() % 2, g_xGameOptions.m_fBPM);
	else//if(g_xGame.m_fCrowdBarPercentage < 1.00f)
		pModel->SetAnimation(BB_PLAYERANIMATION_IDLE4 + rand() % 2, g_xGameOptions.m_fBPM);
}

void xGameOnEnemyAnimationCompleted(xMD3Model *pModel, void *pCallbackObject)
{
#if _DEBUG && 0
	int randnum = BB_ENEMYANIMATION_IDLE + rand() % 4;
	DEBUGLOG("Setting Enemy Animation: %i\n", randnum);
	pModel->SetAnimation(randnum, g_xGameOptions.m_fBPM);
#else
	if(g_xGame.m_bPrePopup || g_xGame.m_bCountdown)
		pModel->SetAnimation(BB_ENEMYANIMATION_INTROLOOP, g_xGameOptions.m_fBPM);
	else if(g_xGame.m_xPlayer.m_bWonGame)
		pModel->SetAnimation(BB_ENEMYANIMATION_LOSTLOOP, g_xGameOptions.m_fBPM);
	else if(g_xGame.m_bFailed)
		pModel->SetAnimation(BB_ENEMYANIMATION_WONLOOP, g_xGameOptions.m_fBPM);
	else
		pModel->SetAnimation(BB_ENEMYANIMATION_IDLE + rand() % 4, g_xGameOptions.m_fBPM);
#endif
}

FORCE_INLINE void xGamePrecache()
{
	xEmitter *pEmitter;

	pEmitter = new xEmitter();
	pEmitter->Init(BB_EMITTER_BLOCKCLEARED, 0.0, 0, NULL);
}

FORCE_INLINE void xGameUpdateCrowdBarArrows(xRenderable2DQuad *pCrowdBarArrow1, xRenderable2DQuad *pCrowdBarArrow2, float fCrowdBarPercentage)
{
	xColor arrowcolor;
	float angle1, angle2, radangle1, radangle2, percentage;

	angle1 = BB_CROWDBAR_ENEMY_ANGLE + (BB_CROWDBAR_PLAYER_ANGLE - BB_CROWDBAR_ENEMY_ANGLE) * (1.0f - fCrowdBarPercentage);
	angle2 = BB_CROWDBAR_PLAYER_ANGLE + (BB_CROWDBAR_PLAYER_ANGLE - BB_CROWDBAR_ENEMY_ANGLE) * fCrowdBarPercentage;
	
	radangle1 = angle1 * PI / 180.0f;
	radangle2 = angle2 * PI / 180.0f;

	if(fCrowdBarPercentage < 0.0625f)
		xColorInit(&arrowcolor, 49.0f / 255.0f, 255.0f / 255.0f, 78.0f / 255.0f, 1.0f);
	else if(fCrowdBarPercentage < 0.1975f)
	{
		percentage = 1.0f - ((fCrowdBarPercentage - 0.0625f) / 0.125f);
		xColorInit(&arrowcolor, 152.0f / 255.0f + (49.0f / 255.0f - 152.0f / 255.0f) * percentage, 233.0f / 255.0f + (255.0f / 255.0f - 233.0f / 255.0f) * percentage, 56.0f / 255.0f + (78.0f / 255.0f - 56.0f / 255.0f) * percentage, 1.0f);
	}
	else if(fCrowdBarPercentage < 0.3125f)
	{
		percentage = 1.0f - ((fCrowdBarPercentage - 0.1975f) / 0.125f);
		xColorInit(&arrowcolor, 203.0f / 255.0f + (152.0f / 255.0f - 203.0f / 255.0f) * percentage, 223.0f / 255.0f + (233.0f / 255.0f - 223.0f / 255.0f) * percentage, 47.0f / 255.0f + (56.0f / 255.0f - 47.0f / 255.0f) * percentage, 1.0f);
	}
	else if(fCrowdBarPercentage < 0.4375f)
	{
		percentage = 1.0f - ((fCrowdBarPercentage - 0.3125f) / 0.125f);
		xColorInit(&arrowcolor, 255.0f / 255.0f + (203.0f / 255.0f - 255.0f / 255.0f) * percentage, 212.0f / 255.0f + (223.0f / 255.0f - 212.0f / 255.0f) * percentage, 35.0f / 255.0f + (47.0f / 255.0f - 35.0f / 255.0f) * percentage, 1.0f);
	}
	else if(fCrowdBarPercentage < 0.5625f)
		xColorInit(&arrowcolor, 255.0f / 255.0f, 212.0f / 255.0f, 35.0f / 255.0f, 1.0f);
	else if(fCrowdBarPercentage < 0.6975f)
	{
		percentage = 1.0f - ((fCrowdBarPercentage - 0.5625f) / 0.125f);
		xColorInit(&arrowcolor, 251.0f / 255.0f + (255.0f / 255.0f - 251.0f / 255.0f) * percentage, 159.0f / 255.0f + (212.0f / 255.0f - 159.0f / 255.0f) * percentage, 26.0f / 255.0f + (35.0f / 255.0f - 26.0f / 255.0f) * percentage, 1.0f);
	}
	else if(fCrowdBarPercentage < 0.8125f)
	{
		percentage = 1.0f - ((fCrowdBarPercentage - 0.6975f) / 0.125f);
		xColorInit(&arrowcolor, 247.0f / 255.0f + (251.0f / 255.0f - 247.0f / 255.0f) * percentage, 106.0f / 255.0f + (159.0f / 255.0f - 106.0f / 255.0f) * percentage, 18.0f / 255.0f + (26.0f / 255.0f - 18.0f / 255.0f) * percentage, 1.0f);
	}
	else if(fCrowdBarPercentage < 0.9375f)
	{
		percentage = 1.0f - ((fCrowdBarPercentage - 0.8125f) / 0.125f);
		xColorInit(&arrowcolor, 238.0f / 255.0f + (247.0f / 255.0f - 238.0f / 255.0f) * percentage, 0.0f / 255.0f + (106.0f / 255.0f - 0.0f / 255.0f) * percentage, 0.0f / 255.0f + (18.0f / 255.0f - 0.0f / 255.0f) * percentage, 1.0f);
	}
	else //if(fCrowdBarPercentage < 1.00)
		xColorInit(&arrowcolor, 238.0f / 255.0f, 0.0f / 255.0f, 0.0f / 255.0f, 1.0f);
	
	xRenderable2DQuadSetPosition(pCrowdBarArrow1, BB_FIELDX + BB_CROWDBAR_ARROW_CENTERX + cos(radangle1) * BB_CROWDBAR_ARROW_DISTANCE - pCrowdBarArrow1->m_xRenderable2D.m_fWidth / 2.0f, BB_FIELDY + BB_CROWDBAR_ARROW_CENTERY - sin(radangle1) * BB_CROWDBAR_ARROW_DISTANCE - pCrowdBarArrow1->m_xRenderable2D.m_fHeight / 2.0f);
	xRenderable2DQuadSetPosition(pCrowdBarArrow2, BB_FIELDX + BB_CROWDBAR_ARROW_CENTERX + cos(radangle2) * BB_CROWDBAR_ARROW_DISTANCE - pCrowdBarArrow2->m_xRenderable2D.m_fWidth / 2.0f, BB_FIELDY + BB_CROWDBAR_ARROW_CENTERY - sin(radangle2) * BB_CROWDBAR_ARROW_DISTANCE - pCrowdBarArrow2->m_xRenderable2D.m_fHeight / 2.0f);
	xRenderable2DQuadSetRotation(pCrowdBarArrow1, -1.0f * (angle1 - 90.0f));
	xRenderable2DQuadSetRotation(pCrowdBarArrow2, -1.0f * (angle2 - 90.0f));
	xRenderable2DQuadUpdate(pCrowdBarArrow1);
	xRenderable2DQuadUpdate(pCrowdBarArrow2);
	xRenderable2DQuadUpdateColor(pCrowdBarArrow1, &arrowcolor);
	xRenderable2DQuadUpdateColor(pCrowdBarArrow2, &arrowcolor);
}

void xGame::UnlockAchievement(int iAchievementID)
{
	bool wasunlocked;
	int i;

	if(g_xCurrentProfile.m_byteAchievementStatus[iAchievementID - 1])
		return;

	wasunlocked = g_xCurrentProfile.m_byteAchievementStatus[iAchievementID - 1] != 0;

	g_xCurrentProfile.m_byteAchievementStatus[iAchievementID - 1] = 1;
	g_xOptions.SavePlayerProfile(&g_xCurrentProfile);

	m_iAchievementStatus[iAchievementID - 1] = 1;

	for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
	{
		if(!m_strAchievementsPending[i])
		{
			m_strAchievementsPending[i] = g_strAchievements[iAchievementID - 1];
			break;
		}
	}

	if(i < BB_PENDINGACHIEVEMENTS_MAX)
	{
		for(i = 0; i < BB_PENDINGACHIEVEMENTS_MAX; i++)
		{
			if(!m_iAchievementsUnlocking[i])
			{
				m_iAchievementsUnlocking[i] = iAchievementID;
				m_dwAchievementUnlockStart[i] = g_xPaperAPI.m_dwCurrentTime;
				xRenderable2DQuadUpdateTexture(m_pAchievementIcon[i], m_pAchievementIconTextures[iAchievementID - 1]);
				break;
			}
		}
	}

	if(!g_xCurrentProfile.m_strLoginName[0] || !g_xCurrentProfile.m_strLoginPassword[0] || wasunlocked)
		return;

	if(m_xHTTPQuery.GetQueryType() == BB_HTTPQUERY_NONE && m_strAchievementsPending[0])
		m_xHTTPQuery.UnlockAchievement(m_strAchievementsPending[0], g_xCurrentProfile.m_strLoginName, g_xCurrentProfile.m_strLoginPassword);
}

FORCE_INLINE void xGameStopSounds()
{
	g_xGame.m_xSong.Stop();

	if(g_xGame.m_pCrowdLoopSound)
	{
		g_xGame.m_pCrowdLoopSound->Stop();
		g_xGame.m_pCrowdLoopExcitedSound->Stop();
		g_xGame.m_pCrowdPositiveSound->Stop();
		g_xGame.m_pCrowdNegativeSound->Stop();
		g_xGame.m_pCrowdGameWonSound->Stop();
		g_xGame.m_pCrowdGameLostSound->Stop();
	}
	
	if(g_xGame.m_pVoiceSoundPlaying)
		g_xGame.m_pVoiceSoundPlaying->Stop();
}